package com.ihidea.core.support.orm.mybatis3.interceptor.dialect;

import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Component;

/**
 * 优化frame.core中的分页
 *
 * @author kevin
 */
@Component
public class MysqlDialect extends Dialect {

	protected CountSqlParser countSqlParser = new CountSqlParser();
	
	/**
	 * 是否支持limit
	 * @return 支持返回true，否则返回false
	 */
	@Override
	public boolean supportsLimit() {
		return true;
	}

	/**
	 * 是否支持偏移游标
	 * @return 支持返回true，否则返回false
	 */
	@Override
	public boolean supportsLimitOffset() {
		return true;
	}

	/**
	 * 将sql变成分页sql语句,提供将offset及limit使用占位符(placeholder)替换.
	 * @param sql 执行的sql语句
	 * @param offset 偏移量
	 * @param limit 范围大
	 * @return 添加分页后的sql语句
	 */
	@Override
	public String getLimitString(String sql, int offset, int limit) {
		final int temp = 11;
		final int tempInt = 100;
		sql = sql.trim();
		boolean isForUpdate = false;
		if (sql.toLowerCase().endsWith(" for update")) {
			sql = sql.substring(0, sql.length() - temp);
			isForUpdate = true;
		}

		StringBuffer pagingSelect = new StringBuffer(sql.length() + tempInt);
		
		// 如果sql中包含union，则使用全集分页；否则直接追加 limit
		if(StringUtils.containsIgnoreCase(sql, "union")) {
			pagingSelect.append("select * from (");
			pagingSelect.append(sql);
			pagingSelect.append(" ) mysqldialect1 limit " + offset + "," + limit);
		} else {
			pagingSelect.append(sql);
			pagingSelect.append(" limit " + offset + "," + limit);
		}

		if (isForUpdate) {
			pagingSelect.append(" for update");
		}

		return pagingSelect.toString();
	}

	@Override
	public String getCountString(String sql) {
		return countSqlParser.getSmartCountSql(sql);
	}

	@Override
	public String getSysdateString() {
		return "select now()";
	}

}
