package com.starcharge.wios.Import;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import cn.hutool.core.date.DateUtil;
import jxl.*;
import org.apache.commons.fileupload.disk.DiskFileItem;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ihidea.core.support.SpringContextLoader;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.servlet.ServletHolderFilter;
import com.ihidea.core.util.FileUtilsEx;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * excel导入控制类
 *
 * @author xgl
 * @version [版本号, 2018年7月25日]
 */
@RestController
@Api(tags = "导入模块")
public class ImportController {
    
    /**
     * excel导入数据解析
     *
     * @param file excel 表格
     * @param serviceName 服务类型名称
     * @param needUpload 是否需要上传文件至oss
     * @param storeName 上传文件存储bucket
     * @param uploadFilePath 上传文件路径
     * @return 导入异常信息
     */
    @PostMapping(value = "import.do")
    @ApiOperation(value = "导入excel")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "serviceName", value = "service名称", dataType = "String", paramType = "query", required = true),})
    public List<?> importCommon(HttpServletRequest request, String serviceName) {
        Map<String, Object> param = ServletHolderFilter.getContext().getParamMap();
        
        List<Object[]> fileList = new ArrayList<Object[]>();
        for (String nameKey : param.keySet()) {
            
            Object _obj = param.get(nameKey);
            
            if (_obj != null && _obj instanceof List && ((List)_obj).size() > 0) {
                
                Object fileItem = ((List)_obj).get(0);
                
                if (fileItem instanceof DiskFileItem) {
                    
                    // 如果是servlet2上传的文件
                    List<DiskFileItem> diskFileItemList = (List<DiskFileItem>)_obj;
                    
                    for (DiskFileItem file : diskFileItemList) {
                        
                        String fileName = FileUtilsEx.getFileNameByPath(file.getName());
                        
                        byte[] fileContent = file.get();
                        
                        fileList.add(new Object[]{fileName, fileContent});
                    }
                }
            }
        }
        
        Import importClass = null;
        if (!StringUtils.isEmpty(serviceName)) {
            // 获取serviceName对应的bean
            importClass = SpringContextLoader.getBean(serviceName, Import.class);
        }
        // 返回的结果list
        List<Object> list = new ArrayList<>();
        // if (!file.isEmpty()) {
        Workbook rwb = null;
        
        try {
            // 输入流获取对象
            rwb = Workbook.getWorkbook(new ByteArrayInputStream((byte[])fileList.get(0)[1]));
            // 获取页数数组
            Sheet[] sheetArr = rwb.getSheets();
            for (Sheet rs : sheetArr) {
                Cell[] headCells = rs.getRow(0);
                int headCellSize = headCells.length;
                // 每页行数
                for (int i = 0; i < rs.getRows(); i++) {
                    List<String> cellList = new ArrayList<>();
                    // 每行单元数组
                    Cell[] cells = rs.getRow(i);
                    for (Cell cell : cells) {
                        if(cell.getType() == CellType.DATE){
                            DateCell dc = (DateCell)cell;
                            cellList.add(DateUtil.date(dc.getDate()).toDateStr());
                        }else{
                            cellList.add(cell.getContents());
                        }
                    }
                    if (headCellSize > cells.length) {
                        for (int k = 0; k < headCellSize - cells.length; k++) {
                            cellList.add("");
                        }
                    }
                    list.add(cellList);
                }
            }
        } catch (Exception e) {
            throw new ServiceException(e.getMessage());
        } finally {
            // 关闭数据流
            if (rwb != null) {
                rwb.close();
            }
        }
        // } else {
        // throw new ServiceException("文件为空");
        // }
        if (importClass != null) {
            // 返回具体实现类处理结果
            List<?> returnList = importClass.handle(list);
            returnList.remove(0);
            return returnList;
        } else {
            list.remove(0);
            return list;
        }
    }
}
