package com.starcharge.wios.Import.impl;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.starcharge.wios.Import.Import;
import com.starcharge.wios.dao.DictionaryDao;
import com.starcharge.wios.dao.OemContractDao;
import com.starcharge.wios.dao.OemDao;
import com.starcharge.wios.dao.OrderDao;
import com.starcharge.wios.dao.RegionDao;
import com.starcharge.wios.dao.entity.Dictionary;
import com.starcharge.wios.dao.entity.Oem;
import com.starcharge.wios.dao.entity.Order;
import com.starcharge.wios.dao.entity.Region;
import com.starcharge.wios.enums.ContractStatus;
import com.starcharge.wios.enums.HouseType;
import com.starcharge.wios.utils.NumberUtil;
import com.starcharge.wios.vo.OemContractVo;

/**
 * 订单导入
 * 
 * @author xgl
 * @version [版本号, 2020年10月22日]
 */
@Service
public class OrderImport implements Import {
    
    @Autowired
    private OrderDao orderDao;
    
    @Autowired
    private OemDao oemDao;
    
    @Autowired
    private RegionDao regionDao;
    
    @Autowired
    private DictionaryDao dicDao;
    
    @Autowired
    private OemContractDao oemContractDao;
    
    @SuppressWarnings("unchecked")
    @Override
    public List<?> handle(List<Object> list) {
        List<String> headList = (List<String>)list.get(0);
        
        List<Object> newList = new ArrayList<>();
        List<Object> childNewList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            childNewList = new ArrayList<>();
            List<String> childList = (List<String>)list.get(i);
            for (int j = 0; j < childList.size() + 1; j++) {
                if (j == 0) {
                    childNewList.add("CA" + NumberUtil.getRandomOrderId(7));
                } else {
                    childNewList.add(childList.get(j - 1));
                }
            }
            newList.add(childNewList);
        }
        List<Dictionary> dicList = dicDao.selectValidByType("wallBox", null);
        Map<String, String> dicMap = dicList.stream().collect(Collectors.toMap(Dictionary::getName, Dictionary::getValue));
        boolean isYiZhuang = true;// 是否移桩订单
        if (headList.size() > 25) {
            isYiZhuang = false;
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        for (int i = 1; i < newList.size(); i++) {
            List<String> childList = (List<String>)newList.get(i);
            String orderId = childList.get(0);// 订单号
            String oemName = childList.get(1);// 主机厂名称
            String oemCode = childList.get(2);// 经销商机构代码
            String regionName = childList.get(3);// 区域名称
            String installAddress = childList.get(4);// 安装地址
            String userName = childList.get(5);// 姓名
            String userSex = childList.get(6);// 性别
            String userPhone = childList.get(7);// 手机
            String surveyBeforeSale = childList.get(8);// 是否为售车勘测
            String wallbox = childList.get(11);// 墙盒品牌
            String outOrderId = childList.get(14);// 车企安装单号
            String invoiceDate = childList.get(15);// 发票日期
            String oemAgentName = childList.get(16);// 经销商名称
            String estateType = childList.get(20);// 小区类型
            String hasIndependentParking = childList.get(21);// 是否独立车位
            String installPosition = childList.get(22);// 室内/室外
            String propertyAgree = childList.get(23);// 物业是否同意安装
            String electricExpand = childList.get(24);// 是否需要电力报装
            String needErectPile = childList.get(25);// 是否需要立柱
            String manuDiapatch = null;
            if (!isYiZhuang) {// 移桩订单不需要这个字段
                manuDiapatch = childList.get(26);// 是否需要人工派单
            }
            
            if (StringUtils.isEmpty(orderId)) {
                childList.add("error:缺少订单号");
                continue;
            }
            if (StringUtils.isEmpty(oemName)) {
                childList.add("error:缺少主机厂名称");
                continue;
            }
            if (StringUtils.isEmpty(installAddress)) {
                childList.add("error:缺少安裝地址");
                continue;
            }
            
            if (StringUtils.isEmpty(oemCode)) {
                childList.add("error:缺少主机厂代码");
                continue;
            }
            
            if (StringUtils.isEmpty(regionName)) {
                childList.add("error:缺少区域");
                continue;
            }
            
            if (StringUtils.isEmpty(wallbox)) {
                childList.add("error:缺少墙盒品牌");
                continue;
            }
            if (StringUtils.isEmpty(outOrderId)) {
                childList.add("error:缺少车企单号");
                continue;
            }
            if (StringUtils.isEmpty(userName)) {
                childList.add("error:缺少用户姓名");
                continue;
            }
            
            String wallboxValue = dicMap.get(wallbox);
            if (StringUtils.isEmpty(wallboxValue)) {
                childList.add("error:墙盒品牌不存在");
                continue;
            } else {
                childList.set(11, wallbox + "_" + wallboxValue);
            }
            
            // List<String> existOrderIdList = orderList.stream().filter(k ->
            // k.equals(orderId)).collect(Collectors.toList());
            // if (existOrderIdList.size() > 1) {
            // childList.add("error:订单号不可重复");
            // continue;
            // }
            
            Order order = orderDao.selectById(orderId);
            if (order != null) {
                childList.add("error:订单号已存在");
                continue;
            }
            
            List<Oem> oemList = oemDao.selectByNameAndCode(oemName, oemCode);
            if (oemList.isEmpty()) {
                childList.add("error:无此主机厂或已被禁用");
                continue;
            } else {
                childList.set(1, oemName + "_" + oemList.get(0).getId());
            }
            Integer oemId = oemList.get(0).getId();
            final int tempOemId = oemId.intValue();
            
            if (!StringUtils.isEmpty(oemAgentName)) {
                List<Oem> oemAgentList = oemDao.selectByName(oemAgentName);
                oemAgentList = oemAgentList.stream().filter(
                    item -> item.getParentId() != null && item.getParentId().intValue() == tempOemId && item.getStatus().intValue() == 1)
                    .collect(Collectors.toList());
                if (!oemAgentList.isEmpty()) {
                    oemId = oemAgentList.get(0).getId();
                    childList.set(16, oemAgentList.get(0).getName() + "_" + oemAgentList.get(0).getId());
                } else {
                    childList.add("error:无此经销商或已被禁用");
                    continue;
                }
            }
            
            List<OemContractVo> oemContractList = oemContractDao.selectByStatusAndOemId(oemId, ContractStatus.EFFECTIVE.getType());
            if (oemContractList.isEmpty()) {
                childList.add("error:该主机厂/经销商没有生效中的合约");
                continue;
            }
            
            Region region = regionDao.selectSingleByName(regionName);
            if (region == null) {
                childList.add("error:区域不存在");
                continue;
            } else {
                childList.set(3, regionName + "_" + region.getId());
            }
            
            if (!StringUtils.isEmpty(userSex)) {
                if ("男".equals(userSex)) {
                    childList.set(6, userSex + "_1");
                } else if ("女".equals(userSex)) {
                    childList.set(6, userSex + "_0");
                } else {
                    childList.add("error:性别不对");
                    continue;
                }
            }
            if (!StringUtils.isEmpty(userPhone)) {
                boolean phoneValidate = NumberUtil.validateMobilePhone(userPhone);
                if (!phoneValidate) {
                    childList.add("error:手机号格式不对");
                    continue;
                }
            } else {
                childList.add("error:缺少用户手机号");
                continue;
            }
            
            if (!StringUtils.isEmpty(invoiceDate)) {
                try {
                    sdf.parse(invoiceDate);
                } catch (ParseException e) {
                    childList.add("error:发票日期输入有误");
                    continue;
                }
            }
            
            if (!StringUtils.isEmpty(surveyBeforeSale)) {
                if ("是".equals(surveyBeforeSale)) {
                    childList.set(8, surveyBeforeSale + "_1");
                } else if ("否".equals(surveyBeforeSale)) {
                    childList.set(8, surveyBeforeSale + "_0");
                } else {
                    childList.add("error:是否为售车勘测字段输入有误");
                    continue;
                }
            } else {
                childList.add("error:是否为售车勘测字段缺少");
                continue;
            }
            
            if (!StringUtils.isEmpty(estateType)) {
                Integer houseType = HouseType.getTypeByValue(estateType);
                if (houseType == null) {
                    childList.add("error:小区类型输入有误");
                    continue;
                }
                childList.set(20, estateType + "_" + houseType);
            }
            
            if (!StringUtils.isEmpty(hasIndependentParking)) {
                if ("是".equals(hasIndependentParking)) {
                    childList.set(21, hasIndependentParking + "_1");
                } else if ("否".equals(hasIndependentParking)) {
                    childList.set(21, hasIndependentParking + "_0");
                } else {
                    childList.add("error:是否独立车位字段输入有误");
                    continue;
                }
            }
            
            if (!StringUtils.isEmpty(installPosition)) {
                if ("室内".equals(installPosition)) {
                    childList.set(22, installPosition + "_1");
                } else if ("室外".equals(installPosition)) {
                    childList.set(22, installPosition + "_2");
                } else {
                    childList.add("error:室内室外字段输入有误");
                    continue;
                }
            }
            
            if (!StringUtils.isEmpty(propertyAgree)) {
                if ("是".equals(propertyAgree)) {
                    childList.set(23, propertyAgree + "_1");
                } else if ("否".equals(propertyAgree)) {
                    childList.set(23, propertyAgree + "_0");
                } else {
                    childList.add("error:物业是否同意字段输入有误");
                    continue;
                }
            }
            
            if (!StringUtils.isEmpty(electricExpand)) {
                if ("是".equals(electricExpand)) {
                    childList.set(24, electricExpand + "_1");
                } else if ("否".equals(electricExpand)) {
                    childList.set(24, electricExpand + "_0");
                } else {
                    childList.add("error:是否需要电力报装字段输入有误");
                    continue;
                }
            }
            
            if (!StringUtils.isEmpty(needErectPile)) {
                if ("是".equals(needErectPile)) {
                    childList.set(25, needErectPile + "_1");
                } else if ("否".equals(needErectPile)) {
                    childList.set(25, needErectPile + "_0");
                } else {
                    childList.add("error:是否需要立柱字段输入有误");
                    continue;
                }
            }
            
            if (!isYiZhuang) {
                if (!StringUtils.isEmpty(manuDiapatch)) {
                    if ("是".equals(manuDiapatch)) {
                        childList.set(26, manuDiapatch + "_1");
                    } else if ("否".equals(manuDiapatch)) {
                        childList.set(26, manuDiapatch + "_0");
                    } else {
                        childList.add("error:是否人工派单字段输入有误");
                        continue;
                    }
                } else {
                    childList.add("error:是否人工派单字段缺少");
                    continue;
                }
            }
            
        }
        return newList;
    }
    
}
