package com.starcharge.wios.Import.impl;

import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.starcharge.warehouse.dto.materials.MaterialsDTO;
import com.starcharge.wios.Import.Import;
import com.starcharge.wios.dao.OemContractDao;
import com.starcharge.wios.dao.OemContractItemDao;
import com.starcharge.wios.dao.OemDao;
import com.starcharge.wios.dao.RegionDao;
import com.starcharge.wios.dao.entity.Oem;
import com.starcharge.wios.dao.entity.Region;
import com.starcharge.wios.enums.ContractStatus;
import com.starcharge.wios.service.WarehouseService;
import com.starcharge.wios.vo.OemContractItemVo;
import com.starcharge.wios.vo.OemContractVo;

@Service
public class SupplierContractPriceImport implements Import {
    
    @Autowired
    private RegionDao regionDao;
    
    @Autowired
    private OemDao oemDao;
    
    @Autowired
    private OemContractDao oemContractDao;
    
    @Autowired
    private OemContractItemDao oemContractItemDao;
    
    @Autowired
    private WarehouseService warehouseService;
    
    @SuppressWarnings("unchecked")
    @Override
    public List<?> handle(List<Object> list) {
        for (int i = 1; i < list.size(); i++) {
            List<String> childList = (List<String>)list.get(i);
            String regionName = childList.get(0);// 区域名称
            String oemName = childList.get(1);// 主机厂名称
            String deviceName = childList.get(2);// 设备名称
            // String surveyPrice = childList.get(3);// 勘测价格（不含税）
            // String installPrice = childList.get(4);// 勘测价格（含税）
            // String remark = childList.get(5);// 备注
            if (StringUtils.isEmpty(regionName) || StringUtils.isEmpty(oemName)) {
                childList.add("error:参数不能未空");
                continue;
            }
            Region region = regionDao.selectSingleByName(regionName);
            if (region == null) {
                childList.add("error:区域不存在");
                continue;
            } else {
                childList.set(0, regionName + "_" + region.getIdTree());
            }
            
            Oem oem = oemDao.selectSingleByName(oemName);
            if (oem == null) {
                childList.add("error:主机厂/经销商不存在");
                continue;
            } else {
                childList.set(1, oemName + "_" + oem.getId());
            }
            
            if (!StringUtils.isEmpty(deviceName)) {
                List<OemContractVo> oemContractList =
                    oemContractDao.selectByStatusAndOemId(oem.getId().intValue(), ContractStatus.EFFECTIVE.getType());
                if (oemContractList.isEmpty()) {
                    childList.add("error:该主机厂/经销商无此设备");
                    continue;
                }
                List<OemContractItemVo> contractItems = oemContractItemDao.selectByContractId(oemContractList.get(0).getId().intValue());
                for (OemContractItemVo vo : contractItems) {
                    MaterialsDTO dto = warehouseService.getMaterialById(vo.getItemId());
                    if (dto != null) {
                        vo.setItemCode(dto.getMaterialsCode());
                        vo.setItemModel(dto.getRegularModel());
                        vo.setItemName(dto.getMaterialsName());
                    }
                }
                
                List<OemContractItemVo> ociList =
                    contractItems.stream().filter(item -> deviceName.equals(item.getItemName())).collect(Collectors.toList());
                if (ociList.isEmpty()) {
                    childList.add("error:设备不存在");
                    continue;
                } else {
                    childList.set(2, deviceName + "_" + ociList.get(0).getItemId());
                }
            }
        }
        return list;
    }
    
}
