package com.starcharge.wios.app.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.base.util.Assert;
import com.starcharge.wios.app.dto.AppOrderDTO;
import com.starcharge.wios.app.dto.AppOrderDetailDTO;
import com.starcharge.wios.app.dto.AppOrderInstallDTO;
import com.starcharge.wios.app.dto.AppPayBillDTO;
import com.starcharge.wios.app.service.AppOrderService;
import com.starcharge.wios.dao.entity.MaterialConfig;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 订单相关接口
 * 
 * @author xgl
 * @version [版本号, 2020年10月27日]
 */
@RestController
@RequestMapping("api2")
@Api(tags = "订单相关接口")
public class AppOrderController {
    @Autowired
    private AppOrderService orderService;
    
    @GetMapping(value = "order/qryList")
    @ApiOperation(value = "查询订单列表")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "camsPhone", value = "cams手机号", dataType = "String", paramType = "query", required = true),})
    public BaseResponse<List<AppOrderDTO>> getList(String camsPhone) {
        Assert.notEmpty(camsPhone, "手机号不能为空");
        return new BaseResponse<>(orderService.getList(camsPhone));
    }
    
    @GetMapping(value = "order/qryInstallDetail")
    @ApiOperation(value = "查询安装订单详情")
    @ApiImplicitParams({@ApiImplicitParam(name = "orderId", value = "工单号", dataType = "String", paramType = "query", required = true),})
    public BaseResponse<AppOrderInstallDTO> qryInstallDetail(String orderId) {
        Assert.notEmpty(orderId, "工单号不能为空");
        AppOrderInstallDTO orderInstallDTO = orderService.getInstallDetail(orderId);
        return new BaseResponse<AppOrderInstallDTO>(orderInstallDTO);
    }
    
    @GetMapping(value = "order/qryOrderDetail")
    @ApiOperation(value = "查询订单详情")
    @ApiImplicitParams({@ApiImplicitParam(name = "orderId", value = "工单号", dataType = "String", paramType = "query", required = true),})
    public BaseResponse<AppOrderDetailDTO> qryOrderDetail(String orderId) {
        Assert.notEmpty(orderId, "工单号不能为空");
        AppOrderDetailDTO result = orderService.getOrderDetail(orderId);
        return new BaseResponse<AppOrderDetailDTO>(result);
    }
    
    @GetMapping(value = "order/qryExpandIntro")
    @ApiOperation(value = "查询报装说明")
    @ApiImplicitParams({@ApiImplicitParam(name = "orderId", value = "工单号", dataType = "String", paramType = "query", required = true),})
    public BaseResponse<String> getExpandIntro(String orderId) {
        Assert.notEmpty(orderId, "缺少参数");
        MaterialConfig mc = orderService.getExpandIntro(orderId);
        if (mc != null)
            return new BaseResponse<String>(mc.getContent());
        else
            return new BaseResponse<String>();
    }
    
    /**
     * 报装
     *
     * @param orderId
     * @param expandStatus
     * @param expandFailReason
     * @param expandFailOperate
     */
    @PostMapping(value = "order/expand")
    @ApiOperation(value = "报装")
    @ApiImplicitParams({@ApiImplicitParam(name = "orderId", value = "订单id", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "expandStatus", value = "报装状态 1：通过 2：不通过", dataType = "int", paramType = "query", required = true),
        @ApiImplicitParam(name = "expandFailReason", value = "报装不通过原因", dataType = "String", paramType = "query", required = false),
        @ApiImplicitParam(name = "expandFailOperate", value = "报装不通过后订单后续操作 1：换其他用电类型继续安装  2：不安装了", dataType = "int", paramType = "query", required = false),})
    public BaseResponse<Object> expand(String orderId, Integer expandStatus, String expandFailReason, Integer expandFailOperate) {
        if (StringUtils.isEmpty(orderId) || expandStatus == null) {
            throw new ServiceException("缺少参数");
        }
        if (expandStatus.intValue() != 1 && expandStatus.intValue() != 2) {
            throw new ServiceException("状态不合法");
        }
        orderService.expand(orderId, expandStatus, expandFailReason, expandFailOperate);
        return new BaseResponse<>();
    }
    
    /**
     * 查询账单列表
     *
     * @param camsPhone
     * @return
     */
    @GetMapping(value = "order/qryBillList")
    @ApiOperation(value = "查询账单列表")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "camsPhone", value = "cams手机号", dataType = "String", paramType = "query", required = false),
        @ApiImplicitParam(name = "orderId", value = "工单号", dataType = "String", paramType = "query", required = false),
        @ApiImplicitParam(name = "status", value = "状态 0-未支付 1-已支付", dataType = "int", paramType = "query", required = false),})
    public BaseResponse<List<AppPayBillDTO>> qryBillList(String camsPhone, String orderId, Integer status) {
        if (!StringUtils.isEmpty(camsPhone) && !StringUtils.isEmpty(orderId)) {
            throw new ServiceException("cams手机号和工单号不能同时为空");
        }
        List<AppPayBillDTO> list = orderService.qryBill(camsPhone, orderId, status);
        return new BaseResponse<>(list);
    }
}
