package com.starcharge.wios.app.controller;

import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.starcharge.wios.app.service.AppOrderService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 回调通知
 * 
 * @author xgl
 * @version [版本号, 2020年10月27日]
 */
@RestController
@RequestMapping("api2")
@Api(tags = "回调通知")
public class CallbackController {
    @Autowired
    private AppOrderService appOrderService;
    
    private Logger logger = LoggerFactory.getLogger(CallbackController.class);
    
    /**
     * 增项费用支付成功
     *
     * @param orderId
     * @param status
     * @return
     */
    @PostMapping(value = "order/paySuccess")
    @ApiOperation(value = "增项费用支付成功")
    @ApiImplicitParams({@ApiImplicitParam(name = "orderId", value = "工单号", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "thirdUserId", value = "用户唯一识别号", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "status", value = "支付状态 1：成功", dataType = "int", paramType = "query", required = true),
        @ApiImplicitParam(name = "payMethod", value = "支付方式 3：微信 4：支付宝", dataType = "int", paramType = "query", required = true),})
    public Map<String, Object> paySuccess(String orderId, String thirdUserId, Integer status, Integer payMethod) {
        logger.info("[增项费用支付成功回调]orderId={},thirdUserId={},status={}", orderId, thirdUserId, status);
        Map<String, Object> returnMap = new HashMap<String, Object>();
        if (StringUtils.isEmpty(orderId) || status == null || payMethod == null || StringUtils.isEmpty(thirdUserId)) {
            returnMap.put("code", 400);
            returnMap.put("msg", "缺少参数");
            return returnMap;
        }
        if (status.intValue() == 1)
            appOrderService.updateInstallOrderPayStatus(orderId, thirdUserId, 1, payMethod.intValue());
        returnMap.put("code", 200);
        returnMap.put("msg", "");
        return returnMap;
    }
}
