package com.starcharge.wios.app.service;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.wios.app.dto.AppOperateLogDTO;
import com.starcharge.wios.app.dto.AppOrderDTO;
import com.starcharge.wios.app.dto.AppOrderDetailDTO;
import com.starcharge.wios.app.dto.AppOrderInstallDTO;
import com.starcharge.wios.app.dto.AppOrderInstallDTO.AdditionFeeItem;
import com.starcharge.wios.app.dto.AppOrderInstallDTO.OrderScore;
import com.starcharge.wios.app.dto.AppPayBillDTO;
import com.starcharge.wios.bo.MaterialConfigBo;
import com.starcharge.wios.bo.OperateLogBo;
import com.starcharge.wios.dao.MaterialConfigDao;
import com.starcharge.wios.dao.OperateLogDao;
import com.starcharge.wios.dao.OrderDao;
import com.starcharge.wios.dao.OrderInstallDao;
import com.starcharge.wios.dao.OrderInstallItemDao;
import com.starcharge.wios.dao.entity.MaterialConfig;
import com.starcharge.wios.dao.entity.OperateLog;
import com.starcharge.wios.dao.entity.Order;
import com.starcharge.wios.dao.entity.OrderInstall;
import com.starcharge.wios.dao.entity.UserPayBill;
import com.starcharge.wios.enums.OperateLogType;
import com.starcharge.wios.enums.OrderStatus;
import com.starcharge.wios.service.UserPayBillService;
import com.starcharge.wios.vo.MaterialConfigVo;
import com.starcharge.wios.vo.OrderInstallItemVo;
import com.starcharge.wios.vo.OrderVo;
import com.starcharge.wios.vo.UserPayBillVo;

/**
 * app订单service
 * 
 * @author xgl
 * @version [版本号, 2020年10月27日]
 */
@Service
public class AppOrderService {
    @Autowired
    private OrderDao orderDao;
    
    @Autowired
    private OrderInstallDao orderInstallDao;
    
    @Autowired
    private OperateLogDao operateLogDao;
    
    @Autowired
    private MaterialConfigDao materialConfigDao;
    
    @Autowired
    private OrderInstallItemDao orderInstallItemDao;
    
    @Autowired
    private UserPayBillService userPayBillService;
    
    public List<AppOrderDTO> getList(String camsPhone) {
        List<AppOrderDTO> returnList = new ArrayList<AppOrderDTO>();
        List<OrderVo> orderList = orderDao.selectByCamsPhoneForApp(camsPhone);
        for (OrderVo ov : orderList) {
            if (ov.getStatus().intValue() == OrderStatus.CLOSED.getType()) {
                ov.setAppSort(4);
            } else if (ov.getStatus().intValue() == OrderStatus.RETURN_VISIT.getType()) {
                ov.setAppSort(3);
            } else if (ov.getStatus().intValue() == OrderStatus.INSTALL_EXAMINE_SUCCESS.getType()) {
                ov.setAppSort(2);
            } else {
                ov.setAppSort(1);
            }
        }
        // 排序
        orderList = orderList.stream().sorted(Comparator.comparing(OrderVo::getAppSort)).collect(Collectors.toList());
        
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        for (OrderVo ov : orderList) {
            Integer transformStatus = transformStatus(ov.getStatus(), ov.getBillStatus());
            AppOrderDTO order = new AppOrderDTO();
            order.setUserName(ov.getUserName());
            order.setUserSex(ov.getUserSex());
            order.setOrderId(ov.getId());
            order.setAddress(ov.getAddress());
            order.setContactName(ov.getInstallStaffName());// 电工名字
            order.setContactPhone(ov.getInstallStaffPhone());// 电工手机号
            // 查询最近一条操作日志
            OperateLog op = operateLogDao.selectLastestByTPId(ov.getId(), OperateLogType.ORDER_STATUS.getType());
            if (op != null)
                order.setTime(sdf.format(op.getCreateTime()));

            if (ov.getDispatchTime() != null)
                order.setDispatchTime(sdf.format(ov.getDispatchTime()));

            if (ov.getSurveyReserveTime() != null)
                order.setSurveyReserveTime(sdf.format(ov.getSurveyReserveTime()));
            
            OperateLog surveyStartLog =
                operateLogDao.selectLastestByTPId(ov.getId(), OperateLogType.ORDER_STATUS.getType(), OrderStatus.SURVEYING.getType() + "");
            if (surveyStartLog != null)
                order.setSurveyStartTime(sdf.format(surveyStartLog.getCreateTime()));
            
            if (ov.getInstallReserveTime() != null)
                order.setInstallReserveTime(sdf.format(ov.getInstallReserveTime()));
            
            OperateLog installStartLog =
                operateLogDao.selectLastestByTPId(ov.getId(), OperateLogType.ORDER_STATUS.getType(), OrderStatus.INSTALLING.getType() + "");
            if (installStartLog != null)
                order.setInstallStartTime(sdf.format(installStartLog.getCreateTime()));
            
            if (ov.getReturnVisitTime() != null)
                order.setReturnVisitTime(sdf.format(ov.getReturnVisitTime()));
            
            order.setRelativeStatus(transformStatus);
            order.setStatus(ov.getStatus());
            order.setStatusText(OrderStatus.getText(ov.getStatus()));
            // order.setFee(ov.getInstallAdditionTotalFee().add(ov.getExpandFee()));
            order.setFee(ov.getBillFee());
            if (ov.getInstallFinishTime() != null)
                order.setInstallFinishTime(sdf.format(ov.getInstallFinishTime()));
            order.setBillId(ov.getBillId());
            order.setBillStatus(ov.getBillStatus());
            order.setExpandStatus(ov.getExpandStatus());
            order.setExpandFailOperate(ov.getExpandFailOperate());
            returnList.add(order);
        }
        return returnList;
    }
    
    public Integer transformStatus(int sourceStatus, Integer billStatus) {
        List<Integer> receiveOrderStatus = new ArrayList<>();
        receiveOrderStatus.add(OrderStatus.DISPATCHING.getType());
        receiveOrderStatus.add(OrderStatus.SURVEY_RESERVING.getType());
        if (receiveOrderStatus.contains(sourceStatus)) {
            return 1;
        }
        List<Integer> surveyReserveStatus = new ArrayList<>();
        surveyReserveStatus.add(OrderStatus.SURVEY_DISPATCHED.getType());
        surveyReserveStatus.add(OrderStatus.SURVEY_RESERVED.getType());
        surveyReserveStatus.add(OrderStatus.SURVEY_RESERVE_EXCEPTION.getType());
        if (surveyReserveStatus.contains(sourceStatus)) {
            return 2;
        }
        List<Integer> surveyStatus = new ArrayList<>();
        surveyStatus.add(OrderStatus.SURVEYING.getType());
        surveyStatus.add(OrderStatus.SURVEY_FINISH.getType());
        surveyStatus.add(OrderStatus.SURVEY_EXAMINING.getType());
        surveyStatus.add(OrderStatus.SURVEY_EXAMINE_SUCCESS.getType());
        surveyStatus.add(OrderStatus.SURVEY_RECTIFYING.getType());
        surveyStatus.add(OrderStatus.SURVEY_RECTIFIED_EXAMINING.getType());
        if (surveyStatus.contains(sourceStatus)) {
            return 3;
        }
        List<Integer> installStatus = new ArrayList<>();
        installStatus.add(OrderStatus.INSTALL_RESERVE_EXCEPTION.getType());
        installStatus.add(OrderStatus.INSTALL_RESERVED.getType());
        installStatus.add(OrderStatus.INSTALL_CHANGE_EXAMINING.getType());
        installStatus.add(OrderStatus.INSTALL_CHANGE_EXAMINE_FAIL.getType());
        installStatus.add(OrderStatus.INSTALL_CHANGE_EXAMINE_SUCCESS.getType());
        installStatus.add(OrderStatus.INSTALLING.getType());
        installStatus.add(OrderStatus.INSTALL_FINISH.getType());
        installStatus.add(OrderStatus.INSTALL_EXAMINING.getType());
        // installStatus.add(OrderStatus.INSTALL_EXAMINE_SUCCESS.getType());
        installStatus.add(OrderStatus.INSTALL_RECTIFYING.getType());
        installStatus.add(OrderStatus.INSTALL_RECTIFY_EXAMINING.getType());
        if (installStatus.contains(sourceStatus)) {
            return 4;
        }
        // 未支付订单还是4
        if (billStatus != null && billStatus.intValue() == 0) {
            return 4;
        }
        return 5;
    }
    
    public AppOrderInstallDTO getInstallDetail(String orderId) {
        AppOrderInstallDTO returnValue = new AppOrderInstallDTO();
        Order order = orderDao.selectById(orderId);
        if (order == null)
            throw new ServiceException("订单不存在");
        OrderInstall orderInstall = orderInstallDao.selectByOrderId(orderId);
        if (orderInstall == null)
            throw new ServiceException("安装订单不存在");

        // 查询账单
        UserPayBill userPayBill = userPayBillService.getByRefId(orderId);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        if (userPayBill != null) {
            returnValue.setBillId(userPayBill.getId());
            returnValue.setBillStatus(userPayBill.getStatus());
            returnValue.setBillPayMethod(userPayBill.getPayType());
            returnValue.setBillUserPhone(userPayBill.getUserPhone());
            if (userPayBill.getPayTime() != null)
                returnValue.setBillPayTime(sdf.format(userPayBill.getPayTime()));
        }
        
        BigDecimal additionTotalFee = orderInstall.getPredictTotalFee() == null ? BigDecimal.ZERO : orderInstall.getPredictTotalFee();
        BigDecimal expandFee = orderInstall.getExpandFee() == null ? BigDecimal.ZERO : orderInstall.getExpandFee();
        returnValue.setOrderId(orderId);
        // returnValue.setOutOrderId(order.getOutOrderId());
        returnValue.setAddress(order.getAddress());
        returnValue.setExpandFee(expandFee);
        returnValue.setAdditionTotalFee(additionTotalFee);
        returnValue.setTotalFee(additionTotalFee.add(expandFee));
        
        if (order.getInstallFinishTime() != null)
            returnValue.setInstallFinishTime(sdf.format(order.getInstallFinishTime()));
        returnValue.setOrderStatus(order.getStatus());
        returnValue.setOrderStatusText(OrderStatus.getText(order.getStatus()));
        List<AppOrderInstallDTO.AdditionFeeItem> expandFeeList = new ArrayList<>();
        
        List<OrderInstallItemVo> oiiList = orderInstallItemDao.selectByOrderIdAndType(orderId, 2);
        for (OrderInstallItemVo oii : oiiList) {
            AppOrderInstallDTO dto = new AppOrderInstallDTO();
            AdditionFeeItem expandFeeItem = dto.new AdditionFeeItem();
            expandFeeItem.setFee(oii.getFee());
            expandFeeItem.setName(oii.getItemName());
            expandFeeItem.setQuantity(oii.getQunatity().intValue());
            expandFeeList.add(expandFeeItem);
        }
        returnValue.setAdditionFeeList(expandFeeList);
        
        OperateLogBo condition = new OperateLogBo();
        condition.setThirdPartyId(orderId);
        condition.setName(OrderStatus.RETURN_VISIT.getType() + "");
        List<OperateLog> olList = operateLogDao.selectByExample(condition);
        if (!olList.isEmpty()) {
            AppOrderInstallDTO dto = new AppOrderInstallDTO();
            OrderScore orderScore = dto.new OrderScore();
            orderScore.setContent(order.getVisitContent());
            orderScore.setFiles(order.getFiles());
            orderScore.setScore(order.getScore());
            orderScore.setTime(sdf.format(olList.get(0).getCreateTime()));
            returnValue.setOrderScore(orderScore);
        }
        
        return returnValue;
    }
    
    public AppOrderDetailDTO getOrderDetail(String orderId) {
        AppOrderDetailDTO returnValue = new AppOrderDetailDTO();
        Order order = orderDao.selectById(orderId);
        if (order == null)
            throw new ServiceException("订单不存在");
        OperateLogBo condition = new OperateLogBo();
        String types = OperateLogType.ORDER_STATUS.getType() + "," + OperateLogType.STAFF_CHANGE.getType() + ","
            + OperateLogType.SUPPLIER_CHANGE.getType();
        condition.setTypes(types);
        condition.setThirdPartyId(orderId);
        List<OperateLog> logList = operateLogDao.selectByExample(condition);
        List<AppOperateLogDTO> list = new ArrayList<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Map<String, String> needShowStatusMap = new HashMap<String, String>();
        needShowStatusMap.put(OrderStatus.SURVEY_RESERVED.getType() + "", "已预约");
        needShowStatusMap.put(OrderStatus.SURVEY_DISPATCHED.getType() + "", "已指派电工，等待上门勘测");
        needShowStatusMap.put(OrderStatus.SURVEY_FINISH.getType() + "", "已完成上门勘测");
        needShowStatusMap.put(OrderStatus.INSTALL_RESERVED.getType() + "", "已完成预约安装");
        needShowStatusMap.put(OrderStatus.INSTALL_FINISH.getType() + "", "已完成上门安装，待评价");
        needShowStatusMap.put(OrderStatus.RETURN_VISIT.getType() + "", "已完成，感谢您的评价");
        needShowStatusMap.put(OrderStatus.CLOSED.getType() + "", "已为您关闭工单");
        boolean hasSurveyReservedModifyLog = false;
        boolean hasInstallReservedModifyLog = false;
        for (OperateLog item : logList) {
            if (needShowStatusMap.containsKey(item.getName()) || item.getType().intValue() == OperateLogType.STAFF_CHANGE.getType()) {
                AppOperateLogDTO dto = new AppOperateLogDTO();
                dto.setName(item.getName());
                if (item.getType().intValue() == OperateLogType.STAFF_CHANGE.getType()) {
                    dto.setValue("已更换电工，等待上门勘测");
                } else {
                    dto.setValue(needShowStatusMap.get(item.getName()));
                }
                if (Integer.valueOf(item.getName()).intValue() == OrderStatus.SURVEY_RESERVED.getType()) {
                    if (!StringUtils.isEmpty(item.getValue()) && item.getValue().contains("修改")) {
                        if (hasSurveyReservedModifyLog)
                            continue;
                        hasSurveyReservedModifyLog = true;
                        dto.setValue("已修改预约信息，等待上门勘察");
                    }
                }
                if (Integer.valueOf(item.getName()).intValue() == OrderStatus.INSTALL_RESERVED.getType()) {
                    if (!StringUtils.isEmpty(item.getValue()) && item.getValue().contains("修改")) {
                        if (hasInstallReservedModifyLog)
                            continue;
                        hasInstallReservedModifyLog = true;
                        dto.setValue("已修改预约信息，等待上门安装");
                    }
                }
                dto.setDate(sdf.format(item.getCreateTime()));
                if (Integer.valueOf(item.getName()).intValue() == OrderStatus.RETURN_VISIT.getType()) {
                    dto.setShowRating(true);
                }
                list.add(dto);
            }
        }
        ;
        returnValue.setAddress(order.getAddress());
        returnValue.setOrderId(orderId);
        returnValue.setStatusText(OrderStatus.getText(order.getStatus()));
        returnValue.setOperateLogList(list);
        returnValue.setUserName(order.getUserName());
        
        UserPayBill userPayBill = userPayBillService.getByRefId(orderId);
        if (userPayBill != null) {
            returnValue.setBillId(userPayBill.getId());
            returnValue.setBillStatus(userPayBill.getStatus());
        }
        return returnValue;
    }
    
    /**
     * 查看报装说明
     *
     * @return
     */
    public MaterialConfig getExpandIntro(String orderId) {
        Order order = orderDao.selectById(orderId);
        if (order == null) {
            throw new ServiceException("订单不存在");
        }
        if (StringUtils.isEmpty(order.getRegionId())) {
            return null;
        }
        MaterialConfigBo condition = new MaterialConfigBo();
        condition.setType(1);
        condition.setStatus(1);
        // condition.setOemId(order.getOemId());
        condition.setRegionId(Integer.valueOf(order.getRegionId()));
        List<MaterialConfigVo> list = materialConfigDao.selectByExample(condition);
        return list.isEmpty() ? null : list.get(0);
    }
    
    /**
     * 更新支付状态
     *
     * @param orderId
     * @param payStatus
     */
    public void updateInstallOrderPayStatus(String orderId, String thirdUserId, Integer payStatus, int payMethod) {
        if (payStatus == 1) {
            userPayBillService.paySuccessByOrderId(orderId, payMethod, thirdUserId, null, new Date(), 3);
        }
    }
    
    /**
     * 报装
     *
     * @param orderId
     * @param expandStatus
     * @param expandFailReason
     * @param expandFailOperate
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void expand(String orderId, Integer expandStatus, String expandFailReason, Integer expandFailOperate) {
        Order order = orderDao.selectById(orderId);
        if (order != null && order.getExpandStatus() != null) {
            throw new ServiceException("该笔订单已报装过");
        }
        Order updateOrder = new Order();
        updateOrder.setExpandStatus(expandStatus);
        if (!StringUtils.isEmpty(expandFailReason))
            updateOrder.setExpandFailReason(expandFailReason);
        if (expandFailOperate != null)
            updateOrder.setExpandFailOperate(expandFailOperate);
        orderDao.updateById(orderId, updateOrder);
        
        OrderInstall orderInstall = new OrderInstall();
        orderInstall.setElectricExpand(0);// app过来的报装信息 不需要再在后台支付费用
        orderInstallDao.updateByOrderId(orderId, orderInstall);
    }
    
    /**
     * 查询账单列表
     *
     * @param phone
     * @return
     */
    public List<AppPayBillDTO> qryBill(String phone, String orderId, Integer status) {
        UserPayBill condition = new UserPayBill();
        condition.setUserPhone(phone);
        condition.setRefId(orderId);
        condition.setStatus(status);
        List<UserPayBillVo> list = userPayBillService.selectListForApp(condition);
        list =
            list.stream().filter(item -> item.getOrderStatus() != null && item.getOrderStatus().intValue() != OrderStatus.CLOSED.getType())
                .collect(Collectors.toList());// 过滤订单状态为已关闭的账单
        List<AppPayBillDTO> returnList = new ArrayList<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        for (UserPayBillVo vo : list) {
            AppPayBillDTO billDto = new AppPayBillDTO();
            billDto.setAmount(vo.getAmount());
            billDto.setBillId(vo.getId());
            billDto.setPayStatus(vo.getStatus());
            billDto.setOrderStatus(vo.getOrderStatus());
            billDto.setOrderStatusText(OrderStatus.getText(vo.getOrderStatus()));
            if (vo.getPayTime() != null)
                billDto.setPayTime(sdf.format(vo.getPayTime()));
            billDto.setOrderId(vo.getRefId());
            if (vo.getInstallFinishTime() != null)
                billDto.setInstallFinishTime(sdf.format(vo.getInstallFinishTime()));
            returnList.add(billDto);
        }
        return returnList;
    }
}
