package com.starcharge.wios.auth.controller;

import java.util.List;

import javax.annotation.Resource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.SpringContextLoader;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.base.util.Assert;
import com.starcharge.wios.auth.bo.AccountBo;
import com.starcharge.wios.auth.dao.model.Role;
import com.starcharge.wios.auth.service.AccountService;
import com.starcharge.wios.auth.service.RoleService;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.auth.vo.AccountVo;
import com.starcharge.wios.contants.Contant;
import com.starcharge.wios.dao.OemDao;
import com.starcharge.wios.dao.entity.Oem;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 账户相关api
 * 
 * @author lilin
 * @version [版本号, 2020年5月9日]
 */
@RestController
@RequestMapping("api")
@Api(tags = "账户管理接口")
public class AccountController {
    /**
     * 账户service
     */
    @Resource
    private AccountService accountService;
    
    /**
     * 角色service
     */
    @Resource
    private RoleService roleService;
    
    @Autowired
    private OemDao oemDao;

    // /**
    // * 查询账户列表
    // *
    // * @param query 查询条件
    // * @return 账户列表
    // */
    // @GetMapping(value = "account/list")
    // @ApiOperation(value = "查询账户列表")
    // @ApiImplicitParams({@ApiImplicitParam(name = "name", value = "姓名", dataType = "String", paramType = "query",
    // required = false),
    // @ApiImplicitParam(name = "phone", value = "手机号", dataType = "String", paramType = "query", required = false),
    // @ApiImplicitParam(name = "status", value = "状态", dataType = "int", paramType = "query", required = false),
    // @ApiImplicitParam(name = "userId", value = "用户id", dataType = "String", paramType = "query", required = false),
    // })
    // public BaseResponse<List<Account>> list(String name, String phone, Integer status, String userId) {
    // Account query = new Account();
    // query.setName(name);
    // query.setPhone(phone);
    // query.setStatus(status);
    // query.setUserId(userId);
    // return new BaseResponse<List<Account>>(accountService.queryList(query));
    // }
    
    /**
     * 查询账户列表
     *
     * @param query 查询条件
     * @return 账户列表
     */
    @GetMapping(value = "account/oem/list")
    @ApiOperation(value = "查询主机厂/经销商账户列表")
    @ApiImplicitParams({@ApiImplicitParam(name = "name", value = "姓名", dataType = "String", paramType = "query", required = false),
        @ApiImplicitParam(name = "status", value = "状态", dataType = "int", paramType = "query", required = false),
        @ApiImplicitParam(name = "oemId", value = "主机厂/经销商id", dataType = "int", paramType = "query", required = false),
        @ApiImplicitParam(name = "type", value = "类型 1:主机厂 2：经销商", dataType = "int", paramType = "query", required = false),
        @ApiImplicitParam(name = "account", value = "登陆账号", dataType = "String", paramType = "query", required = false),
    })
    public BaseResponse<List<AccountVo>> getOemAccountList(String name, Integer status, Integer type, String account) {
        AccountBo query = new AccountBo();
        query.setName(name);
        query.setStatus(status);
        query.setType(type);
        query.setAccount(account);
        return new BaseResponse<List<AccountVo>>(accountService.getOemAccountList(query));
    }
    
    /**
     * 添加主机厂/经销商账户
     *
     * @param accountInfo 账户信息
     * @return 新增成功的条数
     */
    @PostMapping(value = "account/oem/add")
    @ApiOperation(value = "添加主机厂/经销商账户")
    public BaseResponse<Integer> add(AccountBo accountInfo) {
        if (StringUtils.isEmpty(accountInfo.getRoleIds())) {
            throw new ServiceException("缺少角色");
        }
        TokenService tokenService = (TokenService)SpringContextLoader.getBean("tokenService");
        SessionInfo currentUser = tokenService.getUser();
        if ("1".equals(currentUser.getUserId())) {// 超级管理员
            if (accountInfo.getOemId() == null) {
                throw new ServiceException("缺少主机厂/经销商id");
            }
            Oem oem = oemDao.selectById(accountInfo.getOemId());
            Integer parentId = oem.getWbAccountId();
            accountInfo.setParentId(parentId);
        } else {
            accountInfo.setParentId(Integer.valueOf(currentUser.getUserId()));
        }
        accountInfo.setPassword(Contant.ACCOUNT_INIT_PASSWARD);
        return new BaseResponse<Integer>(accountService.addAccount(accountInfo));
    }
    
    
    /**
     * 修改账户
     *
     * @param accountInfo 账户信息
     * @return 修改成功的数量
     */
    @PostMapping(value = "account/update")
    @ApiOperation(value = "修改账户")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "主键", dataType = "int", paramType = "query", required = true),
        @ApiImplicitParam(name = "phone", value = "手机号", dataType = "String", paramType = "query", required = false),
        @ApiImplicitParam(name = "name", value = "账户名称", dataType = "String", paramType = "query", required = false),
        @ApiImplicitParam(name = "account", value = "登陆账号", dataType = "String", paramType = "query", required = false),
        @ApiImplicitParam(name = "roleIds", value = "角色id", dataType = "String", paramType = "query", required = false),
    })
    public BaseResponse<Integer> update(AccountBo accountInfo) {
        return new BaseResponse<Integer>(accountService.updateAccount(accountInfo));
    }
    
    /**
     * 修改密码
     *
     * @param id 账户id
     * @param oldPassword 旧密码
     * @param password 新密码
     * @return 是否修改成功：1成功，其他失败
     */
    @PostMapping(value = "account/updatePassword")
    @ApiOperation(value = "修改密码")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "主键", dataType = "int", paramType = "query", required = true),
        @ApiImplicitParam(name = "oldPassword", value = "老密码", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "password", value = "新密码", dataType = "String", paramType = "query", required = true),})
    public BaseResponse<Integer> updatePassword(Integer id, String oldPassword, String password) {
        return new BaseResponse<Integer>(accountService.updatePassword(id, oldPassword, password));
    }
    
    /**
     * 重置密码
     *
     * @param id
     * @return
     */
    @PostMapping(value = "account/initPassword")
    @ApiOperation(value = "重置密码")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "主键", dataType = "int", paramType = "query", required = true)})
    public BaseResponse<Integer> updatePassword(Integer id) {
        return new BaseResponse<Integer>(accountService.initPassword(id));
    }
    
    /**
     * 发送短信验证码
     *
     * @param phone
     * @return
     */
    @PostMapping(value = "account/sendCaptcha")
    @ApiOperation(value = "发送短信验证码")
    @ApiImplicitParams({@ApiImplicitParam(name = "phone", value = "手机号", dataType = "String", paramType = "query", required = true)})
    public BaseResponse<Object> updatePasswodByCaptcha(String phone) {
        Assert.notEmpty(phone, "phone不能为空");
        accountService.sendCaptcha(phone);
        return new BaseResponse<Object>();
    }
    
    /**
     * 通过短信验证码修改密码
     *
     * @param newPassword
     * @param captcha
     * @param phone
     * @return
     */
    @PostMapping(value = "account/updatePasswodByCaptcha")
    @ApiOperation(value = "通过短信验证码修改密码")
    @ApiImplicitParams({@ApiImplicitParam(name = "newPassword", value = "新密码", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "captcha", value = "验证码", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "phone", value = "手机号", dataType = "String", paramType = "query", required = true)})
    public BaseResponse<Object> updatePasswodByCaptcha(String newPassword, String captcha, String phone) {
        Assert.notEmpty(newPassword, "newPassword不能为空");
        Assert.notEmpty(captcha, "captcha不能为空");
        Assert.notEmpty(phone, "phone不能为空");
        accountService.resetPwd(newPassword, captcha, phone);
        return new BaseResponse<Object>();
    }
    
    /**
     * 根据账户id查询角色列表
     *
     * @param accountId 账户id
     * @return 角色列表
     */
    @GetMapping(value = "account/roleList")
    @ApiOperation(value = "根据账户id查询角色列表")
    @ApiImplicitParam(name = "accountId", value = "账户id", dataType = "int", paramType = "query", required = true)
    public BaseResponse<List<Role>> findAccountRoleList(String accountId) {
        return new BaseResponse<List<Role>>(roleService.findAccountRoleList(accountId));
    }
}
