package com.starcharge.wios.auth.controller;

import javax.annotation.Resource;

import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ihidea.component.api.v2.BaseResponse;
import com.starcharge.wios.auth.service.AccountService;
import com.starcharge.wios.auth.vo.LoginInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 登录相关api
 *
 * @author lilin
 * @version [版本号, 2020年5月9日]
 */
@RestController
// @RequestMapping("api")
@Api(tags = "登陆管理接口")
public class LoginController {
    /**
     * 账户service
     */
    @Resource
    private AccountService accountService;

    /**
     * 登录
     *
     * @param account 账号
     * @param password 密码
     * @return 登录完成后的信息：jwtToken
     */
    @PostMapping(value = "account/login")
    @ApiOperation(value = "登陆")
    @ApiImplicitParams({@ApiImplicitParam(name = "account", value = "账号", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "password", value = "密码", dataType = "String", paramType = "query", required = true),})
    public BaseResponse<LoginInfo> login(String account, String password) {
        return new BaseResponse<LoginInfo>(accountService.login(account, password));
    }

    /**
     * 退出登录
     *
     * @param account 账号
     */
    @PostMapping(value = "account/logout")
    @ApiOperation(value = "退出登陆")
    @ApiImplicitParams({@ApiImplicitParam(name = "account", value = "账号", dataType = "String", paramType = "query", required = true),})
    public BaseResponse<Object> logout(String account) {
        accountService.logout(account);
        return new BaseResponse<>();
    }
}
