package com.starcharge.wios.auth.daoex;

import java.util.List;

import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import com.ihidea.core.base.CoreDao;
import com.starcharge.wios.auth.bo.RoleBo;
import com.starcharge.wios.auth.dao.model.Role;
import com.starcharge.wios.auth.dao.model.RoleMenu;

/**
 * 角色扩展mapper
 * 
 * @author lilin
 * @version [版本号, 2020年5月25日]
 */
public interface RoleMapperEx extends CoreDao {
    /**
     * 批量新增角色/菜单关联关系
     *
     * @param roleMenuList 角色/菜单关联关系集合
     * @return 新增成功的条数
     */
    @Insert("<script>insert into wb_role_menu(role_id, menu_id, create_account_id, create_time) values "
        + "<foreach collection='roleMenuList' item='item' index='index' separator=','>("
        + "#{item.roleId,jdbcType=INTEGER}, #{item.menuId,jdbcType=VARCHAR}, "
        + "#{item.createAccountId,jdbcType=INTEGER}, #{item.createTime,jdbcType=TIMESTAMP}"
        + ")</foreach></script>")
    int batchInsertRoleMenu(@Param("roleMenuList") List<RoleMenu> roleMenuList);
    
    /**
     * 根据账户id查询角色列表
     *
     * @param accountId 账户id
     * @return 角色列表
     */
    @Select("select r.id,r.name,ar.id as status from wb_role r "
        + "left join wb_account_role ar on ar.role_id=r.id " 
        + "where r.`status`=1 and ar.account_id=#{accountId} order by r.id")
    List<Role> findAccountRoleList(@Param("accountId") String accountId);
    
    @Select("select r.* from wb_role r where r.create_account_id=#{createAccountId}")
    List<Role> findRoleListByCreate(@Param("createAccountId") Integer createAccountId);

    @Select("select r.id, r.name, "
            + "(select group_concat(m.e_name) from wb_role_menu rm inner join wb_menu m on rm.menu_id = m.id "
            + " where rm.role_id = r.id and m.app_type = 1) as menuIds "
            + " from wb_account_role ar "
            + " inner join wb_role r on ar.role_id=r.id "
            + " where r.status=1 and ar.account_id= #{accountId} ")
    List<RoleBo> findMinaRoleMenu(@Param("accountId") Integer accountId);
}
