package com.starcharge.wios.auth.filter;

import java.io.IOException;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.ihidea.component.api.json.MJSONResultEntity;
import com.ihidea.core.support.SpringContextLoader;
import com.ihidea.core.util.ServletUtilsEx;
import com.starcharge.base.util.NetUtil;
import com.starcharge.wios.auth.service.AccountService;

/**
 * 登录过滤器，集成jwt
 * 
 * @author lilin
 * @version [版本号, 2020年5月14日]
 */
public class LoginFilter implements Filter {
    /**
     * log
     */
    private static Logger log = LoggerFactory.getLogger(LoginFilter.class);
    
    /**
     * 忽略校验的url
     */
    private String ignoreUrls;
    
    /**
     * 忽略校验的path
     */
    private List<String> ignoreUrlPath;
    
    /**
     * accountService
     */
    private AccountService accountService;
    
    @Override
    public void init(FilterConfig filterConfig) throws ServletException {
        ignoreUrls = filterConfig.getInitParameter("ignoreUrls");
        String path = filterConfig.getInitParameter("ignoreUrlPath");
        if (StringUtils.isNotBlank(path)) {
            ignoreUrlPath = Stream.of(path.split(",")).collect(Collectors.toList());
        }
        accountService = SpringContextLoader.getBean(AccountService.class);
    }
    
    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
        HttpServletRequest req = (HttpServletRequest)request;
        String requestURI = req.getRequestURI();
        if (ignoreUrls.contains(requestURI)) {
            chain.doFilter(request, response);
            return;
        }
        // String path = req.getPathInfo(); // 取除去url-pattern以外的url
        
        // if (path != null) {
        // if (StringUtils.isNotBlank(ignoreUrls) && ignoreUrls.contains(path)) {
        // chain.doFilter(request, response);
        // return;
        // }
        // if (CollectionUtils.isNotEmpty(ignoreUrlPath)) {
        // for (String urlPath : ignoreUrlPath) {
        // if (path.contains(urlPath)) {
        // chain.doFilter(request, response);
        // return;
        // }
        // }
        // }
        // }
        
        String token = req.getHeader("Authorization");
        
        // 认证token
        MJSONResultEntity result = accountService.verifyToken(token, NetUtil.getIpAddr(req));
        // MJSONResultEntity result = null;
        if (result == null) {
            chain.doFilter(request, response);
        } else {
            log.error("token认证失败：{}", result.getText());
            ServletUtilsEx.renderJson((HttpServletResponse)response, result);
        }
    }
    
}
