package com.starcharge.wios.auth.service;

import com.ihidea.component.api.json.MJSONResultEntity;
import com.ihidea.core.support.SpringContextLoader;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.orm.mybatis3.util.IbatisServiceUtils;
import com.ihidea.core.support.pageLimit.PageLimit;
import com.ihidea.core.support.pageLimit.PageLimitHolderFilter;
import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.base.redis.RedisClient;
import com.starcharge.base.util.Assert;
import com.starcharge.wios.auth.bo.AccountBo;
import com.starcharge.wios.auth.dao.AccountMapper;
import com.starcharge.wios.auth.dao.AccountRoleMapper;
import com.starcharge.wios.auth.dao.model.*;
import com.starcharge.wios.auth.daoex.AccountMapperEx;
import com.starcharge.wios.auth.vo.AccountVo;
import com.starcharge.wios.auth.vo.LoginInfo;
import com.starcharge.wios.call.service.PushService;
import com.starcharge.wios.contants.Contant;
import com.starcharge.wios.dao.CamsAccountDao;
import com.starcharge.wios.dao.OemDao;
import com.starcharge.wios.dao.SupplierDao;
import com.starcharge.wios.dao.SupplierStaffDao;
import com.starcharge.wios.dao.entity.CamsAccount;
import com.starcharge.wios.dao.entity.Oem;
import com.starcharge.wios.dao.entity.Supplier;
import com.starcharge.wios.dao.entity.SupplierStaff;
import com.starcharge.wios.utils.NumberUtil;
import com.starcharge.wios.wx.utils.WeChatDecryptUtils;
import com.starcharge.wios.wx.utils.WeChatSessionKeyUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.DigestUtils;

import javax.annotation.Resource;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 账户service
 *
 * @author lilin
 * @version [版本号, 2020年5月9日]
 */
@Service
public class AccountService {
    /**
     * token失效时间
     */
    @Value("${auth.token.validSeconds:7200}")
    private Integer tokenValidSeconds;

    /**
     * 是否允许重复登录
     */
    @Value("${auth.allowRepeatLogin}")
    private Boolean allowRepeatLogin;

    public final static String REDIS_SMS_RESET_PASSWORD_KEY = "wios:sms:resetPwd:";

    public final static int EXPIRE_SECONDS = 180;

    /**
     * 登录账户mapper
     */
    @Resource
    private AccountMapper accountMapper;

    /**
     * 登录账户扩展mapper
     */
    @Resource
    private AccountMapperEx accountMapperEx;

    /**
     * 账户角色mapper
     */
    @Resource
    private AccountRoleMapper accountRoleMapper;

    /**
     * redis客户端
     */
    @Resource
    private RedisClient redisClient;

    /**
     * 菜单service
     */
    // @Autowired
    // private MenuService menuService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private RoleService roleService;

    @Autowired
    private OemDao oemDao;

    @Autowired
    private SupplierDao supplierDao;

    @Autowired
    private CamsAccountDao camsAccountDao;

    @Autowired
    private PushService pushService;

    @Autowired
    private SupplierStaffDao supplierStaffDao;

    /**
     * 本地线程变量，存放当前用户
     */
    // private ThreadLocal<Account> threadLocal = new ThreadLocal<>();

    /**
     * 获取当前登录用户信息
     *
     * @return 当前登录用户信息
     */
    // public Account getCurrentAccount() {
    // return threadLocal.get();
    // }

    /**
     * 设置当前登录用户信息
     *
     * @param account 当前登录用户信息
     */
    // public void setCurrentAccount(Account account) {
    // threadLocal.set(account);
    // }

    /**
     * 通过账号获取账号信息
     *
     * @param account
     * @return
     */
    public Account getByAccount(String account) {
        AccountCriteria example = new AccountCriteria();
        example.createCriteria().andAccountEqualTo(account).andStatusEqualTo(1);
        List<Account> accountList = accountMapper.selectByExample(example);
        if (CollectionUtils.isNotEmpty(accountList)) {
            return accountList.get(0);
        }
        return null;
    }

    public Account selectByPrimaryKey(Integer id) {
        return accountMapper.selectByPrimaryKey(id);
    }

    /**
     * 登录接口
     *
     * @param account  登录账户
     * @param password 登录密码
     * @return jwtToken
     */
    public LoginInfo login(String account, String password) {
        Assert.notEmpty(account, "登录账户不能为空");
        Assert.notEmpty(password, "密码不能为空");
        String md5Password = DigestUtils.md5DigestAsHex(password.getBytes(StandardCharsets.UTF_8));
        Account accountInfo = getByAccount(account);
        Assert.notNull(accountInfo, "登录账户不存在");
        Assert.isTrue(md5Password.equalsIgnoreCase(accountInfo.getPassword()), "密码错误");
        if (accountInfo.getStatus() != null && accountInfo.getStatus().intValue() == 0) {
            throw new ServiceException("账户已被禁用");
        }

        AccountRoleCriteria example = new AccountRoleCriteria();
        example.createCriteria().andAccountIdEqualTo(accountInfo.getId());
        List<AccountRole> arList = accountRoleMapper.selectByExample(example);
        if (arList.isEmpty())
            throw new ServiceException("获取角色失败");
        int roleIdInt = arList.get(0).getRoleId().intValue();
        int roleId = arList.get(0).getRoleId().intValue();

        Integer oemId = null;
        Integer oemType = 0;
        Integer installSupplierId = null;
        Integer installStaffId = null;
        String regionIds = "";
        if (roleIdInt == 2 || roleIdInt == 3) {
            Oem oem = oemDao.selectByAccountId(accountInfo.getId());
            if (oem == null)
                throw new ServiceException("获取主机厂/经销商失败");
            oemType = oem.getType();
            oemId = oem.getId();
        } else if (roleIdInt == 4) {
            Supplier supplier = supplierDao.selectByAccountId(accountInfo.getId());
            if (supplier == null)
                throw new ServiceException("获取服务商失败");
            installSupplierId = Integer.valueOf(supplier.getId());
        } else {
            Integer parentId = accountInfo.getParentId();
            if (parentId != null) {
                List<Role> parentRoleList = roleService.findAccountRoleList(parentId.toString());
                if (parentRoleList.isEmpty())
                    throw new ServiceException("获取用户角色列表失败");

                roleIdInt = parentRoleList.get(0).getId().intValue();
                if (roleIdInt == 2 || roleIdInt == 3) {
                    Oem oem = oemDao.selectByAccountId(parentId);
                    if (oem == null)
                        throw new ServiceException("获取主机厂/经销商失败");

                    oemId = oem.getId();
                } else if (roleIdInt == 4) {
                    Supplier supplier = supplierDao.selectByAccountId(parentId);
                    if (supplier == null)
                        throw new ServiceException("获取服务商失败");
                    installSupplierId = Integer.valueOf(supplier.getId());

                    List<SupplierStaff> supplierStaffList = supplierStaffDao.selectByAccountId(accountInfo.getId());
                    if (!supplierStaffList.isEmpty()) {
                        installStaffId = supplierStaffList.get(0).getId();// 电工id
                    }
                } else {
                    List<CamsAccount> camsAccountList = camsAccountDao.selectByAccountIdAndDeptId(accountInfo.getId(), null);
                    if (!camsAccountList.isEmpty()) {
                        roleIdInt = 999;// cams账号
                        regionIds = camsAccountList.get(0).getRegionId();// 负责区域
                    }
                }

            }
        }

        SessionInfo sessionInfo = new SessionInfo();
        sessionInfo.setUserId(accountInfo.getId().toString());
        sessionInfo.setUserName(accountInfo.getName());
        sessionInfo.setAttribute("roleId", roleIdInt + "");
        sessionInfo.setAttribute("roleIdActual", roleId + "");
        sessionInfo.setAttribute("regionIds", regionIds);
        if (oemId != null) {
            sessionInfo.setAttribute("oemId", oemId.toString());
            sessionInfo.setAttribute("oemType", oemType.toString());
        }
        if (installSupplierId != null)
            sessionInfo.setAttribute("installSupplierId", installSupplierId.toString());
        if (installStaffId != null)
            sessionInfo.setAttribute("installStaffId", installStaffId.toString());
        LoginInfo loginInfo = new LoginInfo();
        loginInfo.setToken(tokenService.saveToken(sessionInfo));
        loginInfo.setUserName(sessionInfo.getUserName());
        loginInfo.setAccountId(accountInfo.getId());
        loginInfo.setRoleId(roleIdInt);
        loginInfo.setInstallStaffId(installStaffId);

        // 更新用户登录信息
        AccountBo updateAccount = new AccountBo();
        updateAccount.setId(accountInfo.getId());
        updateAccount.setLastLoginTime(new Date());
        updateAccount(updateAccount);

        return loginInfo;
    }

    /**
     * 获取登录信息
     *
     * @param accountInfo
     * @return
     */
    private LoginInfo getMinaLoginInfo(Account accountInfo) {
        SessionInfo sessionInfo = new SessionInfo();
        sessionInfo.setUserId(accountInfo.getId().toString());
        sessionInfo.setUserName(accountInfo.getAccount());

        // sessionInfo.setAttribute("ip", NetUtil.getIpAddr(ServletHolderFilter.getContext().getRequest()));
        LoginInfo loginInfo = new LoginInfo();
        loginInfo.setToken(tokenService.saveMinaToken(sessionInfo));
        loginInfo.setUserName(sessionInfo.getUserName());
        loginInfo.setAccountId(accountInfo.getId());

        // 更新用户登录信息
        AccountBo updateAccount = new AccountBo();
        updateAccount.setId(accountInfo.getId());
        updateAccount.setLastLoginTime(new Date());
        updateAccount(updateAccount);

        return loginInfo;
    }

    /**
     * 账号密码登录小程序
     *
     * @param account
     * @param password
     * @return
     */
    public LoginInfo loginMina(String account, String password) {
        Assert.notEmpty(account, "登录账户不能为空");
        Assert.notEmpty(password, "密码不能为空");
        String md5Password = DigestUtils.md5DigestAsHex(password.getBytes(StandardCharsets.UTF_8));
        Account accountInfo = getByAccount(account);
        Assert.notNull(accountInfo, "登录账户不存在");
        Assert.isTrue(md5Password.equalsIgnoreCase(accountInfo.getPassword()), "密码错误");
        if (accountInfo.getStatus() != null && accountInfo.getStatus().intValue() == 0) {
            throw new ServiceException("账户已被禁用");
        }

        return getMinaLoginInfo(accountInfo);
    }

    /**
     * 通过微信授权手机号登录
     *
     * @param openId
     * @param encryptedData
     * @param iv
     * @return
     */
    public LoginInfo loginMinaByWx(String openId, String encryptedData, String iv) {
        String account = WeChatDecryptUtils.decryptWxUser(WeChatSessionKeyUtils.getXcxSessionKey(openId), encryptedData, iv);
        Assert.notBlank(account, "手机号码获取失败");

        Account accountInfo = getByAccount(account);
        Assert.notNull(accountInfo, "登录账户不存在");

        return getMinaLoginInfo(accountInfo);
    }

    /**
     * 校验 token
     *
     * @param token     token
     * @param currentIp 当前访问的ip
     * @return 异常信息
     */
    public MJSONResultEntity verifyToken(String token, String currentIp) {
        // 其他接口需要认证
        if (StringUtils.isBlank(token)) {
            return new MJSONResultEntity("400", "请传入token");
        }
        SessionInfo user = redisClient.get(TokenService.TOKEN_KEY_PREFIX + token, SessionInfo.class, tokenValidSeconds);
        if (user == null) {
            return new MJSONResultEntity("400", "登陆信息失效");
        }
        // String ip = user.getAttribute("ip");
        // if (ip == null || !ip.equals(currentIp)) {
        // return new MJSONResultEntity("400", "检测到设备变更IP地址，请重新登录");
        // }

        // 认证通过，登录续期
        redisClient.expire(TokenService.TOKEN_KEY_PREFIX + token, tokenValidSeconds);

        return null;
    }

    /**
     * 注销接口
     *
     * @param account 登录账户
     */
    public void logout(String account) {
        Assert.notEmpty(account, "登录账户不能为空");
        if (allowRepeatLogin == null || !allowRepeatLogin) {
            // redisClient.del(RedisKeyConstant.REDIS_TOKEN_PREFIX + account);
        }
    }

    // /**
    // * 查询账户列表
    // *
    // * @param query 查询条件
    // * @return 账户列表
    // */
    // public List<Account> queryList(Account query) {
    // AccountCriteria accountCriteria = new AccountCriteria();
    // AccountCriteria.Criteria criteria = accountCriteria.createCriteria();
    // if (StringUtils.isNotBlank(query.getName())) {
    // criteria.andNameLike(query.getName());
    // }
    // if (StringUtils.isNotBlank(query.getPhone())) {
    // criteria.andPhoneEqualTo(query.getPhone());
    // }
    // if (query.getStatus() != null) {
    // criteria.andStatusEqualTo(query.getStatus());
    // }
    // if (StringUtils.isNotBlank(query.getUserId())) {
    // criteria.andUserIdEqualTo(query.getUserId());
    // }
    // List<Account> accountList = accountMapper.selectByExample(accountCriteria);
    // return accountList.stream().map(account -> {
    // account.setPassword(null);
    // return account;
    // }).collect(Collectors.toList());
    // }

    public List<AccountVo> getOemAccountList(AccountBo condition) {
        SessionInfo user = tokenService.getUser();
        if (user != null && !StringUtils.isEmpty(user.getAttribute("oemId"))) {
            condition.setOemId(Integer.valueOf(user.getAttribute("oemId")));
        }
        PageLimit pageLimit = PageLimitHolderFilter.getContext();
        if (pageLimit != null)
            pageLimit.setLimited(true);
        List<AccountVo> managerList = accountMapperEx.getListForOem(condition);
        List<Integer> managerIdList = managerList.stream().map(item -> item.getId()).collect(Collectors.toList());
        Map<Integer, String> oemNameMap = managerList.stream().collect(Collectors.toMap(AccountVo::getId, AccountVo::getOemName));
        Map<Integer, Integer> oemIdMap = managerList.stream().collect(Collectors.toMap(AccountVo::getId, AccountVo::getOemId));
        List<AccountVo> returnList = new ArrayList<AccountVo>();
        if (!managerIdList.isEmpty()) {
            if ("%".equals(condition.getName()))
                condition.setName("\\" + condition.getName());
            List<AccountVo> result = accountMapperEx.selectByParentIds(managerIdList, condition.getAccount(), condition.getName());
            result =
                    result.stream().filter(r -> r.getRoleId().intValue() != 2).filter(r -> r.getRoleId().intValue() != 3)
                            .collect(Collectors.toList());// 去除管理员账号
            if (pageLimit != null)
                result = pageLimit.limitList(result);
            result.stream().forEach(item -> {
                String oemName = oemNameMap.get(item.getId());
                Integer oemId = oemIdMap.get(item.getId());
                if (StringUtils.isEmpty(oemName)) {
                    oemName = oemNameMap.get(item.getParentId());
                }
                if (oemId == null) {
                    oemId = oemIdMap.get(item.getParentId());
                }
                item.setOemName(oemName);
                item.setOemId(oemId);
                returnList.add(item);
            });
        }
        return returnList;
    }

    /**
     * 新增账户
     *
     * @param account 账户信息
     * @return 新增成功的数量
     */
    @Transactional(rollbackFor = Exception.class)
    public int addAccount(AccountBo account) {
        Assert.notEmpty(account.getAccount(), "账号不能为空");
        Assert.notEmpty(account.getPassword(), "密码不能为空");
        Assert.notEmpty(account.getName(), "姓名不能为空");
        if (account.getName().length() > 14) {
            throw new ServiceException("名字过长");
        }
        AccountCriteria accountCriteria = new AccountCriteria();
        accountCriteria.createCriteria().andAccountEqualTo(account.getAccount());

        Assert.isTrue(accountMapper.countByExample(accountCriteria) < 1, "登录账户已存在");
        // TODO: 这里需要新增t_user数据

        account.setPassword(DigestUtils.md5DigestAsHex(account.getPassword().getBytes(StandardCharsets.UTF_8)));
        account.setCreateTime(new Date());
        account.setStatus(1); // 正常
        if (account.getParentId() == null)
            account.setParentId(1);
        int result = accountMapper.insertSelective(account);

        Account parentAccount = accountMapper.selectByPrimaryKey(account.getParentId());
        if (parentAccount != null) {
            if (StringUtils.isEmpty(parentAccount.getIdTree())) {
                account.setIdTree(account.getParentId() + "_" + account.getId());
            } else {
                account.setIdTree(parentAccount.getIdTree() + "_" + account.getId());
            }
        }

        Account record = new Account();
        record.setId(account.getId());
        record.setIdTree(account.getIdTree());
        accountMapper.updateByPrimaryKeySelective(record);

        batchInsertAccountRole(account.getId(), account.getRoleIds());

        return result;
    }

    /**
     * 修改密码
     *
     * @param id          账户id
     * @param oldPassword 旧密码
     * @param password    新密码
     * @return 是否修改成功：1成功，其他失败
     */
    public int updatePassword(Integer id, String oldPassword, String password) {
        Assert.notNull(id, "id不能为空");
        Account account = accountMapper.selectByPrimaryKey(id);
        Assert.notNull(account, "账户不存在");
        Assert.isTrue(account.getStatus() != null && account.getStatus() == 1, "账户已被冻结");
        String md5Password = DigestUtils.md5DigestAsHex(oldPassword.getBytes(StandardCharsets.UTF_8));
        Assert.isTrue(md5Password.equalsIgnoreCase(account.getPassword()), "密码错误");

        Account updateAccount = new Account();
        updateAccount.setId(id);
        updateAccount.setPassword(DigestUtils.md5DigestAsHex(password.getBytes(StandardCharsets.UTF_8)));

        return accountMapper.updateByPrimaryKeySelective(updateAccount);
    }

    /**
     * 重置密码
     *
     * @param id
     * @return
     */
    public int initPassword(int id) {
        Assert.notNull(id, "id不能为空");
        Account updateAccount = new Account();
        updateAccount.setId(id);
        updateAccount.setPassword(DigestUtils.md5DigestAsHex(Contant.ACCOUNT_INIT_PASSWARD.getBytes(StandardCharsets.UTF_8)));
        return accountMapper.updateByPrimaryKeySelective(updateAccount);
    }

    /**
     * 修改账户
     *
     * @param account 账户信息
     * @return 修改成功的数量
     */
    @Transactional(rollbackFor = Exception.class)
    public int updateAccount(AccountBo account) {
        Assert.notNull(account.getId(), "id不能为空");
        if (StringUtils.isNotBlank(account.getAccount())) {
            AccountCriteria example = new AccountCriteria();
            example.createCriteria().andAccountEqualTo(account.getAccount());
            List<Account> list = accountMapper.selectByExample(example);
            if (!list.isEmpty() && list.get(0).getId().intValue() != account.getId().intValue()) {
                throw new ServiceException("账号已存在");
            }
        }
        account.setUpdateTime(new Date());
        if (StringUtils.isNotBlank(account.getPassword())) {
            account.setPassword(DigestUtils.md5DigestAsHex(account.getPassword().getBytes(StandardCharsets.UTF_8)));
        } else {
            account.setPassword(null);
        }
        if (StringUtils.isNotBlank(account.getRoleIds())) {
            AccountRoleCriteria criteria = new AccountRoleCriteria();
            criteria.createCriteria().andAccountIdEqualTo(account.getId());
            accountRoleMapper.deleteByExample(criteria);
            batchInsertAccountRole(account.getId(), account.getRoleIds());
        }

        return accountMapper.updateByPrimaryKeySelective(account);
    }

    /**
     * 批量新增角色
     *
     * @param accountId 账户id
     * @param roleIds   角色id
     * @return 新增成功的条数
     */
    private int batchInsertAccountRole(int accountId, String roleIds) {
        if (StringUtils.isBlank(roleIds)) {
            return 0;
        }
        TokenService tokenService = (TokenService) SpringContextLoader.getBean("tokenService");
        Integer createAccountId = tokenService.getUser() == null ? null : Integer.valueOf(tokenService.getUser().getUserId());
        Date now = new Date();

        return accountMapperEx.batchInsertAccountRole(Stream.of(roleIds.split(",")).map(roleId -> {
            AccountRole accountRole = new AccountRole();
            accountRole.setAccountId(accountId);
            accountRole.setRoleId(Integer.parseInt(roleId));
            accountRole.setCreateAccountId(createAccountId);
            accountRole.setCreateTime(now);
            return accountRole;
        }).collect(Collectors.toList()));
    }

    public Account get(Account entity) {
        return IbatisServiceUtils.get(entity, accountMapper);
    }

    /**
     * 发送短信验证码
     *
     * @param phone
     */
    public void sendCaptcha(String phone) {
        AccountCriteria example = new AccountCriteria();
        example.createCriteria().andAccountEqualTo(phone).andStatusEqualTo(1);
        List<Account> list = accountMapper.selectByExample(example);
        if (list.isEmpty())
            throw new ServiceException("账号未注册，请先联系管理员添加");
        String captcha = NumberUtil.getRandomNumbers(6);
        pushService.sendResetPwdCode(phone, captcha);
        redisClient.put(REDIS_SMS_RESET_PASSWORD_KEY + phone, captcha, EXPIRE_SECONDS);
    }

    /**
     * 通过验证码修改密码
     *
     * @param newPassword
     * @param captcha
     * @param phone
     */
    public void resetPwd(String newPassword, String captcha, String phone) {
        String redisCaptcha = redisClient.get(REDIS_SMS_RESET_PASSWORD_KEY + phone);
        if (!captcha.equals(redisCaptcha)) {
            throw new ServiceException("验证码错误");
        }
        AccountCriteria example = new AccountCriteria();
        example.createCriteria().andAccountEqualTo(phone);
        List<Account> accountList = accountMapper.selectByExample(example);
        if (!accountList.isEmpty()) {
            Account updateAccount = new Account();
            updateAccount.setId(accountList.get(0).getId());
            updateAccount.setPassword(DigestUtils.md5DigestAsHex(newPassword.getBytes(StandardCharsets.UTF_8)));
            accountMapper.updateByPrimaryKeySelective(updateAccount);
        }
    }

    public String getReTokenByAccountId(Integer accountId) {
        Account accountInfo = accountMapper.selectByPrimaryKey(accountId);
        AccountRoleCriteria example = new AccountRoleCriteria();
        example.createCriteria().andAccountIdEqualTo(accountInfo.getId());
        List<AccountRole> arList = accountRoleMapper.selectByExample(example);
        if (arList.isEmpty())
            throw new ServiceException("获取角色失败");
        int roleIdInt = arList.get(0).getRoleId().intValue();
        int roleId = arList.get(0).getRoleId().intValue();

        Integer oemId = null;
        Integer oemType = 0;
        Integer installSupplierId = null;
        Integer installStaffId = null;
        String regionIds = "";
        if (roleIdInt == 2 || roleIdInt == 3) {
            Oem oem = oemDao.selectByAccountId(accountInfo.getId());
            if (oem == null)
                throw new ServiceException("获取主机厂/经销商失败");
            oemType = oem.getType();
            oemId = oem.getId();
        } else if (roleIdInt == 4) {
            Supplier supplier = supplierDao.selectByAccountId(accountInfo.getId());
            if (supplier == null)
                throw new ServiceException("获取服务商失败");
            installSupplierId = Integer.valueOf(supplier.getId());
        } else {
            Integer parentId = accountInfo.getParentId();
            if (parentId != null) {
                List<Role> parentRoleList = roleService.findAccountRoleList(parentId.toString());
                if (parentRoleList.isEmpty())
                    throw new ServiceException("获取用户角色列表失败");

                roleIdInt = parentRoleList.get(0).getId().intValue();
                if (roleIdInt == 2 || roleIdInt == 3) {
                    Oem oem = oemDao.selectByAccountId(parentId);
                    if (oem == null)
                        throw new ServiceException("获取主机厂/经销商失败");

                    oemId = oem.getId();
                } else if (roleIdInt == 4) {
                    Supplier supplier = supplierDao.selectByAccountId(parentId);
                    if (supplier == null)
                        throw new ServiceException("获取服务商失败");
                    installSupplierId = Integer.valueOf(supplier.getId());

                    List<SupplierStaff> supplierStaffList = supplierStaffDao.selectByAccountId(accountInfo.getId());
                    if (!supplierStaffList.isEmpty()) {
                        installStaffId = supplierStaffList.get(0).getId();// 电工id
                    }
                } else {
                    List<CamsAccount> camsAccountList = camsAccountDao.selectByAccountIdAndDeptId(accountInfo.getId(), null);
                    if (!camsAccountList.isEmpty()) {
                        roleIdInt = 999;// cams账号
                        regionIds = camsAccountList.get(0).getRegionId();// 负责区域
                    }
                }

            }
        }

        SessionInfo sessionInfo = new SessionInfo();
        sessionInfo.setUserId(accountInfo.getId().toString());
        sessionInfo.setUserName(accountInfo.getName());
        sessionInfo.setAttribute("roleId", roleIdInt + "");
        sessionInfo.setAttribute("roleIdActual", roleId + "");
        sessionInfo.setAttribute("regionIds", regionIds);
        if (oemId != null) {
            sessionInfo.setAttribute("oemId", oemId.toString());
            sessionInfo.setAttribute("oemType", oemType.toString());
        }
        if (installSupplierId != null)
            sessionInfo.setAttribute("installSupplierId", installSupplierId.toString());
        if (installStaffId != null)
            sessionInfo.setAttribute("installStaffId", installStaffId.toString());
        return tokenService.saveToken(sessionInfo);
    }
}
