package com.starcharge.wios.bo;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.springframework.util.StringUtils;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.StringUtilsEx;
import com.starcharge.wios.dao.entity.OemContract;
import com.starcharge.wios.dao.entity.OemContractItem;
import com.starcharge.wios.dao.entity.OemContractPrice;
import com.starcharge.wios.dao.entity.UploadFile;

import io.swagger.annotations.ApiModelProperty;

public class OemContractBo extends OemContract {

    @ApiModelProperty(value = "充电桩及增项费用json字符串")
    private String itemsJson;
    
    @ApiModelProperty(hidden = true)
    private List<OemContractItem> items;
    
    @ApiModelProperty(value = "区域价格json字符串")
    private String pricesJson;
    
    @ApiModelProperty(hidden = true)
    private List<OemContractPrice> prices;
    
    @ApiModelProperty(value = "文件json字符串")
    private String filesJson;
    
    @ApiModelProperty(hidden = true)
    private List<UploadFile> files;
    
    @ApiModelProperty(value = "开始日期 yyyy-MM-dd")
    private String startDateStr;
    
    @ApiModelProperty(value = "结束日期 yyyy-MM-dd")
    private String endDateStr;
    
    @ApiModelProperty(value = "操作类型 0:变更 1：新增")
    private Integer operateType;// 0:变更 1：新增
    
    public String getItemsJson() {
        return itemsJson;
    }
    
    public List<OemContractItem> getItems() {
        return items;
    }
    
    public String getPricesJson() {
        return pricesJson;
    }
    
    public List<OemContractPrice> getPrices() {
        return prices;
    }
    
    public void setItemsJson(String itemsJson) {
        if (!StringUtils.isEmpty(itemsJson)) {
            itemsJson = StringUtilsEx.unescapeXss(itemsJson);
            this.itemsJson = itemsJson;
            try {
                ObjectMapper mapper = new ObjectMapper();
                mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
                List<OemContractItem> list = mapper.readValue(itemsJson, new TypeReference<List<OemContractItem>>() {
                });
                this.items = list;
            } catch (Exception e) {
                throw new ServiceException("数据转换异常");
            }
        }
    }
    
    public void setItems(List<OemContractItem> items) {
        this.items = items;
    }
    
    public void setPricesJson(String pricesJson) {
        if (!StringUtils.isEmpty(pricesJson)) {
            pricesJson = StringUtilsEx.unescapeXss(pricesJson);
            this.pricesJson = pricesJson;
            try {
                ObjectMapper mapper = new ObjectMapper();
                mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
                List<OemContractPrice> list = mapper.readValue(pricesJson, new TypeReference<List<OemContractPrice>>() {
                });
                this.prices = list;
            } catch (Exception e) {
                throw new ServiceException("数据转换异常");
            }
        }
    }
    
    public void setPrices(List<OemContractPrice> prices) {
        this.prices = prices;
    }
    
    public String getStartDateStr() {
        return startDateStr;
    }
    
    public String getEndDateStr() {
        return endDateStr;
    }
    
    public void setStartDateStr(String startDateStr) {
        if (!StringUtils.isEmpty(startDateStr)) {
            this.startDateStr = startDateStr;
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            try {
                Date start = sdf.parse(startDateStr);
                setStartTime(start);
            } catch (Exception e) {
                throw new ServiceException("开始时间转换失败");
            }
        }
    }
    
    public void setEndDateStr(String endDateStr) {
        if (!StringUtils.isEmpty(endDateStr)) {
            this.endDateStr = endDateStr;
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            try {
                Date end = sdf.parse(endDateStr);
                setEndTime(end);
            } catch (Exception e) {
                throw new ServiceException("结束时间转换失败");
            }
        }
    }
    
    public String getFilesJson() {
        return filesJson;
    }
    
    public List<UploadFile> getFiles() {
        return files;
    }
    
    public void setFilesJson(String filesJson) {
        if (!StringUtils.isEmpty(filesJson)) {
            filesJson = StringUtilsEx.unescapeXss(filesJson);
            this.filesJson = filesJson;
            try {
                ObjectMapper mapper = new ObjectMapper();
                mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
                List<UploadFile> list = mapper.readValue(filesJson, new TypeReference<List<UploadFile>>() {
                });
                this.files = list;
            } catch (Exception e) {
                throw new ServiceException("数据转换异常");
            }
        }
        
    }
    
    public void setFiles(List<UploadFile> files) {
        this.files = files;
    }
    
    public Integer getOperateType() {
        return operateType;
    }
    
    public void setOperateType(Integer operateType) {
        this.operateType = operateType;
    }
    
}
