package com.starcharge.wios.bo;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.StringUtilsEx;
import com.starcharge.wios.dao.entity.OrderInstallItem;
import com.starcharge.wios.dao.entity.OrderSurvey;
import com.starcharge.wios.dao.entity.UploadFile;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.util.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

public class OrderSurveyBo extends OrderSurvey {
    @ApiModelProperty(value = "文件json字符串")
    private String filesJson;
    
    @ApiModelProperty(hidden = true)
    private List<UploadFile> files;
    
    @ApiModelProperty(value = "物料费用json字符串")
    private String itemFeesJson;
    
    @ApiModelProperty(hidden = true)
    private List<OrderInstallItem> itemFees;
    
    @ApiModelProperty(value = "勘测日期字符串")
    private String surveyDateStr;

    @ApiModelProperty(value = "墙盒品牌（勘测）")
    private String surveyWallboxModel;

    @ApiModelProperty(value = "墙盒型号（勘测）")
    private String surveyItemModel;

    
    public String getFilesJson() {
        return filesJson;
    }
    
    public List<UploadFile> getFiles() {
        return files;
    }
    
    public void setFilesJson(String filesJson) {
        if (!StringUtils.isEmpty(filesJson)) {
            filesJson = StringUtilsEx.unescapeXss(filesJson);
            this.filesJson = filesJson;
            try {
                ObjectMapper mapper = new ObjectMapper();
                mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
                List<UploadFile> list = mapper.readValue(filesJson, new TypeReference<List<UploadFile>>() {
                });
                List<UploadFile> useTypeNullList = list.stream().filter(item -> item.getUseType() == null).collect(Collectors.toList());
                if (!useTypeNullList.isEmpty())
                    throw new ServiceException("文件useType不能为空");
                this.files = list;
            } catch (Exception e) {
                throw new ServiceException("数据转换异常");
            }
        }
    }
    
    public void setFiles(List<UploadFile> files) {
        this.files = files;
    }
    
    public String getItemFeesJson() {
        return itemFeesJson;
    }
    
    public List<OrderInstallItem> getItemFees() {
        return itemFees;
    }
    
    public void setItemFeesJson(String itemFeesJson) {
        if (!StringUtils.isEmpty(itemFeesJson)) {
            itemFeesJson = StringUtilsEx.unescapeXss(itemFeesJson);
            this.itemFeesJson = itemFeesJson;
            try {
                itemFeesJson = itemFeesJson.replaceAll("createTime", "createTime1");
                ObjectMapper mapper = new ObjectMapper();
                mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
                List<OrderInstallItem> itemList = mapper.readValue(itemFeesJson, new TypeReference<List<OrderInstallItem>>() {
                });
                itemList.stream().forEach(item -> {
                    if(StringUtils.isEmpty(item.getContractItemId())){
                        item.setContractItemId(item.getId());
                    }
                    item.setId(null);
                });
                List<OrderInstallItem> itemIdNullList =
                    itemList.stream().filter(item -> StringUtils.isEmpty(item.getItemId())).collect(Collectors.toList());
                if (!itemIdNullList.isEmpty())
                    throw new ServiceException("itemId不能为空");
                List<OrderInstallItem> itemNameNullList =
                    itemList.stream().filter(item -> StringUtils.isEmpty(item.getItemName())).collect(Collectors.toList());
                if (!itemNameNullList.isEmpty())
                    throw new ServiceException("itemName不能为空");
                List<OrderInstallItem> contractItemIdNullList =
                    itemList.stream().filter(item -> item.getContractItemId() == null).collect(Collectors.toList());
                if (!contractItemIdNullList.isEmpty())
                    throw new ServiceException("contractItemId不能为空");
                itemList = itemList.stream()
                    .filter(item -> item.getQunatity() != null && item.getFee() != null && item.getQunatity().intValue() >= 0)
                    .collect(Collectors.toList());
                // List<OrderInstallItem> feeNullList = itemList.stream().filter(item -> item.getFee() ==
                // null).collect(Collectors.toList());
                // if (!feeNullList.isEmpty())
                // throw new ServiceException("fee不能为空");
                // List<OrderInstallItem> quantityNullList = itemList.stream()
                // .filter(item -> item.getQunatity() == null || item.getQunatity().intValue() ==
                // 0).collect(Collectors.toList());
                // if (!quantityNullList.isEmpty())
                // throw new ServiceException("quantity不能为空");
                this.itemFees = itemList;
            } catch (JsonProcessingException e) {
                throw new ServiceException("数据转换异常");
            }
        }
    }
    
    public void setItemFees(List<OrderInstallItem> itemFees) {
        this.itemFees = itemFees;
    }
    
    public String getSurveyDateStr() {
        return surveyDateStr;
    }
    
    public void setSurveyDateStr(String surveyDateStr) {
        this.surveyDateStr = surveyDateStr;
        if (!StringUtils.isEmpty(surveyDateStr)) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            try {
                Date surveyDate = sdf.parse(surveyDateStr);
                this.setSurveyDate(surveyDate);
            } catch (ParseException e) {
                throw new ServiceException("勘测日期格式错误");
            }
        }
    }

    public String getSurveyWallboxModel() {
        return surveyWallboxModel;
    }

    public void setSurveyWallboxModel(String surveyWallboxModel) {
        this.surveyWallboxModel = surveyWallboxModel;
    }

    public String getSurveyItemModel() {
        return surveyItemModel;
    }

    public void setSurveyItemModel(String surveyItemModel) {
        this.surveyItemModel = surveyItemModel;
    }
}
