package com.starcharge.wios.call.service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.JSONUtilsEx;
import com.starcharge.wios.call.PushApi;
import com.starcharge.wios.vo.OrderVo;
import org.springframework.util.StringUtils;

/**
 * 推送service
 * 
 * @author xgl
 * @version [版本号, 2020年10月20日]
 */
@Service
public class PushService {
    @Autowired
    private PushApi pushApi;
    
    private Logger logger = LoggerFactory.getLogger(PushService.class);
    
    private final static String CUSTOMER_PHONE = "400-6657-666";// 客服电话
    
    private static final String INSTALL_APPROVE_FINISH = "SMS_207948721";// 安装审批通过1
    
    private static final String INSTALL_ANHOUR_BEFORE = "SMS_207953701";// 上门安装前一个小时1
    
    private static final String INSTALL_RESERVE_MODIFY = "SMS_207973700";// 修改预约安装信息成功1
    
    private static final String INSTALL_RESERVE_SAVE_SUCCESS = "SMS_207973693";// 预约安装信息录入成功1
    
    private static final String SURVEY_ANHOUR_BEFORE = "SMS_207953685";// 上门勘查前一个小时1
    
    private static final String SURVEY_RESERVE_MODITY = "SMS_207948703";// 修改预约勘查信息1
    
    private static final String SURVEY_RESERVE_SAVE_SUCCESS = "SMS_207973683";// 预约勘查信息
    
    private static final String USER_LOGIN_CODE = "SMS_176180051";// 客户登录验证码
    
    private static final String USER_RESET_PASSWORD = "SMS_176180048";// 后台重置密码
    
    /**
     * 异步发送邮件
     *
     * @param toAddress
     * @param subject
     * @param body
     */
    @Async
    public void sendEmailAsync(String toAddress, String subject, String body) {
        sendEmail(toAddress, subject, body);
    }
    
    /**
     * 发送邮件
     *
     * @param toAddress
     * @param subject
     * @param body
     */
    public void sendEmail(String toAddress, String subject, String body) {
        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("toAddress", toAddress);
        paramMap.put("subject", subject);
        paramMap.put("body", body);
        pushApi.gatewayAgent("email/singleMsg/send", paramMap);
    }
    
    /**
     * 发送短信消息
     *
     * @param account
     * @param template
     * @param dyMaps
     */
    public void sendMessage(String account, String template, Map<String, String> dyMaps) {
        try {
            // 调用push发送验证码
            Map<String, String> paramsMap = new HashMap<>(1);
            // 接收短信的手机号码。支持对多个手机号码发送短信，手机号码之间以英文逗号（,）分隔。上限为1000个手机号码。批量调用相对于单条调用及时性稍有延迟。
            paramsMap.put("phoneNumbers", account);
            // 短信模板ID
            paramsMap.put("templateCode", template);
            paramsMap.put("templateParam", JSONUtilsEx.serialize(dyMaps));
            // 表示类型 使用APP的进行发送
            paramsMap.put("application", "wios");
            paramsMap.put("signName", "CAMS充电");
            paramsMap.put("accessConfigId", "3");
            pushApi.gatewayAgent("sms/send", paramsMap, Object.class);
            logger.info("发送短信,手机号:" + account + ",参数-[{}]" + JSONUtilsEx.serialize(paramsMap));
        } catch (Exception e) {
            logger.error("发送短信失败,手机号:" + account + ",错误信息:" + e.getMessage());
            // if (e != null && "短信发送过于频繁，请稍后再试!".equals(e.getMessage())) {
            // throw new ServiceException("短信发送过于频繁，请稍后再试!");
            // } else {
            throw new ServiceException("短信发送失败");
            // }
        }
    }
    
    @Async
    public void sendMessageAsync(String account, String template, Map<String, String> dyMaps) {
        try{
            sendMessage(account, template, dyMaps);
        } catch (Exception e) {
        }
    }
    
    public void sendIntallApproveFinish(String phones, String name) {
        Map<String, String> dyMaps = new HashMap<>();
        dyMaps.put("name", name);
        dyMaps.put("customerPhone", CUSTOMER_PHONE);
        // 2021-01-12注释
        // sendMessageAsync(phones, INSTALL_APPROVE_FINISH, dyMaps);
    }
    
    public void sendInstallAnhourBefore(String phones, String name) {
        Map<String, String> dyMaps = new HashMap<>();
        dyMaps.put("name", name);
        dyMaps.put("customerPhone", CUSTOMER_PHONE);
        // sendMessageAsync(phones, INSTALL_ANHOUR_BEFORE, dyMaps);
    }
    
    public void sendIntallReserveModify(OrderVo order) {
        Date installReserveTime = order.getInstallReserveTime();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日 HH点mm分");
        Map<String, String> dyMaps = new HashMap<>();
        dyMaps.put("name", order.getInstallClientName());
        dyMaps.put("time", sdf.format(installReserveTime));
        dyMaps.put("address", order.getAddress());
        dyMaps.put("staffName", order.getInstallStaffName());
        dyMaps.put("staffPhone", order.getInstallStaffPhone());
        dyMaps.put("customerPhone", CUSTOMER_PHONE);
        // sendMessageAsync(order.getInstallClientPhone(), INSTALL_RESERVE_MODIFY, dyMaps);
    }
    
    public void sendIntallReserveSaveSuccess(OrderVo order) {
        Date installReserveTime = order.getInstallReserveTime();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日 HH点mm分");
        Map<String, String> dyMaps = new HashMap<>();
        dyMaps.put("name", order.getInstallClientName());
        dyMaps.put("time", sdf.format(installReserveTime));
        dyMaps.put("address", order.getAddress());
        dyMaps.put("staffName", order.getInstallStaffName());
        dyMaps.put("staffPhone", order.getInstallStaffPhone());
        dyMaps.put("customerPhone", CUSTOMER_PHONE);
        // sendMessageAsync(order.getInstallClientPhone(), INSTALL_RESERVE_SAVE_SUCCESS, dyMaps);
    }
    
    public void sendSurveyAnhourBefore(String phones, String name) {
        Map<String, String> dyMaps = new HashMap<>();
        dyMaps.put("name", name);
        dyMaps.put("customerPhone", CUSTOMER_PHONE);
        // sendMessageAsync(phones, SURVEY_ANHOUR_BEFORE, dyMaps);
    }
    
    public void sendSurveyReserveModify(OrderVo order) {
        Date surveyReserveTime = order.getSurveyReserveTime();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日 HH点mm分");
        Map<String, String> dyMaps = new HashMap<>();
        dyMaps.put("name", order.getSurveyClientName());
        if(!StringUtils.isEmpty(surveyReserveTime)) {
            dyMaps.put("time", sdf.format(surveyReserveTime));
        }
        dyMaps.put("address", order.getAddress());
        dyMaps.put("staffName", order.getInstallStaffName());
        dyMaps.put("staffPhone", order.getInstallStaffPhone());
        dyMaps.put("customerPhone", CUSTOMER_PHONE);
        // sendMessageAsync(order.getSurveyClientPhone(), SURVEY_RESERVE_MODITY, dyMaps);
    }
    
    public void sendSurveyReserveSaveSucess(OrderVo order) {
        Date surveyReserveTime = order.getSurveyReserveTime();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日 HH点mm分");
        Map<String, String> dyMaps = new HashMap<>();
        dyMaps.put("name", order.getSurveyClientName());
        dyMaps.put("time", sdf.format(surveyReserveTime));
        dyMaps.put("address", order.getAddress());
        dyMaps.put("staffName", order.getInstallStaffName());
        dyMaps.put("staffPhone", order.getInstallStaffPhone());
        dyMaps.put("customerPhone", CUSTOMER_PHONE);
        // sendMessageAsync(order.getSurveyClientPhone(), SURVEY_RESERVE_SAVE_SUCCESS, dyMaps);
    }
    
    public void sendUserLoginCode(String phones, String code) {
        Map<String, String> dyMaps = new HashMap<>();
        dyMaps.put("code", code);
        sendMessage(phones, USER_LOGIN_CODE, dyMaps);
    }
    
    public void sendResetPwdCode(String phones, String code) {
        Map<String, String> dyMaps = new HashMap<>();
        dyMaps.put("code", code);
        sendMessage(phones, USER_RESET_PASSWORD, dyMaps);
    }
}
