package com.starcharge.wios.client;

import com.deppon.dop.module.sdk.shared.util.FastJsonUtil;
import com.deppon.dop.module.sdk.shared.util.SecurityUtil;
import com.dtflys.forest.exceptions.ForestRuntimeException;
import com.dtflys.forest.http.ForestRequest;
import com.dtflys.forest.http.ForestResponse;
import com.dtflys.forest.interceptor.Interceptor;
import com.dtflys.forest.reflection.ForestMethod;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.DigestUtils;

/**
 * <p>
 * 德邦物流请求拦截器，实现自动注入companyCode、digest、timestamp参数
 * <p>
 *
 * @author <a href="mail to: ning.chai@foxmail.com" rel="nofollow">chaining</a>
 */
public class DopInterceptor <T> implements Interceptor<T> {

    private final String companyCode = "EWBXRCHSZX";
    private final String appkey = "fe38a755ce4f7629717342e441cdedff";

    /**
     * 该方法在被调用时
     *
     * @Param request Forest请求对象
     * @Param args 方法被调用时传入的参数数组
     */
    @Override
    public void onInvokeMethod(ForestRequest request, ForestMethod method, Object[] args) {
    }

    @Override
    public boolean beforeExecute(ForestRequest request) {
        request.addHeader("Content-Type", "application/x-www-form-urlencoded;charset=utf-8");
//        String params = request.getBody().get(0)+"";
        long timestamp = System.currentTimeMillis();
        String params = FastJsonUtil.toJSONString(request.getBody().getNameValueBody("params").getValue());
        String digest = SecurityUtil.getStandardDigest(params + appkey + timestamp);
        request.addBody("companyCode", companyCode);
        request.addBody("digest", digest);
        request.addBody("timestamp", timestamp);
        request.addBody("params", params);
        return true;
    }

    /**
     * 该方法在请求成功响应时被调用
     */
    @Override
    public void onSuccess(T data, ForestRequest req, ForestResponse res) {
    }

    /**
     * 该方法在请求发送失败时被调用
     */
    @Override
    public void onError(ForestRuntimeException ex, ForestRequest req, ForestResponse res) {
        // 执行发送请求失败后处理的代码
        int status = res.getStatusCode(); // 获取请求响应状态码
        String content = res.getContent();
    }

    /**
     * 该方法在请求发送之后被调用
     */
    @Override
    public void afterExecute(ForestRequest req, ForestResponse res) {
    }

    private String getDigest(String plainText) {
        return Base64.encodeBase64String(DigestUtils.md5Hex(plainText).getBytes());
    }
}
