package com.starcharge.wios.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.base.util.Assert;
import com.starcharge.wios.bo.CamsAccountBo;
import com.starcharge.wios.service.CamsAccountService;
import com.starcharge.wios.vo.CamsAccountVo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * cams账号相关接口
 * 
 * @author xgl
 * @version [版本号, 2020年9月30日]
 */
@RestController
@RequestMapping("api")
@Api(tags = {"cams账号相关接口"})
public class CamsAccountController {
    @Autowired
    private CamsAccountService camsAccountService;
    
    /**
     * 添加cams账号
     *
     * @param camsAccount
     * @return
     */
    @PostMapping(value = "camsAccount/add")
    @ApiOperation(value = "添加cams账号")
    @ApiImplicitParams({@ApiImplicitParam(name = "account", value = "账户", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "roleId", value = "角色id", dataType = "int", paramType = "query", required = true),
        @ApiImplicitParam(name = "userName", value = "用户姓名", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "departmentId", value = "部门id", dataType = "int", paramType = "query", required = true),
        @ApiImplicitParam(name = "regionId", value = "区域id", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "alarmId", value = "超时提醒id", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "email", value = "邮箱", dataType = "String", paramType = "query", required = true),
    
    })
    public BaseResponse<Object> add(CamsAccountBo camsAccount) {
        if (!StringUtils.isEmpty(camsAccount.getUserName()) && camsAccount.getUserName().length() > 14) {
            throw new ServiceException("名字过长");
        }
        Assert.notNull(camsAccount.getRoleId(), "角色不能为空");
        camsAccountService.add(camsAccount);
        return new BaseResponse<>();
    }
    
    /**
     * 修改cams账号
     *
     * @param camsAccount
     * @return
     */
    @PostMapping(value = "camsAccount/update")
    @ApiOperation(value = "修改cams账号")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "账户id", dataType = "int", paramType = "query", required = true),
        @ApiImplicitParam(name = "departmentId", value = "部门id", dataType = "int", paramType = "query", required = false),
        @ApiImplicitParam(name = "regionId", value = "区域id", dataType = "String", paramType = "query", required = false),
        @ApiImplicitParam(name = "alarmId", value = "超时提醒id", dataType = "String", paramType = "query", required = false),
        @ApiImplicitParam(name = "roleId", value = "角色id", dataType = "int", paramType = "query", required = false),
        @ApiImplicitParam(name = "status", value = "状态 1：正常 2：禁用 3：删除", dataType = "int", paramType = "query", required = false),
        @ApiImplicitParam(name = "email", value = "邮箱", dataType = "String", paramType = "query", required = true),})
    public BaseResponse<Object> update(CamsAccountBo accountBo) {
        if (accountBo.getId() == null) {
            throw new ServiceException("缺少参数");
        }
        if (!StringUtils.isEmpty(accountBo.getUserName()) && accountBo.getUserName().length() > 14) {
            throw new ServiceException("名字过长");
        }
        camsAccountService.update(accountBo.getId(), accountBo);
        return new BaseResponse<>();
    }
    
    /**
     * 查询列表
     *
     * @param account
     * @param userName
     * @return
     */
    @GetMapping(value = "camsAccount/list")
    @ApiOperation(value = "查询列表")
    @ApiImplicitParams({@ApiImplicitParam(name = "account", value = "登陆手机号", dataType = "String", paramType = "query", required = false),
        @ApiImplicitParam(name = "userName", value = "用户姓名", dataType = "String", paramType = "query", required = false),})
    public BaseResponse<Object> list(String account, String userName) {
        CamsAccountBo condition = new CamsAccountBo();
        condition.setAccount(account);
        condition.setUserName(userName);
        return new BaseResponse<Object>(camsAccountService.getList(condition));
    }
    
    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    @GetMapping(value = "camsAccount/detail")
    @ApiOperation(value = "查询详情")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "query", name = "id", value = "id", required = true, dataType = "int"),})
    public BaseResponse<CamsAccountVo> getDetail(Integer id) {
        Assert.notNull(id, "id不能为空");
        CamsAccountVo dept = camsAccountService.getDetailById(id);
        return new BaseResponse<CamsAccountVo>(dept);
    }
}
