package com.starcharge.wios.controller;

import com.ihidea.component.api.v2.BaseResponse;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.convert.DeliveryConvert;
import com.starcharge.wios.dao.entity.Delivery;
import com.starcharge.wios.dao.mappers.DeliveryMapper;
import com.starcharge.wios.dao.mappers.WallboxApplyMapper;
import com.starcharge.wios.service.CommonUpdateService;
import com.starcharge.wios.service.SnowFlakeService;
import com.starcharge.wios.utils.ExcelUtils;
import com.starcharge.wios.validation.CreateEntityGroup;
import com.starcharge.wios.validation.ParamsValidate;
import com.starcharge.wios.validation.UpdateEntityGroup;
import com.starcharge.wios.vo.DeliveryAddVo;
import com.starcharge.wios.vo.DeliveryExcelVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

/**
 * @projectName:xr-wios
 * @packageName:com.starcharge.wios.controller
 * @Description:TODO
 * @Author:luyincheng
 * @CreateDate: 2023/10/818:10
 */
@RestController
@RequestMapping("api/delivery")
@Api(tags = "发货管理接口")
public class DeliveryController {
    @Autowired
    private TokenService tokenService;
    @Autowired
    private DeliveryMapper deliveryMapper;
    @Autowired
    private CommonUpdateService commonUpdateService;
    @Autowired
    private SnowFlakeService snowFlakeService;
    @Autowired
    private WallboxApplyMapper wallboxApplyMapper;
    @PostMapping("/add")
    @ApiOperation(value = "新增发货",notes = "新增发货，返回成功条数")
    @ParamsValidate
    @Transactional
    public BaseResponse<Integer> AddDelivery(@RequestBody @Validated(CreateEntityGroup.class) List<DeliveryAddVo> deliveryAddVoList, BindingResult bindingResult){
        String orderid= snowFlakeService.getNextId(0,0)+"";
        return new BaseResponse(deliveryAddVoList.stream()
                .map(da->{
                    Delivery delivery= DeliveryConvert.INSTANCE.convertAddVo(da);
                    delivery.setDeliveryOrder(orderid);
                    this.commonUpdateService.UpdateColumns(delivery,true);
                    this.wallboxApplyMapper.updateWallBoxTrackingStatusByMC("1",delivery.getMaterialCode());
                    return this.deliveryMapper.insertSelective(delivery);
                }).reduce(0,Integer::sum));
    }
    @PostMapping("/query")
    @ApiOperation(value = "发货列表查询")
    public BaseResponse<List<Delivery>> queryDelivery(Delivery Delivery){
        return new BaseResponse(this.deliveryMapper.selectDeliveryList(Delivery));
    }
    @GetMapping("/detail")
    @ApiOperation(value = "发货详情")
    public BaseResponse<Delivery> getdetail(@RequestParam int id){
        return new BaseResponse(this.deliveryMapper.selectByPrimaryKey(id));
    }
    @GetMapping("/delete")
    @ApiOperation(value = "删除发货信息")
    public BaseResponse<Integer> delete(@RequestParam int id){
        return new BaseResponse(this.deliveryMapper.deleteByPrimaryKey(id));
    }
    @PostMapping("/update")
    @ApiOperation("更新发货单信息")
    @ParamsValidate
    public BaseResponse<Delivery> update(@RequestBody @Validated(UpdateEntityGroup.class) Delivery delivery, BindingResult bindingResult){
        this.commonUpdateService.UpdateColumns(delivery,false);
        return new BaseResponse(this.deliveryMapper.updateByPrimaryKeySelective(delivery));
    }

    @PostMapping("/export")
    @ApiOperation(value = "导出发货单列表")
    public void export(HttpServletResponse response, @RequestBody Delivery Delivery) throws IOException {

//        ExcelUtil<WarehouseInfo> util = new ExcelUtil<WarehouseInfo>(WarehouseInfo.class);
//        util.exportExcel(response, list, "仓库信息管理数据");
        List<DeliveryExcelVo> data = DeliveryConvert.INSTANCE.convertExcelVoList(this
                .deliveryMapper.selectDeliveryList(Delivery));
        // 输出
        ExcelUtils.write(response, "发货管理列表.xls", "发货信息", DeliveryExcelVo.class, data);
    }


}
