package com.starcharge.wios.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.base.util.Assert;
import com.starcharge.wios.bo.OemContractBo;
import com.starcharge.wios.bo.OemContractBo2;
import com.starcharge.wios.service.OemContractService;
import com.starcharge.wios.vo.OemContractVo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 主机厂合同接口
 * 
 * @author xgl
 * @version [版本号, 2020年9月18日]
 */
@RestController
@RequestMapping("api")
@Api(tags = "主机厂合同接口管理接口")
public class OemContractController {
    @Autowired
    private OemContractService oemContractService;
    
    /**
     * 添加合同数据
     *
     * @param contractBo
     */
    @PostMapping(value = "oemContract/add")
    @ApiOperation(value = "添加合同数据")
    @ApiImplicitParams({@ApiImplicitParam(name = "code", value = "合同编号", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "oemId", value = "主机厂id", dataType = "int", paramType = "query", required = true),
        @ApiImplicitParam(name = "deviceSettlementId", value = "设备结算方", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "installSettlementId", value = "安装服务结算方", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "saleAccount", value = "销售员", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "startDateStr", value = "合同开始时间", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "endDateStr", value = "合同结束时间", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "guaranteePeriod", value = "质保期(月)", dataType = "int", paramType = "query", required = true),
        @ApiImplicitParam(name = "settleInterval", value = "结算周期（月）", dataType = "int", paramType = "query", required = true),
        @ApiImplicitParam(name = "standardLength", value = "标准米数", dataType = "int", paramType = "query", required = true),
        @ApiImplicitParam(name = "taxRate", value = "税率", dataType = "double", paramType = "query", required = true),
        @ApiImplicitParam(name = "camsDeviceEnable", value = "是否使用cams设备 0：否 1：是", dataType = "int", paramType = "query", required = true),
    
    })
    public BaseResponse<Object> add(OemContractBo contractBo) {
        if (contractBo.getStartTime() == null || contractBo.getEndTime() == null) {
            throw new ServiceException("缺少日期");
        }
        if (StringUtils.isEmpty(contractBo.getOemId())) {
            throw new ServiceException("缺少主机厂id");
        }
        oemContractService.add(contractBo);
        return new BaseResponse<>();
    }
    
    /**
     * 更新合同数据
     *
     * @param contractBo
     */
    @PostMapping(value = "oemContract/update")
    @ApiOperation(value = "更新合同数据")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "合同id", dataType = "int", paramType = "query", required = true),
    
    })
    public BaseResponse<Object> update(OemContractBo contractBo) {
        Assert.notNull(contractBo.getId(), "缺少id");
        oemContractService.update(contractBo);
        return new BaseResponse<>();
    }
    
    /**
     * 查询列表
     *
     * @param oemBo
     * @return
     */
    @GetMapping(value = "oemContract/list")
    @ApiOperation(value = "查询合同列表")
    @ApiImplicitParams({@ApiImplicitParam(name = "page", value = "一页显示数量", dataType = "int", paramType = "query", required = false),
        @ApiImplicitParam(name = "pagecount", value = "页码", dataType = "int", paramType = "query", required = false),})
    public BaseResponse<List<OemContractVo>> list(OemContractBo2 oemContract) {
        return new BaseResponse<List<OemContractVo>>(oemContractService.getList(oemContract));
    }
    
    /**
     * 查询合同详情
     *
     * @param id
     * @return
     */
    @GetMapping(value = "oemContract/detail")
    @ApiOperation(value = "查询合同详情")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "id", dataType = "int", paramType = "query", required = true)})
    public BaseResponse<OemContractVo> detail(Integer id) {
        Assert.notNull(id, "id不能为空");
        return new BaseResponse<OemContractVo>(oemContractService.getDetail(id));
    }
    
    /**
     * 根据主机厂id查询合同详情
     *
     * @param id
     * @return
     */
    @GetMapping(value = "oemContract/detailByOemId")
    @ApiOperation(value = "根据主机厂id查询合同详情")
    @ApiImplicitParams({@ApiImplicitParam(name = "oemId", value = "主机厂id", dataType = "int", paramType = "query", required = true)})
    public BaseResponse<OemContractVo> detailByOemId(Integer oemId) {
        Assert.notNull(oemId, "oemId不能为空");
        return new BaseResponse<OemContractVo>(oemContractService.getDetailByOemId(oemId));
    }
}
