package com.starcharge.wios.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.wios.dao.entity.OrderScore;
import com.starcharge.wios.service.OrderScoreService;
import com.starcharge.wios.service.OrderService;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 订单评分表 管理 API
 *
 * @author yongfu.liu@wanbangauto.com 2020-10-10 14:08:14
 */
@RestController
@RequestMapping("api")
@Api(tags = "订单评分表的接口")
public class OrderScoreController {

    @Autowired
    private OrderScoreService orderScoreService;

    @Autowired
    private OrderService orderService;

    /**
     * 查询订单评分
     *
     * @return
     */
    @GetMapping(value = "orderScore/getOrderScoreByOrderId")
    @ApiOperation(value = "查询订单评分")
    @ApiImplicitParam(paramType = "query", name = "orderId", value = "订单表ID", required = true, dataType = "String")
    public BaseResponse<Object> list(String orderId) {
        return new BaseResponse<Object>(orderScoreService.getOrderScoreByOrderId(orderId));
    }

    /**
     * 保存订单回访信息
     *
     * @param list
     */
    @PostMapping(value = "orderScore/save")
    @ApiOperation(value = "保存订单回访信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "orderId", value = "订单id", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "visitContent", value = "意见或建议", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "list", value = "格式:[{\"id\":9,\"score\":70},{\"id\":12,\"score\":60}]", dataType = "String", paramType = "query", required = true),
            })
    public BaseResponse<Object> save(String orderId, String visitContent, String list) {

        if (StringUtils.isEmpty(orderId)) {
            throw new ServiceException("缺少参数");
        }

        //更新评分明细
        JSONArray jsonArray = JSONUtil.parseArray(list);
        if(jsonArray.size()>0){
            for(int i = 0 ;i<jsonArray.size();i++){
                JSONObject jsonObject =jsonArray.getJSONObject(i);
                OrderScore orderScore = new OrderScore();
                orderScore.setId(Long.parseLong(String.valueOf(jsonObject.get("id"))));
                orderScore.setScore(Double.valueOf(String.valueOf(jsonObject.get("score"))));
                orderScoreService.update(orderScore);
            }
        }
        //增加已回访的状态
        orderService.visitOrder(orderId, visitContent, null, jsonArray);

        return new BaseResponse<>();
    }

    /**
     * 评分小于60的订单，用于评分报表
     *
     * @return
     */
    @GetMapping(value = "orderScore/getListLessThan60")
    @ApiOperation(value = "评分小于60的订单，用于评分报表")
    @ApiImplicitParam(paramType = "query", name = "scoreSupplierId", value = "订单表ID", required = true, dataType = "int")
    public BaseResponse<Object> getListLessThan60(Long scoreSupplierId) {
        return new BaseResponse<Object>(orderScoreService.getListLessThan60(scoreSupplierId));
    }


}
