package com.starcharge.wios.controller;

import com.ihidea.component.api.v2.BaseResponse;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.convert.ReceivednoteConvert;
import com.starcharge.wios.dao.entity.Receivednote;
import com.starcharge.wios.dao.mappers.ReceivednoteMapper;
import com.starcharge.wios.service.CommonUpdateService;
import com.starcharge.wios.service.SnowFlakeService;
import com.starcharge.wios.utils.ExcelUtils;
import com.starcharge.wios.validation.CreateEntityGroup;
import com.starcharge.wios.validation.ParamsValidate;
import com.starcharge.wios.validation.UpdateEntityGroup;
import com.starcharge.wios.vo.ReceivednoteAddVo;
import com.starcharge.wios.vo.ReceivednoteExcelVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

/**
 * @projectName:xr-wios
 * @packageName:com.starcharge.wios.controller
 * @Description:TODO
 * @Author:luyincheng
 * @CreateDate: 2023/10/814:24
 */
@RestController
@RequestMapping("api/receivednote")
@Api(tags = "收货管理接口")
public class ReceivedNoteController {
    @Autowired
    private TokenService tokenService;
    @Autowired
    private ReceivednoteMapper receivednoteMapper;
    @Autowired
    private CommonUpdateService commonUpdateService;
    @Autowired
    private SnowFlakeService snowFlakeService;

    @PostMapping("/add")
    @ApiOperation(value = "新增收货",notes = "新增收货，返回成功条数")
    @ParamsValidate
    public BaseResponse<Integer> AddReceivednote(@RequestBody @Validated(CreateEntityGroup.class) ReceivednoteAddVo receivednoteAddVo, BindingResult bindingResult){
        String orderid= snowFlakeService.getNextId(0,0)+"";
        return new BaseResponse(receivednoteAddVo.getSingleReceivenotes().stream()
                .map(s->{
                    Receivednote receivednote= ReceivednoteConvert.INSTANCE.convertAddVo(receivednoteAddVo);
                    ReceivednoteConvert.INSTANCE.convertAddSingleVo(s,receivednote);
                    receivednote.setReceivedOrder(orderid);
                    this.commonUpdateService.UpdateColumns(receivednote,true);
                    return this.receivednoteMapper.insertSelective(receivednote);
                }).reduce(0,Integer::sum));
    }
    @PostMapping("/query")
    @ApiOperation(value = "收货列表查询")
    public BaseResponse<List<Receivednote>> queryReceivednote(Receivednote receivednote){
        return new BaseResponse<>(this.receivednoteMapper.selectReceivedNoteList(receivednote));
    }
    @GetMapping("/detail")
    @ApiOperation(value = "收货详情")
    public BaseResponse<Receivednote> getdetail(@RequestParam int id){
        return new BaseResponse<>(this.receivednoteMapper.selectByPrimaryKey(id));
    }
    @GetMapping("/delete")
    @ApiOperation(value = "删除收货信息")
    public BaseResponse<Integer> delete(@RequestParam int id){
        return new BaseResponse(this.receivednoteMapper.deleteByPrimaryKey(id));
    }
    @PostMapping("/update")
    @ApiOperation("更新收货单信息")
    @ParamsValidate
    public BaseResponse<Receivednote> update(@RequestBody @Validated(UpdateEntityGroup.class) Receivednote receivednote, BindingResult bindingResult){
        this.commonUpdateService.UpdateColumns(receivednote,false);
        return new BaseResponse(this.receivednoteMapper.updateByPrimaryKeySelective(receivednote));
    }

    @PostMapping("/export")
    @ApiOperation(value = "导出收货单列表")
    public void export(HttpServletResponse response, @RequestBody Receivednote receivednote) throws IOException {

//        ExcelUtil<WarehouseInfo> util = new ExcelUtil<WarehouseInfo>(WarehouseInfo.class);
//        util.exportExcel(response, list, "仓库信息管理数据");
        List<ReceivednoteExcelVo> data = ReceivednoteConvert.INSTANCE.convertExcelVoList(this
                .receivednoteMapper.selectReceivedNoteList(receivednote));
        // 输出
        ExcelUtils.write(response, "收货管理列表.xls", "收货信息", ReceivednoteExcelVo.class, data);
    }
}
