package com.starcharge.wios.controller;

import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.convert.RejectConvert;
import com.starcharge.wios.dao.entity.Reject;
import com.starcharge.wios.dao.mappers.RejectMapper;
import com.starcharge.wios.enums2.TauxiliarymaterialStatusEnum;
import com.starcharge.wios.service.CommonUpdateService;
import com.starcharge.wios.service.SnowFlakeService;
import com.starcharge.wios.utils.ExcelUtils;
import com.starcharge.wios.validation.CreateEntityGroup;
import com.starcharge.wios.validation.ParamsValidate;
import com.starcharge.wios.validation.UpdateEntityGroup;
import com.starcharge.wios.vo.RejectAddVo;
import com.starcharge.wios.vo.RejectCheckVo;
import com.starcharge.wios.vo.RejectExcelVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.var;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @projectName:xr-wios
 * @packageName:com.starcharge.wios.controller
 * @Description:TODO
 * @Author:luyincheng
 * @CreateDate: 2023/10/819:15
 */
@RestController
@RequestMapping("api/reject")
@Api(tags = "退货管理接口")
public class RejectController {
    @Autowired
    private TokenService tokenService;
    @Autowired
    private RejectMapper rejectMapper;
    @Autowired
    private CommonUpdateService commonUpdateService;
    @Autowired
    private SnowFlakeService snowFlakeService;

    @PostMapping("/add")
    @ApiOperation(value = "新增退货")
    @ParamsValidate
    public BaseResponse<Integer> AddReject(@RequestBody @Validated(CreateEntityGroup.class) RejectAddVo rejectAddVo, BindingResult bindingResult){
        String orderid= snowFlakeService.getNextId(0,0)+"";
        Reject reject= RejectConvert.INSTANCE.convertAddVo(rejectAddVo);
        reject.setRejectOrder(orderid);
        reject.setStatus(TauxiliarymaterialStatusEnum.已申请.name());
        this.commonUpdateService.UpdateColumns(reject,true);
        return new BaseResponse<>(this.rejectMapper.insertSelective(reject));
    }
    @PostMapping("/query")
    @ApiOperation(value = "退货列表查询")
    public BaseResponse<List<Reject>> queryReject(Reject Reject){
        return new BaseResponse(this.rejectMapper.selectRejectList(Reject));
    }
    @GetMapping("/detail")
    @ApiOperation(value = "退货详情")
    public BaseResponse<Reject> getdetail(@RequestParam int id){
        return new BaseResponse(this.rejectMapper.selectByPrimaryKey(id));
    }
    @GetMapping("/delete")
    @ApiOperation(value = "删除退货信息")
    public BaseResponse<Integer> delete(@RequestParam int id){
        return new BaseResponse(this.rejectMapper.deleteByPrimaryKey(id));
    }
    @PostMapping("/update")
    @ApiOperation("更新退货单信息")
    @ParamsValidate
    public BaseResponse<Reject> update(@RequestBody @Validated(UpdateEntityGroup.class) RejectAddVo rejectAddVo, BindingResult bindingResult){
        var data=RejectConvert.INSTANCE.convertAddVo(rejectAddVo);
        this.commonUpdateService.UpdateColumns(data,false);
        return new BaseResponse(this.rejectMapper.updateByPrimaryKeySelective(data));
    }

    @PostMapping("/export")
    @ApiOperation(value = "导出退货单列表")
    public void export(HttpServletResponse response, @RequestBody Reject Reject) throws IOException {

//        ExcelUtil<WarehouseInfo> util = new ExcelUtil<WarehouseInfo>(WarehouseInfo.class);
//        util.exportExcel(response, list, "仓库信息管理数据");
        List<RejectExcelVo> data = RejectConvert.INSTANCE.convertExcelVoList(this
                .rejectMapper.selectRejectList(Reject));
        // 输出
        ExcelUtils.write(response, "退货管理列表.xls", "退货信息", RejectExcelVo.class, data);
    }
    @PostMapping("/check")
    @ApiOperation("审批退货单")
    @ParamsValidate
    public BaseResponse<Boolean> check(@RequestBody @Validated RejectCheckVo rejectCheckVo,BindingResult bindingResult){
        var status=TauxiliarymaterialStatusEnum.getEnumByName(0,rejectCheckVo.getStatus());
        if(status==null){
            throw new ServiceException(400,"status必须是枚举："+TauxiliarymaterialStatusEnum.getStatusTypeEnums(0).stream().map(e->e.name()).reduce("",(a,b)->a+","+b));
        }
        SessionInfo sessionInfo = tokenService.getUser();
        var data=RejectConvert.INSTANCE.convertCheckVo(rejectCheckVo);
        data.setCheckUser(sessionInfo.getUserId());
        data.setCheckUserName(sessionInfo.getUserName());
        data.setCheckTime(LocalDateTime.now());
        this.commonUpdateService.UpdateColumns(data,false);
        return new BaseResponse(this.rejectMapper.updateByPrimaryKeySelective(data));
    }


}
