package com.starcharge.wios.controller;

import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.base.util.Assert;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.bo.ReportOrderResponseBo;
import com.starcharge.wios.dao.RegionDao;
import com.starcharge.wios.dao.SupplierDao;
import com.starcharge.wios.dao.entity.Region;
import com.starcharge.wios.dao.entity.Supplier;
import com.starcharge.wios.dto.ReportOrderDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ihidea.component.api.v2.BaseResponse;
import com.starcharge.wios.bo.ReportOrderBo;
import com.starcharge.wios.service.ReportService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import java.text.ParseException;
import java.util.List;

/**
 * 数据报表的接口 API
 *
 * @author yongfu.liu@wanbangauto.com 2020-10-10 14:08:14
 */
@RestController
@RequestMapping("api")
@Api(tags = "数据报表的接口")
public class ReportController {

    @Autowired
    private ReportService reportService;

    @Autowired
    private RegionDao regionDao;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private SupplierDao supplierDao;

    /**
     * 安装进度等数据统计
     *
     * @return
     */
    @GetMapping(value = "report/installData")
    @ApiOperation(value = "安装进度等数据统计")
    @ApiImplicitParams({@ApiImplicitParam(name = "regionId", value = "区域id", dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "beginTime", value = "开始时间", dataType = "DateTime", paramType = "query", required = false),
            @ApiImplicitParam(name = "endTime", value = "截止时间", dataType = "DateTime", paramType = "query", required = false),
            @ApiImplicitParam(name = "supplierIds", value = "服务商ID,多个用逗号分割", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "vehicleModel", value = "车型", dataType = "String", paramType = "query", required = false),
    })
    public BaseResponse<Object> installData(ReportOrderBo reportOrderBo) {
        if (!StringUtils.isEmpty(reportOrderBo.getBeginTime())) {
            reportOrderBo.setBeginTime(reportOrderBo.getBeginTime() + " 00:00:00");
        }
        if (!StringUtils.isEmpty(reportOrderBo.getEndTime())) {
            reportOrderBo.setEndTime(reportOrderBo.getEndTime() + " 23:59:59");
        }
        return new BaseResponse<Object>(reportService.installData((reportOrderBo)));
    }

    /**
     * 电桩安装报表数据统计
     *
     * @return
     */
    @GetMapping(value = "report/installDataTable")
    @ApiOperation(value = "电桩安装报表数据统计")
    @ApiImplicitParams({@ApiImplicitParam(name = "regionId", value = "区域id", dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "beginTime", value = "开始时间", dataType = "DateTime", paramType = "query", required = false),
            @ApiImplicitParam(name = "endTime", value = "截止时间", dataType = "DateTime", paramType = "query", required = false),
            @ApiImplicitParam(name = "supplierIds", value = "服务商ID,多个用逗号分割", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "vehicleModel", value = "车型", dataType = "String", paramType = "query", required = false),
    })
    public BaseResponse<Object> installDataTable(ReportOrderBo reportOrderBo) {
        if (!StringUtils.isEmpty(reportOrderBo.getBeginTime())) {
            reportOrderBo.setBeginTime(reportOrderBo.getBeginTime() + " 00:00:00");
        }
        if (!StringUtils.isEmpty(reportOrderBo.getEndTime())) {
            reportOrderBo.setEndTime(reportOrderBo.getEndTime() + " 23:59:59");
        }
        String regionName = "";
        if (reportOrderBo.getRegionId()!=null) {
            Region region = regionDao.selectById(reportOrderBo.getRegionId().intValue());
            reportOrderBo.setRegionIdTree(region.getIdTree());
            regionName = region.getName();
        }
        List<ReportOrderDTO> reportOrderDTOS = reportService.installDataTable((reportOrderBo));
        if (reportOrderBo.getRegionId()!=null) {
            final String finalRegionName = regionName;
            reportOrderDTOS.stream().forEach(item->{
                item.setRegionId(reportOrderBo.getRegionId());
                item.setRegionName(finalRegionName);
            });
        }
        return new BaseResponse<Object>(reportOrderDTOS);
    }

    /**
     * 安装进度等数据统计
     *
     * @return
     */
    @GetMapping(value = "report/installEfficiency")
    @ApiOperation(value = "安装效率统计")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "beginTime", value = "开始时间", dataType = "DateTime", paramType = "query", required = false),
            @ApiImplicitParam(name = "endTime", value = "截止时间", dataType = "DateTime", paramType = "query", required = false),
            @ApiImplicitParam(name = "supplierIds", value = "服务商ID,多个用逗号分割", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "vehicleModel", value = "车型", dataType = "String", paramType = "query", required = false),
    })
    public BaseResponse<Object> installEfficiency(ReportOrderBo reportOrderBo) {
        if (!StringUtils.isEmpty(reportOrderBo.getBeginTime())) {
            reportOrderBo.setBeginTime(reportOrderBo.getBeginTime() + " 00:00:00");
        }
        if (!StringUtils.isEmpty(reportOrderBo.getEndTime())) {
            reportOrderBo.setEndTime(reportOrderBo.getEndTime() + " 23:59:59");
        }
        return new BaseResponse<Object>(reportService.installEfficiency((reportOrderBo)));
    }

    /**
     * @Description 接单时效统计(管理员后台)
     * @Param [orderResponseBo]
     * @author liwenxiang
     * @date 2021/8/26 9:44
     * @return com.ihidea.component.api.v2.BaseResponse<java.lang.Object>
     */
    @GetMapping(value = "report/orderResponseToAdmin")
    @ApiOperation(value = "接单时效统计(管理员后台)")
    @ApiImplicitParams({@ApiImplicitParam(name = "regionId", value = "区域id", dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "dispatchBeginTime", value = "开始时间(YYYY-MM-DD)", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "dispatchEndTime", value = "截止时间(YYYY-MM-DD)", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "supplierId", value = "服务商id（多个逗号分隔）", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "oemId", value = "主机厂id", dataType = "String", paramType = "query", required = false),
    })
    public BaseResponse<Object> orderResponseToAdmin(ReportOrderResponseBo reportOrderResponseBo) {
        if (!StringUtils.isEmpty(reportOrderResponseBo.getDispatchBeginTime())) {
            reportOrderResponseBo.setDispatchBeginTime(reportOrderResponseBo.getDispatchBeginTime() + " 00:00:00");
        }
        if (!StringUtils.isEmpty(reportOrderResponseBo.getDispatchEndTime())) {
            reportOrderResponseBo.setDispatchEndTime(reportOrderResponseBo.getDispatchEndTime() + " 23:59:59");
        }
        if (reportOrderResponseBo.getRegionId()!=null) {
            reportOrderResponseBo.setRegionIdTree(regionDao.selectById(reportOrderResponseBo.getRegionId()).getIdTree());
        }
        return new BaseResponse<Object>(reportService.orderResponseToAdmin((reportOrderResponseBo)));
    }

    /**
     * @Description 勘测时效统计(管理员后台)
     * @Param [orderResponseBo]
     * @author liwenxiang
     * @date 2021/8/26 9:44
     * @return com.ihidea.component.api.v2.BaseResponse<java.lang.Object>
     */
    @GetMapping(value = "report/surveyResponseToAdmin")
    @ApiOperation(value = "勘测时效统计(管理员后台)")
    @ApiImplicitParams({@ApiImplicitParam(name = "regionId", value = "区域id", dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "dispatchBeginTime", value = "开始时间(YYYY-MM-DD)", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "dispatchEndTime", value = "截止时间(YYYY-MM-DD)", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "supplierId", value = "服务商id（多个逗号分隔）", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "oemId", value = "主机厂id", dataType = "String", paramType = "query", required = false),
    })
    public BaseResponse<Object> surveyResponseToAdmin(ReportOrderResponseBo reportOrderResponseBo) {
        if (!StringUtils.isEmpty(reportOrderResponseBo.getDispatchBeginTime())) {
            reportOrderResponseBo.setDispatchBeginTime(reportOrderResponseBo.getDispatchBeginTime() + " 00:00:00");
        }else{
            reportOrderResponseBo.setDispatchBeginTime("2000-01-01 00:00:00");
        }
        if (!StringUtils.isEmpty(reportOrderResponseBo.getDispatchEndTime())) {
            reportOrderResponseBo.setDispatchEndTime(reportOrderResponseBo.getDispatchEndTime() + " 23:59:59");
        }
        if (reportOrderResponseBo.getRegionId()!=null) {
            reportOrderResponseBo.setRegionIdTree(regionDao.selectById(reportOrderResponseBo.getRegionId()).getIdTree());
        }
        return new BaseResponse<Object>(reportService.surveyResponseToAdmin((reportOrderResponseBo)));
    }

    /**
     * @Description 安装时效统计(管理员后台)
     * @Param [orderResponseBo]
     * @author liwenxiang
     * @date 2021/8/26 9:44
     * @return com.ihidea.component.api.v2.BaseResponse<java.lang.Object>
     */
    @GetMapping(value = "report/installResponseToAdmin")
    @ApiOperation(value = "安装时效统计(管理员后台)")
    @ApiImplicitParams({@ApiImplicitParam(name = "regionId", value = "区域id", dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "dispatchBeginTime", value = "开始时间(YYYY-MM-DD)", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "dispatchEndTime", value = "截止时间(YYYY-MM-DD)", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "supplierId", value = "服务商id（多个逗号分隔）", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "oemId", value = "主机厂id", dataType = "String", paramType = "query", required = false),
    })
    public BaseResponse<Object> installResponseToAdmin(ReportOrderResponseBo reportOrderResponseBo) {
        if (!StringUtils.isEmpty(reportOrderResponseBo.getDispatchBeginTime())) {
            reportOrderResponseBo.setDispatchBeginTime(reportOrderResponseBo.getDispatchBeginTime() + " 00:00:00");
        }else{
            reportOrderResponseBo.setDispatchBeginTime("2000-01-01 00:00:00");
        }
        if (!StringUtils.isEmpty(reportOrderResponseBo.getDispatchEndTime())) {
            reportOrderResponseBo.setDispatchEndTime(reportOrderResponseBo.getDispatchEndTime() + " 23:59:59");
        }
        if (reportOrderResponseBo.getRegionId()!=null) {
            reportOrderResponseBo.setRegionIdTree(regionDao.selectById(reportOrderResponseBo.getRegionId()).getIdTree());
        }
        return new BaseResponse<Object>(reportService.installResponseToAdmin((reportOrderResponseBo)));
    }

    /**
     * @Description 安装完成率统计(管理员后台)
     * @Param [orderResponseBo]
     * @author liwenxiang
     * @date 2021/8/26 9:44
     * @return com.ihidea.component.api.v2.BaseResponse<java.lang.Object>
     */
    @GetMapping(value = "report/installEfficientToAdmin")
    @ApiOperation(value = "安装完成率统计(管理员后台)")
    @ApiImplicitParams({@ApiImplicitParam(name = "regionId", value = "区域id", dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "dispatchBeginTime", value = "开始时间(YYYY-MM-DD)", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "dispatchEndTime", value = "截止时间(YYYY-MM-DD)", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "supplierId", value = "服务商id（多个逗号分隔）", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "oemId", value = "主机厂id", dataType = "String", paramType = "query", required = false),
    })
    public BaseResponse<Object> installEfficientToAdmin(ReportOrderResponseBo reportOrderResponseBo) {
        if (!StringUtils.isEmpty(reportOrderResponseBo.getDispatchBeginTime())) {
            reportOrderResponseBo.setDispatchBeginTime(reportOrderResponseBo.getDispatchBeginTime() + " 00:00:00");
        }
        if (!StringUtils.isEmpty(reportOrderResponseBo.getDispatchEndTime())) {
            reportOrderResponseBo.setDispatchEndTime(reportOrderResponseBo.getDispatchEndTime() + " 23:59:59");
        }
        if (reportOrderResponseBo.getRegionId()!=null) {
            reportOrderResponseBo.setRegionIdTree(regionDao.selectById(reportOrderResponseBo.getRegionId()).getIdTree());
        }
        return new BaseResponse<Object>(reportService.installEfficientToAdmin((reportOrderResponseBo)));
    }

    /**
     * @Description 报告通过率统计(管理员后台)
     */
    @GetMapping(value = "report/reportPassToAdmin")
    @ApiOperation(value = "报告通过率统计(管理员后台)")
    @ApiImplicitParams({@ApiImplicitParam(name = "regionId", value = "区域id", dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "dispatchBeginTime", value = "开始时间(YYYY-MM-DD)", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "dispatchEndTime", value = "截止时间(YYYY-MM-DD)", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "supplierId", value = "服务商id（多个逗号分隔）", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "oemId", value = "主机厂id", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "type", value = "类型 1:勘测 2:安装", dataType = "int", paramType = "query", required = true),
    })
    public BaseResponse<Object> reportPassToAdmin(ReportOrderResponseBo reportOrderResponseBo) {
        if (!StringUtils.isEmpty(reportOrderResponseBo.getDispatchBeginTime())) {
            reportOrderResponseBo.setDispatchBeginTime(reportOrderResponseBo.getDispatchBeginTime() + " 00:00:00");
        }
        if (!StringUtils.isEmpty(reportOrderResponseBo.getDispatchEndTime())) {
            reportOrderResponseBo.setDispatchEndTime(reportOrderResponseBo.getDispatchEndTime() + " 23:59:59");
        }
        if (reportOrderResponseBo.getRegionId()!=null) {
            reportOrderResponseBo.setRegionIdTree(regionDao.selectById(reportOrderResponseBo.getRegionId()).getIdTree());
        }
        if(reportOrderResponseBo.getType() == null){
            reportOrderResponseBo.setType(1);
        }
        return new BaseResponse<Object>(reportService.reportPassToAdmin((reportOrderResponseBo)));
    }



    /**
     * @Description 接单时效统计(服务商后台)
     * @Param [orderResponseBo]
     * @author liwenxiang
     * @date 2021/8/26 9:44
     * @return com.ihidea.component.api.v2.BaseResponse<java.lang.Object>
     */
    @GetMapping(value = "report/orderResponseToSupplier")
    @ApiOperation(value = "接单时效统计(服务商后台)")
    @ApiImplicitParams({@ApiImplicitParam(name = "regionId", value = "区域id", dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "dispatchBeginTime", value = "开始月份(YYYY-MM)", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "dispatchEndTime", value = "截止月份(YYYY-MM)", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "oemId", value = "主机厂id", dataType = "String", paramType = "query", required = false),
    })
    public BaseResponse<Object> orderResponseToSupplier(ReportOrderResponseBo reportOrderResponseBo) throws ParseException {
        Assert.notEmpty(reportOrderResponseBo.getDispatchBeginTime(),"派单月份不能为空");
        Assert.notEmpty(reportOrderResponseBo.getDispatchEndTime(),"派单月份不能为空");
        if (reportOrderResponseBo.getRegionId()!=null) {
            reportOrderResponseBo.setRegionIdTree(regionDao.selectById(reportOrderResponseBo.getRegionId()).getIdTree());
        }
        SessionInfo user = tokenService.getUser();
        Supplier supplier = supplierDao.selectByAccountId(Integer.parseInt(user.getUserId()));
        if (ObjectUtils.isEmpty(supplier)) {
            throw new ServiceException("无权限查看");
        }
        reportOrderResponseBo.setSupplierId(supplier.getId());

        return new BaseResponse<Object>(reportService.orderResponseToSupplier((reportOrderResponseBo)));
    }

    /**
     * @Description 勘测时效统计(服务商后台)
     * @Param [orderResponseBo]
     * @author liwenxiang
     * @date 2021/8/26 9:44
     * @return com.ihidea.component.api.v2.BaseResponse<java.lang.Object>
     */
    @GetMapping(value = "report/surveyResponseToSupplier")
    @ApiOperation(value = "勘测时效统计(服务商后台)")
    @ApiImplicitParams({@ApiImplicitParam(name = "regionId", value = "区域id", dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "dispatchBeginTime", value = "开始月份(YYYY-MM)", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "dispatchEndTime", value = "截止月份(YYYY-MM)", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "oemId", value = "主机厂id", dataType = "String", paramType = "query", required = false),
    })
    public BaseResponse<Object> surveyResponseToSupplier(ReportOrderResponseBo reportOrderResponseBo) throws ParseException {
        Assert.notEmpty(reportOrderResponseBo.getDispatchBeginTime(),"派单月份不能为空");
        Assert.notEmpty(reportOrderResponseBo.getDispatchEndTime(),"派单月份不能为空");

        if (reportOrderResponseBo.getRegionId()!=null) {
            reportOrderResponseBo.setRegionIdTree(regionDao.selectById(reportOrderResponseBo.getRegionId()).getIdTree());
        }
        SessionInfo user = tokenService.getUser();
        Supplier supplier = supplierDao.selectByAccountId(Integer.parseInt(user.getUserId()));
        if (ObjectUtils.isEmpty(supplier)) {
            throw new ServiceException("无权限查看");
        }
        reportOrderResponseBo.setSupplierId(supplier.getId());
        return new BaseResponse<Object>(reportService.surveyResponseToSupplier((reportOrderResponseBo)));
    }

    /**
     * @Description 安装时效统计(服务商后台)
     * @Param [orderResponseBo]
     * @author liwenxiang
     * @date 2021/8/26 9:44
     * @return com.ihidea.component.api.v2.BaseResponse<java.lang.Object>
     */
    @GetMapping(value = "report/installResponseToSupplier")
    @ApiOperation(value = "安装时效统计(服务商后台)")
    @ApiImplicitParams({@ApiImplicitParam(name = "regionId", value = "区域id", dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "dispatchBeginTime", value = "开始月份(YYYY-MM)", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "dispatchEndTime", value = "截止月份(YYYY-MM)", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "oemId", value = "主机厂id", dataType = "String", paramType = "query", required = false),
    })
    public BaseResponse<Object> installResponseToSupplier(ReportOrderResponseBo reportOrderResponseBo) throws ParseException {
        Assert.notEmpty(reportOrderResponseBo.getDispatchBeginTime(),"派单月份不能为空");
        Assert.notEmpty(reportOrderResponseBo.getDispatchEndTime(),"派单月份不能为空");
        if (reportOrderResponseBo.getRegionId()!=null) {
            reportOrderResponseBo.setRegionIdTree(regionDao.selectById(reportOrderResponseBo.getRegionId()).getIdTree());
        }
        SessionInfo user = tokenService.getUser();
        Supplier supplier = supplierDao.selectByAccountId(Integer.parseInt(user.getUserId()));
        if (ObjectUtils.isEmpty(supplier)) {
            throw new ServiceException("无权限查看");
        }
        reportOrderResponseBo.setSupplierId(supplier.getId());
        return new BaseResponse<Object>(reportService.installResponseToSupplier((reportOrderResponseBo)));
    }

    /**
     * @Description 安装完成率统计(服务商后台)
     * @Param [orderResponseBo]
     * @author liwenxiang
     * @date 2021/8/26 9:44
     * @return com.ihidea.component.api.v2.BaseResponse<java.lang.Object>
     */
    @GetMapping(value = "report/installEfficientToSupplier")
    @ApiOperation(value = "安装完成率统计(服务商后台)")
    @ApiImplicitParams({@ApiImplicitParam(name = "regionId", value = "区域id", dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "dispatchBeginTime", value = "开始月份(YYYY-MM)", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "dispatchEndTime", value = "截止月份(YYYY-MM)", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "oemId", value = "主机厂id", dataType = "String", paramType = "query", required = false),
    })
    public BaseResponse<Object> installEfficientToSupplier(ReportOrderResponseBo reportOrderResponseBo) throws ParseException {
        Assert.notEmpty(reportOrderResponseBo.getDispatchBeginTime(),"派单月份不能为空");
        Assert.notEmpty(reportOrderResponseBo.getDispatchEndTime(),"派单月份不能为空");
        if (reportOrderResponseBo.getRegionId()!=null) {
            reportOrderResponseBo.setRegionIdTree(regionDao.selectById(reportOrderResponseBo.getRegionId()).getIdTree());
        }
        SessionInfo user = tokenService.getUser();
        Supplier supplier = supplierDao.selectByAccountId(Integer.parseInt(user.getUserId()));
        if (ObjectUtils.isEmpty(supplier)) {
            throw new ServiceException("无权限查看");
        }
        reportOrderResponseBo.setSupplierId(supplier.getId());
        return new BaseResponse<Object>(reportService.installEfficientToSupplier((reportOrderResponseBo)));
    }

    /**
     * @Description 报告通过率统计(服务商后台)
     */
    @GetMapping(value = "report/reportPassToSupplier")
    @ApiOperation(value = "安装完成率统计(服务商后台)")
    @ApiImplicitParams({@ApiImplicitParam(name = "regionId", value = "区域id", dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "dispatchBeginTime", value = "开始月份(YYYY-MM)", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "dispatchEndTime", value = "截止月份(YYYY-MM)", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "oemId", value = "主机厂id", dataType = "String", paramType = "query", required = false),
            @ApiImplicitParam(name = "type", value = "类型 1:勘测 2:安装", dataType = "int", paramType = "query", required = true),
    })
    public BaseResponse<Object> reportPassToSupplier(ReportOrderResponseBo reportOrderResponseBo) throws ParseException {
        Assert.notEmpty(reportOrderResponseBo.getDispatchBeginTime(),"审核月份不能为空");
        Assert.notEmpty(reportOrderResponseBo.getDispatchEndTime(),"审核月份不能为空");
        if (reportOrderResponseBo.getRegionId()!=null) {
            reportOrderResponseBo.setRegionIdTree(regionDao.selectById(reportOrderResponseBo.getRegionId()).getIdTree());
        }
        if(reportOrderResponseBo.getType() == null){
            reportOrderResponseBo.setType(1);
        }
        SessionInfo user = tokenService.getUser();
        Supplier supplier = supplierDao.selectByAccountId(Integer.parseInt(user.getUserId()));
        if (ObjectUtils.isEmpty(supplier)) {
            throw new ServiceException("无权限查看");
        }
        reportOrderResponseBo.setSupplierId(supplier.getId());
        return new BaseResponse<Object>(reportService.reportPassToSupplier((reportOrderResponseBo)));
    }
}
