package com.starcharge.wios.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ihidea.component.api.v2.BaseResponse;
import com.starcharge.wios.contants.ScoreConstant;
import com.starcharge.wios.dao.entity.ScoreConfig;
import com.starcharge.wios.service.ScoreConfigService;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 评价配置表 管理 API
 *
 * @author yongfu.liu@wanbangauto.com 2020-10-10 14:08:14
 */
@RestController
@RequestMapping("api")
@Api(tags = "评分配置接口")
public class ScoreConfigController {

    @Autowired
    private ScoreConfigService scoreConfigService;

    /**
     * 查询列表
     *
     * @return
     */
    @GetMapping(value = "scoreConfig/list")
    @ApiOperation(value = "查询列表")
    @ApiImplicitParams({})
    public BaseResponse<Object> list() {
        return new BaseResponse<Object>(scoreConfigService.getListByType("Supplier"));
    }

    /**
     * 批量更新或新增配置项
     *
     * @param list
     */
    @PostMapping(value = "scoreConfig/updateBatch")
    @ApiOperation(value = "批量更新或新增配置项")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "主键id", dataType = "int", paramType = "query", required = true),
            @ApiImplicitParam(name = "code", value = "编码", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "name", value = "名称", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "weight", value = "权重", dataType = "int", paramType = "query", required = true),
            @ApiImplicitParam(name = "parentCode", value = "父配置项编码", dataType = "String", paramType = "query", required = true),
    })
    public BaseResponse<Object> updateBatch( String list) {
        JSONArray jsonArray = JSONUtil.parseArray(list);
        if(jsonArray.size()>0){
            for(int i = 0 ;i<jsonArray.size();i++){
                JSONObject jsonObject =jsonArray.getJSONObject(i);
                ScoreConfig scoreConfig =new ScoreConfig();
                if(jsonObject.get("code") != null){
                    scoreConfig.setCode(String.valueOf(jsonObject.get("code")));
                }
                if(jsonObject.get("name") != null){
                    scoreConfig.setName(String.valueOf(jsonObject.get("name")));
                }
                if(jsonObject.get("parentCode") != null){
                    scoreConfig.setParentCode(String.valueOf(jsonObject.get("parentCode")));
                }
                if(jsonObject.get("weight") != null){
                    scoreConfig.setWeight(Double.valueOf(String.valueOf(jsonObject.get("weight"))));
                }
                if(String.valueOf(jsonObject.get("id")).indexOf("new") == -1){
                    scoreConfig.setId(Long.parseLong(String.valueOf(jsonObject.get("id"))));
                    scoreConfigService.update(scoreConfig);
                }else{
                    scoreConfig.setType("Supplier");
                    if (jsonObject.get("parentCode") != null && jsonObject.get("parentCode").toString().equals(ScoreConstant.CUSTOMER)) {
                        scoreConfig.setSource("五星评分");
                    } else {
                        scoreConfig.setSource("手动评分");
                    }
                    scoreConfigService.insert(scoreConfig);
                }
            }
        }

        return new BaseResponse<>();
    }

    /**
     * 批量删除
     *
     * @param ids  id, 多个时用逗号分割
     * @return
     */
    @PostMapping(value = "scoreConfig/deleteBatch")
    @ApiOperation(value = "批量删除")
    @ApiImplicitParam(paramType = "query", name = "ids", value = "配置ID,多个用逗号分割", required = true, dataType = "String")
    public BaseResponse<Object> deleteBatch( String ids) {
        String[] idList = ids.split(",");
        for(String id : idList){
            ScoreConfig scoreConfig = new ScoreConfig();
            scoreConfig.setId(Long.parseLong(id));
            scoreConfig.setStatus(0L);
            scoreConfigService.update(scoreConfig);
        }

        return new BaseResponse<>();
    }

}
