package com.starcharge.wios.controller;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.base.util.Assert;
import com.starcharge.wios.bo.OemBo;
import com.starcharge.wios.contants.ScoreConstant;
import com.starcharge.wios.dao.entity.*;
import com.starcharge.wios.dto.ScoreSupplierDTO;
import com.starcharge.wios.service.ScoreSupplierDetailService;
import com.starcharge.wios.service.ScoreSupplierService;
import com.starcharge.wios.vo.OrderVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * 供应商评分相关api
 * 
 * @author lyf
 * @version [版本号, 2020年10月8日]
 */
@RestController
@RequestMapping("api")
@Api(tags = "服务商评分接口")
public class ScoreSupplierController {
    @Autowired
    private ScoreSupplierService scoreSupplierService;

    @Autowired
    private ScoreSupplierDetailService scoreSupplierDetailService;
    
    /**
     * 查询列表
     *
     * @param scoreSupplierDTO
     * @return
     */
    @GetMapping(value = "scoreSupplier/list")
    @ApiOperation(value = "查询列表")
    @ApiImplicitParams({@ApiImplicitParam(name = "page", value = "一页显示数量", dataType = "int", paramType = "query", required = false),
        @ApiImplicitParam(name = "pagecount", value = "页码", dataType = "int", paramType = "query", required = false),
        @ApiImplicitParam(name = "regionId", value = "区域ID", dataType = "int", paramType = "query", required = false),
        @ApiImplicitParam(name = "supplierName", value = "服务商名称", dataType = "String", paramType = "query", required = false),
        @ApiImplicitParam(name = "month", value = "月份", dataType = "String", paramType = "query", required = false),})
    public BaseResponse<Object> list(ScoreSupplierDTO scoreSupplierDTO) {
        //scoreSupplierDTO.setStatus();
        return new BaseResponse<Object>(scoreSupplierService.getList(scoreSupplierDTO));
    }

//    /**
//     * 更新投诉次数
//     *
//     * @param
//     */
//    @PostMapping(value = "scoreSupplier/complaintCount")
//    @ApiOperation(value = "更新投诉次数")
//    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "主键id", dataType = "int", paramType = "query", required = true),
//        @ApiImplicitParam(name = "complaintCount", value = "投诉次数", dataType = "int", paramType = "query", required = false),
//    })
//    public BaseResponse<Object> complaintCount(Long id, Long complaintCount) {
//        //保存到明细表的相关项
//        ScoreSupplierDetail scoreSupplierDetail = new ScoreSupplierDetail();
//        scoreSupplierDetail.setCode(ScoreConstant.DEDUCT_ITEM2);
//        scoreSupplierDetail.setScoreSupplierId(id);
//        scoreSupplierDetail.setResultCalc(Double.valueOf(complaintCount));
//        scoreSupplierDetailService.updateByCode(scoreSupplierDetail);
//        return new BaseResponse<>();
//    }

    /**
     * 取服务商评价详情信息
     *
     * @param id
     * @return
     */
    @GetMapping(value = "scoreSupplier/detail")
    @ApiOperation(value = "取服务商评价详情信息")
    @ApiImplicitParam(paramType = "query", name = "id", value = "评分主表ID", required = true, dataType = "Long")
    public BaseResponse<Object> detail( Long id) {
        return new BaseResponse<Object>(scoreSupplierService.detail(id));
    }

    /**
     * 删除评分表
     *
     * @param id
     * @return
     */
    @PostMapping(value = "scoreSupplier/delete")
    @ApiOperation(value = "删除")
    @ApiImplicitParam(paramType = "query", name = "id", value = "评分主表ID", required = true, dataType = "Long")
    public BaseResponse<Object> deleteBatch( Long id) {
        scoreSupplierService.delete(id);
        return new BaseResponse<>();
    }

    /**
     * 根据服务商id取最新的采购评分
     *
     * @param supplierId
     * @return
     */
    @GetMapping(value = "scoreSupplier/getPurchasingScoreList")
    @ApiOperation(value = "根据服务商id取最新的采购评分")
    @ApiImplicitParam(paramType = "query", name = "supplierId", value = "评分主表ID", required = true, dataType = "int")
    public BaseResponse<Object> getPurchasingScoreList( Long supplierId) {
        return new BaseResponse<Object>(scoreSupplierService.getPurchasingScoreList(supplierId));
    }

    /**
     * 更新采购评分
     *
     * @param list
     * @return
     */
    @PostMapping(value = "scoreSupplier/updatePurchasingScore")
    @ApiOperation(value = "更新采购评分")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "主键id", dataType = "int", paramType = "query", required = true),
            @ApiImplicitParam(name = "score", value = "评分", dataType = "Double", paramType = "query", required = true),
    })
    public BaseResponse<Object> updatePurchasingScore(String list) {
        List<ScoreSupplierDetail> scoreSupplierDetailList = new ArrayList<>();
        JSONArray jsonArray = JSONUtil.parseArray(list);
        if(jsonArray.size()>0){
            for(int i = 0 ;i<jsonArray.size();i++){
                JSONObject jsonObject =jsonArray.getJSONObject(i);
                ScoreSupplierDetail scoreSupplierDetail = new ScoreSupplierDetail();
                scoreSupplierDetail.setId(Long.parseLong(String.valueOf(jsonObject.get("id"))));
                scoreSupplierDetail.setScore(Double.valueOf(String.valueOf(jsonObject.get("score"))));
                scoreSupplierDetailList.add(scoreSupplierDetail);
            }
        }
        scoreSupplierService.updatePurchasingScore(scoreSupplierDetailList);
        return new BaseResponse<>();
    }

}
