package com.starcharge.wios.controller;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.ihidea.component.api.v2.BaseResponse;
import com.starcharge.wios.contants.ScoreConstant;
import com.starcharge.wios.dao.entity.OrderScore;
import com.starcharge.wios.dao.entity.ScoreSupplierDetail;
import com.starcharge.wios.service.ScoreSupplierDetailService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 评价配置表 管理 API
 *
 * @author yongfu.liu@wanbangauto.com 2020-10-10 14:08:14
 */
@RestController
@RequestMapping("api")
@Api(tags = "评分表明细项的接口")
public class ScoreSupplierDetailController {

    @Autowired
    private ScoreSupplierDetailService scoreSupplierDetailService;

    /**
     * 查询评分表的明细项
     *
     * @return
     */
    @GetMapping(value = "scoreSupplierDetail/list")
    @ApiOperation(value = "查询列表")
    @ApiImplicitParam(paramType = "query", name = "scoreSupplierId", value = "评分表ID", required = true, dataType = "int")
    public BaseResponse<Object> list( Long scoreSupplierId) {
        return new BaseResponse<Object>(scoreSupplierDetailService.getListByScoreSupplierId(scoreSupplierId));
    }

    /**
     * 批量更新明细项
     *
     * @param list
     */
    @PostMapping(value = "scoreSupplierDetail/updateBatch")
    @ApiOperation(value = "批量更新")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "主键id", dataType = "int", paramType = "query", required = true),
            @ApiImplicitParam(name = "code", value = "编码", dataType = "string", paramType = "query", required = true),
            @ApiImplicitParam(name = "score", value = "实际评分", dataType = "double", paramType = "query", required = true),
            @ApiImplicitParam(name = "resultCalc", value = "统计数据，次数", dataType = "double", paramType = "query", required = true),
    })
    public BaseResponse<Object> updateBatch(String list) {
        JSONArray jsonArray = JSONUtil.parseArray(list);
        if(jsonArray.size()>0){
            for(int i = 0 ;i<jsonArray.size();i++){
                JSONObject jsonObject =jsonArray.getJSONObject(i);
                ScoreSupplierDetail scoreSupplierDetail = new ScoreSupplierDetail();
                scoreSupplierDetail.setId(Long.parseLong(String.valueOf(jsonObject.get("id"))));
                if(ScoreConstant.DEDUCT_ITEM1.equals(String.valueOf(jsonObject.get("code")))
                    || ScoreConstant.DEDUCT_ITEM2.equals(String.valueOf(jsonObject.get("code")))){
                    Double resultCalc = Double.valueOf(String.valueOf(jsonObject.get("resultCalc")));
                    scoreSupplierDetail.setResultCalc(resultCalc);
                    scoreSupplierDetail.setScore(resultCalc * 5);
                }else{
                    scoreSupplierDetail.setScore(Double.valueOf(String.valueOf(jsonObject.get("score"))));
                }
                scoreSupplierDetailService.update(scoreSupplierDetail);
            }
        }
        return new BaseResponse<>();
    }


}
