package com.starcharge.wios.controller;

import java.math.BigDecimal;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.base.util.Assert;
import com.starcharge.wios.bo.SupplierContractBo;
import com.starcharge.wios.service.SupplierContractService;
import com.starcharge.wios.vo.SupplierContractVo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 服务商合约接口
 * 
 * @author xgl
 * @version [版本号, 2020年9月24日]
 */
@RestController
@RequestMapping("api")
@Api(tags = "服务商合同管理接口")
public class SupplierContractController {
    @Autowired
    private SupplierContractService supplierContractService;
    
    /**
     * 添加合同数据
     *
     * @param contractBo
     */
    @PostMapping(value = "supplierContract/add")
    @ApiOperation(value = "添加合同数据")
    @ApiImplicitParams({@ApiImplicitParam(name = "code", value = "合同编号", dataType = "String", paramType = "query", required = true),
        @ApiImplicitParam(name = "supplierId", value = "服务商id", dataType = "int", paramType = "query", required = true),
        @ApiImplicitParam(name = "taxRate", value = "税率", dataType = "double", paramType = "query", required = true),
    
    })
    public BaseResponse<Object> add(SupplierContractBo contractBo) {
        if (contractBo.getStartTime() == null || contractBo.getEndTime() == null) {
            throw new ServiceException("缺少日期");
        }
        supplierContractService.add(contractBo);
        return new BaseResponse<>();
    }
    
    /**
     * 更新合同数据
     *
     * @param contractBo
     */
    @PostMapping(value = "supplierContract/update")
    @ApiOperation(value = "更新合同数据")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "合同id", dataType = "int", paramType = "query", required = true),
    
    })
    public BaseResponse<Object> update(SupplierContractBo contractBo) {
        Assert.notNull(contractBo.getId(), "缺少id");
        supplierContractService.update(contractBo);
        return new BaseResponse<>();
    }
    
    /**
     * 查询列表
     *
     * @param supplierBo
     * @return
     */
    @GetMapping(value = "supplierContract/list")
    @ApiOperation(value = "查询合同列表")
    @ApiImplicitParams({@ApiImplicitParam(name = "page", value = "一页显示数量", dataType = "int", paramType = "query", required = false),
        @ApiImplicitParam(name = "pagecount", value = "页码", dataType = "int", paramType = "query", required = false),
        @ApiImplicitParam(name = "code", value = "合同编号", dataType = "int", paramType = "query", required = false),
        @ApiImplicitParam(name = "supplierName", value = "服务商名称", dataType = "String", paramType = "query", required = false),
        @ApiImplicitParam(name = "taxRate", value = "税率", dataType = "double", paramType = "query", required = false),
        @ApiImplicitParam(name = "status", value = "状态", dataType = "int", paramType = "query", required = false),})
    public BaseResponse<Object> list(String code, String supplierName, BigDecimal taxRate, Integer status) {
        SupplierContractBo supplierContract = new SupplierContractBo();
        supplierContract.setCode(code);
        supplierContract.setSupplierName(supplierName);
        supplierContract.setTaxRate(taxRate);
        supplierContract.setStatus(status);
        return new BaseResponse<Object>(supplierContractService.getList(supplierContract));
    }
    
    /**
     * 查询合同详情
     *
     * @param id
     * @return
     */
    @GetMapping(value = "supplierContract/detail")
    @ApiOperation(value = "查询合同详情")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "id", dataType = "int", paramType = "query", required = true)})
    public BaseResponse<SupplierContractVo> detail(Integer id) {
        Assert.notNull(id, "id不能为空");
        return new BaseResponse<SupplierContractVo>(supplierContractService.getDetail(id));
    }
}
