package com.starcharge.wios.dao;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.bo.OemContractBo2;
import com.starcharge.wios.dao.entity.OemContract;
import com.starcharge.wios.dao.mappers.OemContractMapper;
import com.starcharge.wios.enums.ContractStatus;
import com.starcharge.wios.vo.OemContractVo;

/**
 * 主机厂合约dao
 * 
 * @author xgl
 * @version [版本号, 2020年9月16日]
 */
@Repository
public class OemContractDao {
    
    @Autowired
    private OemContractMapper oemContractMapper;
    
    @Autowired
    private TokenService tokenService;
    
    public OemContract selectById(int id) {
        return oemContractMapper.selectById(id);
    }
    
    public List<OemContract> selectByCode(String code) {
        return oemContractMapper.selectByCode(code);
    }
    
    public int insert(OemContract contract) {
        contract.setCreateTime(new Date());
        if (tokenService.getUser() != null)
            contract.setCreateAccount(tokenService.getUser().getUserName());
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date date1 = null;
        try {
            date1 = sdf.parse(sdf.format(new Date()));
        } catch (ParseException e) {
            throw new ServiceException("数据转换失败");
        }
        if (contract.getStartTime().compareTo(date1) > 0) {
            contract.setStatus(ContractStatus.NOT_START.getType());
        } else {
            if (contract.getEndTime().compareTo(date1) >= 0) {
                contract.setStatus(ContractStatus.EFFECTIVE.getType());
            } else {
                throw new ServiceException("结束日期必须大于今天");
            }
        }
        return oemContractMapper.insert(contract);
    }
    
    public List<OemContractVo> selectByStatusAndOemId(int oemId, int status) {
        OemContractBo2 condition = new OemContractBo2();
        condition.setOemId(oemId + "");
        condition.setStatus(status);
        List<OemContractVo> list = oemContractMapper.selectByExample(condition);
        return list;
    }
    
    public List<OemContract> selectSimpleByStatusAndOemId(int oemId, int status) {
        return oemContractMapper.selectByStatusAndOemId(status, oemId);
    }
    
    public List<OemContractVo> selectValid(Integer oemId) {
        OemContractBo2 condition = new OemContractBo2();
        if (oemId != null)
            condition.setOemId(oemId.intValue() + "");
        List<Integer> statusInList = new ArrayList<Integer>();
        statusInList.add(ContractStatus.NOT_START.getType());
        statusInList.add(ContractStatus.EFFECTIVE.getType());
        condition.setStatusInList(statusInList);
        condition.setOrderBy(2);
        List<OemContractVo> list = oemContractMapper.selectByExample(condition);
        return list;
    }
    
    public List<OemContractVo> getList(OemContractBo2 condition) {
        return oemContractMapper.selectByExample(condition);
    }
    
    public int update(int id, OemContract contract) {
        contract.setId(id);
        contract.setModifyTime(new Date());
        if (tokenService.getUser() != null)
            contract.setModifyAccount(tokenService.getUser().getUserName());

        if (contract.getStartTime() != null && contract.getEndTime() != null) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            Date date1 = null;
            try {
                date1 = sdf.parse(sdf.format(new Date()));
            } catch (ParseException e) {
                throw new ServiceException("数据转换失败");
            }
            if (contract.getStartTime().compareTo(date1) > 0) {
                contract.setStatus(ContractStatus.NOT_START.getType());
            } else {
                if (contract.getEndTime().compareTo(date1) >= 0) {
                    contract.setStatus(ContractStatus.EFFECTIVE.getType());
                } else {
                    throw new ServiceException("结束日期必须大于今天");
                }
            }
        }
        return oemContractMapper.updateDynamic(contract);
    }
    
    public OemContractVo selectDetailById(int contractId) {
        return oemContractMapper.selectDetailById(contractId);
    }
    
    public static void main(String[] args) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date date1 = null;
        try {
            date1 = sdf.parse(sdf.format(new Date()));
        } catch (ParseException e) {
            throw new ServiceException("数据转换失败");
        }
        System.out.println(date1);
    }
}
