package com.starcharge.wios.dao;

import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.bo.OemBo;
import com.starcharge.wios.dao.entity.Oem;
import com.starcharge.wios.dao.mappers.OemMapper;
import com.starcharge.wios.vo.OemVo;

/**
 * 主机厂/4s店dao层
 * 
 * @author xgl
 * @version [版本号, 2020年9月8日]
 */
@Repository
public class OemDao {
    
    @Autowired
    private OemMapper oemMapper;
    
    @Autowired
    private TokenService tokenService;
    
    /**
     * 查询列表
     *
     * @param condition
     * @return
     */
    public List<OemVo> getList(OemBo condition) {
        return oemMapper.selectByExample(condition);
    }
    
    public List<OemVo> selectByType(int type) {
        OemBo condition = new OemBo();
        condition.setType(type);
        // condition.setStatus(1);
        return oemMapper.selectByExample(condition);
    }
    
    public List<Oem> selectSelfAndChildren(Integer id) {
        return oemMapper.selectSelfAndChildren(id);
    }
    
    public Oem selectById(Integer id) {
        return oemMapper.selectById(id);
    }
    
    /**
     * 单个新增
     *
     * @param oem
     * @return
     */
    public int insert(Oem oem) {
        oem.setCreateTime(new Date());
        oem.setStatus(1);
        if (tokenService.getUser() != null)
            oem.setCreateAccount(tokenService.getUser().getUserId());
        return oemMapper.insert(oem);
    }
    
    /**
     * 单个更新
     *
     * @param oem
     * @return
     */
    public int update(Oem oem) {
        oem.setModifyTime(new Date());
        if (tokenService.getUser() != null)
            oem.setModifyAccount(tokenService.getUser().getUserId());
        return oemMapper.updateDynamic(oem);
    }
    
    public List<Oem> selectOnly(String code, String name) {
        return oemMapper.selectOnly(code, name);
    }
    
    public List<Oem> selectByNameAndCode(String name, String code) {
        return oemMapper.selectByNameAndCode(name, code);
    }
    
    public List<Oem> selectByName(String name) {
        return oemMapper.selectByName(name);
    }
    
    public Oem selectSingleByName(String name) {
        List<Oem> list = selectByName(name);
        return list.isEmpty() ? null : list.get(0);
    }
    
    public Oem selectByAccountId(Integer accountId) {
        List<Oem> list = oemMapper.selectByAccountId(accountId);
        return list.isEmpty() ? null : list.get(0);
    }
}
