package com.starcharge.wios.dao;

import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.bo.OperateLogBo;
import com.starcharge.wios.dao.entity.OperateLog;
import com.starcharge.wios.dao.mappers.OperateLogMapper;
import com.starcharge.wios.enums.OperateLogType;
import com.starcharge.wios.enums.OrderStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 操作日志dao
 * 
 * @author xgl
 * @version [版本号, 2020年9月27日]
 */
@Repository
public class OperateLogDao {
    private static final Logger logger= LoggerFactory.getLogger(OperateLogDao.class);

    @Autowired
    private OperateLogMapper operateLogMapper;
    
    @Autowired
    private TokenService tokenService;
    
    public int insert(OperateLog operateLog) {
        operateLog.setCreateTime(new Date());
        if (tokenService.getUser() != null) {
            try{
                operateLog.setCreateAccount(Integer.valueOf(tokenService.getUser().getUserId()));
            }catch (Exception e){
                logger.error("获取当前用户编号出现异常:"+e.getMessage());
            }
            operateLog.setCreateAccountName(tokenService.getUser().getUserName());
        }
        return operateLogMapper.insert(operateLog);
    }
    
    public int insertOrderLog(String orderId, Integer name, String value, String desc) {
        OperateLog operateLog = new OperateLog();
        operateLog.setThirdPartyId(orderId);
        operateLog.setName(name == null ? "" : name.toString());
        operateLog.setValue(value);
        operateLog.setDesc(desc);
        operateLog.setType(OperateLogType.ORDER_STATUS.getType());
        return insert(operateLog);
    }
    
    public int insertOrderLog(String orderId, Integer name, String desc) {
        return insertOrderLog(orderId, name, OrderStatus.getText(name), desc);
    }
    
    // public int insertOrderLog(String orderId, Integer name) {
    // return insertOrderLog(orderId, name, null);
    // }
    
    public List<OperateLog> selectByExample(OperateLogBo condition) {
        if (!StringUtils.isEmpty(condition.getTypes())) {
            String[] arr = condition.getTypes().split(",");
            condition.setTypeList(Arrays.asList(arr));
        }
        return operateLogMapper.selectByExample(condition);
    }

    public OperateLog selectById(Integer id) {
        return operateLogMapper.selectById(id);
    }
    
    /**
     * 查询最近一天记录
     *
     * @param thId
     * @param type
     * @return
     */
    public OperateLog selectLastestByTPId(String thId, Integer type) {
        return selectLastestByTPId(thId, type, "");
    }
    
    public OperateLog selectLastestByTPId(String thId, Integer type, String name) {
        OperateLogBo condition = new OperateLogBo();
        condition.setThirdPartyId(thId);
        condition.setType(type);
        condition.setName(name);
        List<OperateLog> list = operateLogMapper.selectByExample(condition);
        return list.isEmpty() ? null : list.get(0);
    }

    public OperateLog getNewestCheckFailByOrderId(String orderId,String name) {
        return operateLogMapper.getNewestCheckFailByOrderId(orderId,name);
    }

    public void deleteByOrderId(String orderId) {
        operateLogMapper.deleteByOrderId(orderId);
    }
}
