package com.starcharge.wios.dao;

import cn.hutool.core.util.StrUtil;
import com.ihidea.core.support.pageLimit.PageLimit;
import com.ihidea.core.support.pageLimit.PageLimitHolderFilter;
import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.bo.OrderBo;
import com.starcharge.wios.bo.WarrantyOrderBo;
import com.starcharge.wios.constant.Constant;
import com.starcharge.wios.dao.entity.*;
import com.starcharge.wios.dao.mappers.*;
import com.starcharge.wios.dto.WarrantyOrder;
import com.starcharge.wios.enums.OrderStatus;
import com.starcharge.wios.vo.OrderVo;
import com.starcharge.wios.vo.SupplierStaffVo;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 订单dao
 * 
 * @author xgl
 * @version [版本号, 2020年9月16日]
 */
@Repository
public class OrderDao {
    @Autowired
    private OrderMapper orderMapper;
    
    @Autowired
    private TokenService tokenService;
    
    @Autowired
    private RegionDao regionDao;

    @Autowired
    private OemMapper oemMapper;

    @Autowired
    private SupplierMapper supplierMapper;

    @Autowired
    private SupplierStaffMapper supplierStaffMapper;

    @Autowired
    private SecretPhoneMapper secretPhoneMapper;

    @Autowired
    private SystemInfoDao systemInfoDao;

    @Autowired
    private InvoiceDao invoiceDao;

    public Order selectById(String id) {
        return orderMapper.selectById(id);
    }
    public int insert(Order order) {
        order.setCreateTime(new Date());
        order.setModifyTime(new Date());
        order.setStatus(OrderStatus.DISPATCHING.getType());
        if (order.getType() == null)
            order.setType(1);
        SessionInfo user = tokenService.getUser();
        if (user != null)
            order.setCreateAccount(user.getUserId());
        return orderMapper.insert(order);
    }
    
    public int batchInsert(List<Order> list) {
        SessionInfo user = tokenService.getUser();
        list.stream().forEach(order -> {
            order.setCreateTime(new Date());
            order.setModifyTime(new Date());
            order.setStatus(OrderStatus.DISPATCHING.getType());
            if (order.getType() == null)
                order.setType(1);
            if (user != null)
                order.setCreateAccount(user.getUserId());
        });
        return orderMapper.batchInsert(list);
    }
    
    public List<OrderVo> getList(OrderBo condition) {
        if (!StringUtils.isEmpty(condition.getRegionId())) {
            List<Region> regionList = regionDao.selectSelfAndChildren(Integer.valueOf(condition.getRegionId()));
            List<Integer> regionIdList = regionList.stream().map(Region::getId).collect(Collectors.toList());
            condition.setRegionIdList(regionIdList);
        }
        if (!StringUtils.isEmpty(condition.getStatusIn())) {
            String statusIn = condition.getStatusIn();
            String[] statusInArr = statusIn.split(",");
            List<Integer> statusInList = new ArrayList<>();
            for (String status : statusInArr) {
                statusInList.add(Integer.valueOf(status));
            }
            condition.setStatusInList(statusInList);
        }
        if (!StringUtils.isEmpty(condition.getIdIn())) {
            String idIn = condition.getIdIn();
            String[] idInArr = idIn.split(",");
            condition.setIdInList(Arrays.asList(idInArr));
        }
        
        // 查询待报装的订单(只有勘测阶段)
        if (condition.getExpandStatusInt() != null && condition.getExpandStatusInt().intValue() == 0) {
            List<Integer> statusInList = new ArrayList<>();
            statusInList.add(OrderStatus.SURVEY_DISPATCHED.getType());
            statusInList.add(OrderStatus.SURVEYING.getType());
            statusInList.add(OrderStatus.SURVEY_FINISH.getType());
            statusInList.add(OrderStatus.SURVEY_EXAMINING.getType());
            statusInList.add(OrderStatus.SURVEY_RECTIFYING.getType());
            statusInList.add(OrderStatus.SURVEY_RECTIFIED_EXAMINING.getType());
            condition.setStatusInList(statusInList);
        }

        return selectByCondition(condition);
    }
    
    public List<Order> selectAll(OrderBo condition) {
        return orderMapper.selectAll(condition);
    }
    
    public List<Order> selectAllForSchedule(String id) {
        OrderBo condition = new OrderBo();
        if (!StringUtils.isEmpty(id))
            condition.setId(id);
        List<Integer> statusNotInList = new ArrayList<>();
        statusNotInList.add(OrderStatus.CLOSED.getType());
        statusNotInList.add(OrderStatus.RETURN_VISIT.getType());
        // statusNotInList.add(OrderStatus.PLACE_ON_FILE.getType());
        condition.setStatusNotInList(statusNotInList);
        
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, -15);
        condition.setCreateStartTime(sdf.format(cal.getTime()));
        condition.setCreateEndTime(sdf.format(new Date()));
        return orderMapper.selectAll(condition);
    }
    
    public List<OrderVo> getRegionId(String regionId) {
        OrderBo condition = new OrderBo();
        condition.setRegionId(regionId);
        return selectByCondition(condition);
    }

    /**
     * 更新订单
     *
     * @param id
     * @param order
     * @return
     */
    public int updateById(String id, Order order) {
        order.setId(id);
        order.setModifyTime(new Date());
        SessionInfo user = tokenService.getUser();
        if (user != null)
            order.setModifyAccount(user.getUserId());
        return orderMapper.updateDynamic(order);
    }
    
    public int clearInstallStaff(String orderId) {
        return orderMapper.clearInstallStaff(orderId);
    }
    
    public OrderVo selectDetailById(String id) {
        return orderMapper.selectDetailById(id);
    }
    
    /**
     * 更新订单状态
     *
     * @param id
     * @param status
     * @return
     */
    public int updateStatusById(String id, int status) {
        Order order = new Order();
        order.setStatus(status);
        return updateById(id, order);
    }
    
    /**
     * 查询时间段内的订单
     *
     * @param oemIds
     * @param installFinishStartTime
     * @param installFinishEndTime
     * @return
     */
    public List<OrderVo> selectOemOrders(String oemIds, String installFinishStartTime, String installFinishEndTime) {
        String[] oemIdArr = oemIds.split(",");
        List<String> oemIdList = Arrays.asList(oemIdArr);
        List<Integer> oemIdIntList = new ArrayList<Integer>();
        oemIdList.stream().forEach(item -> {
            oemIdIntList.add(Integer.valueOf(item));
        });
        OrderBo condition = new OrderBo();
        condition.setInstallFinishStartTime(installFinishStartTime);
        condition.setInstallFinishEndTime(installFinishEndTime);
        condition.setOemIdList(oemIdIntList);
        return selectByCondition(condition);
    }
    
    public List<OrderVo> selectPeriodOrders(String installFinishStartTime, String installFinishEndTime) {
        OrderBo condition = new OrderBo();
        condition.setInstallFinishStartTime(installFinishStartTime);
        condition.setInstallFinishEndTime(installFinishEndTime);
        condition.setType(1);
        condition.setIsClosed(64);
        return selectByCondition(condition);
    }
    
    public List<WarrantyOrder> selectWarrantyOrders(WarrantyOrderBo condition) {
        if (condition.getOrderBy() == null) {
            condition.setOrderBy(1);
        }
        return orderMapper.selectWarrantyOrders(condition);
    }
    
    public List<OrderVo> selectByCamsPhone(String camsPhone) {
        return orderMapper.selectByCamsPhone(camsPhone);
    }
    
    public List<OrderVo> selectByCamsPhoneForApp(String camsPhone) {
        return orderMapper.selectByCamsPhoneForApp(camsPhone);
    }

    public Integer countByCamsPhone(String camsPhone) {
        return orderMapper.countByCamsPhone(camsPhone);
    }

    public List<OrderVo> selectOwnOrder(OrderBo condition){
        return orderMapper.selectOwnOrder(condition);
    }

    public List<OrderVo> selectByCondition (OrderBo condition){
        List<Oem> oemList = oemMapper.selectAll(null);
        if(!StringUtils.isEmpty(condition.getOemName())){
            List<Integer> filterOemIds = oemList.stream().filter(v -> v.getName().contains(condition.getOemName())).map(Oem::getId).collect(Collectors.toList());
            if (filterOemIds.size()>0) {
                condition.setFilterOemIds(filterOemIds);
            }

        }

        //
        List<Supplier> supplierList = supplierMapper.selectAll(null);
        if (!StringUtils.isEmpty(condition.getSupplierName())) {
            List<String> filterSupplierIds = supplierList.stream().filter(v -> v.getName().contains(condition.getSupplierName())).map(Supplier::getId).collect(Collectors.toList());
            if (filterSupplierIds.size() == 0) {
                filterSupplierIds.add("-1");
            }
            condition.setFilterSupplierIds(filterSupplierIds);
        }

        PageLimit pageLimit = PageLimitHolderFilter.getContext();
        if (pageLimit != null) {
            pageLimit.setLimited(false);
        }

        //按墙盒设备编号精确查找
        if (!StringUtils.isEmpty(condition.getWallboxNo())) {
            condition.setFromExport(1);
        }
        List<OrderVo> orderList = orderMapper.selectByExample(condition);
        List<SupplierStaffVo> supplierStaffList = supplierStaffMapper.selectAll();
        SecretPhone conditionSecretPhone = new SecretPhone();
        conditionSecretPhone.setStatus(1);
        List<SecretPhone> secretPhoneList = secretPhoneMapper.selectByExample(conditionSecretPhone);
//        List<SystemInfo> systemInfoList=systemInfoDao.getOrderInOverTime();
//        Map<String, String> mapSystemInfo = systemInfoList.stream().distinct().collect(Collectors.toMap(p -> p.getThirdPartyId() + "|" + p.getOrderStatus(), SystemInfo::getThirdPartyId, (v1, v2) -> v2));
        Map<String, String> invoiceMap = new HashMap<>();
        if (condition.getFromExport()!=null && condition.getFromExport()==1) {
            List<Invoice> invoiceList =invoiceDao.getadditionalCostInvoiceList();
            invoiceMap = invoiceList.stream().collect(Collectors.toMap(Invoice::getOrderId, Invoice::getAdditionalCostInvoiceNumber, (v1, v2) -> v2));
        }
        Map<String, String> finalInvoiceMap = invoiceMap;
        orderList.stream().forEach(order ->{
                    for (Oem oem:oemList){
                        if (order.getOemId()!=null){
                            if (order.getOemId().equals(oem.getId())) {
                                order.setOemName(oem.getName());
                            }
                        }
                        if (order.getOemAgentId()!=null) {
                            if(order.getOemAgentId().equals(oem.getId())){
                                order.setOemAgentName(oem.getName());
                            }
                        }

                    }
                    for(Supplier supplier:supplierList){
                        if(order.getInstallSupplierId()!=null){
                            if ((order.getInstallSupplierId().toString()).equals(supplier.getId())) {
                                order.setSupplierName(supplier.getName());
                            }
                        }
                    }
                    for (SupplierStaffVo supplierStaffVo:supplierStaffList){
                        if(order.getSurveyStaffId()!=null){
                            if(order.getSurveyStaffId().equals(supplierStaffVo.getId())){
                                order.setSurveyStaffName(supplierStaffVo.getName());
                                order.setSurveyStaffPhone(supplierStaffVo.getPhone());
                            }
                        }
                        if(order.getInstallStaffId()!=null){
                            if(order.getInstallStaffId().equals(supplierStaffVo.getId())){
                                order.setInstallStaffName(supplierStaffVo.getName());
                                order.setInstallStaffPhone(supplierStaffVo.getPhone());
                            }
                        }
                    }
                    for(SecretPhone secretPhone:secretPhoneList){
                        if (!StringUtils.isEmpty(order.getUserPhone())) {
                            if (order.getUserPhone().equals(secretPhone.getPhone())) {
                                order.setVirtualUserPhone(secretPhone.getSecretNo());
                            }
                        }
                    }

                    /*if(mapSystemInfo.containsKey((order.getId()+"|"+order.getStatus()))){
                        order.setOvertimeFlag(Constant.OVERTIME_FLAG_YES);
                    }else{
                        order.setOvertimeFlag(Constant.OVERTIME_FLAG_NO);
                    }*/

                    //安装墙盒型号，勘察墙盒型号，订单新建墙盒型号：优先级有高到低取
                    if(StrUtil.isBlank(order.getInstallWallboxModel())){
                        if(StrUtil.isBlank(order.getSurveyWallboxModel())){
                            order.setWallboxModel(order.getWallboxModel());
                        }else{
                            order.setWallboxModel(order.getSurveyWallboxModel());
                        }
                    }else{
                        order.setWallboxModel(order.getInstallWallboxModel());
                    }

                    if (condition.getFromExport()!=null && condition.getFromExport()==1) {
                        order.setInvoiceNumber(finalInvoiceMap.get(order.getId()));
                    }

                }
        );
        return orderList;
    }

    public void delete(String id) {
        orderMapper.delete(id);
    }


    public int clearSurveyFollowTime(String orderId) {
        return orderMapper.clearSurveyFollowTime(orderId);
    }

    public int clearInstallFollowTime(String orderId) {
        return orderMapper.clearInstallFollowTime(orderId);
    }

    public List<Order> selectSurveyPeriodOrders(String dateStart, String dateEnd) {
        return orderMapper.selectSurveyPeriodOrders(dateStart,dateEnd);
    }

    public List<Order> selectInstallPeriodOrders(String dateStart, String dateEnd) {
        return orderMapper.selectInstallPeriodOrders(dateStart,dateEnd);
    }

    public Order selectByOutOrderId(String outOrderId) {
        OrderBo orderVo = new OrderBo();
        orderVo.setOutOrderId(outOrderId);
        orderVo.setStatusNotInList(Arrays.asList(OrderStatus.CLOSED.getType()));
        List<Order> orders = orderMapper.selectAll(orderVo);
        if(orders.size()>0){
            return orders.get(0);
        }
        return null;
    }
}
