package com.starcharge.wios.dao;

import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.dao.entity.Order;
import com.starcharge.wios.dao.entity.OrderScore;
import com.starcharge.wios.dao.mappers.OrderMapper;
import com.starcharge.wios.dao.mappers.OrderScoreMapper;
import com.starcharge.wios.utils.NumberUtil;

@Repository
public class OrderScoreDao {
    @Autowired
    private OrderScoreMapper orderScoreMapper;

    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    TokenService tokenService;
    
    public int insert(OrderScore orderScore) {
        orderScore.setCreateAccount(this.getUserId());
        orderScore.setCreateTime(new Date());
        orderScore.setModifyAccount(this.getUserId());
        orderScore.setModifyTime(new Date());
        orderScore.setStatus(1L);
        return orderScoreMapper.insert(orderScore);
    }

    public int insertOrderScore(String orderId){
        return orderScoreMapper.insertOrderScore(orderId);
    }

    public int update( OrderScore orderScore) {
        orderScore.setModifyAccount(this.getUserId());
        orderScore.setModifyTime(new Date());
        int ret = orderScoreMapper.update(orderScore);
        if(orderScore.getScore() !=null){
            //更新该订单的客户评分合计
            OrderScore tmpOrderScore = orderScoreMapper.selectById(orderScore.getId());
            if (tmpOrderScore != null) {
                Double sumScore = orderScoreMapper.getScoreByOrderId(tmpOrderScore.getOrderId());
                Order order = new Order();
                order.setId(String.valueOf(tmpOrderScore.getOrderId()));
                order.setScore(NumberUtil.roundDouble(sumScore));
                orderMapper.updateDynamic(order);
            }
        }
        return ret;
    }

    /**
     * 通过编码和主表id更新明细表记录
     * @param orderScore
     * @return
     */
    public int updateByCode( OrderScore orderScore) {
        orderScore.setModifyAccount(this.getUserId());
        orderScore.setModifyTime(new Date());
        return orderScoreMapper.updateByCode(orderScore);
    }

    public OrderScore selectById(Long id) {
        return orderScoreMapper.selectById(id);
    }

    public Double getScoreByOrderId(String orderId){
        return orderScoreMapper.getScoreByOrderId(orderId);
    }
    
    public List<OrderScore> selectByTypeAndOrderIds(int type, List<String> orderIds) {
        return orderScoreMapper.selectByTypeAndOrderIds(type, orderIds);
    }

    public List<OrderScore> getListByOrderId(String orderId){
        return orderScoreMapper.getListByOrderId(orderId);
    }

    public List<Order> getOrderListLessThan60(Long supplierId, Long regionId, String month){
        return orderScoreMapper.getOrderListLessThan60(supplierId, regionId, month);
    }

    public String getUserId(){
        SessionInfo user = tokenService.getUser();
        if(user != null){
            return user.getUserId();
        }else{
            return "1";
            //throw new ServiceException("无效的用户，请登录!");
        }
    }

}
