package com.starcharge.wios.dao;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.wios.dao.entity.Region;
import com.starcharge.wios.dao.mappers.RegionMapper;

/**
 * 区域dao
 * 
 * @author xgl
 * @version [版本号, 2020年9月21日]
 */
@Repository
public class RegionDao {
    @Autowired
    private RegionMapper regionMapper;
    
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public int insert(Region region) {
        List<Region> regionList = selectByParentId(region.getParentId());
        regionList = regionList.stream().filter(r -> r.getName().equals(region.getName())).collect(Collectors.toList());// 筛选同级别的区域
        if (!regionList.isEmpty())
            throw new ServiceException("同一级别下的区域名称不可相同");
        
        region.setCreateTime(new Date());
        region.setStatus(1);
        regionMapper.insert(region);
        
        Region pRegion = selectById(region.getParentId());
        String idTree = pRegion.getIdTree();
        if (StringUtils.isEmpty(idTree)) {
            region.setIdTree(pRegion.getId() + "_" + region.getId());
        } else {
            region.setIdTree(idTree + "_" + region.getId());
        }
        
        Region updateRegion = new Region();
        updateRegion.setId(region.getId());
        updateRegion.setIdTree(region.getIdTree());
        return regionMapper.updateDynamic(updateRegion);
    }
    
    public Region selectById(int id) {
        return regionMapper.selectById(id);
    }
    
    public List<Region> selectByParentId(int parentId) {
        return regionMapper.selectByParentId(parentId);
    }
    
    public List<Region> selectAll(Integer status) {
        return regionMapper.selectAll(status, null);
    }
    
    public List<Region> selectAll(Integer status, List<Integer> ids) {
        return regionMapper.selectAll(status, ids);
    }
    
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public int update(int id, Region region) {
        region.setId(id);
        region.setUpdateTime(new Date());
        Region reg = regionMapper.selectById(id);
        if (region.getParentId() != null) {
            Region parentRegion = regionMapper.selectById(region.getParentId().intValue());
            String parentIdTree = parentRegion.getIdTree();
            if (StringUtils.isEmpty(parentIdTree)) {
                parentIdTree = parentRegion.getId().toString();
            }
            String idTree = (parentIdTree + "_" + id);
            
            List<Region> regionList = selectByParentId(reg.getParentId());
            regionList = regionList.stream()
                .filter(r -> r.getName().equals(region.getName())).collect(Collectors.toList());// 筛选同级别的区域
            if (!regionList.isEmpty() && regionList.get(0).getId().intValue() != id)
                throw new ServiceException("同一级别下的区域名称不可相同");

            region.setIdTree(idTree);
            regionMapper.updateIdTree(idTree, reg.getIdTree());
        }
        regionMapper.updateDynamic(region);
        return regionMapper.updateDynamic(region);
    }
    
    public List<Region> selectByName(String regionName) {
        return regionMapper.selectByName(regionName);
    }
    
    public Region selectSingleByName(String regionName) {
        List<Region> regionList = selectByName(regionName);
        return regionList.isEmpty() ? null : regionList.get(0);
    }
    
    public List<Region> selectSelfAndChildren(Integer id) {
        Region region = regionMapper.selectById(id);
        return regionMapper.selectSelfAndChildren(region.getIdTree());
    }
    
    public List<Region> selectByCode(String code) {
        return regionMapper.selectByCode(code);
    }
}
