package com.starcharge.wios.dao;

import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.dao.entity.ScoreSupplier;
import com.starcharge.wios.dao.mappers.ScoreSupplierMapper;
import com.starcharge.wios.dto.ScoreSupplierDTO;
import com.starcharge.wios.enums.ScoreStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;

@Repository
public class ScoreSupplierDao {
    @Autowired
    private ScoreSupplierMapper scoreSupplierMapper;

    @Autowired
    TokenService tokenService;
    
    public int insert(ScoreSupplier scoreSupplier) {
        scoreSupplier.setCreateAccount(this.getUserId());
        scoreSupplier.setCreateTime(new Date());
        scoreSupplier.setModifyAccount(this.getUserId());
        scoreSupplier.setModifyTime(new Date());
        scoreSupplier.setStatus(ScoreStatus.PROCESSING.getType());
        return scoreSupplierMapper.insert(scoreSupplier);
    }

    public int update( ScoreSupplier scoreSupplier) {
        scoreSupplier.setModifyAccount(this.getUserId());
        scoreSupplier.setModifyTime(new Date());
        return scoreSupplierMapper.update(scoreSupplier);
    }
    public int delete(Long id) {
        return scoreSupplierMapper.delete(id);
    }

    public ScoreSupplierDTO selectById(Long id) {
        return scoreSupplierMapper.selectById(id);
    }
    public ScoreSupplierDTO selectLastBySupplierId(Long supplierId) {
        return scoreSupplierMapper.selectLastBySupplierId(supplierId);
    }


    public List<ScoreSupplierDTO> selectByExample(ScoreSupplierDTO scoreSupplierDTO){
        return scoreSupplierMapper.selectByExample(scoreSupplierDTO);
    }

    public int insertMonthDetail(Long scoreSupplierId){
        return scoreSupplierMapper.insertMonthDetail(scoreSupplierId);
    }

    /**
     * 统计某个状态的订单数
     * @param supplierId
     * @param reginId
     * @param orderStatus
     * @param month
     * @return
     */
    public int countOrder(Long supplierId, Long reginId, String orderStatus,String month){
        return scoreSupplierMapper.countOrder(supplierId, reginId, orderStatus, month);
    }

    /**
     * 统计更换服务商次数
     * @param supplierId
     * @param month
     * @return
     */
    public int countChangeSupplier(Long supplierId, String threeMonthBefore,String month){
        return scoreSupplierMapper.countChangeSupplier(supplierId,threeMonthBefore, month);
    }

    /**
     * 计算工单完成的平均时间（小时）
     * @param supplierId
     * @param reginId
     * @param month
     * @return
     */
    public Integer calcOrderFinishedTime(Long supplierId, Long reginId, String month) {
        return scoreSupplierMapper.calcOrderFinishedTime(supplierId, reginId, month);
    }
    /**
     * 计算接单响应的平均时间（小时）
     * @param supplierId
     * @param reginId
     * @param month
     * @return
     */
    public Integer calcOrderResponseTime(Long supplierId, Long reginId, String month) {
        return scoreSupplierMapper.calcOrderResponseTime(supplierId, reginId, month);
    }

    /**
     * 计算客户评价的平均分
     * @param supplierId
     * @param regionIdTree
     * @param month
     * @return
     */
    public Double calcOrderScore(Long supplierId, String regionIdTree,String threeMonthBefore,String month){
        return scoreSupplierMapper.calcOrderScore(supplierId, regionIdTree,threeMonthBefore, month);
    }


    /**
     * 计算勘查完成的平均时间（小时），T3上门服务
     * @param supplierId
     * @param reginId
     * @param month
     * @return
     */
    public Integer calcOrderSurveyTime(Long supplierId, Long reginId, String month) {
        return scoreSupplierMapper.calcOrderResponseTime(supplierId, reginId, month);
    }

    public String getUserId(){
        SessionInfo user = tokenService.getUser();
        if(user != null){
            return user.getUserId();
        }else{
            return "1";
            //throw new ServiceException("无效的用户，请登录!");
        }
    }
    
    /**
     * 查询服务商最新评分记录
     *
     * @param supplierId
     * @param regionId
     * @return
     */
    public ScoreSupplier selectLastestScoreInfo(int supplierId, int regionId) {
        return scoreSupplierMapper.selectLastestScoreInfo(supplierId, regionId);
    }


    public int countOrderByCondition(Long supplierId, String regionIdTree,Integer status, String threeMonthBefore, String month) {
        return scoreSupplierMapper.countOrderByCondition(supplierId,regionIdTree,status, threeMonthBefore,month);
    }

    public int countOrderHasFinished(Long supplierId, String regionIdTree,String threeMonthBefore, String month) {
        return scoreSupplierMapper.countOrderHasFinished(supplierId,regionIdTree,threeMonthBefore,month);
    }

    public Integer countOrderFinishedTime(Long supplierId, String regionIdTree, String threeMonthBefore, String month) {
        return scoreSupplierMapper.countOrderFinishedTime(supplierId,regionIdTree,threeMonthBefore,month);
    }


    public Integer countOrderResponseTime(Long supplierId, String regionIdTree, String threeMonthBefore, String month) {
        return scoreSupplierMapper.countOrderResponseTime(supplierId,regionIdTree,threeMonthBefore,month);
    }

    public Integer countOrderSurveyTime(Long supplierId, String regionIdTree, String threeMonthBefore, String month) {
        return scoreSupplierMapper.countOrderSurveyTime(supplierId,regionIdTree,threeMonthBefore,month);
    }

    public int countOrderResponse(Long supplierId, String regionIdTree, String threeMonthBefore, String month) {
        return scoreSupplierMapper.countOrderResponse(supplierId,regionIdTree,threeMonthBefore,month);
    }

    public int countOrderFinished(Long supplierId, String regionIdTree, String threeMonthBefore, String month) {
        return scoreSupplierMapper.countOrderFinished(supplierId,regionIdTree,threeMonthBefore,month);
    }

    public int countOrderSurvey(Long supplierId, String regionIdTree, String threeMonthBefore, String month) {
        return scoreSupplierMapper.countOrderSurvey(supplierId,regionIdTree,threeMonthBefore,month);
    }
}
