package com.starcharge.wios.dao;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.bo.SupplierContractBo;
import com.starcharge.wios.dao.entity.SupplierContract;
import com.starcharge.wios.dao.mappers.SupplierContractMapper;
import com.starcharge.wios.enums.ContractStatus;
import com.starcharge.wios.vo.SupplierContractVo;

/**
 * 服务商合同dao
 * 
 * @author xgl
 * @version [版本号, 2020年9月16日]
 */
@Repository
public class SupplierContractDao {
    @Autowired
    private SupplierContractMapper supplierContractMapper;
    
    @Autowired
    private TokenService tokenService;
    
    public SupplierContract selectById(int id) {
        return supplierContractMapper.selectById(id);
    }
    
    public List<SupplierContract> selectByCode(String code) {
        return supplierContractMapper.selectByCode(code);
    }
    
    public int insert(SupplierContract contract) {
        contract.setCreateTime(new Date());
        if (tokenService.getUser() != null)
            contract.setCreateAccount(tokenService.getUser().getUserName());
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date date1 = null;
        try {
            date1 = sdf.parse(sdf.format(new Date()));
        } catch (ParseException e) {
            throw new ServiceException("数据转换失败");
        }
        if (contract.getStartTime().compareTo(date1) > 0) {
            contract.setStatus(ContractStatus.NOT_START.getType());
        } else {
            if (contract.getEndTime().compareTo(date1) >= 0) {
                contract.setStatus(ContractStatus.EFFECTIVE.getType());
            } else {
                throw new ServiceException("结束日期必须大于今天");
            }
        }
        return supplierContractMapper.insert(contract);
    }
    
    public int update(int id, SupplierContract contract) {
        contract.setId(id);
        contract.setModifyTime(new Date());
        if (tokenService.getUser() != null)
            contract.setModifyAccount(tokenService.getUser().getUserName());

        if (contract.getStartTime() != null && contract.getEndTime() != null) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            Date date1 = null;
            try {
                date1 = sdf.parse(sdf.format(new Date()));
            } catch (ParseException e) {
                throw new ServiceException("数据转换失败");
            }
            if (contract.getStartTime().compareTo(date1) > 0) {
                contract.setStatus(ContractStatus.NOT_START.getType());
            } else {
                if (contract.getEndTime().compareTo(date1) >= 0) {
                    contract.setStatus(ContractStatus.EFFECTIVE.getType());
                } else {
                    throw new ServiceException("结束日期必须大于今天");
                }
            }
        }
        return supplierContractMapper.updateDynamic(contract);
    }
    
    public List<SupplierContractVo> selectByStatusAndSupplierId(int supplierId, int status) {
        SupplierContractBo condition = new SupplierContractBo();
        condition.setSupplierId(supplierId + "");
        condition.setStatus(status);
        List<SupplierContractVo> list = supplierContractMapper.selectByExample(condition);
        return list;
    }
    
    public List<SupplierContractVo> selectValid(Integer supplierId) {
        SupplierContractBo condition = new SupplierContractBo();
        if (supplierId != null)
            condition.setSupplierId(supplierId.intValue() + "");
        List<Integer> statusInList = new ArrayList<Integer>();
        statusInList.add(ContractStatus.NOT_START.getType());
        statusInList.add(ContractStatus.EFFECTIVE.getType());
        condition.setStatusInList(statusInList);
        condition.setOrderBy(2);
        List<SupplierContractVo> list = supplierContractMapper.selectByExample(condition);
        return list;
    }
    
    public List<SupplierContractVo> getList(SupplierContractBo condition) {
        return supplierContractMapper.selectByExample(condition);
    }
    
    public List<SupplierContract> selectByRegionId(List<String> regionIdList,Integer checkStatus) {
        return supplierContractMapper.selectByRegionId(regionIdList, ContractStatus.EFFECTIVE.getType(),checkStatus);
    }
    
    public List<SupplierContract> selectEffectiveContract(String supplierId, String regionId, Integer status) {
        return supplierContractMapper.selectEffectiveContract(supplierId, regionId, status);
    }
}
