package com.starcharge.wios.dao;

import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.dao.entity.UploadFile;
import com.starcharge.wios.dao.mappers.UploadFileMapper;

/**
 * 上传文件dao
 * 
 * @author xgl
 * @version [版本号, 2020年10月10日]
 */
@Repository
public class UploadFileDao {
    @Autowired
    private UploadFileMapper ufMapper;
    
    @Autowired
    private TokenService tokenService;
    
    public UploadFile selectById(Integer id) {
        return ufMapper.selectById(id);
    }
    
    public int insert(UploadFile file) {
        file.setCreateTime(new Date());
        file.setStatus(1);
        if (tokenService.getUser() != null) {
            file.setCreateBy(tokenService.getUser().getUserName());
        }
        return ufMapper.insert(file);
    }
    
    public int batchInsert(List<UploadFile> list) {
        list.stream().forEach(file -> {
            file.setCreateTime(new Date());
            file.setStatus(1);
            if (tokenService.getUser() != null) {
                file.setCreateBy(tokenService.getUser().getUserName());
            }
        });
        return ufMapper.batchInsert(list);
    }
    
    public int update(int id, UploadFile file) {
        file.setId(id);
        file.setUpdateTime(new Date());
        if (tokenService.getUser() != null) {
            file.setUpdateBy(tokenService.getUser().getUserName());
        }
        return ufMapper.updateDynamic(file);
    }
    
    // public List<UploadFile> selectByExample(UploadFile condition) {
    // return ufMapper.selectByExample(condition);
    // }
    
    public List<UploadFile> selectValidByTypeAndTPId(int type, String tpId) {
        UploadFile condition = new UploadFile();
        condition.setStatus(1);
        condition.setType(type);
        condition.setThirdPartyId(tpId);
        return ufMapper.selectByExample(condition);
    }
    
    public int deleteByTypeAndTPId(int type, String tPId) {
        return ufMapper.deleteByTypeAndTPId(type, tPId);
    }
    
    public int delete(int id) {
        UploadFile file = new UploadFile();
        file.setId(id);
        file.setStatus(2);
        file.setUpdateTime(new Date());
        if (tokenService.getUser() != null) {
            file.setUpdateBy(tokenService.getUser().getUserName());
        }
        return ufMapper.updateDynamic(file);
    }
}
