package com.starcharge.wios.dao.mappers;

import com.starcharge.wios.dao.entity.ScoreSupplier;
import com.starcharge.wios.dto.ScoreSupplierDTO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface ScoreSupplierMapper {
    int delete(Long id);

    int insert(ScoreSupplier scoreSupplier);

    int update(ScoreSupplier scoreSupplier);

    ScoreSupplierDTO selectById(Long id);

    ScoreSupplierDTO selectLastBySupplierId(Long supplierId);

    List<ScoreSupplierDTO> selectByExample(ScoreSupplierDTO scoreSupplierDTO);

    //Integer findPageWithCount(ScoreSupplierDTO scoreSupplierDTO);

    int insertMonthDetail(Long scoreSupplierId);

    /**
     * 统计某个状态的订单数
     *
     * @param supplierId
     * @param reginId
     * @param orderStatus
     * @param month
     * @return
     */
    int countOrder(@Param("supplierId") Long supplierId, @Param("regionId") Long reginId, @Param("orderStatus") String orderStatus,
                   @Param("month") String month);

    /**
     * 统计更换服务商次数
     *
     * @param supplierId
     * @param month
     * @return
     */
    int countChangeSupplier(@Param("supplierId") Long supplierId, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    /**
     * 计算工单完成的平均时间（小时）
     *
     * @param supplierId
     * @param reginId
     * @param month
     * @return
     */
    Integer calcOrderFinishedTime(@Param("supplierId") Long supplierId, @Param("regionId") Long reginId, @Param("month") String month);

    /**
     * 计算接单响应的平均时间（小时）
     *
     * @param supplierId
     * @param reginId
     * @param month
     * @return
     */
    Integer calcOrderResponseTime(@Param("supplierId") Long supplierId, @Param("regionId") Long reginId, @Param("month") String month);

    /**
     * 计算勘查完成的平均时间（小时），T3上门服务
     *
     * @param supplierId
     * @param reginId
     * @param month
     * @return
     */
    int calcOrderSurveyTime(@Param("supplierId") Long supplierId, @Param("regionId") Long reginId, @Param("month") String month);

    ScoreSupplier selectLastestScoreInfo(@Param("supplierId") int supplierId, @Param("regionId") int regionId);

    /**
     * 计算客户评价的平均分
     *
     * @param supplierId
     * @param regionIdTree
     * @param month
     * @return
     */
    Double calcOrderScore(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    int countOrderByCondition(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("status") Integer status, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    int countOrderHasFinished(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    Integer countOrderFinishedTime(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    Integer countOrderResponseTime(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    Integer countOrderSurveyTime(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    int countOrderResponse(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    int countOrderFinished(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);

    int countOrderSurvey(@Param("supplierId") Long supplierId, @Param("regionIdTree") String regionIdTree, @Param("threeMonthBefore") String threeMonthBefore, @Param("month") String month);
}