package com.starcharge.wios.dto;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.springframework.util.StringUtils;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.StringUtilsEx;
import com.starcharge.wios.dao.entity.SupplierContract;
import com.starcharge.wios.dao.entity.SupplierContractItem;
import com.starcharge.wios.dao.entity.SupplierContractPackPrice;
import com.starcharge.wios.dao.entity.SupplierContractPrice;
import com.starcharge.wios.dao.entity.UploadFile;

import io.swagger.annotations.ApiModelProperty;

public class SupplierContractDTO extends SupplierContract {
    @ApiModelProperty(value = "常规价格json")
    private String pricesJson;
    
    @ApiModelProperty(hidden = true)
    private List<SupplierContractPrice> prices;
    
    @ApiModelProperty(value = "套包价格json")
    private String packPricesJson;
    
    @ApiModelProperty(hidden = true)
    private List<SupplierContractPackPrice> packPrices;
    
    @ApiModelProperty(value = "增项价格json")
    private String itemsJson;
    
    @ApiModelProperty(hidden = true)
    private List<SupplierContractItem> items;
    
    @ApiModelProperty(value = "文件json字符串")
    private String filesJson;
    
    @ApiModelProperty(hidden = true)
    private List<UploadFile> files;
    
    @ApiModelProperty(value = "开始日期 yyyy-MM-dd")
    private String startDateStr;
    
    @ApiModelProperty(value = "结束日期 yyyy-MM-dd")
    private String endDateStr;
    
    public String getPricesJson() {
        return pricesJson;
    }
    
    public List<SupplierContractPrice> getPrices() {
        return prices;
    }
    
    public String getPackPricesJson() {
        return packPricesJson;
    }
    
    public List<SupplierContractPackPrice> getPackPrices() {
        return packPrices;
    }
    
    public String getItemsJson() {
        return itemsJson;
    }
    
    public List<SupplierContractItem> getItems() {
        return items;
    }
    
    public String getFilesJson() {
        return filesJson;
    }
    
    public List<UploadFile> getFiles() {
        return files;
    }
    
    public void setPricesJson(String pricesJson) {
        if (!StringUtils.isEmpty(pricesJson)) {
            pricesJson = StringUtilsEx.unescapeXss(pricesJson);
            this.pricesJson = pricesJson;
            try {
                ObjectMapper mapper = new ObjectMapper();
                mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
                List<SupplierContractPrice> list = mapper.readValue(pricesJson, new TypeReference<List<SupplierContractPrice>>() {
                });
                this.prices = list;
            } catch (Exception e) {
                throw new ServiceException("数据转换异常");
            }
        }
    }
    
    public void setPrices(List<SupplierContractPrice> prices) {
        this.prices = prices;
    }
    
    public void setPackPricesJson(String packPricesJson) {
        if (!StringUtils.isEmpty(packPricesJson)) {
            itemsJson = StringUtilsEx.unescapeXss(packPricesJson);
            this.packPricesJson = packPricesJson;
            try {
                ObjectMapper mapper = new ObjectMapper();
                mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
                List<SupplierContractPackPrice> list =
                    mapper.readValue(packPricesJson, new TypeReference<List<SupplierContractPackPrice>>() {
                    });
                this.packPrices = list;
            } catch (Exception e) {
                throw new ServiceException("数据转换异常");
            }
        }
    }
    
    public void setPackPrices(List<SupplierContractPackPrice> packPrices) {
        this.packPrices = packPrices;
    }
    
    public void setItemsJson(String itemsJson) {
        if (!StringUtils.isEmpty(itemsJson)) {
            itemsJson = StringUtilsEx.unescapeXss(itemsJson);
            this.itemsJson = itemsJson;
            try {
                ObjectMapper mapper = new ObjectMapper();
                mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
                List<SupplierContractItem> list = mapper.readValue(itemsJson, new TypeReference<List<SupplierContractItem>>() {
                });
                this.items = list;
            } catch (Exception e) {
                throw new ServiceException("数据转换异常");
            }
        }
    }
    
    public void setItems(List<SupplierContractItem> items) {
        this.items = items;
    }
    
    public void setFilesJson(String filesJson) {
        if (!StringUtils.isEmpty(filesJson)) {
            filesJson = StringUtilsEx.unescapeXss(filesJson);
            this.filesJson = filesJson;
            try {
                ObjectMapper mapper = new ObjectMapper();
                mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
                List<UploadFile> list = mapper.readValue(filesJson, new TypeReference<List<UploadFile>>() {
                });
                this.files = list;
            } catch (Exception e) {
                throw new ServiceException("数据转换异常");
            }
        }
    }
    
    public void setFiles(List<UploadFile> files) {
        this.files = files;
    }
    
    public String getStartDateStr() {
        return startDateStr;
    }
    
    public String getEndDateStr() {
        return endDateStr;
    }
    
    public void setStartDateStr(String startDateStr) {
        if (!StringUtils.isEmpty(startDateStr)) {
            this.startDateStr = startDateStr;
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            try {
                Date start = sdf.parse(startDateStr);
                setStartTime(start);
            } catch (Exception e) {
                throw new ServiceException("开始时间转换失败");
            }
        }
    }
    
    public void setEndDateStr(String endDateStr) {
        if (!StringUtils.isEmpty(endDateStr)) {
            this.endDateStr = endDateStr;
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            try {
                Date end = sdf.parse(endDateStr);
                setEndTime(end);
            } catch (Exception e) {
                throw new ServiceException("结束时间转换失败");
            }
        }
    }
}
