package com.starcharge.wios.export;

import java.io.*;
import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import java.util.zip.Adler32;
import java.util.zip.CheckedOutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.servlet.http.HttpServletResponse;

import cn.hutool.core.util.ZipUtil;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.util.DateUtilsEx;
import com.starcharge.component.datastore.FileController;
import com.starcharge.component.datastore.FileSupportService;
import com.starcharge.component.datastore.fileio.FileIoEntity;
import com.starcharge.component.datastore.fileio.FileIoFactory;
import com.starcharge.component.datastore.fileio.FileIoOSS;
import com.starcharge.component.datastore.fileio.IFileIo;
import com.starcharge.wios.bo.OrderBo;
import com.starcharge.wios.constant.Constant;
import com.starcharge.wios.contants.Contant;
import com.starcharge.wios.controller.OrderController;
import com.starcharge.wios.dao.OrderInstallDao;
import com.starcharge.wios.dao.entity.OrderInstall;
import com.starcharge.wios.service.OrderInstallService;
import com.starcharge.wios.service.OrderService;
import com.starcharge.wios.vo.OrderVo;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

import com.deepoove.poi.data.HyperLinkTextRenderData;
import com.deepoove.poi.data.NumbericRenderData;
import com.deepoove.poi.data.PictureRenderData;
import com.deepoove.poi.data.RenderData;
import com.deepoove.poi.data.TextRenderData;
import com.deepoove.poi.util.BytePictureUtils;
import com.ihidea.core.support.SpringContextLoader;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.JSONUtilsEx;
import com.starcharge.wios.dao.OrderInstallItemDao;
import com.starcharge.wios.dao.UploadFileDao;
import com.starcharge.wios.dao.entity.UploadFile;
import com.starcharge.wios.enums.UploadFileType;
import com.starcharge.wios.export.bo.OrderInstallExportBo;
import com.starcharge.wios.export.bo.OrderSurveyExportBo;
import com.starcharge.wios.export.bo.UploadPic;
import com.starcharge.wios.service.ScoreConfigService;
import com.starcharge.wios.vo.OrderInstallItemVo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 通用导出controller
 *
 * @author xgl
 * @version [版本号, 2018年8月27日]
 */
@RestController
@Api(tags = "导出模块")
public class ExportController {
    @Autowired
    ScoreConfigService scoreConfigService;

    @Autowired
    private UploadFileDao uploadFileDao;

    @Autowired
    private OrderInstallItemDao oiiDao;

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderInstallDao orderInstallDao;

    @Autowired
    private FileSupportService fileSupportService;

    @Autowired
    private FileSupportService service;

    @Autowired
    private FileIoOSS fileIoOSS;


    /**
     * 导出
     *
     * @param response
     * @param serviceName service对应的bean名字
     * @param fileName    文件名
     * @param searchJson  查询条件json
     * @throws Exception
     */
    @GetMapping(value = "export.do")
    @ApiOperation(value = "导出excel")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "serviceName", value = "service名称", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "fileName", value = "文件名称", dataType = "String", paramType = "query", required = true),
            @ApiImplicitParam(name = "searchJson", value = "查询条件json字符串", dataType = "String", paramType = "query", required = false),})
    public void export(HttpServletResponse response, String serviceName, String fileName, String searchJson) throws Exception {
        Export<?> export = SpringContextLoader.getBean(serviceName + "Export", Export.class);
        Contants contants = new Contants();
        Field field = Contants.class.getDeclaredField("export_" + serviceName + "_field");
        String fieldStr = field.get(contants).toString();
        Field title = Contants.class.getDeclaredField("export_" + serviceName + "_title");
        String titleStr = title.get(contants).toString();

        // String[] titleArray = "score_supplier".equals(serviceName) ? scoreConfigService.getExcelTitles() :
        // titleStr.split(",");
        String[][] titleArray = null;
        if ("score_supplier".equals(serviceName)) {
            String[] excelTitles = scoreConfigService.getExcelTitles();
            titleArray = new String[2][excelTitles.length];
            titleArray[0] = excelTitles;
            String[] newArr = Arrays.copyOf(excelTitles, excelTitles.length);
            newArr[5] = "总分";
            titleArray[1] = newArr;
        } else {
            titleArray = new String[][]{titleStr.split(",")};
        }

        String[] fieldStrArray = fieldStr.split(",");
        List<?> exportList = export.getExportList(searchJson);
        PoiUtil.exportExcelToWebsite(response, fileName, titleArray, fieldStrArray, exportList);

    }

    @SuppressWarnings("unchecked")
    @PostMapping(value = "/exportSurveyOrderDocx.do")
    @ApiOperation(value = "导出勘察订单word")
    public void exportDocx(HttpServletResponse response, OrderSurveyExportBo bo) throws Exception {
        if (StringUtils.isEmpty(bo.getOrderId()) || StringUtils.isEmpty(bo.getId()) || bo.getType() == null) {
            throw new ServiceException("缺少参数");
        }
        List<OrderInstallItemVo> oiiList = oiiDao.selectByOrderIdAndType(bo.getOrderId(), bo.getType().intValue() == 1 ? 1 : 3);
        List<UploadFile> files = uploadFileDao.selectValidByTypeAndTPId(
                bo.getType().intValue() == 1 ? UploadFileType.SURVEY_ORDER.getType() : UploadFileType.INSTALL_CHANGE_ORDER.getType(),
                bo.getId());
        List<UploadFile> picFiles =
                files.stream().filter(file -> file.getUseType() != null && file.getUseType().intValue() == 1).collect(Collectors.toList());
        List<UploadFile> attachFiles =
                files.stream().filter(file -> file.getUseType() != null && file.getUseType().intValue() == 2).collect(Collectors.toList());
        Map<String, Object> dataMap = JSONUtilsEx.deserialize(JSONUtilsEx.serialize(bo), Map.class);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        List<UploadPic> picList = new ArrayList<>();
        picFiles.stream().forEach(pic -> {
            try {
                UploadPic up = new UploadPic();
                up.setPrd(new PictureRenderData(50, 50, ".png", BytePictureUtils.getUrlBufferedImage(pic.getUrl())));
                up.setName(pic.getCreateBy());
                up.setTime(sdf.format(pic.getCreateTime()));
                picList.add(up);
            } catch (Exception e) {
            }
        });
        dataMap.put("picFiles", picList);

        List<RenderData> attachFileList = new ArrayList<>();
        attachFiles.stream().forEach(file -> {
            HyperLinkTextRenderData row = new HyperLinkTextRenderData(file.getName(), file.getUrl());
            attachFileList.add(row);
        });
        dataMap.put("attachFiles", new NumbericRenderData(attachFileList));

        List<RenderData> additionList = new ArrayList<>();
        oiiList.stream().forEach(oii -> {
            StringBuffer sb = new StringBuffer();
            sb.append(oii.getItemName()).append("  ").append("X").append(oii.getQunatity().intValue()).append("  ")
                    .append(oii.getItemModel()).append("  ").append(oii.getFee()).append("元");
            TextRenderData row = new TextRenderData(sb.toString());
            additionList.add(row);
        });
        dataMap.put("additionFee", new NumbericRenderData(additionList));

        PoiUtil.exportOrderDocx(response, dataMap, bo.getType().intValue() == 1 ? "勘察订单详情.docx" : "安装变更订单详情.docx", "勘察订单导出模板");
    }

    @SuppressWarnings("unchecked")
    @PostMapping(value = "/exportInstallOrderDocx.do")
    @ApiOperation(value = "导出安装订单word")
    public void exportInstallDocx(HttpServletResponse response, OrderInstallExportBo bo) throws Exception {
        if (StringUtils.isEmpty(bo.getOrderId()) || StringUtils.isEmpty(bo.getId())) {
            throw new ServiceException("缺少参数");
        }
        List<OrderInstallItemVo> oiiList = oiiDao.selectByOrderIdAndType(bo.getOrderId(), 2);
        List<UploadFile> files = uploadFileDao.selectValidByTypeAndTPId(UploadFileType.INSTALL_ORDER.getType(), bo.getId().toString());
        List<UploadFile> picFiles =
                files.stream().filter(file -> file.getUseType() != null && file.getUseType().intValue() != 5).collect(Collectors.toList());
        List<UploadFile> attachFiles =
                files.stream().filter(file -> file.getUseType() != null && file.getUseType().intValue() == 5).collect(Collectors.toList());
        Map<String, Object> dataMap = JSONUtilsEx.deserialize(JSONUtilsEx.serialize(bo), Map.class);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        List<UploadPic> picList = new ArrayList<>();
        picFiles.stream().forEach(pic -> {
            try {
                UploadPic up = new UploadPic();
                up.setPrd(new PictureRenderData(50, 50, ".png", BytePictureUtils.getUrlBufferedImage(pic.getUrl())));
                up.setName(pic.getCreateBy());
                up.setTime(sdf.format(pic.getCreateTime()));
                picList.add(up);
            } catch (Exception e) {
            }
        });
        dataMap.put("picFiles", picList);

        List<RenderData> attachFileList = new ArrayList<>();
        attachFiles.stream().forEach(file -> {
            HyperLinkTextRenderData row = new HyperLinkTextRenderData(file.getName(), file.getUrl());
            attachFileList.add(row);
        });
        dataMap.put("attachFiles", new NumbericRenderData(attachFileList));

        List<RenderData> additionList = new ArrayList<>();
        oiiList.stream().forEach(oii -> {
            StringBuffer sb = new StringBuffer();
            sb.append(oii.getItemName()).append("  ").append("X").append(oii.getQunatity().intValue()).append("  ")
                    .append(oii.getItemModel()).append("  ").append(oii.getFee()).append("元");
            TextRenderData row = new TextRenderData(sb.toString());
            additionList.add(row);
        });
        dataMap.put("additionFee", new NumbericRenderData(additionList));

        List<RenderData> chargeCheckList = new ArrayList<>();// 充电设施检查
        String chargeCheck = bo.getChargeCheck();
        if (!StringUtils.isEmpty(chargeCheck)) {
            String[] chargeCheckArr = chargeCheck.split(",");
            for (String cc : chargeCheckArr) {
                String[] ccArr = cc.split("_");
                chargeCheckList.add(new TextRenderData(ccArr[0] + "   " + ccArr[1]));
            }
        }
        dataMap.put("chargeCheckList", new NumbericRenderData(chargeCheckList));

        List<RenderData> cableCheckList = new ArrayList<>();// 电缆铺设检查
        String cableCheck = bo.getCableCheck();
        if (!StringUtils.isEmpty(cableCheck)) {
            String[] cableCheckArr = cableCheck.split(",");
            for (String cc : cableCheckArr) {
                String[] ccArr = cc.split("_");
                cableCheckList.add(new TextRenderData(ccArr[0] + "   " + ccArr[1]));
            }
        }
        dataMap.put("cableCheckList", new NumbericRenderData(cableCheckList));

        List<RenderData> trainContentList = new ArrayList<>();// 现场培训内容
        String trainContent = bo.getTrainContent();
        if (!StringUtils.isEmpty(trainContent)) {
            String[] trainContentArr = trainContent.split(",");
            for (String content : trainContentArr) {
                trainContentList.add(new TextRenderData(content));
            }
        }
        dataMap.put("trainContentList", new NumbericRenderData(trainContentList));

        PoiUtil.exportOrderDocx(response, dataMap, "安装订单详情.docx", "安装订单导出模板");
    }

    @GetMapping(value = "/exportInstallAttachFiles.do")
    @ApiOperation(value = "导出安装附件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "searchJson", value = "查询条件json字符串", dataType = "String", paramType = "query", required = false),})
    public BaseResponse<String> export(String searchJson) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        OrderBo condition = mapper.readValue(searchJson, OrderBo.class);
        List<OrderVo> orderList = orderService.getList(condition);
        File zipFile = null;
        zipFile = File.createTempFile("安装附件_", ".zip");
        FileOutputStream f = new FileOutputStream(zipFile);
        ZipOutputStream zos = new ZipOutputStream(f);
        List<UploadFile> uploadFileList = new ArrayList<>();
        for (OrderVo orderVo : orderList) {
            OrderInstall orderInstall = orderInstallDao.selectByOrderId(orderVo.getId());
            if (ObjectUtils.isEmpty(orderInstall)) {
                continue;
            }
            List<UploadFile> uploadFiles = uploadFileDao.selectValidByTypeAndTPId(2, orderInstall.getId().toString());
            List<UploadFile> uploadFilesFilter = uploadFiles.stream().filter(p -> (p.getName().contains("安装报告") || p.getName().contains("验收单"))).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(uploadFilesFilter)) {
                continue;
            }
            uploadFileList.addAll(uploadFilesFilter);
        }
        if (CollectionUtils.isEmpty(uploadFileList)) {
            zos.close();
            f.close();
            zipFile.delete();
            throw new ServiceException("无安装附件下载");
        }
        BufferedInputStream bis = null;
        for (UploadFile uploadFile : uploadFileList) {
            String urlStr = uploadFile.getUrl().substring(uploadFile.getUrl().lastIndexOf("/") + 1);
            FileIoEntity fileIoEntity = fileSupportService.get(urlStr);
            zos.putNextEntry(new ZipEntry(uploadFile.getName()));
            int bytesRead;
            byte[] buffs = new byte[1024 * 10];
            bis = new BufferedInputStream(new ByteArrayInputStream(fileIoEntity.getContent()), 1024 * 10);
            while ((bytesRead = bis.read(buffs, 0, 1024 * 10)) != -1) {
                zos.write(buffs, 0, bytesRead);
            }
        }
        zos.close();

        byte[] buffer = null;
        try {
            FileInputStream fis = new FileInputStream(zipFile);
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            byte[] b = new byte[1024];
            int n;
            while ((n = fis.read(b)) != -1) {
                bos.write(b, 0, n);
            }
            fis.close();
            bos.close();
            buffer = bos.toByteArray();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        //上传到oss
        String url = service.add(zipFile.getName(), buffer, "ds_upload");
        if (!StringUtils.startsWithIgnoreCase(url, "http")) {
            url = Contant.FILE_UPLOAD_DEFAULT_URL + url;
        }
        f.close();
        zipFile.delete();
        return new BaseResponse<>(url);
    }
}
