package com.starcharge.wios.export.impl;

import java.util.List;

import com.starcharge.wios.dao.entity.Region;
import com.starcharge.wios.service.RegionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.starcharge.wios.bo.ReportOrderBo;
import com.starcharge.wios.dto.ReportOrderDTO;
import com.starcharge.wios.export.Export;
import com.starcharge.wios.service.ReportService;
import org.springframework.util.StringUtils;

@Service
public class InstallDataTableExport implements Export<ReportOrderDTO> {
    
    @Autowired
    private ReportService reportService;
    @Autowired
    private RegionService regionService;
    
    @Override
    public List<ReportOrderDTO> getExportList(String json) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        ReportOrderBo reportOrderBo = mapper.readValue(json, ReportOrderBo.class);
        if (!StringUtils.isEmpty(reportOrderBo.getBeginTime())) {
            reportOrderBo.setBeginTime(reportOrderBo.getBeginTime() + " 00:00:00");
        }
        if (!StringUtils.isEmpty(reportOrderBo.getEndTime())) {
            reportOrderBo.setEndTime(reportOrderBo.getEndTime() + " 23:59:59");
        }
        String regionName = "";
        if (reportOrderBo.getRegionId()!=null) {
            Region region = regionService.getById(reportOrderBo.getRegionId().intValue());
            reportOrderBo.setRegionIdTree(region.getIdTree());
            regionName = region.getName();
        }
        List<ReportOrderDTO> reportOrderDTOS = reportService.installDataTable(reportOrderBo);
        if (reportOrderBo.getRegionId()!=null) {
            final String finalRegionName = regionName;
            reportOrderDTOS.stream().forEach(item->{
                item.setRegionId(reportOrderBo.getRegionId());
                item.setRegionName(finalRegionName);
            });
        }
        return reportOrderDTOS;
    }
    
}
