package com.starcharge.wios.export.impl;

import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.starcharge.warehouse.dto.materials.MaterialsDTO;
import com.starcharge.warehouse.query.MaterialsQuery;
import com.starcharge.wios.export.Export;
import com.starcharge.wios.service.WarehouseService;

/**
 * 物料库存导出
 * 
 * @author xgl
 * @version [版本号, 2020年10月28日]
 */
@Service
public class MaterialInventoryExport implements Export<MaterialsDTO> {
    
    @Autowired
    private WarehouseService warehouseService;
    
    @Override
    public List<MaterialsDTO> getExportList(String json) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        MaterialsQuery materialsQuery = mapper.readValue(json, MaterialsQuery.class);
        
        List<MaterialsDTO> list = new ArrayList<>();
        int page = 1;
        int pagecount = 10;
        while (true) {
            materialsQuery.setPage(page);
            materialsQuery.setPagecount(pagecount);
            List<MaterialsDTO> list1 = warehouseService.qryStockMaterials(materialsQuery);
            if (list1 == null || list1.isEmpty()) {
                break;
            }
            list.addAll(list1);
            if (list1.size() < pagecount)
                break;
            page++;
        }
        list.stream().forEach(item -> {
            if (item.getTotalNumber() != null)
                item.setTotalNumber(item.getTotalNumber().setScale(0, RoundingMode.HALF_DOWN));
            if (item.getHasNumber() != null)
                item.setHasNumber(item.getHasNumber().setScale(0, RoundingMode.HALF_DOWN));
            if (item.getRemainingNumber() != null)
                item.setRemainingNumber(item.getRemainingNumber().setScale(0, RoundingMode.HALF_DOWN));
            if (item.getStayWithNumber() != null)
                item.setStayWithNumber(item.getStayWithNumber().setScale(0, RoundingMode.HALF_DOWN));
        });
        
        return list;
    }
    
}
