package com.starcharge.wios.export.impl;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.starcharge.warehouse.dto.materials.MaterialsDTO;
import com.starcharge.warehouse.query.MaterialsQuery;
import com.starcharge.wios.bo.OperateLogBo;
import com.starcharge.wios.bo.OrderBo;
import com.starcharge.wios.dao.DictionaryDao;
import com.starcharge.wios.dao.OperateLogDao;
import com.starcharge.wios.dao.RegionDao;
import com.starcharge.wios.dao.entity.Dictionary;
import com.starcharge.wios.dao.entity.OperateLog;
import com.starcharge.wios.dao.entity.Region;
import com.starcharge.wios.enums.OperateLogType;
import com.starcharge.wios.enums.OrderStatus;
import com.starcharge.wios.export.Export;
import com.starcharge.wios.service.OrderService;
import com.starcharge.wios.service.WarehouseService;
import com.starcharge.wios.vo.OrderVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 订单导出
 * 
 * @author xgl
 * @version [版本号, 2020年10月13日]
 */
@Service
public class OrderExport implements Export<Map<String, Object>> {
    
    @Autowired
    private OrderService orderService;
    
    @Autowired
    private RegionDao regionDao;
    
    @Autowired
    private OperateLogDao olDao;

    @Autowired
    private DictionaryDao dicDao;
    @Autowired
    private WarehouseService warehouseService;

    @SuppressWarnings("unchecked")
    @Override
    public List<Map<String, Object>> getExportList(String json) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        OrderBo condition = mapper.readValue(json, OrderBo.class);
        condition.setFromExport(1);
        List<OrderVo> orderList = orderService.getList(condition);
        List<Map<String, Object>> returnList = new ArrayList<Map<String, Object>>();
        int no = 1;
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat sdf2 = new SimpleDateFormat("yyyyMMdd");
        List<Region> regionList = regionDao.selectAll(null);
        Map<Integer, Region> regionMap = regionList.stream().collect(Collectors.toMap(Region::getId, a -> a));
        OperateLogBo olCheckFailCondition = new OperateLogBo();
        olCheckFailCondition.setType(OperateLogType.ORDER_STATUS.getType());
        olCheckFailCondition.setName(OrderStatus.SURVEY_RECTIFYING.getType()+"");
        List<OperateLog> surveryCheckFailList = olDao.selectByExample(olCheckFailCondition);
        OperateLogBo olCheckFailConditionAdd = new OperateLogBo();
        olCheckFailConditionAdd.setType(OperateLogType.ORDER_STATUS.getType());
        olCheckFailConditionAdd.setName(OrderStatus.INSTALL_RECTIFYING.getType()+"");
        List<OperateLog> installCheckFailList = olDao.selectByExample(olCheckFailConditionAdd);
        // stream流分组统计订单审核不通过次数
        Map<String, Long> surveryCheckFailMap = surveryCheckFailList.stream().collect(Collectors.groupingBy(OperateLog::getThirdPartyId, Collectors.counting()));
        Map<String, Long> installCheckFailMap = installCheckFailList.stream().collect(Collectors.groupingBy(OperateLog::getThirdPartyId, Collectors.counting()));
        OperateLogBo olCondition = new OperateLogBo();
        olCondition.setType(OperateLogType.ORDER_REMARK.getType());
        List<OperateLog> olList = olDao.selectByExample(olCondition);
        Map<String, List<OperateLog>> olMap = olList.stream().collect(Collectors.toMap(OperateLog::getThirdPartyId, p -> {
            List<OperateLog> ll = new ArrayList<>();
            ll.add(p);
            return ll;
        }, (List<OperateLog> value1, List<OperateLog> value2) -> {
            value1.addAll(value2);
            return value1;
        }));

        List<Dictionary> dicList = dicDao.selectValidByType("wallBox", null);
        Map<String, String> dicMap = dicList.stream().collect(Collectors.toMap(Dictionary::getValue, Dictionary::getName));

        //充电桩类型的物料字典
        MaterialsQuery query = new MaterialsQuery();
        query.setType(3);
        query.setPage(1);
        query.setPagecount(1000);
        List<MaterialsDTO> materialsDTOList = warehouseService.qryMaterials(query);
        Map<String, String> itemMap = materialsDTOList.stream().collect(Collectors.toMap(MaterialsDTO::getId, MaterialsDTO::getMaterialsName));

        for (OrderVo order : orderList) {
            Map<String, Object> map = mapper.readValue(mapper.writeValueAsString(order), Map.class);
            map.put("no", no++);
            if (order.getStatus() != null)
                map.put("statusStr", OrderStatus.getText(order.getStatus()));
            if (order.getCreateTime() != null) {
                map.put("createTimeStr", sdf.format(order.getCreateTime()));
                map.put("registerTimeStr", sdf.format(order.getCreateTime()));
            }
            if (order.getDispatchTime() != null)
                map.put("dispatchTimeStr", sdf.format(order.getDispatchTime()));
            if (order.getSurveyReserveTime() != null)
                map.put("surveyReserveTimeStr", sdf.format(order.getSurveyReserveTime()));
            if (order.getSurveyFinishTime() != null)
                map.put("surveyFinishTimeStr", sdf.format(order.getSurveyFinishTime()));
            if (order.getSurveySubmitExamineTime() != null)
                map.put("surveySubmitExamineTimeStr", sdf.format(order.getSurveySubmitExamineTime()));
            if (order.getSurveyApproveTime() != null)
                map.put("surveyApproveTimeStr", sdf.format(order.getSurveyApproveTime()));
            if (order.getInstallReserveTime() != null)
                map.put("installReserveTimeStr", sdf.format(order.getInstallReserveTime()));
            if (order.getInstallFinishTime() != null)
                map.put("installFinishTimeStr", sdf.format(order.getInstallFinishTime()));
            if (order.getInstallSubmitExamineTime() != null)
                map.put("installSubmitExamineTimeStr", sdf.format(order.getInstallSubmitExamineTime()));
            if (order.getInstallApproveTime() != null)
                map.put("installApproveTimeStr", sdf.format(order.getInstallApproveTime()));
            if (order.getReturnVisitTime() != null)
                map.put("returnVisitTimeStr", sdf.format(order.getReturnVisitTime()));
            if (order.getCloseTime() != null)
                map.put("closeTimeStr", sdf.format(order.getCloseTime()));
            if (!StringUtils.isEmpty(order.getWallboxModel())) {
                map.put("wallboxModelStr", dicMap.get(order.getWallboxModel()));
            }
            if (StringUtils.isEmpty(order.getInstallItemModel())) {
                order.setInstallItemModel(order.getWallboxItemId());
            }
            if (!StringUtils.isEmpty(order.getInstallItemModel())) {
                map.put("installItemModelStr", itemMap.get(order.getInstallItemModel()));
            }
            if (order.getSurveyBeforeSaleEnable() != null)
                map.put("surveyBeforeSaleEnableStr", order.getSurveyBeforeSaleEnable().intValue() == 0 ? "否" : "是");
            if (order.getExpandStatus() != null)
                map.put("expandStatusStr", order.getExpandStatus().intValue() == 1 ? "通过" : "不通过");
            if (order.getElectricExpand() != null)
                map.put("electricExpandStr", (order.getElectricExpand() == null || order.getElectricExpand().intValue() == 0) ? "否" : "是");
            if (order.getInstallPosition() != null)
                map.put("installPositionStr", order.getInstallPosition().intValue() == 1 ? "室内" : "室外");
            if (order.getNeedErectPile() != null)
                map.put("needErectPileStr", order.getNeedErectPile().intValue() == 0 ? "否" : "是");
            if (!StringUtils.isEmpty(order.getRegionId())) {
                // Region region = regionDao.selectById(Integer.valueOf(order.getRegionId()));
                // String[] arr = region.getIdTree().split("_");
                // List<Integer> ids = new ArrayList<>();
                // for (String rId : arr) {
                // ids.add(Integer.valueOf(rId));
                // }
                // List<Region> regionList = regionDao.selectAll(null, ids);
                // Map<Integer, String> regionMap = regionList.stream().collect(Collectors.toMap(Region::getId,
                // Region::getName));
                Integer rId = Integer.valueOf(order.getRegionId());
                Region region = regionMap.get(rId);
                if (region != null) {
                    String idTree = region.getIdTree();
                    String[] arr = idTree.split("_");
                    if (arr.length > 1)
                        map.put("area",
                            regionMap.get(Integer.valueOf(arr[1])) == null ? null : regionMap.get(Integer.valueOf(arr[1])).getName());
                    if (arr.length > 2)
                        map.put("province",
                            regionMap.get(Integer.valueOf(arr[2])) == null ? null : regionMap.get(Integer.valueOf(arr[2])).getName());
                    if (arr.length > 3)
                        map.put("city",
                            regionMap.get(Integer.valueOf(arr[3])) == null ? null : regionMap.get(Integer.valueOf(arr[3])).getName());
                }
            }
            
            // OperateLogBo olCondition = new OperateLogBo();
            // olCondition.setThirdPartyId(order.getId());
            // String types = OperateLogType.ORDER_STATUS.getType() + "," + OperateLogType.STAFF_CHANGE.getType() + ","
            // + OperateLogType.SUPPLIER_CHANGE.getType() + "," + OperateLogType.ORDER_REMARK.getType();
            // olCondition.setTypes(types);
            // List<OperateLog> olList = olDao.selectByExample(olCondition);
            // StringBuffer remark = new StringBuffer();
            // olList.stream().forEach(item -> {
            // remark.append(sdf2.format(item.getCreateTime()))
            // .append(item.getType().intValue() == OperateLogType.ORDER_REMARK.getType() ? "备注" : "").append("：")
            // .append(item.getValue());
            // remark.append(";");
            // });
            // map.put("remark", remark.toString());
            StringBuffer remark = new StringBuffer();
            List<OperateLog> logList = olMap.get(order.getId());
            if (logList != null && !logList.isEmpty()) {
                logList.stream().forEach(item -> {
                    remark.append(sdf2.format(item.getCreateTime()))
                        .append("：")
                        .append(item.getValue());
                    remark.append(";");
                });
                map.put("remark", remark.toString());

                String desc = logList.get(0).getDesc();
                if(StrUtil.isNotBlank(desc)){
                    JSONObject jsonObject = JSONUtil.parseObj(desc);
                    map.put("nextTime", jsonObject.getStr("nextTime"));
                    map.put("tag", jsonObject.getStr("tag"));
                }
            }
            // put订单审核不通过次数
            map.put("surveyCheckFailNum",surveryCheckFailMap.get(order.getId())==null? 0 : surveryCheckFailMap.get(order.getId()));
            map.put("installCheckFailNum",installCheckFailMap.get(order.getId())==null? 0 : installCheckFailMap.get(order.getId()));
            returnList.add(map);
        }
        return returnList;
    }
    
    public static void main(String[] args) {
    }
}
