package com.starcharge.wios.export.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.starcharge.warehouse.dto.single.SingleDTO;
import com.starcharge.warehouse.query.SingleQuery;
import com.starcharge.wios.enums.WarehouseSingleType;
import com.starcharge.wios.export.Export;
import com.starcharge.wios.service.WarehouseService;

/**
 * 单据导出
 * 
 * @author xgl
 * @version [版本号, 2020年10月28日]
 */
@Service
public class SingleExport implements Export<Map<String, Object>> {
    
    @Autowired
    private WarehouseService warehouseService;
    
    @SuppressWarnings("unchecked")
    @Override
    public List<Map<String, Object>> getExportList(String json) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        SingleQuery singleQuery = mapper.readValue(json, SingleQuery.class);
        
        List<SingleDTO> list = new ArrayList<>();
        int page = 1;
        int pagecount = 10;
        while (true) {
            singleQuery.setPage(page);
            singleQuery.setPagecount(pagecount);
            List<SingleDTO> list1 = warehouseService.qrySingal(singleQuery);
            if (list1 == null || list1.isEmpty()) {
                break;
            }
            list.addAll(list1);
            if (list1.size() < pagecount)
                break;
            page++;
        }
        
        List<Map<String, Object>> returnList = new ArrayList<Map<String, Object>>();
        for (SingleDTO dto : list) {
            Map<String, Object> map = mapper.readValue(mapper.writeValueAsString(dto), Map.class);
            if (dto.getType() != null)
                map.put("typeStr", WarehouseSingleType.getText(dto.getType().intValue()));
            returnList.add(map);
        }
        return returnList;
    }
    
}
