package com.starcharge.wios.export.impl;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.starcharge.wios.bo.SupplierContractBo;
import com.starcharge.wios.enums.ContractStatus;
import com.starcharge.wios.export.Export;
import com.starcharge.wios.service.SupplierContractService;
import com.starcharge.wios.vo.SupplierContractVo;

/**
 * 服务商合同导出
 * 
 * @author xgl
 * @version [版本号, 2020年10月23日]
 */
@Service
public class SupplierContractExport implements Export<Map<String, Object>> {
    
    @Autowired
    private SupplierContractService supplierContractService;
    
    @SuppressWarnings("unchecked")
    @Override
    public List<Map<String, Object>> getExportList(String json) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        SupplierContractBo condition = mapper.readValue(json, SupplierContractBo.class);
        List<SupplierContractVo> contractList = supplierContractService.getList(condition);
        List<Map<String, Object>> returnList = new ArrayList<Map<String, Object>>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        int no = 1;
        for (SupplierContractVo contract : contractList) {
            Map<String, Object> map = mapper.readValue(mapper.writeValueAsString(contract), Map.class);
            map.put("no", no++);
            if (contract.getStartTime() != null)
                map.put("startDateStr", sdf.format(contract.getStartTime()));
            if (contract.getEndTime() != null)
                map.put("endDateStr", sdf.format(contract.getEndTime()));
            if (contract.getSignDate() != null)
                map.put("signDateStr", sdf.format(contract.getSignDate()));
            if (contract.getStatus() != null)
                map.put("statusStr", ContractStatus.getText(contract.getStatus().intValue()));
            returnList.add(map);
        }
        return returnList;
    }
    
}
