package com.starcharge.wios.export.impl;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.wios.dao.SettlementDetailSupplierDao;
import com.starcharge.wios.dao.entity.SettlementDetailSupplier;
import com.starcharge.wios.export.Export;

/**
 * 服务商结算单详情导出
 * 
 * @author xgl
 * @version [版本号, 2020年10月23日]
 */
@Service
public class SupplierSettlementDetailExport implements Export<Map<String, Object>> {
    
    @Autowired
    private SettlementDetailSupplierDao settlementDao;
    
    @SuppressWarnings("unchecked")
    @Override
    public List<Map<String, Object>> getExportList(String json) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        Map map = mapper.readValue(json, Map.class);
        Object settlementId = map.get("settlementId");
        if (settlementId == null) {
            throw new ServiceException("缺少参数");
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<Map<String, Object>> returnList = new ArrayList<Map<String, Object>>();
        List<SettlementDetailSupplier> list = settlementDao.selectBySettlementId(settlementId.toString());
        for (SettlementDetailSupplier detail : list) {
            Map<String, Object> m = mapper.readValue(mapper.writeValueAsString(detail), Map.class);
            if (detail.getFinishTime() != null)
                m.put("finishTimeStr", sdf.format(detail.getFinishTime()));
            returnList.add(m);
        }
        return returnList;
    }
    
}
