package com.starcharge.wios.export.impl;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.starcharge.wios.bo.SupplierStaffBo;
import com.starcharge.wios.export.Export;
import com.starcharge.wios.service.SupplierStaffService;
import com.starcharge.wios.vo.SupplierStaffVo;

/**
 * 电工导出
 * 
 * @author xgl
 * @version [版本号, 2020年11月5日]
 */
@Service
public class SupplierStaffExport implements Export<Map<String, Object>> {
    
    @Autowired
    private SupplierStaffService supplierStaffService;
    
    @SuppressWarnings("unchecked")
    @Override
    public List<Map<String, Object>> getExportList(String json) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        SupplierStaffBo condition = mapper.readValue(json, SupplierStaffBo.class);
        List<SupplierStaffVo> staffList = supplierStaffService.getList(condition);
        List<Map<String, Object>> returnList = new ArrayList<Map<String, Object>>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        int no = 1;
        for (SupplierStaffVo staff : staffList) {
            Map<String, Object> map = mapper.readValue(mapper.writeValueAsString(staff), Map.class);
            map.put("no", no++);
            if (staff.getIsInsure() != null)
                map.put("insureStr", staff.getIsInsure().intValue() == 0 ? "否" : "是");
            if (staff.getCertExpire() != null)
                map.put("certExpireStr", sdf.format(staff.getCertExpire()));
            returnList.add(map);
        }
        return returnList;
    }
    
}
