package com.starcharge.wios.export.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.starcharge.wios.dto.WarehouseDTO2;
import com.starcharge.wios.dto.WarehouseQuery2;
import com.starcharge.wios.export.Export;
import com.starcharge.wios.service.WarehouseService;

/**
 * 仓库导出
 * 
 * @author xgl
 * @version [版本号, 2020年10月28日]
 */
@Service("WareHouseExport")
public class WarehouseExport implements Export<Map<String, Object>> {
    
    @Autowired
    private WarehouseService warehouseService;
    
    @SuppressWarnings("unchecked")
    @Override
    public List<Map<String, Object>> getExportList(String json) throws Exception {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        WarehouseQuery2 warehouseQuery = mapper.readValue(json, WarehouseQuery2.class);
        List<WarehouseDTO2> list = new ArrayList<>();
        int page = 1;
        int pagecount = 10;
        while (true) {
            warehouseQuery.setPage(page);
            warehouseQuery.setPagecount(pagecount);
            List<WarehouseDTO2> list1 = warehouseService.qryWarehouseDetailList(warehouseQuery);
            if (list1 == null || list1.isEmpty()) {
                break;
            }
            list.addAll(list1);
            if (list1.size() < pagecount)
                break;
            page++;
        }
        List<Map<String, Object>> returnList = new ArrayList<Map<String, Object>>();
        for (WarehouseDTO2 dto : list) {
            Map<String, Object> map = mapper.readValue(mapper.writeValueAsString(dto), Map.class);
            if (!StringUtils.isEmpty(dto.getWarehouseAttribution()))
                map.put("attributionName", dto.getWarehouseAttribution().equals("1") ? "CAMS仓库" : "安装服务商仓库");
            returnList.add(map);
        }
        return returnList;
    }
    
}
