package com.starcharge.wios.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.starcharge.wios.dao.UploadFileDao;
import com.starcharge.wios.enums.IOperator;
import com.starcharge.wios.utils.ClassUtil;

/**
 * 通用service
 * 
 * @author xgl
 * @version [版本号, 2020年9月17日]
 */
@Service
public class CommonService {
    
    // 枚举类map
    public static Map<String, IOperator[]> enumMap = new HashMap<String, IOperator[]>();
    
    @Autowired
    private UploadFileDao uploadFileDao;
    
    /**
     * 初始化方法
     *
     */
    @PostConstruct
    public void init() {
        // 获取包下所有枚举类
        List<Class<?>> enumClzList = ClassUtil.getClasssFromPackage("com.starcharge.wios.enums");
        for (Class<?> clz : enumClzList) {
            if (clz.isEnum()) {
                String clzName = clz.getName();
                String[] arr = clzName.split("\\.");
                String name = arr[arr.length - 1];
                enumMap.put(name, (IOperator[])clz.getEnumConstants());
            }
        }
    }
    
    /**
     * 获取枚举map
     *
     * @param enumName
     * @return
     */
    public Map<String, Map<Integer, String>> getEnums(String enumNames) {
        Map<String, Map<Integer, String>> returnMap = new HashMap<>();
        String[] arr = enumNames.split(",");
        for (String enumName : arr) {
            Map<Integer, String> map = new HashMap<>();
            IOperator[] iOperators = enumMap.get(enumName);
            if (iOperators == null)
                continue;
            for (IOperator io : iOperators) {
                map.put(io.getType(), io.getValue());
            }
            returnMap.put(enumName, map);
        }
        return returnMap;
    }

    public Map<String, List<Map<String, Object>>> getEnumsArr(String enumNames) {
        Map<String, List<Map<String, Object>>> returnMap = new HashMap<>();
        String[] arr = enumNames.split(",");
        for (String enumName : arr) {
            List<Map<String, Object>> enumList=new ArrayList<>();
            IOperator[] iOperators = enumMap.get(enumName);
            if (iOperators == null){
                continue;
            }
            for (IOperator io : iOperators) {
                Map<String, Object> map = new HashMap<>();
                map.put("id", io.getType());
                map.put("name", io.getValue());
                enumList.add(map);
            }
            returnMap.put(enumName, enumList);
        }
        return returnMap;
    }
    
    /**
     * 删除
     *
     * @param id
     */
    public void deleteUploadFile(int id) {
        uploadFileDao.delete(id);
    }
}
