package com.starcharge.wios.service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.wios.dao.DepartmentDao;
import com.starcharge.wios.dao.RegionDao;
import com.starcharge.wios.dao.entity.Department;
import com.starcharge.wios.dao.entity.Region;
import com.starcharge.wios.vo.DepartmentVo;

/**
 * 部门service
 * 
 * @author xgl
 * @version [版本号, 2020年9月24日]
 */
@Service
public class DepartmentService {
    @Autowired
    private DepartmentDao departmentDao;
    
    @Autowired
    private RegionDao regionDao;
    
    public void add(Department department) {
        departmentDao.insert(department);
    }
    
    public void update(int id, Department department) {
        if (department.getStatus() != null && department.getStatus().intValue() == 2) {
            List<Department> childDeptList = departmentDao.selectByParentId(id);
            if (!childDeptList.isEmpty()) {
                throw new ServiceException("有下级部门，不可删除");
            }
        }
        departmentDao.update(id, department);
    }
    
    public List<DepartmentVo> getList(Integer status) {
        List<DepartmentVo> list = departmentDao.getList(status);
        
        for (DepartmentVo item : list) {
            String[] idTreeArr = item.getIdTree().split("_");
            item.setIdLevelCount(idTreeArr.length);
        }
        // 排序
        list = list.stream().sorted(Comparator.comparing(DepartmentVo::getIdLevelCount)).collect(Collectors.toList());
        
        List<DepartmentVo> returnList = new ArrayList<DepartmentVo>();
        for (Department department : list) {
            Integer parentId = department.getParentId();
            if (parentId != null) {
                DepartmentVo departmentVo = getDepartmentVo(returnList, parentId);
                if (departmentVo == null)
                    continue;
                DepartmentVo departmentVoNew = new DepartmentVo();
                BeanUtils.copyProperties(department, departmentVoNew);
                List<DepartmentVo> children = departmentVo.getChildren();
                if (children == null)
                    children = new ArrayList<>();
                children.add(departmentVoNew);
                departmentVo.setChildren(children);
            } else {
                DepartmentVo departmentVo = new DepartmentVo();
                BeanUtils.copyProperties(department, departmentVo);
                returnList.add(departmentVo);
            }
        }
        return returnList;
    }
    
    public DepartmentVo getDepartmentVo(List<DepartmentVo> returnList, int id) {
        for (DepartmentVo departmentVo : returnList) {
            if (departmentVo.getId().intValue() == id) {
                return departmentVo;
            }
            List<DepartmentVo> children = departmentVo.getChildren();
            if (children != null) {
                DepartmentVo departmentVo2 = getDepartmentVo(children, id);
                if (departmentVo2 != null)
                    return departmentVo2;
            }
        }
        return null;
    }
    
    public DepartmentVo getById(Integer id) {
        Department dept = departmentDao.selectById(id);
        DepartmentVo returnValue = new DepartmentVo();
        BeanUtils.copyProperties(dept, returnValue);
        if (dept != null) {
            if (dept.getRegionId() != null) {
                Region region = regionDao.selectById(dept.getRegionId());
                returnValue.setRegion(region);
            }
        }
        return returnValue;
    }
}
