package com.starcharge.wios.service;

import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import cn.hutool.core.date.DateUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.ihidea.core.util.BeanUtilsEx;
import com.ihidea.core.util.JSONUtilsEx;
import com.ihidea.core.util.StringUtilsEx;
import com.starcharge.base.redis.RedisClient;
import com.starcharge.base.util.Assert;
import com.starcharge.wios.call.service.PushService;
import com.starcharge.wios.constant.Constant;
import com.starcharge.wios.dao.InfoConfigDao;
import com.starcharge.wios.dao.OrderInstallDao;
import com.starcharge.wios.dao.UserPayBillDao;
import com.starcharge.wios.dao.entity.InfoConfig;
import com.starcharge.wios.dao.entity.OrderInstall;
import com.starcharge.wios.dao.entity.UserPayBill;
import com.starcharge.wios.enums.InfoConfigTypes;
import com.starcharge.wios.vo.InvoiceVo;
import com.starcharge.wios.wx.vo.WxInvoiceVO;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.wios.bo.InvoiceQueryBo;
import com.starcharge.wios.call.InvoiceApi;
import com.starcharge.wios.dao.OrderInstallItemDao;
import com.starcharge.wios.dto.InvoiceDTO;
import com.starcharge.wios.dto.InvoiceDetailDTO;
import com.starcharge.wios.dto.InvoiceDetailDTO.AdditionFee;
import com.starcharge.wios.dto.InvoiceOrderDTO;
import com.starcharge.wios.vo.OrderInstallItemVo;

/**
 * 发票service
 *
 * @author xgl
 * @version [版本号, 2020年10月13日]
 */
@Service
public class InvoiceService {
    @Autowired
    private InvoiceApi invoiceApi;

    @Autowired
    private OrderInstallDao orderInstallDao;

    @Autowired
    private OrderInstallItemDao orderInstallItemDao;

    @Autowired
    private UserPayBillService userPayBillService;
    @Autowired
    private UserPayBillDao userPayBillDao;

    @Value("${appkey.invoice}")
    private String invoiceAppkey;

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private InfoConfigDao infoConfigDao;

    @Autowired
    private PushService pushService;
    @Autowired
    private OrderInvoiceService orderInvoiceService;

    public List<InvoiceDTO> getList(InvoiceQueryBo condition) {
        Map<String, String> paramMap = new HashMap<>();
        if (condition.getPage() != null)
            paramMap.put("page", condition.getPage().toString());
        if (condition.getPagecount() != null)
            paramMap.put("pagecount", condition.getPagecount().toString());
        paramMap.put("invoiceTitle", condition.getInvoiceTitle());
        paramMap.put("applyAccount", condition.getApplyAccount());
        if (condition.getBusinessType() != null)
            paramMap.put("businessType", condition.getBusinessType().toString());
        if (condition.getInvoiceStatus() != null)
            paramMap.put("status", condition.getInvoiceStatus().toString());
        if (condition.getInvoiceType() != null)
            paramMap.put("invoiceType", condition.getInvoiceType().toString());
        paramMap.put("type", "8");
        paramMap.put("scope", "3");// 出票范围,1星充出票，2第三方出票，3全部
        Object obj = invoiceApi.gatewayAgent("merchant/total/list", paramMap);
        if (obj != null) {
            try {
                ObjectMapper mapper = new ObjectMapper();
                mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
                List<InvoiceDTO> list = mapper.readValue(mapper.writeValueAsString(obj), new TypeReference<List<InvoiceDTO>>() {
                });
                return list;

            } catch (Exception e) {
                throw new ServiceException("数据转换异常");
            }
        }
        return null;
    }

    public InvoiceDetailDTO getDetail(String invoiceId) {
        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("invoiceId", invoiceId);
        Object obj = invoiceApi.gatewayAgent("merchant/invoice/detail", paramMap);
        if (obj != null) {
            ObjectMapper mapper = new ObjectMapper();
            mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
            try {
                InvoiceDetailDTO dto = mapper.readValue(mapper.writeValueAsBytes(obj), InvoiceDetailDTO.class);
                List<InvoiceOrderDTO> orderList = getOrdersByInvoiceId(invoiceId);
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                List<AdditionFee> feeList = new ArrayList<AdditionFee>();
                if (orderList != null) {
                    List<String> billIdList = orderList.stream().map(InvoiceOrderDTO::getOrderId).collect(Collectors.toList());
                    for (String billId : billIdList) {
                        // billId = "CA20201221134507";// 测试用的
                        UserPayBill payBill = userPayBillDao.getById(billId);
                        if(payBill != null){
                            List<OrderInstallItemVo> oiiList = orderInstallItemDao.selectByOrderIdAndType(payBill.getRefId(), 2);
                            for (OrderInstallItemVo oii : oiiList) {
                                if (oii.getQunatity() != null && oii.getContractItemId() != null) {
                                    AdditionFee fee = dto.new AdditionFee();
                                    fee.setCreateTime(sdf.format(oii.getCreateTime()));
                                    fee.setFee(oii.getFee());
                                    fee.setFeeWithTax(oii.getRatePrice().multiply(oii.getQunatity()).setScale(2, RoundingMode.HALF_UP));
                                    fee.setOrderId(payBill.getRefId());
                                    fee.setName(oii.getItemName());
                                    fee.setTax(fee.getFeeWithTax().subtract(fee.getFee()));
                                    fee.setTaxRate(oii.getTaxRate());
                                    feeList.add(fee);
                                }
                            }
                        }

                    }
                }
                dto.setAdditionFeeList(feeList);
                return dto;
            } catch (Exception e) {
                throw new ServiceException("数据转换异常");
            }
        }
        return null;
    }

    /**
     * 查询发票对应的订单列表
     *
     * @param invoiceId
     * @return
     */
    public List<InvoiceOrderDTO> getOrdersByInvoiceId(String invoiceId) {
        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("invoiceId", invoiceId);
        Object obj = invoiceApi.gatewayAgent("merchant/invoice/order/list", paramMap);
        if (obj != null) {
            try {
                ObjectMapper mapper = new ObjectMapper();
                mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
                List<InvoiceOrderDTO> list = mapper.readValue(mapper.writeValueAsString(obj), new TypeReference<List<InvoiceOrderDTO>>() {
                });
                return list;

            } catch (Exception e) {
                throw new ServiceException("数据转换异常");
            }
        }
        return null;
    }

    /**
     * 判断工单账单是否已申请发票 0-未申请 1-已申请 2-未知（异常）
     * @param billId
     * @return
     */
    public Integer hasInvoiceByBillId(String billId){
        try{
            Map<String, String> paramMap=new HashMap<>();
            paramMap.put("invoiceAppkey", invoiceAppkey);
            paramMap.put("orderid", billId);
            paramMap.put("ordertype", "6");

            Integer result=(Integer)invoiceApi.gatewayAgent("/invoice/checkInvoice", paramMap);

            return result > 0 ? 1 : 0;
        }catch (Exception e){
            return 2;
        }
    }

    /**
     * 根据工单账单编号查询发票详情
     * @param billId
     * @return
     * @throws Exception
     */
    public InvoiceVo getInvoiceByBillId(String billId){
        Map<String, String> paramMap=new HashMap<>();
        paramMap.put("invoiceAppkey", invoiceAppkey);
        paramMap.put("orderId", billId);

        Object result=invoiceApi.gatewayAgent("/invoice/wios/info", paramMap);

        return result!=null ? JSONUtilsEx.deserialize(JSONUtilsEx.serialize(result), InvoiceVo.class) : null;
    }

    /**
     * 发票申请
     * @param wxInvoiceVO
     * @throws Exception
     */
    public void applyInvoice(WxInvoiceVO wxInvoiceVO) throws Exception{
        Assert.notBlank(wxInvoiceVO.getBillId(), "订单编号不能为空");
        wxInvoiceVO.setInvoiceTitle(wxInvoiceVO.getInvoiceTitle().trim().replaceAll("\n", "").replaceAll("\r", ""));
        wxInvoiceVO.setPayerRegisterNo(wxInvoiceVO.getPayerRegisterNo().trim().replaceAll("\n", "").replaceAll("\r", ""));
        InvoiceVo invoiceVo = BeanUtilsEx.copyPropertiesUnConvert(wxInvoiceVO, InvoiceVo.class);
        invoiceVo.setOutInvoiceId(wxInvoiceVO.getBillId());
        invoiceVo.setOrderIds(wxInvoiceVO.getBillId());

        String lockId="applyInvoice:"+invoiceVo.getOutInvoiceId();
        String requestId= StringUtilsEx.getUUID();
        if(redisClient.lock(lockId,requestId, 30)){
            try{
                UserPayBill bill = userPayBillService.getById(wxInvoiceVO.getBillId());
                Assert.notNull(bill, "安装订单不存在");
                Assert.isTrue(bill.getStatus() == 1, "安装订单未支付");

                OrderInstall orderInstall= orderInstallDao.selectByOrderId(bill.getRefId());
                Assert.notNull(orderInstall, "安装工单不存在");
                Assert.isTrue(orderInstall.getPayStatus() == 1, "安装工单未支付");

//                if(DateUtil.isSameDay(bill.getPayTime(), new Date())){
//                    throw new ServiceException("支付后当日不能申请发票，请次日申请");
//                }

                invoiceVo.setUserId(bill.getUserPhone());
                //根据实际收费金额开票
                invoiceVo.setInvoiceAmount(bill.getAmount());
                invoiceVo.setInvoiceAppkey(invoiceAppkey);
                invoiceVo.setExpressPayer(0);
                invoiceVo.setType(8);
                invoiceVo.setRemark("安装订单号:" + bill.getRefId() + ";" + invoiceVo.getRemark());

                Map<String, String> paramMap=BeanUtils.describe(invoiceVo);
                try{
                    invoiceApi.gatewayAgent("/invoice/apply", paramMap);
                    //纸质发票开票申请发送至指定邮箱
                    if (invoiceVo.getInvoiceMaterial()== 1) {
                        List<InfoConfig> mailList = infoConfigDao.selectValidNameType(InfoConfigTypes.INVOICE_MAIL.getType(), null);
                        mailList.stream().forEach(item-> {
                            if (StringUtils.isNotEmpty(item.getName())) {
                                pushService.sendEmailAsync(item.getName(), "纸质发票开票申请通知",
                                        "<p>您好：</p>" + "<p>安装订单：" + orderInstall.getOrderId() + "已申请开具纸质发票，请及时查看</p><p>本邮件是系统自动发送，无需直接回复</p>");
                            }

                        });
                    }else{
                        //更新开票状态
                        orderInvoiceService.updateAdditionalByBillId(wxInvoiceVO.getBillId());
                    }
                }catch (ServiceException e){
                    throw e;
                }
            }finally {
                redisClient.releaseLock(lockId, requestId);
            }
        } else {
            throw new ServiceException("系统忙，请稍后再试");
        }
    }

    /**
     * 纸票出票申请
     * @param outInvoiceIds
     */
    public void print(String outInvoiceIds) {
        Map<String, String> paramMap=new HashMap<>();
        paramMap.put("invoiceAppkey", invoiceAppkey);
        paramMap.put("outInvoiceIds", outInvoiceIds);
        invoiceApi.gatewayAgent("/invoice/print", paramMap);

        //更新开票状态
        orderInvoiceService.updateAdditionalByBillId(outInvoiceIds);
    }

    /**
     * 纸票发货
     * @param invoiceId
     * @param logisticsComp
     * @param logisticsNo
     */
    public void setLogistics(String invoiceId,String logisticsComp,String logisticsNo) {
        Map<String, String> paramMap=new HashMap<>();
        paramMap.put("invoiceAppkey", invoiceAppkey);
        paramMap.put("invoiceId", invoiceId);
        paramMap.put("logisticsComp", logisticsComp);
        paramMap.put("logisticsNo", logisticsNo);
        paramMap.put("logisticsAmount", "0");
        invoiceApi.gatewayAgent("/invoice/setLogistics", paramMap);
    }
}
