package com.starcharge.wios.service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.pageLimit.PageLimit;
import com.ihidea.core.support.pageLimit.PageLimitHolderFilter;
import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.warehouse.dto.materials.MaterialsDTO;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.bo.OemContractBo;
import com.starcharge.wios.bo.OemContractBo2;
import com.starcharge.wios.contants.Contant;
import com.starcharge.wios.dao.OemContractDao;
import com.starcharge.wios.dao.OemContractItemDao;
import com.starcharge.wios.dao.OemContractPriceDao;
import com.starcharge.wios.dao.UploadFileDao;
import com.starcharge.wios.dao.entity.OemContract;
import com.starcharge.wios.dao.entity.OemContractItem;
import com.starcharge.wios.dao.entity.OemContractPrice;
import com.starcharge.wios.dao.entity.UploadFile;
import com.starcharge.wios.enums.ContractStatus;
import com.starcharge.wios.enums.UploadFileType;
import com.starcharge.wios.vo.OemContractItemVo;
import com.starcharge.wios.vo.OemContractPriceVo;
import com.starcharge.wios.vo.OemContractVo;

/**
 * 主机厂合同service
 *
 * @author xgl
 * @version [版本号, 2020年9月18日]
 */
@Service
public class OemContractService {

    @Autowired
    private OemContractDao oemContractDao;

    @Autowired
    private OemContractItemDao oemContractItemDao;

    @Autowired
    private OemContractPriceDao oemContractPriceDao;

    @Autowired
    private UploadFileDao uploadFileDao;

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private TokenService tokenService;

    /**
     * 添加数据
     *
     * @param contractBo
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void add(OemContractBo contractBo) {
        List<OemContract> contractListByCode = oemContractDao.selectByCode(contractBo.getCode());
        if (!contractListByCode.isEmpty())
            throw new ServiceException("合同编号已存在");
        // 获取已添加的有效的合同列表
        List<OemContractVo> validContractList = oemContractDao.selectValid(Integer.valueOf(contractBo.getOemId()));
        for (OemContractVo contract : validContractList) {
            if (contractBo.getOperateType() == null || contractBo.getOperateType().intValue() == 0) {
                OemContract updateContract = new OemContract();
                if (contract.getEndTime().compareTo(contractBo.getEndTime()) <= 0) {
                    if (contract.getStartTime().compareTo(contractBo.getStartTime()) >= 0) {
                        // 时间包含则设为失效
                        updateContract.setStatus(ContractStatus.INVALID.getType());
                    } else {
                        if (contract.getEndTime().compareTo(contractBo.getStartTime()) >= 0) {
                            // 时间有交集则更新该合同的结束时间
                            Calendar cal = Calendar.getInstance();
                            cal.setTime(contractBo.getStartTime());
                            cal.add(Calendar.DATE, -1);
                            updateContract.setEndTime(cal.getTime());
                            if (contractBo.getStartTime().compareTo(new Date()) <= 0)// 结束时间小于当前时间
                                updateContract.setStatus(ContractStatus.INVALID.getType());
                        } else {
                            continue;
                        }
                    }
                } else {
                    throw new ServiceException("结束日期不能小于已添加合同的结束日期");
                }
                oemContractDao.update(contract.getId(), updateContract);
            } else {
                if((contractBo.getStartTime().compareTo(contract.getStartTime())>=0
                        && contractBo.getStartTime().compareTo(contract.getEndTime())<=0)
                    || (contractBo.getEndTime().compareTo(contract.getStartTime())>=0
                        && contractBo.getEndTime().compareTo(contract.getEndTime())<=0)){
                    throw new ServiceException("与已有合同时间冲突");
                }
            }
        }
        oemContractDao.insert(contractBo);// 添加合同数据
        List<OemContractItem> items = contractBo.getItems();
        if (items != null && !items.isEmpty()) {
            items.stream().forEach(item -> {
                item.setContractId(contractBo.getId().toString());
                item.setId(null);
            });
            oemContractItemDao.batchInsert(items);// 添加item
        }

        List<OemContractPrice> prices = contractBo.getPrices();
        if (prices != null && !prices.isEmpty()) {
            prices.stream().forEach(price -> {
                price.setContractId(contractBo.getId());
                price.setId(null);
            });
            oemContractPriceDao.batchInsert(prices);// 添加价格
        }

        List<UploadFile> files = contractBo.getFiles();
        if (files != null && !files.isEmpty()) {
            files.stream().forEach(file -> {
                file.setId(null);
                file.setType(UploadFileType.OEM_CONTRACT.getType());
                file.setThirdPartyId(contractBo.getId().toString());
                if (!file.getUrl().startsWith("http"))
                    file.setUrl(Contant.FILE_UPLOAD_DEFAULT_URL + file.getUrl());
            });
            uploadFileDao.batchInsert(files);// 添加文件
        }
    }

    /**
     * 查询列表
     *
     * @param condition
     * @return
     */
    public List<OemContractVo> getList(OemContractBo2 condition) {
        SessionInfo user = tokenService.getUser();
        if (user != null && !StringUtils.isEmpty(user.getAttribute().get("oemId"))) {
            condition.setOemId(user.getAttribute().get("oemId"));
        }
        return oemContractDao.getList(condition);
    }

    /**
     * 更新合同
     *
     * @param contractBo
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void update(OemContractBo contractBo) {
        if (!StringUtils.isEmpty(contractBo.getCode())) {
            List<OemContract> contractListByCode = oemContractDao.selectByCode(contractBo.getCode());
            if (!contractListByCode.isEmpty()) {
                if (contractListByCode.get(0).getId().intValue() != contractBo.getId().intValue()) {
                    throw new ServiceException("合同编号已存在");
                }
            }
        }
        oemContractDao.update(contractBo.getId(), contractBo);
        oemContractItemDao.deleteByContractId(contractBo.getId());
        oemContractPriceDao.deleteByContractId(contractBo.getId());
        uploadFileDao.deleteByTypeAndTPId(UploadFileType.OEM_CONTRACT.getType(), contractBo.getId().toString());

        List<OemContractItem> items = contractBo.getItems();
        if (items != null && !items.isEmpty()) {
            items.stream().forEach(item -> item.setContractId(contractBo.getId().toString()));
            oemContractItemDao.batchInsert(items);// 添加item
        }

        List<OemContractPrice> prices = contractBo.getPrices();
        if (prices != null && !prices.isEmpty()) {
            prices.stream().forEach(price -> price.setContractId(contractBo.getId()));
            oemContractPriceDao.batchInsert(prices);// 添加价格
        }

        List<UploadFile> files = contractBo.getFiles();
        if (files != null && !files.isEmpty()) {
            files.stream().forEach(file -> {
                file.setId(null);
                file.setType(UploadFileType.OEM_CONTRACT.getType());
                file.setThirdPartyId(contractBo.getId().toString());
                if (!file.getUrl().startsWith("http"))
                    file.setUrl(Contant.FILE_UPLOAD_DEFAULT_URL + file.getUrl());
            });
            uploadFileDao.deleteByTypeAndTPId(UploadFileType.OEM_CONTRACT.getType(), contractBo.getId().toString());
            uploadFileDao.batchInsert(files);// 添加文件
        }
    }

    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    public OemContractVo getDetail(int id) {
        OemContractVo contract = oemContractDao.selectDetailById(id);
        if (contract == null) {
            return null;
        }

        List<OemContractItemVo> contractItems = oemContractItemDao.selectByContractId(id);
        for (OemContractItemVo vo : contractItems) {
            MaterialsDTO dto = warehouseService.getMaterialById(vo.getItemId());
            if (dto != null) {
                vo.setItemCode(dto.getMaterialsCode());
                vo.setItemModel(dto.getRegularModel());
                vo.setItemName(dto.getMaterialsName());
            }
        }

        PageLimit pageLimit = PageLimitHolderFilter.getContext();
        if (pageLimit != null)
            pageLimit.setLimited(true);
        List<OemContractPriceVo> contractPrices = oemContractPriceDao.selectByContractId(id);
        List<UploadFile> files = uploadFileDao.selectValidByTypeAndTPId(UploadFileType.OEM_CONTRACT.getType(), id + "");

        contract.setItems(contractItems);
        contract.setPrices(contractPrices);
        contract.setFiles(files);
        return contract;
    }

    /**
     * 主机厂id查询合同详情
     *
     * @param oemId
     * @return
     */
    public OemContractVo getDetailByOemId(int oemId) {
        List<OemContractVo> list = oemContractDao.selectByStatusAndOemId(oemId, ContractStatus.EFFECTIVE.getType());
        if (list.isEmpty())
            return null;
        return getDetail(list.get(0).getId());
    }

    /**
     * 获取主机厂有效的增项列表
     * @param oemId
     * @param regionId
     * @return
     */
    public List<OemContractItemVo> getContractItems(Integer oemId, String regionId){
        OemContractVo contract=getDetailByOemId(oemId);
        return contract!=null ? contract.getItems() : null;
    }

    /**
     * 更新合同状态
     *
     * @throws ParseException
     *
     */
    public void contractTiming() throws ParseException {
        List<OemContractVo> contractList = oemContractDao.selectValid(null);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        for (OemContractVo contract : contractList) {
            Date startDate = contract.getStartTime();
            Date endDate = contract.getEndTime();
            OemContract updateContract = new OemContract();
            endDate = sdf1.parse(sdf.format(endDate) + " 23:59:59");
            if (endDate.compareTo(new Date()) < 0) {
                updateContract.setStatus(ContractStatus.INVALID.getType());
            } else {
                if (contract.getStatus() != null && contract.getStatus().intValue() == ContractStatus.NOT_START.getType()
                    && startDate.compareTo(new Date()) <= 0) {
                    updateContract.setStatus(ContractStatus.EFFECTIVE.getType());
                }
            }
            if (updateContract.getStatus() != null) {
                oemContractDao.update(contract.getId(), updateContract);
            }

        }
    }

}
