package com.starcharge.wios.service;

import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.StringUtilsEx;
import com.starcharge.base.redis.RedisClient;
import com.starcharge.base.util.Assert;
import com.starcharge.warehouse.dto.materials.MaterialsDTO;
import com.starcharge.wios.bo.OrderInstallBo;
import com.starcharge.wios.contants.Contant;
import com.starcharge.wios.dao.*;
import com.starcharge.wios.dao.entity.*;
import com.starcharge.wios.enums.UploadFileType;
import com.starcharge.wios.vo.OrderInstallItemVo;
import com.starcharge.wios.vo.OrderInstallVo;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;

/**
 * 安装订单service
 * 
 * @author xgl
 * @version [版本号, 2020年9月17日]
 */
@Service
public class OrderInstallService {
    
    @Autowired
    private OrderInstallDao orderInstallDao;
    
    @Autowired
    private UploadFileDao uploadFileDao;
    
    @Autowired
    private ExpandPriceDao expandPriceDao;
    
    @Autowired
    private OrderDao orderDao;
    
    @Autowired
    private OrderInstallItemDao orderInstallItemDao;
    
    @Autowired
    private UserPayBillService userPayBillService;
    
    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private OperateLogDao operateLogDao;

    @Autowired
    private RedisClient redisClient;
    
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void add(OrderInstallBo orderInstall) {
        String lockId = "addOrderInstall:"+orderInstall.getOrderId();
        String requestId = StringUtilsEx.getUUID();
        if(redisClient.lock(lockId, requestId, 60)){
            try{
                OrderInstall oi = orderInstallDao.selectByOrderId(orderInstall.getOrderId());
                if (oi != null)
                    throw new ServiceException("安装订单已存在");
                Order order = orderDao.selectById(orderInstall.getOrderId());
                // 2020-01-06注释
                // if (StringUtils.isEmpty(order.getWarehouseId())) {
                // throw new ServiceException("订单没有仓库");
                // }
                if (StringUtils.isEmpty(orderInstall.getWallboxItemId())) {
                    throw new ServiceException("缺少充电桩");
                }
                // 2020-01-06注释
                // List<MaterialsDTO> materialList = warehouseService.getStockMaterialById(order.getWarehouseId(),
                // orderInstall.getWallboxItemId());
                // if (materialList.isEmpty()) {
                // throw new ServiceException("仓库无此物料");
                // } else {
                // BigDecimal total = materialList.stream().map(MaterialsDTO::getRemainingNumber).reduce(BigDecimal.ZERO,
                // BigDecimal::add);
                // if (total.compareTo(BigDecimal.ZERO) == 0) {
                // throw new ServiceException("充电桩库存不足");
                // }
                // }

                // 锁定仓库充电桩(2020-01-04注释)
                // if (order != null && !StringUtils.isEmpty(orderInstall.getWallboxItemId()) &&
                // !StringUtils.isEmpty(order.getWarehouseId()))
                // warehouseService.materailNumberUpdate(orderInstall.getWallboxItemId(), 1, 1, order.getWarehouseId());

                List<OrderInstallItem> itemList = orderInstall.getItemFees();
                if (itemList != null && !itemList.isEmpty()) {
                    itemList.stream().forEach(item -> {
                        item.setItemType(2);// 增项
                        item.setType(2);// 安装订单
                        item.setOrderId(orderInstall.getOrderId());
                    });

                    orderInstallItemDao.batchInsert(itemList);
                    orderInstall.setPredictTotalFee(itemList.stream().map(OrderInstallItem::getFee).reduce(BigDecimal.ZERO, BigDecimal::add));
                }

                if (orderInstall.getElectricExpand() != null && orderInstall.getElectricExpand().intValue() == 1) {
                    if (order != null) {
                        List<ExpandPrice> priceList = expandPriceDao.selectByRegionId(Integer.valueOf(order.getRegionId()));
                        if (!priceList.isEmpty())
                            orderInstall.setExpandFee(priceList.get(0).getPrice());
                    }
                }
                if (orderInstall.getExpandFee() == null)
                    orderInstall.setExpandFee(BigDecimal.ZERO);

                // 存入注册cams phone，用于后面开具发票
                orderInstall.setThirdUserId(order.getSurveyCamsPhone());
                orderInstallDao.insert(orderInstall);// 插入数据库
                List<UploadFile> files = orderInstall.getFiles();
                if (files != null && !files.isEmpty()) {
                    files.stream().forEach(file -> {
                        file.setId(null);
                        file.setType(UploadFileType.INSTALL_ORDER.getType());
                        file.setThirdPartyId(orderInstall.getId().toString());
                        if (!StringUtils.startsWithIgnoreCase(file.getUrl(), "http")) {
                            file.setUrl(Contant.FILE_UPLOAD_DEFAULT_URL + file.getUrl());
                        }
                    });
                    uploadFileDao.batchInsert(files);// 添加文件
                }
                // 修改安装订单修改订单的墙盒型号和墙盒设备型号
                Order orderUpdate = new Order();
                orderUpdate.setInstallWallboxModel(orderInstall.getInstallWallboxModel());
                orderUpdate.setInstallItemModel(orderInstall.getWallboxItemId());
                orderDao.updateById(orderInstall.getOrderId(), orderUpdate);
            }finally {
                redisClient.releaseLock(lockId, requestId);
            }
        }else{
            throw new ServiceException("系统忙，请稍后");
        }
    }
    
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void update(int id, OrderInstallBo orderInstall) {
        OrderInstall oi = orderInstallDao.selectById(id);
        if (oi == null) {
            throw new ServiceException("无此安装单");
        }
        
        // 如果已经生成账单，1)不允许变更报装 2)不允许变更增项
        if (hasGenBill(oi.getOrderId())) {
            orderInstall.setElectricExpand(null);
            orderInstall.setExpandMethod(null);
            orderInstall.setItemFeesJson(null);
            orderInstall.setItemFees(null);
        }
        
        List<OrderInstallItem> itemList = orderInstall.getItemFees();
        if (CollectionUtils.isNotEmpty(itemList)) {
            itemList.stream().forEach(item -> {
                item.setItemType(2);// 增项
                item.setType(2);// 安装订单
                item.setOrderId(oi.getOrderId());
            });
            orderInstallItemDao.delete(oi.getOrderId(), 2);
            orderInstallItemDao.batchInsert(itemList);
            orderInstall.setPredictTotalFee(itemList.stream().map(OrderInstallItem::getFee).reduce(BigDecimal.ZERO, BigDecimal::add));
        }
        orderInstallDao.update(id, orderInstall);// 更新
        
        uploadFileDao.deleteByTypeAndTPId(UploadFileType.INSTALL_ORDER.getType(), orderInstall.getId().toString());
        List<UploadFile> files = orderInstall.getFiles();
        if (files != null && !files.isEmpty()) {
            files.stream().forEach(file -> {
                file.setId(null);
                file.setType(UploadFileType.INSTALL_ORDER.getType());
                file.setThirdPartyId(orderInstall.getId().toString());
                if (!StringUtils.startsWithIgnoreCase(file.getUrl(), "http")) {
                    file.setUrl(Contant.FILE_UPLOAD_DEFAULT_URL + file.getUrl());
                }
            });
            uploadFileDao.deleteByTypeAndTPId(UploadFileType.INSTALL_ORDER.getType(), orderInstall.getId().toString());
            uploadFileDao.batchInsert(files);// 添加文件
        }
        // 修改安装订单修改订单的墙盒型号和墙盒设备型号
        Order order = new Order();
        order.setInstallWallboxModel(orderInstall.getInstallWallboxModel());
        order.setInstallItemModel(orderInstall.getWallboxItemId());
        orderDao.updateById(oi.getOrderId(), order);
    }
    
    /**
     * 更新订单增项、报装
     * 
     * @param orderId
     * @param type 订单类型 1-勘察订单 2-安装订单 3-安装变更
     * @param itemList
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveInstallItem(String orderId, Integer type, List<OrderInstallItem> itemList, Integer electricExpand, Integer expandMethod, BigDecimal expandFee) {
        Assert.notBlank(orderId, "订单号不能为空");
        Assert.notNull(type, "订单类型不能为空");

        Order order = orderDao.selectById(orderId);
        Assert.notNull(order, "订单不存在");

        if (type == 2) {
            if (hasGenBill(orderId)) {
                throw new ServiceException("当前订单已生成账单，不支持修改增项和报装");
            }
        }
        
        orderInstallItemDao.delete(orderId, type);
        BigDecimal totalFee = BigDecimal.ZERO;
        if (CollectionUtils.isNotEmpty(itemList)) {
            for (OrderInstallItem item : itemList) {
                // 增项
                item.setItemType(2);
                item.setType(type);
                item.setOrderId(orderId);
                
                totalFee = totalFee.add(item.getFee());
            }
            orderInstallItemDao.batchInsert(itemList);
        }
        
        if (type == 2) {
            OrderInstall oi = orderInstallDao.selectByOrderId(orderId);
            // 如果安装报告还未创建，则先创建
            if (oi == null) {
                OrderInstall orderInstall = new OrderInstall();
                orderInstall.setOrderId(orderId);
                orderInstall.setThirdUserId(order.getSurveyCamsPhone());
                orderInstall.setPayStatus(0);
                orderInstall.setPredictTotalFee(totalFee);
                orderInstall.setElectricExpand(electricExpand);
                orderInstall.setExpandMethod(expandMethod);
                orderInstall.setExpandFee(expandFee);
                orderInstallDao.insert(orderInstall);
            }else{
                OrderInstall orderInstall = new OrderInstall();
                orderInstall.setId(oi.getId());
                orderInstall.setPredictTotalFee(totalFee);
                orderInstall.setElectricExpand(electricExpand);
                orderInstall.setExpandMethod(expandMethod);
                orderInstall.setExpandFee(expandFee);
                orderInstallDao.update(oi.getId(), orderInstall);
            }
        }
    }
    
    public OrderInstallVo getByOrderId(String orderId) {
        OrderInstall orderInstall = orderInstallDao.selectByOrderId(orderId);
        if (orderInstall == null)
            return null;
        OrderInstallVo result = new OrderInstallVo();
        String wallboxItemId = orderInstall.getWallboxItemId();
        if (!StringUtils.isEmpty(wallboxItemId)) {
            MaterialsDTO dto = warehouseService.getMaterialById(wallboxItemId);
            if (dto != null) {
                result.setWallboxItemName(dto.getMaterialsName());
                result.setWallboxItemModel(dto.getRegularModel());
            }
        }
        BeanUtils.copyProperties(orderInstall, result);
        Order order = orderDao.selectById(orderId);
        if (order == null)
            return null;
        result.setRegionId(Integer.valueOf(order.getRegionId()));
        
        List<UploadFile> files =
            uploadFileDao.selectValidByTypeAndTPId(UploadFileType.INSTALL_ORDER.getType(), orderInstall.getId().toString());
        result.setFiles(files);
        List<OrderInstallItemVo> itemFeeList = orderInstallItemDao.selectByOrderIdAndType(orderId, 2);
        result.setItemFees(itemFeeList);
        
        result.setHasGenBill(hasGenBill(orderId) ? 1 : 0);
        result.setInstallWallboxModel(order.getInstallWallboxModel());

        //加入上次审核不通过原因描述
        OperateLog operateLog=operateLogDao.getNewestCheckFailByOrderId(orderId,"55");
        if (Objects.nonNull(operateLog)) {
            result.setCheckFailDesc(operateLog.getDesc());
        }else{
            result.setCheckFailDesc("");
        }

        return result;
    }
    
    public void updateByOrderId(String orderId, OrderInstall orderInstall) {
        orderInstallDao.updateByOrderId(orderId, orderInstall);
    }
    
    public boolean hasGenBill(String orderId) {
        return userPayBillService.countByRefId(orderId) > 0;
    }
    
}
