package com.starcharge.wios.service;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.StrUtil;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.wios.dao.InvoiceDao;
import com.starcharge.wios.dao.OrderDao;
import com.starcharge.wios.dao.SupplierDao;
import com.starcharge.wios.dao.UserPayBillDao;
import com.starcharge.wios.dao.entity.*;
import com.starcharge.wios.enums.OrderStatus;
import com.starcharge.wios.vo.InvoiceVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * @author licheng
 * @Description OrderInvoiceService
 * @date 2021/7/24 13:41
 * @return
 */
@Service
public class OrderInvoiceService {
    private Logger log = LoggerFactory.getLogger(getClass());
    @Autowired
    private InvoiceDao invoiceDao;
    @Autowired
    private UserPayBillDao userPayBillDao;
    @Autowired
    private InvoiceService invoiceService;

    @Autowired
    private SapRemoteService sapRemoteService;

    @Autowired
    private OrderDao orderDao;

    @Autowired
    private SupplierDao supplierDao;

    /**
     * 批量更新
     *
     * @param supplierInvoiceJson
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void batchUpdate(String supplierInvoiceJson) {
        ObjectMapper mapper = new ObjectMapper();
        try {
            List<Invoice> list = mapper.readValue(supplierInvoiceJson, new TypeReference<List<Invoice>>() {
            });
            list.forEach(invoice -> {
                invoice.setModifyTime(new Date());
                //提交项（1-服务开票，2-设备开票，两个同时-3）
                Integer submititem=invoice.getSubmitItem();

                int i =0;
                if(submititem==null){
                    //不含税
                    BigDecimal supplierFee=invoice.getSupplierFeeWithTax();
                    invoice.setSupplierFeeWithTax(invoice.getSupplierTax().add(supplierFee));
                    i = invoiceDao.updateSupplier(invoice);
                    if (i == 0) {
                        invoice.setCreateTime(new Date());
                        i=invoiceDao.insertSelective(invoice);
                    }
                    //服务商收票后调用
                    if(i==1&&invoice.getSupplierReceiveInvoiceStatus()!=null&&invoice.getSupplierReceiveInvoiceStatus()==1){
                        Order order=orderDao.selectById(invoice.getOrderId());
                        if(order==null){
                            return;
                        }
                        //wios->sap订单服务商结算数据同步
                        ThreadUtil.execAsync(()->sapRemoteService.syncSettlementData(order,invoice));

                    }
                }else{
                    if(submititem==1){
                        i = invoiceDao.updateInstall(invoice);
                    }else if(submititem==2){
                        i = invoiceDao.updateDevice(invoice);
                    }else if(submititem==3){
                        i = invoiceDao.updateByPrimaryKeySelective(invoice);
                    }

                    if (i == 0) {
                        invoice.setCreateTime(new Date());
                        i=invoiceDao.insertSelective(invoice);
                    }
                    //套包内安装服务开票后调用订单推送
                    if((submititem==1||submititem==3)&&invoice.getPackageInstallInvoiceStatus()!=null&&invoice.getPackageInstallInvoiceStatus()==1){
                        invoice.setSubmitItem(1);
                        Order order=orderDao.selectById(invoice.getOrderId());
                        if(order==null){
                            return;
                        }
                        if(order.getStatus()==OrderStatus.INSTALL_EXAMINE_SUCCESS.getType()||order.getStatus()==OrderStatus.RETURN_VISIT.getType()){
                            //wios->sap订单推送
                            log.info(order.getId()+",安装服务开票开始同步SAP");
                            Invoice invoice1=new Invoice();
                            Order order1=new Order();
                            BeanUtils.copyProperties(invoice,invoice1);
                            BeanUtils.copyProperties(order,order1);
                            ThreadUtil.execAsync(() -> sapRemoteService.pushOrderOrInvoice(order1,invoice1));
                        }
                    }
                    if((submititem==2||submititem==3)&&invoice.getPackageDeviceInvoiceStatus()!=null&&invoice.getPackageDeviceInvoiceStatus()==1){
                        invoice.setSubmitItem(2);
                        Order order=orderDao.selectById(invoice.getOrderId());
                        if(order==null){
                            return;
                        }
                        if(order.getStatus()==OrderStatus.INSTALL_EXAMINE_SUCCESS.getType()||order.getStatus()==OrderStatus.RETURN_VISIT.getType()){
                            //wios->sap订单推送
                            log.info(order.getId()+",设备开票开始同步SAP");
                            Invoice invoice2=new Invoice();
                            Order order2=new Order();
                            BeanUtils.copyProperties(invoice,invoice2);
                            BeanUtils.copyProperties(order,order2);
                            ThreadUtil.execAsync(()->sapRemoteService.pushOrderOrInvoice(order2,invoice2));
                        }
                    }
                }
            });
        } catch (JsonProcessingException e) {
            log.error("json转换失败",e);
            throw new ServiceException("转换失败");
        }
    }


    public Invoice getByOrderId(String orderId) {
        return invoiceDao.selectByPrimaryKey(orderId);
    }

    /**
     * 更新增项开票状态
     */
    public void updateAdditionalByBillId(String billId) {
        if (StrUtil.isNotBlank(billId)) {
            UserPayBill payBill = userPayBillDao.getById(billId);
            if (payBill != null) {
                Invoice record = new Invoice();
                record.setOrderId(payBill.getRefId());
                record.setModifyTime(new Date());
                int i = invoiceDao.updateByPrimaryKeySelective(record);
                if (i == 0) {
                    record.setCreateTime(new Date());
                    invoiceDao.insertSelective(record);
                }
            }
        }
    }

    /**
     * 更新增项开票状态
     */
    public void invoiceUpdater() {
        InvoiceCriteria example = new InvoiceCriteria();
        example.createCriteria().andAdditionalCostInvoiceStatusIsNull();
        List<Invoice> invoices = invoiceDao.selectByExample(example);
        for (Invoice invoice : invoices) {
            UserPayBill entity = new UserPayBill();
            entity.setRefId(invoice.getOrderId());
            List<UserPayBill> list = userPayBillDao.list(entity);
            if (list.size() > 0) {
                UserPayBill payBill = list.get(0);
                try {
                    InvoiceVo invoiceVo = invoiceService.getInvoiceByBillId(payBill.getId());
                    if (invoiceVo != null && invoiceVo.getStatus() > 0) {
                        Invoice record = new Invoice();
                        record.setOrderId(payBill.getRefId());
                        record.setAdditionalCostInvoiceStatus(1);
                        record.setAdditionalCostInvoiceNumber(invoiceVo.getInvoiceNumber());
                        record.setAdditionalCostInvoiceDate(invoiceVo.getInvoiceTime());
                        record.setModifyTime(new Date());
                        invoiceDao.updateByPrimaryKeySelective(record);
                    }
                }catch (Exception e){
                }

            }
        }
    }
}
