package com.starcharge.wios.service;

import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.StringUtilsEx;
import com.starcharge.base.redis.RedisClient;
import com.starcharge.wios.bo.OrderSurveyBo;
import com.starcharge.wios.contants.Contant;
import com.starcharge.wios.dao.*;
import com.starcharge.wios.dao.entity.*;
import com.starcharge.wios.enums.UploadFileType;
import com.starcharge.wios.vo.OrderInstallItemVo;
import com.starcharge.wios.vo.OrderSurveyVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

/**
 * 勘察订单service
 * 
 * @author xgl
 * @version [版本号, 2020年9月17日]
 */
@Service
public class OrderSurveyService {
    @Autowired
    private OrderSurveyDao orderSurveyDao;
    
    @Autowired
    private UploadFileDao uploadFileDao;
    
    @Autowired
    private OrderInstallItemDao orderInstallItemDao;
    
    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderDao orderDao;

    @Autowired
    private OperateLogDao operateLogDao;

    @Autowired
    private RedisClient redisClient;

    /**
     * 新增
     *
     * @param orderSurvey
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void add(OrderSurveyBo orderSurvey) {
        String lockId = "addOrderSurvey:"+orderSurvey.getOrderId()+"-"+orderSurvey.getType();
        String requestId = StringUtilsEx.getUUID();
        if(redisClient.lock(lockId, requestId, 60)){
            try{
                OrderSurvey os = orderSurveyDao.selectByOrderId(orderSurvey.getOrderId(), orderSurvey.getType());
                if (os != null)
                    throw new ServiceException("订单已存在");
                List<OrderInstallItem> itemList = orderSurvey.getItemFees();
                if (itemList != null && !itemList.isEmpty()) {
                    itemList.stream().forEach(item -> {
                        item.setItemType(2);// 增项
                        if(orderSurvey.getType().intValue() == 1){
                            item.setType(1);// 勘察订单
                        }else{
                            item.setType(3);// 变更安装订单
                        }
                        item.setOrderId(orderSurvey.getOrderId());
                    });
                    orderInstallItemDao.batchInsert(itemList);
                }
                orderSurveyDao.insert(orderSurvey);

                List<UploadFile> files = orderSurvey.getFiles();
                if (files != null && !files.isEmpty()) {
                    files.stream().forEach(file -> {
                        file.setId(null);
                        file.setType(orderSurvey.getType().intValue() == 1 ? UploadFileType.SURVEY_ORDER.getType()
                            : UploadFileType.INSTALL_CHANGE_ORDER.getType());
                        file.setThirdPartyId(orderSurvey.getId().toString());
                        if(!StringUtils.startsWithIgnoreCase(file.getUrl(), "http")){
                            file.setUrl(Contant.FILE_UPLOAD_DEFAULT_URL + file.getUrl());
                        }
                    });
                    uploadFileDao.batchInsert(files);// 添加文件
                }

                if (orderSurvey.getType() != null && orderSurvey.getType().intValue() == 1) {
                    // 勘测订单修改订单的墙盒型号和墙盒设备型号
                    Order order = new Order();
                    order.setSurveyWallboxModel(orderSurvey.getSurveyWallboxModel());
                    order.setSurveyItemModel(orderSurvey.getSurveyItemModel());
                    orderDao.updateById(orderSurvey.getOrderId(), order);
                }

                if (orderSurvey.getType() != null && orderSurvey.getType().intValue() == 2) {// 安装变更
                    orderService.installChange(orderSurvey.getOrderId(), null, null);
                }
            }finally {
                redisClient.releaseLock(lockId, requestId);
            }
        }else{
            throw new ServiceException("系统忙，请稍后");
        }
        
    }
    
    /**
     * 修改
     *
     * @param id
     * @param orderSurvey
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void update(int id, OrderSurveyBo orderSurvey) {
        orderSurvey.setType(null);
        List<OrderInstallItem> itemList = orderSurvey.getItemFees();
        OrderSurvey os = orderSurveyDao.selectById(id);
        if (os == null)
            throw new ServiceException("无此勘察单");
        if (itemList!=null && !itemList.isEmpty()) {
            itemList.stream().forEach(item -> {
                item.setItemType(2);// 增项
                if (os.getType().intValue() == 1) {
                    item.setType(1);// 勘察订单
                } else {
                    item.setType(3);// 变更安装订单
                }
                item.setOrderId(os.getOrderId());
            });
            orderInstallItemDao.delete(os.getOrderId(), os.getType().intValue() == 1 ? 1 : 3);
            orderInstallItemDao.batchInsert(itemList);
        }
        orderSurveyDao.update(id, orderSurvey);
        
        uploadFileDao.deleteByTypeAndTPId(
            os.getType().intValue() == 1 ? UploadFileType.SURVEY_ORDER.getType() : UploadFileType.INSTALL_CHANGE_ORDER.getType(), id + "");
        List<UploadFile> files = orderSurvey.getFiles();
        if (files != null && !files.isEmpty()) {
            files.stream().forEach(file -> {
                file.setId(null);
                file.setType(os.getType().intValue() == 1 ? UploadFileType.SURVEY_ORDER.getType()
                    : UploadFileType.INSTALL_CHANGE_ORDER.getType());
                file.setThirdPartyId(id + "");
                if(!StringUtils.startsWithIgnoreCase(file.getUrl(), "http")){
                    file.setUrl(Contant.FILE_UPLOAD_DEFAULT_URL + file.getUrl());
                }
            });
            uploadFileDao.batchInsert(files);// 添加文件
        }

        if (os.getType() != null && os.getType().intValue() == 1) {
            // 勘测订单修改订单的墙盒型号和墙盒设备型号
            Order order = new Order();
            order.setSurveyWallboxModel(orderSurvey.getSurveyWallboxModel());
            order.setSurveyItemModel(orderSurvey.getSurveyItemModel());
            orderDao.updateById(os.getOrderId(), order);
        }
        
        if (os.getType() != null && os.getType().intValue() == 2) {// 安装变更
            orderService.installChange(os.getOrderId(), null, null);
        }
    }
    
    /**
     * 根据订单id查询
     *
     * @param orderId
     * @return
     */
    public OrderSurveyVo getByOrderId(String orderId, Integer type) {
        OrderSurvey orderSurvey = orderSurveyDao.selectByOrderId(orderId, type);
        if (orderSurvey == null)
            return null;
        OrderSurveyVo result = new OrderSurveyVo();
        BeanUtils.copyProperties(orderSurvey, result);
        
        List<UploadFile> files = uploadFileDao.selectValidByTypeAndTPId(
            type.intValue() == 1 ? UploadFileType.SURVEY_ORDER.getType() : UploadFileType.INSTALL_CHANGE_ORDER.getType(),
            orderSurvey.getId().toString());
        result.setFiles(files);
        List<OrderInstallItemVo> itemFeeList = orderInstallItemDao.selectByOrderIdAndType(orderId, type.intValue() == 1 ? 1 : 3);
        result.setItemFees(itemFeeList);
        //加入上次审核不通过原因描述
        OperateLog operateLog=operateLogDao.getNewestCheckFailByOrderId(orderId,"28");
        if (Objects.nonNull(operateLog)) {
            result.setCheckFailDesc(operateLog.getDesc());
        }else{
            result.setCheckFailDesc("");
        }
        Order order = orderDao.selectById(orderId);
        result.setSurveyWallboxModel(order.getSurveyWallboxModel());
        result.setSurveyItemModel(order.getSurveyItemModel());
        return result;
    }
}
