package com.starcharge.wios.service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.starcharge.wios.dao.entity.DispatchConfig;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.dao.RegionDao;
import com.starcharge.wios.dao.entity.Region;
import com.starcharge.wios.dto.RegionDTO;
import com.starcharge.wios.vo.RegionVo;

/**
 * 区域管理service
 * 
 * @author xgl
 * @version [版本号, 2020年9月21日]
 */
@Service
public class RegionService {
    @Autowired
    private RegionDao regionDao;
    
    @Autowired
    private TokenService tokenService;
    
    public void add(Region region) {
        regionDao.insert(region);
    }
    
    public Region getById(int id) {
        return regionDao.selectById(id);
    }
    
    public List<Region> getByParentId(int parentId) {
        return regionDao.selectByParentId(parentId);
    }
    
    /**
     * 查询列表并组装数据
     *
     * @param status
     * @return
     */
    public List<RegionVo> selectAll(Integer status) {
        return selectAll(status, new ArrayList<>());
    }
    
    /**
     * 查询列表并组装数据
     *
     * @param status
     * @return
     */
    public List<RegionVo> selectAll(Integer status, List<Integer> selectedIds) {
        SessionInfo sessionInfo = tokenService.getUser();
        List<Integer> regionIdList = null;
        if (sessionInfo != null) {
            String regionIds = sessionInfo.getAttribute().get("regionIds");
            if (!StringUtils.isEmpty(regionIds)) {
                regionIdList = new ArrayList<>();
                String[] arr = regionIds.split(",");
                for (String rId : arr) {
                    regionIdList.add(Integer.valueOf(rId));
                }
            }
        }
        List<Region> list = regionDao.selectAll(status, regionIdList);
        
        List<RegionDTO> dtoList = new ArrayList<RegionDTO>();
        for (Region item : list) {
            RegionDTO dto = new RegionDTO();
            BeanUtils.copyProperties(item, dto);
            String[] idTreeArr = item.getIdTree().split("_");
            dto.setIdLevelCount(idTreeArr.length);
            dtoList.add(dto);
        }
        // 排序
        dtoList = dtoList.stream().sorted(Comparator.comparing(RegionDTO::getIdLevelCount)).collect(Collectors.toList());
        
        List<RegionVo> returnList = new ArrayList<RegionVo>();
        for (Region region : dtoList) {
            Integer parentId = region.getParentId();
            if (!returnList.isEmpty()) {
                RegionVo regionVo = getRegionVo(returnList, parentId);
                if (regionVo == null) {
                    regionVo = new RegionVo();
                    BeanUtils.copyProperties(region, regionVo);
                    returnList.add(regionVo);
                    continue;
                }
                RegionVo regionVoNew = new RegionVo();
                if (selectedIds.contains(region.getId().intValue()))
                    regionVoNew.setChecked(true);
                BeanUtils.copyProperties(region, regionVoNew);
                List<RegionVo> children = regionVo.getChildren();
                if (children == null)
                    children = new ArrayList<>();
                children.add(regionVoNew);
                regionVo.setChildren(children);
            } else {
                RegionVo regionVo = new RegionVo();
                BeanUtils.copyProperties(region, regionVo);
                returnList.add(regionVo);
            }
        }
        return returnList;
    }

    public List<RegionVo> convertRegions(List<Region> list, Map<Integer, DispatchConfig> dispatchConfigMap){
        List<RegionDTO> dtoList = new ArrayList<RegionDTO>();
        for (Region item : list) {
            RegionDTO dto = new RegionDTO();
            BeanUtils.copyProperties(item, dto);
            String[] idTreeArr = item.getIdTree().split("_");
            if(idTreeArr.length > 4){
                continue;
            }
            dto.setIdLevelCount(idTreeArr.length);

            DispatchConfig dispatchConfig = dispatchConfigMap.get(item.getId());
            if(dispatchConfig != null){
                dto.setDispatchConfigId(dispatchConfig.getId());
                dto.setDispatchNum(dispatchConfig.getDispatchNum());
            }

            dtoList.add(dto);
        }
        // 排序
        dtoList = dtoList.stream().sorted(Comparator.comparing(RegionDTO::getIdLevelCount)).collect(Collectors.toList());

        List<RegionVo> returnList = new ArrayList<RegionVo>();
        for (RegionDTO region : dtoList) {
            Integer parentId = region.getParentId();
            if (!returnList.isEmpty()) {
                RegionVo regionVo = getRegionVo(returnList, parentId);
                if (regionVo == null) {
                    regionVo = new RegionVo();
                    BeanUtils.copyProperties(region, regionVo);
                    returnList.add(regionVo);
                    continue;
                }
                RegionVo regionVoNew = new RegionVo();
                BeanUtils.copyProperties(region, regionVoNew);
                List<RegionVo> children = regionVo.getChildren();
                if (children == null)
                    children = new ArrayList<>();
                children.add(regionVoNew);
                regionVo.setChildren(children);
            } else {
                RegionVo regionVo = new RegionVo();
                BeanUtils.copyProperties(region, regionVo);
                returnList.add(regionVo);
            }
        }
        return returnList;
    }
    
    /**
     * 递归获取区域
     *
     * @param returnList
     * @param id
     * @return
     */
    public RegionVo getRegionVo(List<RegionVo> returnList, int id) {
        for (RegionVo regionVo : returnList) {
            if (regionVo.getId().intValue() == id) {
                return regionVo;
            }
            List<RegionVo> children = regionVo.getChildren();
            if (children != null) {
                RegionVo regionVo2 = getRegionVo(children, id);
                if (regionVo2 != null)
                    return regionVo2;
            }
        }
        return null;
    }
    
    public void update(int id, Region region) {
        if (region.getStatus() != null && region.getStatus().intValue() == 2) {
            List<Region> childRegionList = regionDao.selectByParentId(id);
            if (!childRegionList.isEmpty())
                throw new ServiceException("该区域存在下级，不可删除");
        }
        regionDao.update(id, region);
    }
    
    // public void refreshRegion(){
    // List<Region> regionList = regionDao.selectAll(1);
    // }
}
