package com.starcharge.wios.service;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.starcharge.warehouse.dto.materials.MaterialsDTO;
import com.starcharge.wios.call.service.PushService;
import com.starcharge.wios.dao.*;
import com.starcharge.wios.dao.entity.*;
import com.starcharge.wios.utils.HttpUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * sap系统接口调用
 */
@Service
public class SapRemoteService {

    private Logger log = LoggerFactory.getLogger(getClass());

    @Value("${sap.url}")
    private String sapUrl;

    @Value("${sap.username}")
    private String sapUsername;

    @Value("${sap.password}")
    private String sapPassword;

    @Autowired
    private OemDao oemDao;

    @Autowired
    private SupplierDao supplierDao;

    @Autowired
    private DictionaryDao dictionaryDao;

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private SettlementService settlementService;

    @Autowired
    private SupplierContractDao supplierContractDao;
    @Autowired
    private OrderInstallDao orderInstallDao;

    /**
     * 订单推送(安装审核通过)
     */
    public Integer pushOrder(Order order){
        String orderId=order.getId();
        Map<String,Object> isHead=new HashMap<>();
        isHead.put("ZNO",orderId);
        //订单安桩审核状态
        String zstatus1="X";
        //服务开票状态
        String zstatus2="";
        //安装开票日期
        String zdate2="";
        //服务开票状态
        String packageInstallInvoiceStatus="";
        //设备开票状态
        String packageDeviceInvoiceStatus="";
        String packageInstallInvoiceNumber="";
        String packageDeviceInvoiceNumber="";

        //安装服务费（含税）
        BigDecimal installServiceFeeWithTax=BigDecimal.ZERO;
        //服务税
        BigDecimal serviceTax=BigDecimal.ZERO;
        //设备价格（含税）
        BigDecimal deviceFeeWithTax=BigDecimal.ZERO;
        //设备税
        BigDecimal deviceTax=BigDecimal.ZERO;

        Dictionary dictionary=null;
        if(StringUtils.isBlank(order.getInstallWallboxModel())){
            order.setInstallWallboxModel(order.getWallboxModel());
        }
        if(StringUtils.isNotBlank(order.getInstallWallboxModel())) {
            List<Dictionary> dictionaryList = dictionaryDao.selectValidByValue("wallbox", order.getInstallWallboxModel());
            if (CollectionUtils.isNotEmpty(dictionaryList)) {
                dictionary = dictionaryList.get(0);
            }
        }
        //ZBRANDTXT 墙盒品牌描述
        String zbrandtxt="";
        String zbrandid="";
        //产品编码
        String materialsCode="";
        //车企安装单号
        String outOrderId=order.getOutOrderId();
        //设备编号
        String wallboxNo="";
        //车型
        String vehicleModel=order.getVehicleModel();

        int size=1;
        //isSetting=1 就是有安装服务和设备 0就是只有安装服务
        if(dictionary!=null){
            zbrandtxt=dictionary.getName();
            zbrandid = dictionary.getValue();
        }
        OrderInstall orderInstall = orderInstallDao.selectByOrderId(orderId);
        if(orderInstall!=null){
            wallboxNo = orderInstall.getWallboxNo();
        }
        if(dictionary!=null&&dictionary.getIsSettle()==1){
            if(StringUtils.isBlank(order.getInstallItemModel()) && orderInstall!=null){
                order.setInstallItemModel(orderInstall.getWallboxItemId());
            }
            if(StringUtils.isNotBlank(order.getInstallItemModel())) {
                MaterialsDTO materialsDTO=warehouseService.getMaterialById(order.getInstallItemModel());
                if(materialsDTO==null){
                    log.warn("获取materialsDTO失败,订单id:{}",orderId);
                    return 0;
                }
                materialsCode=materialsDTO.getMaterialsCode();
                size=2;
            }
        }
        SettlementDetail settlementDetail=settlementService.getSettlementDetail(order,dictionary);
        if(settlementDetail==null){
            log.warn("获取settlementDetail失败,订单id:{}",orderId);
            return 0;
        }
        installServiceFeeWithTax=settlementDetail.getInstallServiceFeeWithTax();
        if(installServiceFeeWithTax!=null){
            serviceTax=installServiceFeeWithTax.multiply(BigDecimal.valueOf(0.09)).divide(BigDecimal.valueOf(1.09),2);
            serviceTax=serviceTax.setScale(2,RoundingMode.HALF_DOWN);
        }

        deviceFeeWithTax=settlementDetail.getDeviceFeeWithTax();
        if(deviceFeeWithTax!=null){
            deviceTax=deviceFeeWithTax.multiply(BigDecimal.valueOf(0.13)).divide(BigDecimal.valueOf(1.13),2);
            deviceTax=deviceTax.setScale(2,RoundingMode.HALF_DOWN);
        }
        isHead.put("ZSTATUS1",zstatus1);
        isHead.put("ZSTATUS2",zstatus2);
        isHead.put("BUKRS","1000");
        isHead.put("VKBUR","101");
        Oem oem=oemDao.selectById(order.getOemId());
        if(oem==null){
            log.warn("获取oem失败,订单id:{}",orderId);
            return 0;
        }
        //结算客户编号 测试1000
        isHead.put("KUNNR",oem.getCode());
        isHead.put("ZDATE1",order.getInstallApproveTime()==null?"":DateUtil.format(order.getInstallApproveTime(), "yyyy-MM-dd"));
        isHead.put("ZDATE2",zdate2);
        isHead.put("ZSHIPNO",order.getShipno());
        isHead.put("ZCOL81","");
        isHead.put("ZCOL82",outOrderId);
        isHead.put("ZCOL83","");
        isHead.put("ZCOL84","");
        isHead.put("ZCOL85","");
        isHead.put("ZCOL86","");

        Supplier supplier=supplierDao.selectById(order.getInstallSupplierId());
        if(supplier==null){
            log.warn("获取supplier失败,订单id:{}",orderId);
            return 0;
        }

        Integer supplierContractId = order.getSupplierContractId();
        if (supplierContractId == null)
            return 0;
        SupplierContract contract = supplierContractDao.selectById(supplierContractId);
        if (contract == null)
            return 0;
        SettlementDetailSupplier settlementDetailSupplier=settlementService.getSettlementDetailSupplier(order,contract);

        BigDecimal zpric=BigDecimal.ZERO;
        if(settlementDetailSupplier!=null){
            zpric=settlementDetailSupplier.getInstallFeeWithTax().setScale(2, RoundingMode.HALF_DOWN);
        }
        List items=new ArrayList(size);
        for(int i=1;i<=size;i++){
            //i 0是服务 1设备
            Map<String,Object> item=new HashMap<>();
            item.put("ZNO",orderId);
            //1开始自增
            item.put("ZITEM",i);
            item.put("ZSTATUS1",zstatus1);
            item.put("ZSTATUS2",i==1?packageInstallInvoiceStatus:packageDeviceInvoiceStatus);
            //第三方服务商编号
            item.put("LIFNR",supplier.getCode());
            item.put("ZBRANDID",zbrandid);
            item.put("ZBRANDTXT",zbrandtxt);
            //安装服务固定值：HCFWT000001 设备(如果设备需要结算，根据墙盒信息is_settle判断) WarehouseService.getMaterialById(t_order.install_item_model)
            item.put("MATNR",i==1?"HCFWT000001":materialsCode);
            item.put("KWMENT",1);
            item.put("NETWR",i==1?installServiceFeeWithTax:deviceFeeWithTax);
            //税额 必须大于0

            item.put("MWSBP",i==1?serviceTax:deviceTax);
            item.put("LGORT","");
            item.put("ZPRIC",i==1?zpric:"");
            item.put("ZINVNO",i==1?packageInstallInvoiceNumber:packageDeviceInvoiceNumber);
            item.put("ZVENDTYPE",supplier.getLevel());

            item.put("ZCOL01","");
            item.put("ZCOL02","");
            item.put("ZCOL03","");
            item.put("ZCOL04","");
            item.put("ZCOL05","");
            item.put("ZCOL06","");
            item.put("ZCOL07","");
            item.put("ZCOL08","");
            item.put("ZCOL09","");
            item.put("ZCOL10","");
            item.put("ZCOL11","");
            item.put("ZCOL12","");
            item.put("ZCOL13","");
            item.put("ZCOL14","");
            item.put("ZCOL15","");
            item.put("ZCOL16","");
            item.put("ZCOL17","");
            item.put("ZCOL18","");
            item.put("ZCOL19","");
            item.put("ZCOL20","");
            item.put("ZCOL21","");
            item.put("ZCOL22","");
            item.put("ZCOL23","");
            item.put("ZCOL24","");
            item.put("ZCOL25","");
            item.put("ZCOL26","");
            item.put("ZCOL27","");
            item.put("ZCOL28","");
            item.put("ZCOL29","");
            item.put("ZCOL30","");
            item.put("ZCOL91","");
            item.put("ZCOL92",wallboxNo);
            item.put("ZCOL93",vehicleModel);
            item.put("ZCOL94","");
            item.put("ZCOL95","");
            item.put("ZCOL96","");
            items.add(item);
        }
        Map<String,Object> param=new HashMap<>();
        param.put("IS_HEAD",isHead);
        Map<String,Object> itItem=new HashMap<>();
        itItem.put("item",items);
        param.put("IT_ITEM",itItem);

        String basicAuth= HttpUtil.buildBasicAuth(sapUsername,sapPassword,Charset.forName("UTF-8"));
        Map<String,String> headers=new HashMap<>();
        headers.put("Authorization",basicAuth);
        headers.put("Content-Type","application/json");
        String url=sapUrl+"/http/post/zfmsd003";
        log.info("订单推送(安装审核通过)[{}] pushOrder请求参数：{}",orderId,JSONObject.toJSONString(param));
        String response= HttpUtil.doPost(url,param,headers);
        if(StringUtils.isNotBlank(response)){
            try{
                JSONObject resp=JSONObject.parseObject(response);
                String retCode=resp.getString("ZTYPE");
                if("S".equals(retCode)){
                    return 1;
                }
            }catch (Exception e){
            }
        }
        log.info("订单推送失败(安装审核通过)[{}] pushOrder请求结果：{}",orderId,response);
        sendWarningEmail(orderId,response,"安装审核通过");
        return 0;
    }

    private static List<String> mailList = new ArrayList<>();
    static{
        mailList.add("jianfeng.gu@camsnetec.com");
        mailList.add("juan.song@camsnetec.com");
        mailList.add("liuhong.fan@camsnetec.com");
        mailList.add("jiaxin.tang@camsnetec.com");
        mailList.add("yijie.fang@camsnetec.com");
        mailList.add("wei.wang@camsnetec.com");
    }
    @Autowired
    private PushService pushService;

    private void sendWarningEmail(String orderId,String response,String title){
        mailList.stream().forEach(email -> {
            pushService.sendEmailAsync(email, "SAP订单推送失败("+title+")",
                    "<p>您好：</p>" + "<p>订单号为" + orderId + "的订单，"+title+"时同步SAP失败，请及时查看</p><p>SAP返回值："+response+"</p><p>本邮件是系统自动发送，无需直接回复</p>");
        });
    }

    /**
     * 订单推送(更新开票状态)
     */
    public synchronized Integer pushOrderOrInvoice(Order order,Invoice invoice){
        String orderId=order.getId();
        Map<String,Object> isHead=new HashMap<>();
        isHead.put("ZNO",orderId);
        //订单安桩审核状态
        String zstatus1="X";
        //安装开票日期
        String zdate2="";
        //服务开票状态
        String packageInstallInvoiceStatus="";
        //设备开票状态
        String packageDeviceInvoiceStatus="";

        //服务商合同约定的安装价格
        BigDecimal zpric=BigDecimal.ZERO;
        //ZBRANDTXT 墙盒品牌描述
        String zbrandtxt="";
        String zbrandid="";
        if(StringUtils.isBlank(order.getInstallWallboxModel())){
            order.setInstallWallboxModel(order.getWallboxModel());
        }
        Dictionary dictionary=null;
        if(StringUtils.isNotBlank(order.getInstallWallboxModel())) {
            List<Dictionary> dictionaryList = dictionaryDao.selectValidByValue("wallbox", order.getInstallWallboxModel());
            if (CollectionUtils.isNotEmpty(dictionaryList)) {
                dictionary = dictionaryList.get(0);
            }
        }
        if(dictionary!=null){
            zbrandtxt=dictionary.getName();
            zbrandid=dictionary.getValue();
        }
        //产品编码
        String materialsCode="";

        BigDecimal tax=BigDecimal.ZERO;
        BigDecimal feeWithTax=BigDecimal.ZERO;
        String invoiceNumber="";

        //提交项（1-服务开票，2-设备开票）
        int submititem=invoice.getSubmitItem();
        if(submititem==1&&invoice.getPackageInstallInvoiceStatus()==1){
            packageInstallInvoiceStatus="X";
            tax=invoice.getPackageInstallTax();
            feeWithTax=invoice.getPackageInstallFeeWithTax();
            invoiceNumber=invoice.getPackageInstallInvoiceNumber();
            materialsCode="HCFWT000001";
            zdate2=invoice.getPackageInstallInvoiceDate()==null?"":DateUtil.format(invoice.getPackageInstallInvoiceDate(), "yyyy-MM-dd");

            Integer supplierContractId = order.getSupplierContractId();
            if (supplierContractId == null)
                return 0;
            SupplierContract contract = supplierContractDao.selectById(supplierContractId);
            if (contract == null)
                return 0;
            SettlementDetailSupplier settlementDetailSupplier=settlementService.getSettlementDetailSupplier(order,contract);

            if(settlementDetailSupplier!=null){
                zpric=settlementDetailSupplier.getInstallFeeWithTax().setScale(2, RoundingMode.HALF_DOWN);
            }
        }
        if(submititem==2&&invoice.getPackageDeviceInvoiceStatus()==1){
            packageDeviceInvoiceStatus="X";
            tax=invoice.getPackageDeviceTax();
            feeWithTax=invoice.getPackageDeviceFeeWithTax();
            invoiceNumber=invoice.getPackageDeviceInvoiceNumber();
            zdate2=invoice.getPackageDeviceInvoiceDate()==null?"":DateUtil.format(invoice.getPackageDeviceInvoiceDate(), "yyyy-MM-dd");

            if(dictionary!=null&&dictionary.getIsSettle()==1){
                if(StringUtils.isBlank(order.getInstallItemModel())){
                    OrderInstall orderInstall = orderInstallDao.selectByOrderId(orderId);
                    if(orderInstall!=null){
                        order.setInstallItemModel(orderInstall.getWallboxItemId());
                    }
                }
                MaterialsDTO materialsDTO=warehouseService.getMaterialById(order.getInstallItemModel());
                if(materialsDTO==null){
                    log.warn("获取materialsDTO失败,订单id:{}",orderId);
                    return 0;
                }
                materialsCode=materialsDTO.getMaterialsCode();
            }
        }
        isHead.put("ZSTATUS1",zstatus1);
        isHead.put("ZSTATUS2",packageInstallInvoiceStatus);
        isHead.put("BUKRS","1000");
        isHead.put("VKBUR","101");
        Oem oem=oemDao.selectById(order.getOemId());
        if(oem==null){
            log.warn("获取oem失败,订单id:{}",orderId);
            return 0;
        }
        //结算客户编号 测试1000
        isHead.put("KUNNR",oem.getCode());
        isHead.put("ZDATE1",order.getInstallApproveTime()==null?"":DateUtil.format(order.getInstallApproveTime(), "yyyy-MM-dd"));
        isHead.put("ZDATE2",zdate2);
        isHead.put("ZSHIPNO",order.getShipno());
        isHead.put("ZCOL81",packageDeviceInvoiceStatus);
        isHead.put("ZCOL82","");
        isHead.put("ZCOL83","");
        isHead.put("ZCOL84","");
        isHead.put("ZCOL85","");
        isHead.put("ZCOL86","");

        Supplier supplier=supplierDao.selectById(order.getInstallSupplierId());
        if(supplier==null){
            log.warn("获取supplier失败,订单id:{}",orderId);
            return 0;
        }
        List items=new ArrayList(1);
        Map<String,Object> item=new HashMap<>();
        item.put("ZNO",orderId);

        item.put("ZITEM",submititem);
        item.put("ZSTATUS1",zstatus1);
        item.put("ZSTATUS2",packageInstallInvoiceStatus);
        //第三方服务商编号
        item.put("LIFNR",supplier.getCode());
        item.put("ZBRANDID",zbrandid);
        item.put("ZBRANDTXT",zbrandtxt);
        item.put("MATNR",materialsCode);
        item.put("KWMENT",1);
        item.put("NETWR",feeWithTax);
        //税额 必须大于0
        item.put("MWSBP",tax);
        item.put("LGORT","");
        item.put("ZPRIC",zpric);
        item.put("ZINVNO",invoiceNumber);
        item.put("ZVENDTYPE",supplier.getLevel());
        item.put("ZCOL01","");
        item.put("ZCOL02","");
        item.put("ZCOL03","");
        item.put("ZCOL04","");
        item.put("ZCOL05","");
        item.put("ZCOL06","");
        item.put("ZCOL07","");
        item.put("ZCOL08","");
        item.put("ZCOL09","");
        item.put("ZCOL10","");
        item.put("ZCOL11","");
        item.put("ZCOL12","");
        item.put("ZCOL13","");
        item.put("ZCOL14","");
        item.put("ZCOL15","");
        item.put("ZCOL16","");
        item.put("ZCOL17","");
        item.put("ZCOL18","");
        item.put("ZCOL19","");
        item.put("ZCOL20","");
        item.put("ZCOL21","");
        item.put("ZCOL22","");
        item.put("ZCOL23","");
        item.put("ZCOL24","");
        item.put("ZCOL25","");
        item.put("ZCOL26","");
        item.put("ZCOL27","");
        item.put("ZCOL28","");
        item.put("ZCOL29","");
        item.put("ZCOL30","");
        item.put("ZCOL91",packageDeviceInvoiceStatus);
        item.put("ZCOL92","");
        item.put("ZCOL93","");
        item.put("ZCOL94","");
        item.put("ZCOL95","");
        item.put("ZCOL96","");
        items.add(item);
        Map<String,Object> param=new HashMap<>();
        param.put("IS_HEAD",isHead);
        Map<String,Object> itItem=new HashMap<>();
        itItem.put("item",items);
        param.put("IT_ITEM",itItem);

        String basicAuth= HttpUtil.buildBasicAuth(sapUsername,sapPassword,Charset.forName("UTF-8"));
        Map<String,String> headers=new HashMap<>();
        headers.put("Authorization",basicAuth);
        headers.put("Content-Type","application/json");
        String url=sapUrl+"/http/post/zfmsd003";

        log.info("订单推送(更新开票状态{})[{}] pushOrderOrInvoice请求参数：{}",submititem,orderId,JSONObject.toJSONString(param));
        String response= HttpUtil.doPost(url,param,headers);
        if(StringUtils.isNotBlank(response)){
            try{
                JSONObject resp=JSONObject.parseObject(response);
                String retCode=resp.getString("ZTYPE");
                if("S".equals(retCode)){
                    return 1;
                }
            }catch (Exception e){
            }
        }
        log.info("订单推送失败(更新开票状态{})[{}] pushOrderOrInvoice请求结果：{}",submititem,orderId,response);
        sendWarningEmail(orderId,response,"更新开票状态");
        return 0;
    }

    /**
     * 同步订单服务商结算数据
     */
    public synchronized Integer syncSettlementData(Order order,Invoice invoice){
        String orderId=order.getId();
        String url=sapUrl+"/http/post/zfmsd004";
        Map<String,Object> paramMap=new HashMap<>();
        Map<String,Object> param=new HashMap<>();
        param.put("ZNO",orderId);

        Supplier supplier=supplierDao.selectById(order.getInstallSupplierId());
        param.put("LIFNR",supplier.getCode());
        param.put("ZDATE",DateUtil.format(invoice.getSupplierInvoiceDate(), "yyyy-MM-dd"));

        //税额
        BigDecimal supplierTax=invoice.getSupplierTax();
        //安装付款金额（不含税）
        BigDecimal supplierFee=invoice.getSupplierFeeWithTax().subtract(supplierTax);
        //税率
        param.put("ZTAX1",0.09);
        param.put("ZTAX2",supplierTax);
        param.put("ZNETPR1",supplierFee);
        param.put("ZINVNO1",invoice.getSupplierInvoiceNumber());
        List items=new ArrayList();
        items.add(param);
        Map<String,Object> item=new HashMap<>();
        item.put("item",items);
        paramMap.put("LT_INT",item);

        String basicAuth= HttpUtil.buildBasicAuth(sapUsername,sapPassword,Charset.forName("UTF-8"));
        Map<String,String> headers=new HashMap<>();
        headers.put("Authorization",basicAuth);
        headers.put("Content-Type","application/json");
        log.info("同步订单服务商结算数据[{}] syncSettlementData请求参数：{}",orderId,JSONObject.toJSONString(paramMap));
        String response= HttpUtil.doPost(url,paramMap,headers);
        if(StringUtils.isNotBlank(response)){
            try{
                JSONObject resp=JSONObject.parseObject(response);
                String retCode=resp.getJSONObject("ZFMSD004_RESPONSE").getJSONObject("ET_OUT").getJSONObject("item").getString("ZMESSAGE");
                if("S".equals(retCode)){
                    return 1;
                }
            }catch (Exception e){
            }
        }
        log.info("同步订单服务商结算数据[{}] syncSettlementData请求结果：{}",orderId,response);
        sendWarningEmail(orderId,response,"同步订单服务商结算数据");
        return 0;
    }

    /**
     * 校验服务商录入
     * @param code 服务商编码
     */
    public Integer verifyServiceProvider(String code){
        String url=sapUrl+"/http/post_cams/zfmmm007";
        Map<String,Object> paramMap=new HashMap<>();
        Map<String,Object> lsinput=new HashMap<>();
        lsinput.put("ZCJRQ","");
        lsinput.put("ZJSRQ","");
        lsinput.put("ZWBXT","WIOS");
        paramMap.put("LS_INPUT",lsinput);
        Map<String,Object> ltStcd5=new HashMap<>();
        List items=new ArrayList();
        Map<String,Object> stcd5=new HashMap<>();
        stcd5.put("STCD5",code);
        items.add(stcd5);
        ltStcd5.put("item",items);
        paramMap.put("LT_STCD5",ltStcd5);

        String basicAuth= HttpUtil.buildBasicAuth(sapUsername,sapPassword,Charset.forName("UTF-8"));
        Map<String,String> headers=new HashMap<>();
        headers.put("Authorization",basicAuth);
        headers.put("Content-Type","application/json");
        log.info("校验服务商录入 syncSettlementData请求参数：{}",JSONObject.toJSONString(paramMap));
        String response= HttpUtil.doPost(url,paramMap,headers);
        log.info("校验服务商录入 syncSettlementData请求结果：{}",response);
        if(StringUtils.isNotBlank(response)){
            try{
                JSONObject resp=JSONObject.parseObject(response);
                String retCode=resp.getJSONObject("ZFMMM007_Response").getJSONObject("LT_MESG").getJSONObject("item").getString("ZTYPE");
                if("S".equals(retCode)){
                    return 1;
                }
            }catch (Exception e){
                return 0;
            }
        }
        return 0;
    }


}
