package com.starcharge.wios.service;

import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.wios.auth.bo.AccountBo;
import com.starcharge.wios.auth.service.AccountService;
import com.starcharge.wios.bo.OemBo;
import com.starcharge.wios.contants.Contant;
import com.starcharge.wios.contants.ScoreConstant;
import com.starcharge.wios.dao.ScoreConfigDao;
import com.starcharge.wios.dao.entity.ScoreConfig;
import com.starcharge.wios.dao.entity.ScoreSupplier;
import com.starcharge.wios.vo.OemVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.List;

/**
 * 评价配置表应用服务
 *
 * @author yongfu.liu@wanbangauto.com 2020-10-10 14:08:14
 */
@Service
public class ScoreConfigService {

    @Autowired
    private ScoreConfigDao scoreConfigDao;

    /**
     * 查询列表
     *
     * @param type
     * @return
     */
    public List<ScoreConfig> getListByType(String type) {
        return scoreConfigDao.getListByType(type);
    }

    /**
     * 单个更新
     *
     * @param scoreConfig
     */
    public void update(ScoreConfig scoreConfig) {
        if(scoreConfig.getId() == null || scoreConfig.getId()==0){
            throw new ServiceException("没有该条记录的 ID");
        }
        scoreConfigDao.update(scoreConfig);
    }

    /**
     * 单个新增
     * @param scoreConfig
     */
    public void insert(ScoreConfig scoreConfig){
        scoreConfigDao.insert(scoreConfig);
    }

    public String[] getExcelTitles(){
        List<ScoreConfig> list = scoreConfigDao.getListByType("Supplier");

        String titles =  String.format("安装服务商,所属区域,评价月份,综合评分,采购评分(%d%%),工程评分(%d%%),线上评分(%d%%),线下评分(%d%%),客户评分(%d%%),扣分",
                this.getWeightByCode(list, ScoreConstant.FINAL_PURCHASING),
                this.getWeightByCode(list, ScoreConstant.FINAL_ENGINEERING),
                this.getWeightByCode(list, ScoreConstant.ONLINE),
                this.getWeightByCode(list, ScoreConstant.OFFLINE),
                this.getWeightByCode(list, ScoreConstant.CUSTOMER) );
        return titles.split(",");
    }

    protected Integer getWeightByCode(List<ScoreConfig> list, String code){
        for(ScoreConfig scoreConfig : list){
            if(code.equals(scoreConfig.getCode())){
                return scoreConfig.getWeight().intValue();
            }
        }
        return 0;
    }
}
