package com.starcharge.wios.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.wios.dao.SecretPhoneDao;
import com.starcharge.wios.dao.entity.SecretPhone;
import com.starcharge.wios.dto.SecretBindDTO;
import com.starcharge.wios.utils.PhoneNumProtectionUtil;

@Service
public class SecretPhoneService {

    @Autowired
    private SecretPhoneDao secretPhoneDao;

    /**
     * 绑定虚拟号码
     *
     * @param phone
     * @return
     * @throws Exception
     */
    public SecretBindDTO bind(String phone) throws Exception {
        if (StringUtils.isEmpty(phone))
            throw new ServiceException("缺少参数");
        SecretPhone sp = secretPhoneDao.selectByPhone(phone);
        if (sp != null && sp.getStatus() != null && sp.getStatus().intValue() == 1) {
            throw new ServiceException("该号码已绑定虚拟号码");
        }
        SecretBindDTO result = PhoneNumProtectionUtil.bindAxn(phone);
        String code = result.getCode();
        if (!StringUtils.isEmpty(code) && code.equals("isv.NO_AVAILABLE_NUMBER")) {
            PhoneNumProtectionUtil.buySecretNo();// 购买号码
            Thread.sleep(1000);// 等一会儿
            result = PhoneNumProtectionUtil.bindAxn(phone);// 再次绑定
            String code2 = result.getCode();
            if (!StringUtils.isEmpty(code2) && !code2.equals("OK")) {
                throw new ServiceException("再次绑定失败，错误码：" + code2);
            }
        } else if (!StringUtils.isEmpty(code) && !code.equals("OK") && !code.equals("isv.NO_AVAILABLE_NUMBER")) {
            throw new ServiceException("绑定失败，错误码：" + code);
        }
        
        SecretPhone secretPhone = new SecretPhone();
        secretPhone.setPhone(phone);
        secretPhone.setPoolKey(PhoneNumProtectionUtil.ALI_POOLKEY);
        secretPhone.setExpiredTime(result.getExpiredDate());
        secretPhone.setSecretNo(result.getSecretNo());
        secretPhone.setSubsId(result.getSubsId());
        secretPhoneDao.insert(secretPhone);
        
        return result;
    }
    
    @Async
    public SecretBindDTO bindAsync(String phone) throws Exception {
        return bind(phone);
    }
    
    /**
     * 解绑虚拟号码
     *
     * @param phone
     * @throws Exception
     */
    public void unbind(String phone) throws Exception {
        if (StringUtils.isEmpty(phone))
            throw new ServiceException("缺少参数");
        SecretPhone sp = secretPhoneDao.selectByPhone(phone);
        if (sp == null) {
            throw new ServiceException("号码不存在");
        }
        PhoneNumProtectionUtil.unbindSubscription(sp.getSubsId(), sp.getSecretNo());
        
        // SecretPhone secretPhone = new SecretPhone();
        // secretPhone.setPhone(phone);
        // secretPhone.setStatus(0);
        // secretPhoneDao.updateDynamic(secretPhone);
        secretPhoneDao.delete(phone);
    }
    
    public SecretPhone get(String phone) {
        return secretPhoneDao.selectByPhone(phone);
    }
    
    public void releaseExpired() throws Exception {
        List<SecretPhone> list = secretPhoneDao.selectExpired();
        for (SecretPhone sp : list) {
            PhoneNumProtectionUtil.unbindSubscription(sp.getSubsId(), sp.getSecretNo());
            Thread.sleep(500);
            PhoneNumProtectionUtil.releaseSecretNo(sp.getSecretNo());
            secretPhoneDao.delete(sp.getPhone());
        }
    }
}
