package com.starcharge.wios.service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.warehouse.dto.materials.MaterialsDTO;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.bo.SupplierContractBo;
import com.starcharge.wios.contants.Contant;
import com.starcharge.wios.dao.RegionDao;
import com.starcharge.wios.dao.SupplierContractDao;
import com.starcharge.wios.dao.SupplierContractItemDao;
import com.starcharge.wios.dao.SupplierContractPackPriceDao;
import com.starcharge.wios.dao.SupplierContractPriceDao;
import com.starcharge.wios.dao.SupplierDao;
import com.starcharge.wios.dao.UploadFileDao;
import com.starcharge.wios.dao.entity.Region;
import com.starcharge.wios.dao.entity.Supplier;
import com.starcharge.wios.dao.entity.SupplierContract;
import com.starcharge.wios.dao.entity.SupplierContractItem;
import com.starcharge.wios.dao.entity.SupplierContractPackPrice;
import com.starcharge.wios.dao.entity.SupplierContractPrice;
import com.starcharge.wios.dao.entity.UploadFile;
import com.starcharge.wios.enums.ContractStatus;
import com.starcharge.wios.enums.UploadFileType;
import com.starcharge.wios.vo.SupplierContractItemVo;
import com.starcharge.wios.vo.SupplierContractPackPriceVo;
import com.starcharge.wios.vo.SupplierContractPriceVo;
import com.starcharge.wios.vo.SupplierContractVo;

/**
 * 服务商合同service
 * 
 * @author xgl
 * @version [版本号, 2020年10月10日]
 */
@Service
public class SupplierContractService {

    private static final Logger logger= LoggerFactory.getLogger(SupplierContractService.class);

    @Autowired
    private SupplierContractDao supplierContractDao;
    
    @Autowired
    private SupplierContractItemDao supplierContractItemDao;
    
    @Autowired
    private SupplierContractPriceDao supplierContractPriceDao;
    
    @Autowired
    private SupplierContractPackPriceDao supplierContractPackPriceDao;
    
    @Autowired
    private UploadFileDao uploadFileDao;
    
    @Autowired
    private RegionDao regionDao;
    
    @Autowired
    private SupplierDao supplierDao;
    
    @Autowired
    private WarehouseService warehouseService;
    
    @Autowired
    private TokenService tokenService;
    
    /**
     * 添加数据
     *
     * @param contractBo
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void add(SupplierContractBo contractBo) {
        List<SupplierContract> contractListByCode = supplierContractDao.selectByCode(contractBo.getCode());
        if (!contractListByCode.isEmpty())
            throw new ServiceException("合同编号已存在");
        // 获取已添加的有效的合同列表
        List<SupplierContractVo> validContractList = supplierContractDao.selectValid(Integer.valueOf(contractBo.getSupplierId()));
        for (SupplierContractVo contract : validContractList) {
            if (contractBo.getOperateType() == null || contractBo.getOperateType().intValue() == 0) {
                SupplierContract updateContract = new SupplierContract();
                if (contract.getEndTime().compareTo(contractBo.getEndTime()) < 0) {
                    if (contract.getStartTime().compareTo(contractBo.getStartTime()) >= 0) {
                        // 时间包含则设为失效
                        updateContract.setStatus(ContractStatus.INVALID.getType());
                    } else {
                        if (contract.getEndTime().compareTo(contractBo.getStartTime()) >= 0) {
                            // 时间有交集则更新该合同的结束时间
                            Calendar cal = Calendar.getInstance();
                            cal.setTime(contractBo.getStartTime());
                            cal.add(Calendar.DATE, -1);
                            updateContract.setEndTime(cal.getTime());
                            if (contractBo.getStartTime().compareTo(new Date()) <= 0)// 结束时间小于当前时间
                                updateContract.setStatus(ContractStatus.INVALID.getType());
                        } else {
                            continue;
                        }
                    }
                } else {
                    throw new ServiceException("结束日期不能小于已添加合同的结束日期");
                }
                supplierContractDao.update(contract.getId(), updateContract);
            } else {
                if ((contract.getStartTime().compareTo(contractBo.getStartTime()) >= 0
                    && contract.getStartTime().compareTo(contractBo.getEndTime()) <= 0)
                    || (contract.getEndTime().compareTo(contractBo.getStartTime()) >= 0
                        && contract.getEndTime().compareTo(contractBo.getEndTime()) <= 0)) {
                    throw new ServiceException("与已有合同时间冲突");
                }
            }
        }
        supplierContractDao.insert(contractBo);// 添加合同数据
        List<SupplierContractItem> items = contractBo.getItems();
        if (items != null && !items.isEmpty()) {
            items.stream().forEach(item -> {
                item.setContractId(contractBo.getId());
                item.setId(null);
            });
            supplierContractItemDao.batchInsert(items);// 添加item
        }
        
        List<SupplierContractPrice> prices = contractBo.getPrices();
        if (prices != null && !prices.isEmpty()) {
            prices.stream().forEach(price -> {
                price.setContractId(contractBo.getId());
                price.setId(null);
            });
            supplierContractPriceDao.batchInsert(prices);// 添加价格
        }
        
        List<SupplierContractPackPrice> packPrices = contractBo.getPackPrices();
        if (packPrices != null && !packPrices.isEmpty()) {
            packPrices.stream().forEach(packPrice -> {
                packPrice.setContractId(contractBo.getId());
                packPrice.setId(null);
            });
            supplierContractPackPriceDao.batchInsert(packPrices);// 添加套包价格
        }
        
        List<UploadFile> files = contractBo.getFiles();
        if (files != null && !files.isEmpty()) {
            files.stream().forEach(file -> {
                file.setId(null);
                file.setType(UploadFileType.SUPPLIER_CONTRACT.getType());
                file.setThirdPartyId(contractBo.getId().toString());
                if (!file.getUrl().startsWith("http"))
                    file.setUrl(Contant.FILE_UPLOAD_DEFAULT_URL + file.getUrl());
            });
            
            uploadFileDao.batchInsert(files);// 添加文件
        }
    }
    
    /**
     * 更新合同
     *
     * @param contractBo
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void update(SupplierContractBo contractBo) {
        if (!StringUtils.isEmpty(contractBo.getCode())) {
            List<SupplierContract> contractListByCode = supplierContractDao.selectByCode(contractBo.getCode());
            if (!contractListByCode.isEmpty()) {
                if (contractListByCode.get(0).getId().intValue() != contractBo.getId().intValue()) {
                    throw new ServiceException("合同编号已存在");
                }
            }
        }
        supplierContractDao.update(contractBo.getId(), contractBo);
        supplierContractItemDao.deleteByContractId(contractBo.getId());
        supplierContractPriceDao.deleteByContractId(contractBo.getId());
        supplierContractPackPriceDao.deleteByContractId(contractBo.getId());
        uploadFileDao.deleteByTypeAndTPId(UploadFileType.SUPPLIER_CONTRACT.getType(), contractBo.getId().toString());
        
        List<SupplierContractItem> items = contractBo.getItems();
        if (items != null && !items.isEmpty()) {
            items.stream().forEach(item -> item.setContractId(contractBo.getId()));
            supplierContractItemDao.batchInsert(items);// 添加item
        }
        
        List<SupplierContractPrice> prices = contractBo.getPrices();
        if (prices != null && !prices.isEmpty()) {
            prices.stream().forEach(price -> price.setContractId(contractBo.getId()));
            supplierContractPriceDao.batchInsert(prices);// 添加价格
        }
        
        List<SupplierContractPackPrice> packPrices = contractBo.getPackPrices();
        if (packPrices != null && !packPrices.isEmpty()) {
            packPrices.stream().forEach(packPrice -> packPrice.setContractId(contractBo.getId()));
            supplierContractPackPriceDao.batchInsert(packPrices);// 添加套包价格
        }
        
        List<UploadFile> files = contractBo.getFiles();
        if (files != null && !files.isEmpty()) {
            files.stream().forEach(file -> {
                file.setId(null);
                file.setType(UploadFileType.SUPPLIER_CONTRACT.getType());
                file.setThirdPartyId(contractBo.getId().toString());
                if (!file.getUrl().startsWith("http"))
                    file.setUrl(Contant.FILE_UPLOAD_DEFAULT_URL + file.getUrl());
            });
            uploadFileDao.deleteByTypeAndTPId(UploadFileType.SUPPLIER_CONTRACT.getType(), contractBo.getId().toString());
            uploadFileDao.batchInsert(files);// 添加文件
        }
    }
    
    /**
     * 查询列表
     *
     * @param condition
     * @return
     */
    public List<SupplierContractVo> getList(SupplierContractBo condition) {
        SessionInfo user = tokenService.getUser();
        if (user != null && !StringUtils.isEmpty(user.getAttribute().get("installSupplierId"))) {
            condition.setSupplierId(user.getAttribute().get("installSupplierId"));
        }
        List<SupplierContractVo> returnList = supplierContractDao.getList(condition);
        List<Region> regionList = regionDao.selectAll(null);
        Map<Integer, String> regionMap = regionList.stream().collect(Collectors.toMap(Region::getId, Region::getName));
        returnList.stream().forEach(item -> {
            String regionId = item.getRegionId();
            if (!StringUtils.isEmpty(regionId)) {
                String[] regionArr = regionId.split(",");
                List<String> namelist = new ArrayList<>();
                for (String rId : regionArr) {
                    String name = regionMap.get(Integer.valueOf(rId));
                    if (!StringUtils.isEmpty(name)) {
                        namelist.add(name);
                    }
                }
                item.setRegionName(StringUtils.collectionToCommaDelimitedString(namelist));
            }
        });
        
        return returnList;
    }
    
    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    public SupplierContractVo getDetail(int id) {
        SupplierContract contract = supplierContractDao.selectById(id);
        if (contract == null) {
            return null;
        }
        Supplier supplier = supplierDao.selectById(Integer.valueOf(contract.getSupplierId()));
        SupplierContractVo result = new SupplierContractVo();
        BeanUtils.copyProperties(contract, result);
        if (supplier != null)
            result.setSupplierName(supplier.getName());

        List<SupplierContractItemVo> contractItems = supplierContractItemDao.selectByContractId(id);
        for (SupplierContractItemVo vo : contractItems) {
            MaterialsDTO dto = warehouseService.getMaterialById(vo.getItemId());
            if (dto != null) {
                vo.setItemCode(dto.getMaterialsCode());
                vo.setItemModel(dto.getRegularModel());
                vo.setItemName(dto.getMaterialsName());
            }
        }
        
        List<SupplierContractPriceVo> contractPrices = supplierContractPriceDao.selectByContractId(id);
        for (SupplierContractPriceVo vo : contractPrices) {
            MaterialsDTO dto = warehouseService.getMaterialById(vo.getDeviceModel());
            if (dto != null) {
                vo.setDeviceName(dto.getMaterialsName());
            }
        }
        
        List<SupplierContractPackPriceVo> contractPackPrices = supplierContractPackPriceDao.selectByContractId(id);
        for (SupplierContractPackPriceVo vo : contractPackPrices) {
            MaterialsDTO dto = warehouseService.getMaterialById(vo.getDeviceModel());
            if (dto != null) {
                vo.setDeviceName(dto.getMaterialsName());
            }
        }
        
        List<UploadFile> files = uploadFileDao.selectValidByTypeAndTPId(UploadFileType.SUPPLIER_CONTRACT.getType(), id + "");
        result.setItems(contractItems);
        result.setPrices(contractPrices);
        result.setPackPrices(contractPackPrices);
        result.setFiles(files);
        return result;
    }


    public SupplierContract getEffectiveContract(String supplierId, String regionId){
        List<SupplierContract> list=supplierContractDao.selectEffectiveContract(supplierId, regionId, 2);
        return !CollectionUtils.isEmpty(list) ? list.get(0) : null;
    }

    public List<SupplierContractItemVo> getContractItems(String supplierId, String regionId){
        SupplierContract contract = getEffectiveContract(supplierId, null);
        return contract!=null ? getContractItems(contract.getId()) : null;
    }

    public List<SupplierContractItemVo> getContractItems(Integer contractId){
        if(contractId==null){
            return null;
        }
        List<SupplierContractItemVo> contractItems = supplierContractItemDao.selectByContractId(contractId);
        for (SupplierContractItemVo vo : contractItems) {
            try{
                MaterialsDTO dto = warehouseService.getMaterialById(vo.getItemId());
                if (dto != null) {
                    vo.setItemCode(dto.getMaterialsCode());
                    vo.setItemModel(dto.getRegularModel());
                    vo.setItemName(dto.getMaterialsName());
                }
            }catch (Exception e){
                logger.error("加载远程仓库数据出现异常："+e.getMessage(), e);
            }
        }
        return contractItems;
    }
    
    /**
     * 更新合同状态
     * 
     * @throws ParseException
     *
     */
    public void contractTiming() throws ParseException {
        List<SupplierContractVo> contractList = supplierContractDao.selectValid(null);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        for (SupplierContractVo contract : contractList) {
            Date startDate = contract.getStartTime();
            Date endDate = contract.getEndTime();
            SupplierContract updateContract = new SupplierContract();
            endDate = sdf1.parse(sdf.format(endDate) + " 23:59:59");
            if (endDate.compareTo(new Date()) < 0) {
                updateContract.setStatus(ContractStatus.INVALID.getType());
            } else {
                if (contract.getStatus() != null && contract.getStatus().intValue() == ContractStatus.NOT_START.getType()
                    && startDate.compareTo(new Date()) <= 0) {
                    updateContract.setStatus(ContractStatus.EFFECTIVE.getType());
                }
            }
            if (updateContract.getStatus() != null) {
                supplierContractDao.update(contract.getId(), updateContract);
            }
        }
    }
}
