package com.starcharge.wios.service;

import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.wios.auth.bo.AccountBo;
import com.starcharge.wios.auth.dao.AccountRoleMapper;
import com.starcharge.wios.auth.dao.model.AccountRole;
import com.starcharge.wios.auth.dao.model.AccountRoleCriteria;
import com.starcharge.wios.auth.service.AccountService;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.bo.SupplierStaffBo;
import com.starcharge.wios.contants.Contant;
import com.starcharge.wios.dao.SupplierDao;
import com.starcharge.wios.dao.SupplierStaffDao;
import com.starcharge.wios.dao.entity.Supplier;
import com.starcharge.wios.dao.entity.SupplierStaff;
import com.starcharge.wios.vo.SupplierStaffVo;

/**
 * 服务商员工service
 * 
 * @author xgl
 * @version [版本号, 2020年9月23日]
 */
@Service
public class SupplierStaffService {
    @Autowired
    private SupplierStaffDao supplierStaffDao;
    
    @Autowired
    private AccountService accountService;
    
    @Autowired
    private TokenService tokenService;
    
    @Autowired
    private SupplierDao supplierDao;
    
    @Autowired
    private AccountRoleMapper arMapper;
    
    /**
     * 添加服务商员工
     *
     * @param staff
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void add(SupplierStaffBo staff) {
        AccountBo account = new AccountBo();
        account.setAccount(staff.getPhone());
        account.setName(staff.getName());
        account.setPassword(Contant.ACCOUNT_INIT_PASSWARD);
        account.setPhone(staff.getPhone());
        account.setRoleIds(staff.getRoleIds());
        
        SessionInfo user = tokenService.getUser();
        if (user == null) {
            throw new ServiceException("缺少登陆用户信息");
        }
        if ("1".equals(user.getUserId())) {// 超级管理员
            if (staff.getSupplierId() == null) {
                throw new ServiceException("缺少服务商id");
            }
            Supplier supplier = supplierDao.selectById(staff.getSupplierId());
            Integer parentId = supplier.getAccountId();
            account.setParentId(parentId);
        } else {
            account.setParentId(Integer.valueOf(user.getUserId()));
            Supplier supplier = supplierDao.selectByAccountId(Integer.valueOf(user.getUserId()));
            if (supplier == null)
                throw new ServiceException("缺少管理员账号对应的服务商");
            staff.setSupplierId(Integer.valueOf(supplier.getId()));
        }
        
        accountService.addAccount(account);
        staff.setAccountId(account.getId());
        supplierStaffDao.insert(staff);
    }
    
    public void update(int id, SupplierStaffBo staff) {
        if (!StringUtils.isEmpty(staff.getRoleIds())) {
            SupplierStaff ss = supplierStaffDao.selectById(id);
            AccountRoleCriteria example = new AccountRoleCriteria();
            example.createCriteria().andAccountIdEqualTo(ss.getAccountId());
            arMapper.deleteByExample(example);
            AccountRole record = new AccountRole();
            record.setAccountId(ss.getAccountId());
            record.setRoleId(Integer.valueOf(staff.getRoleIds()));
            arMapper.insert(record);
        }
        supplierStaffDao.update(id, staff);
    }
    
    public List<SupplierStaffVo> getList(SupplierStaffBo condition) {
        SessionInfo user = tokenService.getUser();
        if (user != null && !StringUtils.isEmpty(user.getAttribute().get("installSupplierId"))) {
            condition.setSupplierId(Integer.valueOf(user.getAttribute().get("installSupplierId")));
        }
        if (user != null && !StringUtils.isEmpty(user.getUserId())) {
            List<SupplierStaff> staffList = supplierStaffDao.selectByAccountId(Integer.valueOf(user.getUserId()));
            if (!staffList.isEmpty()) {
                condition.setId(staffList.get(0).getId());
            }
        }
        return supplierStaffDao.selectByExample(condition);
    }
    
    public SupplierStaffVo getInfoByAccount(String account) {
    	SupplierStaffVo staffVo = null;
    	
    	if (StringUtils.isNotBlank(account)) {
    		SupplierStaffBo condition = new SupplierStaffBo();
    		condition.setAccount(account);
    		condition.setStatus(1);
    		List<SupplierStaffVo> list = getList(condition);
    		if (CollectionUtils.isNotEmpty(list)) {
    			staffVo = list.get(0);
			}
		}
    	
    	return staffVo;
	}

    public SupplierStaff getByAccountId(Integer accountId){
        List<SupplierStaff> list=supplierStaffDao.selectByAccountId(accountId);
        if(CollectionUtils.isNotEmpty(list)){
            return list.get(0);
        }
        return null;
    }
}
