package com.starcharge.wios.service;

        import java.util.Date;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.util.HtmlUtils;

import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.base.util.Assert;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.bo.SystemNoticeBo;
import com.starcharge.wios.call.service.PushService;
import com.starcharge.wios.contants.Contant;
import com.starcharge.wios.dao.OperateLogDao;
import com.starcharge.wios.dao.RegionDao;
import com.starcharge.wios.dao.SupplierDao;
import com.starcharge.wios.dao.SystemInfoDao;
import com.starcharge.wios.dao.SystemNoticeDao;
import com.starcharge.wios.dao.entity.SystemNotice;
import com.starcharge.wios.vo.SystemNoticeVo;

/**
 * 订单service
 *
 * @author xgl
 * @version [版本号, 2020年9月16日]
 */
@Service
public class SystemNoticeService {

    @Autowired
    private SystemNoticeDao systemNoticeDao;

    @Autowired
    private SupplierDao supplierDao;

    @Autowired
    private RegionDao regionDao;

    @Autowired
    private OperateLogDao operateLogDao;

    @Autowired
    private DictionaryService dicService;

    @Autowired
    private SystemInfoDao systemInfoDao;

    @Autowired
    private CamsAccountService camsAccountService;

    @Autowired
    private PushService pushService;

    @Autowired
    private TokenService tokenService;

    private Logger logger = LoggerFactory.getLogger(SystemNoticeService.class);



    /**
     * 添加公告/附件
     *
     * @param systemNotice
     */
    public void add(SystemNotice systemNotice) {
        Assert.notNull(systemNotice.getStatus(), "公告状态不能为空");
        if(systemNotice.getType().equals(1)) {
            Assert.notNull(systemNotice.getTitle(), "公告标题不能为空");
            Assert.notNull(systemNotice.getContent(), "公告内容不能为空");
            Assert.notNull(systemNotice.getDisplayEndStr(), "公告展示时间不能为空");
            Assert.notNull(systemNotice.getDisplayFromStr(), "公告展示时间不能为空");
            systemNotice.setContent(HtmlUtils.htmlEscapeHex(systemNotice.getContent()));
            systemNoticeDao.insert(systemNotice);

        }else if (systemNotice.getType().equals(2)){
            Assert.notNull(systemNotice.getTitle(), "标题不能为空");
            Assert.notNull(systemNotice.getFileAnnex(), "附件不能为空");
            systemNotice.setFileAnnex(Contant.FILE_UPLOAD_DEFAULT_URL + systemNotice.getFileAnnex());
            systemNoticeDao.insert(systemNotice);
        }

    }
    /**
     * 查询列表
     *
     * @param condition
     * @return
     */
    public List<SystemNoticeVo> getList(SystemNoticeBo condition) {

        List<SystemNoticeVo> list = systemNoticeDao.getList(condition);
        return list;

    }

    /**
     * 查询明细
     *
     * @param id
     * @return Detail
     */
    public SystemNoticeVo getDetail(Long id) {
        SystemNoticeBo condition = new SystemNoticeBo();
        condition.setId(id);
        SystemNoticeVo Detail = systemNoticeDao.getDetail(condition);
        return Detail;
    }

    /**
     * 更新明细
     *
     * @param id
     * @return Detail
     */
    public void updateNotice(SystemNoticeBo systemNotice,Long id) {
        SystemNoticeVo checkSystemNotice =  this.getDetail(id);
        Assert.notNull(checkSystemNotice, "更新公告不存在");
        Assert.notNull(systemNotice.getStatus(), "公告状态不确定");
        if (!checkSystemNotice.getType().equals(systemNotice.getType())){
            throw new ServiceException("公告类型type不匹配");
        }
        SystemNoticeBo systemNoticeUpdate = new SystemNoticeBo();
        systemNoticeUpdate.setModifyTime(new Date());
        systemNoticeUpdate.setId(id);
        systemNoticeUpdate.setStatus(systemNotice.getStatus());
        SessionInfo user = tokenService.getUser();
        if (user != null){
            systemNoticeUpdate.setModifyAccount(user.getUserId());
        }else{
            throw new ServiceException("用户信息未获取");
        }
        if(checkSystemNotice.getType().equals(1)) {
            Assert.notNull(systemNotice.getTitle(), "更新公告标题不能为空");
            Assert.notNull(systemNotice.getContent(), "更新公告内容不能为空");
            Assert.notNull(systemNotice.getDisplayEndStr(), "更新公告展示时间不能为空");
            Assert.notNull(systemNotice.getDisplayFromStr(), "更新公告展示时间不能为空");
            systemNoticeUpdate.setTitle(systemNotice.getTitle());
            systemNoticeUpdate.setContent(systemNotice.getContent());
            systemNoticeUpdate.setDisplayEnd(systemNotice.getDisplayEnd());
            systemNoticeUpdate.setDisplayFrom(systemNotice.getDisplayFrom());
            systemNoticeDao.updateNotice(systemNoticeUpdate);
        }else if (checkSystemNotice.getType().equals(2)){
            Assert.notNull(systemNotice.getTitle(), "更新标题不能为空");
            Assert.notNull(systemNotice.getFileAnnex(), "更新附件不能为空");
            systemNoticeUpdate.setTitle(systemNotice.getTitle());
            if (!systemNotice.getFileAnnex().startsWith("http"))
                systemNoticeUpdate.setFileAnnex(Contant.FILE_UPLOAD_DEFAULT_URL + systemNotice.getFileAnnex());
            systemNoticeDao.updateNotice(systemNoticeUpdate);
        }
    }


    /**
     * 删除公告
     *
     * @param id,systemNotice
     * @return
     */
    public void deleteNotice(SystemNoticeBo systemNotice,Long id) {
        SystemNoticeVo checkSystemNotice =  this.getDetail(id);
        Assert.notNull(checkSystemNotice, "公告不存在");
        SystemNoticeBo systemNoticeUpdate = new SystemNoticeBo();
        systemNoticeUpdate.setModifyTime(new Date());
        systemNoticeUpdate.setId(id);
        if(!systemNotice.getStatus().equals(0)){
            throw new ServiceException("公告状态有问题");
        }
        systemNoticeUpdate.setStatus(0);
        SessionInfo user = tokenService.getUser();
        if (user != null){
            systemNoticeUpdate.setModifyAccount(user.getUserId());
        }else{
            systemNoticeUpdate.setModifyAccount("4");
        }
        systemNoticeDao.updateNotice(systemNoticeUpdate);
    }

}