package com.starcharge.wios.service;

import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import com.ihidea.core.util.StringUtilsEx;
import com.starcharge.base.redis.RedisClient;
import com.starcharge.base.util.Assert;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.dao.UserDao;
import com.starcharge.wios.dao.entity.User;
import com.starcharge.wios.utils.SpringContextUtil;
import com.starcharge.wios.vo.UserLoginInfoVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author kevin
 * @create 2020/11/16 15:38
 */
@Service
public class UserService {
    @Autowired
    private UserDao userDao;

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private TokenService tokenService;

    public List<User> list(User entity){
        return userDao.list(entity);
    }

    public User get(User entity){
        List<User> list=userDao.list(entity);
        return CollectionUtils.isNotEmpty(list) ? list.get(0) : null;
    }

    public User getByAccount(String account){
        Assert.notBlank(account, "手机号不能为空");
        User entity=new User();
        entity.setAccount(account);
        return get(entity);
    }

    public Integer countByAccount(String account){
        Assert.notBlank(account, "手机号不能为空");
        User entity=new User();
        entity.setAccount(account);
        return userDao.count(entity);
    }

    public int add(User record){
        Assert.notBlank(record.getAccount(), "手机号不能为空");

        String lockId="user_add:"+record.getAccount();
        String requestId=StringUtilsEx.getUUID();
        if(redisClient.lock(lockId,requestId, 30)){
            try{
                if(record.getStatus() == null){
                    record.setStatus(1);
                }
                record.setCreateTime(new Date());
                return userDao.add(record);
            }finally {
                redisClient.releaseLock(lockId, requestId);
            }
        }else{
            throw new ServiceException("系统忙，请稍后再试");
        }
    }

    public int update(User record){
        Assert.notNull(record.getId(), "ID不能为空");
        return userDao.update(record);
    }

    public User getById(Integer id){
        Assert.notNull(id, "ID不能为空");
        return userDao.getById(id);
    }

    public int deleteById(Integer id){
        return userDao.deleteById(id);
    }

    public UserLoginInfoVO login(String phone, String openId){
        Assert.notBlank(phone, "手机号不能为空");
        User user = getByAccount(phone);
        // 账户不存在自动创建
        if(user==null){
            user = new User();
            user.setAccount(phone);
            user.setOpenId(openId);
            user.setLastLoginTime(new Date());

            add(user);

            user = getByAccount(user.getAccount());
        }else{
            Assert.isTrue(user.getStatus()==1, "账号已禁用");

            // 更新账户登录时间
            User newUser = new User();
            newUser.setId(user.getId());
            if(StringUtils.isNotBlank(openId)){
                newUser.setOpenId(openId);
            }
            newUser.setLastLoginTime(new Date());
            userDao.update(newUser);
        }

        SessionInfo sessionInfo = new SessionInfo();
        sessionInfo.setUserId(String.valueOf(user.getId()));
        sessionInfo.setUserName(user.getAccount());
        sessionInfo.setAttribute("openId", StringUtils.isNotBlank(openId) ? openId : user.getOpenId());
        sessionInfo.setAttribute("userType", "customer");
        // 有效期10天
        String token=tokenService.saveWxToken(sessionInfo);

        UserLoginInfoVO loginInfo = new UserLoginInfoVO();
        loginInfo.setUserId(String.valueOf(user.getId()));
        loginInfo.setUserAccount(user.getAccount());
        loginInfo.setOpenId(StringUtils.isNotBlank(openId) ? openId : user.getOpenId());
        loginInfo.setToken(token);
        return loginInfo;
    }

    public void logout(){
        tokenService.invalidToken(SpringContextUtil.getToken());
    }

}
