package com.starcharge.wios.service;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.dao.OrderDao;
import com.starcharge.wios.dao.OrderInstallDao;
import com.starcharge.wios.dao.SupplierDao;
import com.starcharge.wios.dao.SupplierStaffDao;
import com.starcharge.wios.dao.entity.Order;
import com.starcharge.wios.dao.entity.OrderInstall;
import com.starcharge.wios.dao.entity.SupplierStaff;
import com.starcharge.wios.dao.entity.WallboxApply;
import com.starcharge.wios.dao.mappers.WallboxApplyMapper;
import com.starcharge.wios.dto.WallboxApplyBatchCheckDTO;
import com.starcharge.wios.dto.WallboxApplyCheckDTO;
import com.starcharge.wios.dto.WallboxApplyDetailQueryDTO;
import com.starcharge.wios.vo.WallboxApplyCheckVO;
import com.starcharge.wios.vo.WallboxApplyDetailVO;
import com.starcharge.wios.vo.WallboxApplyInstallInfoVO;
import com.starcharge.wios.vo.WallboxApplyInstallListVO;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * description
 * <p>
 *
 * @author <a href="mail to: ning.chai@foxmail.com" rel="nofollow">chaining</a>
 */
@Service
public class WallboxApplyService {

    @Autowired
    private WallboxApplyMapper wallboxApplyMapper;
    @Autowired
    private OrderDao orderDao;
    @Autowired
    private OrderInstallDao orderInstallDao;
    @Autowired
    private TokenService tokenService;
    @Autowired
    private SupplierDao supplierDao;
    @Autowired
    private SupplierStaffDao supplierStaffDao;

    private final Integer CHECK_STATUS_REJECT = 2;

    /**
     * 查询墙盒申请
     *
     * @param id 墙盒申请主键
     * @return 墙盒申请
     */

    public WallboxApplyInstallInfoVO selectWallboxApplyById(Long id) {
        return wallboxApplyMapper.selectWallboxApplyById(id);
    }



    /**
     * 墙盒详情(墙盒状态)
     *
     * @param orderId 订单id
     * @return 墙盒申请
     */
    public WallboxApplyDetailVO selectWallboxDetail(String orderId) {
        SessionInfo loginUser = tokenService.getUser();
        String userId = loginUser.getUserId();
        List<SupplierStaff> supplier = supplierStaffDao.selectByAccountId(Integer.valueOf(userId));
        WallboxApplyDetailQueryDTO query = new WallboxApplyDetailQueryDTO();
        query.setOrderId(orderId);
        query.setSupplierId(supplier.stream().findFirst().get().getSupplierId().toString());
        return wallboxApplyMapper.selectWallboxDetail(query);
    }

    /**
     * 查询墙盒申请列表
     *
     * @param WallboxApply 墙盒申请
     * @return 墙盒申请
     */
    public List<WallboxApply> selectWallboxApplyList(WallboxApply WallboxApply) {
        return wallboxApplyMapper.selectWallboxApplyList(WallboxApply);
    }

    /**
     * 根据订单表查询墙盒申请列表
     *
     * @param WallboxApply 墙盒申请
     * @return 墙盒申请
     */
    public List<WallboxApplyInstallListVO> selectWallboxApplyListFromOrder(WallboxApply WallboxApply) {
        List<WallboxApplyInstallListVO> list =
                wallboxApplyMapper.selectWallboxApplyListFromOrder(WallboxApply);
        list.stream().map(l->{

            return l;
        });
        return list;
    }


    public WallboxApply calculateTimeRemaining(WallboxApply wallboxApply) {

        return wallboxApply;
    }

    /**
     * 新增墙盒申请
     *
     * @param req 墙盒申请
     * @return 结果
     */

    public int insertWallboxApply(WallboxApply req) {
        //判断是否可以新增
        //validateApplyExists(req.getOrderId());

        WallboxApply condition = new WallboxApply();
        condition.setOrderId(req.getOrderId());
        condition.setCheckStatus(1L);
        List<WallboxApply> retList = wallboxApplyMapper.selectWallboxApplyList(condition);
        if (CollectionUtils.isNotEmpty(retList)){
            throw new ServiceException("存在已审核通过的安装单，无法重复申请");
        }
        condition.setSupplierId(req.getSupplierId());
        List<WallboxApply> retList2 = wallboxApplyMapper.selectWallboxApplyList(condition);
        if (CollectionUtils.isNotEmpty(retList2)){
            throw new ServiceException("该订单已申请");
        }
        //补全相关信息
        Order order = orderDao.selectById(req.getOrderId());
        req.setRegionId(order.getRegionId());
        String wallboxModel = order.getWallboxModel();
        if (StrUtil.isBlank(wallboxModel)) {
            throw new ServiceException("订单数据异常，缺失墙盒信息，请联系系统管理员");
        }
        req.setWallboxItemId(order.getWallboxModel());
        req.setWallboxModel(order.getWallboxModel() == null ? "" : order.getWallboxModel());
        req.setOutOrderId(order.getOutOrderId());
        req.setInstallReserveTime(order.getInstallReserveTime());
        req.setAddress(order.getAddress());
        req.setTrackingNumber(order.getShipno());
        req.setInstallProcess(Integer.toUnsignedLong(order.getStatus()));
        req.setInstallStatus(Integer.toString(order.getStatus()));
        req.setOutOrderId(order.getOutOrderId());
        return wallboxApplyMapper.insertWallboxApply(req);
    }

    /**
     * 新增墙盒申请
     *
     * @param orderId 订单号
     * @return 结果
     */

    public int insertWallboxApply(int orderId) {
        //判断订单状态
//        orderInstallDao.selectByOrderId()
        return 0;
    }

    /**
     * 修改墙盒申请
     *
     * @return 结果
     */

    public int updateWallboxApply(WallboxApply wallboxApply) {
        WallboxApply record = wallboxApplyMapper.selectWallboxApplyByOrderId(wallboxApply.getOutOrderId());
//        if (!CHECK_STATUS_REJECT.equals(record.getCheckStatus())) {
//            throw new ServiceException("非驳回状态订单无法修改信息！");
//        }
        wallboxApply.setCheckTime(new Date());
        return wallboxApplyMapper.updateWallboxApply(wallboxApply);
    }

    /**
     *  审核
     * @param req
     * @return
     */
    public int checkWallboxApply(WallboxApplyCheckVO req){
        WallboxApply cond = new WallboxApply();
        cond.setOrderId(req.getOrderId());
        List<WallboxApply> list = wallboxApplyMapper.selectWallboxApplyList(cond);
        List<WallboxApply> filteredList = list.stream()
                .filter(l -> l.getCheckStatus() == 2)
                .collect(Collectors.toList());
        if (!filteredList.isEmpty()) {
            throw new ServiceException("同订单已审核");
        }

        WallboxApply update = new WallboxApply();
        update.setId(req.getId());
        update.setOrderId(req.getOrderId());
        update.setCheckStatus(req.getCheckStatus());
        update.setCheckRemarks(req.getCheckRemarks());
        update.setCheckTime(new Date());
        update.setCheckAccount(tokenService.getUser().getUserId());
        return wallboxApplyMapper.updateWallboxApply(update);
    }

    /**
     * 批量审核
     */
    public WallboxApplyBatchCheckDTO checkBatch(WallboxApplyBatchCheckDTO req) {
        List<WallboxApplyCheckDTO> checkList = req.getCheckList();
        List<WallboxApply> toUpdateList = new ArrayList<>();
        boolean allowedUpdate = true;

        for (WallboxApplyCheckDTO wallboxApply : checkList) {
            WallboxApply cond = new WallboxApply();
            cond.setOrderId(wallboxApply.getOrderId());
            List<WallboxApply> list = wallboxApplyMapper.selectWallboxApplyList(cond);
            List<WallboxApply> filteredList = list.stream()
                    .filter(l -> l.getCheckStatus() == 2)
                    .collect(Collectors.toList());

            if (!filteredList.isEmpty()) {
                wallboxApply.setDataStatus("0");
                wallboxApply.setErrorMessage("该订单已审核通过");
                allowedUpdate = false;
            }

            if (allowedUpdate){
                WallboxApply update = new WallboxApply();
                update.setId(wallboxApply.getId());
                update.setCheckStatus(req.getCheckStatus());
                update.setCheckRemarks(req.getCheckRemarks());
                update.setCheckAccount(tokenService.getUser().getUserId());
                toUpdateList.add(update);
            }
        }

        if (allowedUpdate){
            // 批量更新操作
            toUpdateList.forEach(l -> wallboxApplyMapper.updateWallboxApply(l));
        }

        return req;
    }



    void validateApplyExists(String id) {
        OrderInstall orderInstall = orderInstallDao.selectByOrderId(id);
        if (ObjectUtil.isNull(orderInstall)) {
            throw new ServiceException("不存在安装单号");
        }
    }
}
