package com.starcharge.wios.service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.pageLimit.PageLimit;
import com.ihidea.core.support.pageLimit.PageLimitHolderFilter;
import com.ihidea.core.support.session.SessionInfo;
import com.ihidea.core.util.JSONUtilsEx;
import com.starcharge.warehouse.dto.materials.MaterialsDTO;
import com.starcharge.warehouse.dto.single.SingleDTO;
import com.starcharge.warehouse.dto.single.SingleMaterialsDTO;
import com.starcharge.warehouse.dto.warehouse.WarehouseDTO;
import com.starcharge.warehouse.dto.warehouse.WarehouseLevenDTO;
import com.starcharge.warehouse.query.MaterialsListQuery;
import com.starcharge.warehouse.query.MaterialsQuery;
import com.starcharge.warehouse.query.MaterialsSaveQuery;
import com.starcharge.warehouse.query.SingleQuery;
import com.starcharge.warehouse.query.SingleSaveQuery;
import com.starcharge.warehouse.query.WarehouseLevenSaveQuery;
import com.starcharge.warehouse.query.WarehouseMaterialsQuery;
import com.starcharge.warehouse.query.WarehouseQuery;
import com.starcharge.warehouse.query.WarehouseSaveQuery;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.bo.SupplierBo;
import com.starcharge.wios.dao.OemContractItemDao;
import com.starcharge.wios.dao.OemDao;
import com.starcharge.wios.dao.RegionDao;
import com.starcharge.wios.dao.SupplierContractItemDao;
import com.starcharge.wios.dao.SupplierDao;
import com.starcharge.wios.dao.entity.Oem;
import com.starcharge.wios.dao.entity.OemContractItem;
import com.starcharge.wios.dao.entity.Region;
import com.starcharge.wios.dao.entity.Supplier;
import com.starcharge.wios.dao.entity.SupplierContractItem;
import com.starcharge.wios.dto.SingleSaveQuery2;
import com.starcharge.wios.dto.WarehouseDTO2;
import com.starcharge.wios.dto.WarehouseLevelDTO2;
import com.starcharge.wios.dto.WarehouseQuery2;
import com.starcharge.wios.dto.WarehouseSelectorData;
import com.starcharge.wios.interfac.MaterialsConsumer;
import com.starcharge.wios.interfac.SingleConsumer;
import com.starcharge.wios.interfac.WarehouseConsumer;
import com.starcharge.wios.vo.SupplierVo;

/**
 * 仓库service
 * 
 * @author xgl
 * @version [版本号, 2020年10月23日]
 */
@Service
public class WarehouseService {
    
    @Value("${appkey.warehouse}")
    private String appkey;
    
    @Autowired
    private WarehouseConsumer warehouseConsumer;
    
    @Autowired
    private SingleConsumer singleConsumer;
    
    @Autowired
    private MaterialsConsumer materialConsumer;
    
    @Autowired
    private TokenService tokenService;
    
    @Autowired
    private RegionDao regionDao;
    
    @Autowired
    private SupplierDao supplierDao;
    
    @Autowired
    private OemDao oemDao;
    
    @Autowired
    private OemContractItemDao ociDao;
    
    @Autowired
    private SupplierContractItemDao sciDao;
    
    private Logger logger = LoggerFactory.getLogger(getClass());
    
    /**
     * 新增修改仓库
     *
     * @param warehouse
     * @return
     */
    public void saveWarehouse(WarehouseSaveQuery warehouse) {
        if (StringUtils.isEmpty(warehouse.getAccountId())) {
            if (tokenService.getUser() != null)
                warehouse.setAccountId(tokenService.getUser().getUserId());
        }
        warehouse.setAppkey(appkey);
        BaseResponse<Void> response = warehouseConsumer.saveWarehouse(warehouse);
        if (response.success()) {
        } else {
            logger.error("saveWarehouse调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
        
    }
    
    /**
     * 新增/修改仓库等级
     *
     * @param warehouseLevenSaveQuery
     * @return
     */
    public void saveWarehouseLevel(WarehouseLevenSaveQuery warehouseLevenSaveQuery) {
        if (StringUtils.isEmpty(warehouseLevenSaveQuery.getAccountId())) {
            if (tokenService.getUser() != null)
                warehouseLevenSaveQuery.setAccountId(tokenService.getUser().getUserId());
        }
        warehouseLevenSaveQuery.setAppkey(appkey);
        BaseResponse<Void> response = warehouseConsumer.saveWarehouseLeven(warehouseLevenSaveQuery);
        if (response.success()) {
        } else {
            logger.error("saveWarehouseLevel调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }
    
    /**
     * 查询仓库
     *
     * @param warehouseQuery
     * @return
     */
    public List<WarehouseDTO2> qryWarehouseDetailList(WarehouseQuery2 warehouseQuery2) {
        SessionInfo user = tokenService.getUser();
        if (user != null) {
            String installSupplierId = user.getAttribute().get("installSupplierId");
            warehouseQuery2.setServiceProvider(installSupplierId);
        }
        
        if (!StringUtils.isEmpty(warehouseQuery2.getServiceProviderName())) {
            SupplierBo condition = new SupplierBo();
            condition.setName(warehouseQuery2.getServiceProviderName());
            List<SupplierVo> supplierList = supplierDao.getList(condition);
            if (!supplierList.isEmpty()) {
                List<String> supplierIdList = supplierList.stream().map(SupplierVo::getId).collect(Collectors.toList());
                warehouseQuery2.setServiceProvider(StringUtils.collectionToCommaDelimitedString(supplierIdList));
            } else {
                return new ArrayList<>();
            }
        }
        
        List<WarehouseDTO2> returnList = new ArrayList<WarehouseDTO2>();
        List<WarehouseDTO> result = qryWarehouseList(warehouseQuery2);
        
        // PageLimit pageLimit = PageLimitHolderFilter.getContext();
        // if (pageLimit != null)
        // pageLimit.setLimited(true);
        
        List<Region> regionList = regionDao.selectAll(null);
        Map<Integer, Region> regionMap = regionList.stream().collect(Collectors.toMap(Region::getId, region -> region));
        List<Supplier> supplierList = supplierDao.selectAll();
        Map<String, String> supplierMap = supplierList.stream().collect(Collectors.toMap(Supplier::getId, Supplier::getName));
        result.stream().forEach(item -> {
            WarehouseDTO2 dto = new WarehouseDTO2();
            BeanUtils.copyProperties(item, dto);
            Region region = regionMap.get(Integer.valueOf(dto.getArea()));
            if (region != null)
                dto.setAreaName(region.getName());
            if (region != null)
                dto.setAreaPathCode(region.getIdTree().replaceAll("_", "\\/"));
            dto.setProviderName(supplierMap.get(dto.getServiceProvider()));
            returnList.add(dto);
        });
        // return pageLimit.limitList(returnList);
        return returnList;
    }
    
    public List<WarehouseDTO> qryWarehouseList(WarehouseQuery warehouseQuery) {
        warehouseQuery.setAppkey(appkey);
        logger.info("qryWarehouseList请求数据:{}", JSONUtilsEx.serialize(warehouseQuery));
        BaseResponse<List<WarehouseDTO>> response = warehouseConsumer.qryWarehouseList(warehouseQuery);
        if (response.success()) {
            PageLimit pageLimit = response.getPageLimit();
            if (pageLimit != null)
                PageLimitHolderFilter.setContext(pageLimit.getCurrentPageNo(), pageLimit.getPageLength(), pageLimit.getTotalCount());
            PageLimit pageLimit1 = PageLimitHolderFilter.getContext();
            if (pageLimit1 != null)
                pageLimit1.setLimited(true);
            // logger.info("qryWarehouseList返回数据:{}", JSONUtilsEx.serialize(response.getData()));
            return response.getData();
        } else {
            logger.error("qryWarehouseList调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }
    
    /**
     * 查询仓库等级
     *
     * @param appkey
     * @param serviceProvider
     * @param parentId
     * @return
     */
    public List<WarehouseLevelDTO2> qryWarehouseLevel(String serviceProvider, Integer parentId, String name) {
        BaseResponse<List<WarehouseLevenDTO>> response = warehouseConsumer.qryWarehouseLeven(null, appkey, serviceProvider, parentId, name);
        if (response.success()) {
            List<WarehouseLevenDTO> warehouseLevelList = response.getData();
            
            // 根据条件查询是补全上一级等级
            {
                Set<String> pathCodeSet = new HashSet<>();
                if (!StringUtils.isEmpty(name)) {
                    for (WarehouseLevenDTO dto : warehouseLevelList) {
                        String pathCode = dto.getPathCode();
                        String[] pathCodeArr = pathCode.split("\\/");
                        for (int i = 0; i < pathCodeArr.length - 1; i++) {
                            pathCodeSet.add(pathCodeArr[i]);
                        }
                    }
                    Iterator<String> it = pathCodeSet.iterator();
                    while (it.hasNext()) {
                        String pathCode = it.next();
                        BaseResponse<List<WarehouseLevenDTO>> response2 =
                            warehouseConsumer.qryWarehouseLeven(Integer.valueOf(pathCode), appkey, null, null, null);
                        if (response2.success()) {
                            if (!response2.getData().isEmpty()) {
                                warehouseLevelList.add(response2.getData().get(0));
                            }
                        }
                    }
                    // 从小到大排序（总仓要放第一个）
                    warehouseLevelList =
                        warehouseLevelList.stream().sorted(Comparator.comparing(WarehouseLevenDTO::getId)).collect(Collectors.toList());
                }
            }
            
            // PageLimit pageLimit = response.getPageLimit();
            // if (pageLimit != null)
            // PageLimitHolderFilter.setContext(pageLimit.getCurrentPageNo(), pageLimit.getPageLength(),
            // pageLimit.getTotalCount());
            // PageLimit pageLimit1 = PageLimitHolderFilter.getContext();
            // if (pageLimit1 != null)
            // pageLimit1.setLimited(true);
            
            List<WarehouseLevelDTO2> returnList = new ArrayList<WarehouseLevelDTO2>();
            for (WarehouseLevenDTO dto : warehouseLevelList) {
                String pId = dto.getParentId();
                if (!StringUtils.isEmpty(pId)) {
                    WarehouseLevelDTO2 dto2 = getLevelVo(returnList, pId);
                    if (dto2 == null)
                        continue;
                    WarehouseLevelDTO2 dto2New = new WarehouseLevelDTO2();
                    BeanUtils.copyProperties(dto, dto2New);
                    List<WarehouseLevelDTO2> children = dto2.getChildren();
                    if (children == null)
                        children = new ArrayList<>();
                    children.add(dto2New);
                    dto2.setChildren(children);
                } else {
                    WarehouseLevelDTO2 dto2 = new WarehouseLevelDTO2();
                    BeanUtils.copyProperties(dto, dto2);
                    returnList.add(dto2);
                }
            }
            return returnList;
        } else {
            logger.error("qryWarehouseLevel调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }
    
    public WarehouseLevelDTO2 getLevelVo(List<WarehouseLevelDTO2> returnList, String id) {
        for (WarehouseLevelDTO2 dto : returnList) {
            if (dto.getId().equals(id)) {
                return dto;
            }
            List<WarehouseLevelDTO2> children = dto.getChildren();
            if (children != null) {
                WarehouseLevelDTO2 dto2 = getLevelVo(children, id);
                if (dto2 != null)
                    return dto2;
            }
        }
        return null;
    }
    
    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    public WarehouseLevenDTO qryLevelDetail(Integer id) {
        BaseResponse<List<WarehouseLevenDTO>> response = warehouseConsumer.qryWarehouseLeven(id, appkey, null, null, null);
        if (response.success()) {
            List<WarehouseLevenDTO> list = response.getData();
            return list.isEmpty() ? null : list.get(0);
        } else {
            logger.error("qryWarehouseLeven调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }
    
    /**
     * 新增调拨单
     *
     * @param singleSaveQuery
     * @return
     */
    public void saveSingle(SingleSaveQuery2 singleSaveQuery) {
        if (StringUtils.isEmpty(singleSaveQuery.getAccountId())) {
            if (tokenService.getUser() != null) {
                String accountName = tokenService.getUser().getUserName();
                String oemId = tokenService.getUser().getAttribute().get("oemId");
                String installSupplierId = tokenService.getUser().getAttribute().get("installSupplierId");
                if (!StringUtils.isEmpty(oemId)) {
                    Oem oem = oemDao.selectById(Integer.valueOf(oemId));
                    if (oem != null)
                        accountName = oem.getName();
                }
                if (!StringUtils.isEmpty(installSupplierId)) {
                    Supplier supplier = supplierDao.selectById(Integer.valueOf(installSupplierId));
                    if (supplier != null)
                        accountName = supplier.getName();
                }
                singleSaveQuery.setAccountId(accountName);
            }
        }
        if (!StringUtils.isEmpty(singleSaveQuery.getWarehouseMaterialsQueryJson())) {
            try {
                ObjectMapper mapper = new ObjectMapper();
                mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
                List<WarehouseMaterialsQuery> list =
                    mapper.readValue(singleSaveQuery.getWarehouseMaterialsQueryJson(), new TypeReference<List<WarehouseMaterialsQuery>>() {
                    });
                singleSaveQuery.setWarehouseMaterialsQueryList(list);
            } catch (Exception e) {
                throw new ServiceException("数据转换异常");
            }
        }
        singleSaveQuery.setAppkey(appkey);
        SingleSaveQuery query = new SingleSaveQuery();
        BeanUtils.copyProperties(singleSaveQuery, query);
        BaseResponse<Void> response = singleConsumer.saveSingle(query);
        if (response.success()) {
        } else {
            logger.error("saveSingle调用失败text={}", response.getText());
            throw new com.ihidea.core.support.exception.ServiceException(response.getText());
        }
    }
    
    /**
     * 查询调拨单
     *
     * @param singleQuery
     * @return
     */
    public List<SingleDTO> qrySingal(SingleQuery singleQuery) {
        SessionInfo user = tokenService.getUser();
        if (user != null) {
            String supplierId = user.getAttribute().get("installSupplierId");
            if (!StringUtils.isEmpty(supplierId)) {
                singleQuery.setServiceProvider(supplierId);
            }
        }
        singleQuery.setAppkey(appkey);
        BaseResponse<List<SingleDTO>> response = singleConsumer.qrySingal(singleQuery);
        if (response.success()) {
            PageLimit pageLimit = response.getPageLimit();
            if (pageLimit != null)
                PageLimitHolderFilter.setContext(pageLimit.getCurrentPageNo(), pageLimit.getPageLength(), pageLimit.getTotalCount());
            PageLimit pageLimit1 = PageLimitHolderFilter.getContext();
            if (pageLimit1 != null)
                pageLimit1.setLimited(true);
        } else {
            logger.error("qrySingal调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
        return response.getData();
    }
    
    /**
     * 查询单据物料
     *
     * @param singleId
     * @param page
     * @param pagecount
     * @return
     */
    public List<SingleMaterialsDTO> qrySingleMaterialsBySingleId(String singleId, Integer page, Integer pagecount) {
        BaseResponse<List<SingleMaterialsDTO>> response = singleConsumer.qrySingleMaterialsBySingleId(singleId, page, pagecount);
        if (response.success()) {
            PageLimit pageLimit = response.getPageLimit();
            if (pageLimit != null)
                PageLimitHolderFilter.setContext(pageLimit.getCurrentPageNo(), pageLimit.getPageLength(), pageLimit.getTotalCount());
            PageLimit pageLimit1 = PageLimitHolderFilter.getContext();
            if (pageLimit1 != null)
                pageLimit1.setLimited(true);
        } else {
            logger.error("qrySingleMaterialsBySingleId调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
        return response.getData();
    }
    
    /**
     * 新增物料
     *
     * @param materialsListQuery
     * @return
     */
    public void saveMaterials(String materialsListQueryJson) {
        MaterialsListQuery materialsListQuery = new MaterialsListQuery();
        if (tokenService.getUser() != null)
            materialsListQuery.setAccountId(tokenService.getUser().getUserId());
        materialsListQuery.setAppkey(appkey);
        try {
            ObjectMapper mapper = new ObjectMapper();
            mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
            List<MaterialsSaveQuery> list = mapper.readValue(materialsListQueryJson, new TypeReference<List<MaterialsSaveQuery>>() {
            });
            materialsListQuery.setMaterialsSaveQueryList(list);
        } catch (Exception e) {
            throw new ServiceException("数据转换异常");
        }
        // List<MaterialsSaveQuery> qList = materialsListQuery.getMaterialsSaveQueryList();
        // for (MaterialsSaveQuery q : qList) {
        // String materialsCode = q.getMaterialsCode();
        // if (!StringUtils.isEmpty(materialsCode)) {
        // List<MaterialsDTO> mList = qryMaterialsByCode(materialsCode);
        // if (!mList.isEmpty()) {
        // throw new ServiceException("此编码已存在：" + materialsCode);
        // }
        // }
        // }
        BaseResponse<Void> response = materialConsumer.saveMaterials(materialsListQuery);
        if (response.success()) {
        } else {
            logger.error("saveMaterials调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }

    /**
     * 新增物料
     * @return
     */
    public void saveMaterials(MaterialsSaveQuery materialsSaveQuery) {
        MaterialsListQuery materialsListQuery = new MaterialsListQuery();
        materialsListQuery.setAccountId("sap");
        materialsListQuery.setAppkey(appkey);
        List<MaterialsSaveQuery> list = new ArrayList<>();
        list.add(materialsSaveQuery);
        materialsListQuery.setMaterialsSaveQueryList(list);
        BaseResponse<Void> response = materialConsumer.saveMaterials(materialsListQuery);
        if (response.success()) {
        } else {
            logger.error("sap-saveMaterials调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }
    
    /**
     * 更新单个物料
     *
     * @param materialsSaveQuery
     */
    public void updateMaterial(MaterialsSaveQuery materialsSaveQuery) {
        // if (!StringUtils.isEmpty(materialsSaveQuery.getMaterialsCode())) {
        // List<MaterialsDTO> mList = qryMaterialsByCode(materialsSaveQuery.getMaterialsCode());
        // if (!mList.isEmpty() && !mList.get(0).getId().equals(materialsSaveQuery.getId())) {
        // throw new ServiceException("此编码已存在：" + materialsSaveQuery.getMaterialsCode());
        // }
        // }
        MaterialsListQuery materialsListQuery = new MaterialsListQuery();
        if (tokenService.getUser() != null)
            materialsListQuery.setAccountId(tokenService.getUser().getUserId());
        materialsListQuery.setAppkey(appkey);
        List<MaterialsSaveQuery> list = new ArrayList<>();
        list.add(materialsSaveQuery);
        materialsListQuery.setMaterialsSaveQueryList(list);
        BaseResponse<Void> response = materialConsumer.saveMaterials(materialsListQuery);
        if (response.success()) {
        } else {
            logger.error("updateMaterial调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }
    
    /**
     * 查询物料
     *
     * @param materialsQuery
     * @return
     */
    public List<MaterialsDTO> qryMaterials(MaterialsQuery materialsQuery) {

        materialsQuery.setAppkey(appkey);
        logger.info("qryMaterials请求数据:{}", JSONUtilsEx.serialize(materialsQuery));
        BaseResponse<List<MaterialsDTO>> response = materialConsumer.qryMaterials(materialsQuery);
        if (response.success()) {
            PageLimit pageLimit = response.getPageLimit();
            if (pageLimit != null)
                PageLimitHolderFilter.setContext(pageLimit.getCurrentPageNo(), pageLimit.getPageLength(), pageLimit.getTotalCount());
            PageLimit pageLimit1 = PageLimitHolderFilter.getContext();
            if (pageLimit1 != null)
                pageLimit1.setLimited(true);
            // logger.info("qryMaterials返回数据:{}", JSONUtilsEx.serialize(response.getData()));
        } else {
            logger.error("qryMaterials调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
        return response.getData();
    }
    
    /**
     * 根据物料编码查询物料
     *
     * @param materialCode
     * @return
     */
    public List<MaterialsDTO> qryMaterialsByCode(String materialCode) {
        MaterialsQuery materialsQuery = new MaterialsQuery();
        materialsQuery.setPage(1);
        materialsQuery.setPagecount(1);
        materialsQuery.setMaterialsCode(materialCode);
        return qryMaterials(materialsQuery);
    }
    
    /**
     * 删除物料
     *
     * @param materialsId
     * @return
     */
    public void delMaterials(String materialsId) {
        List<MaterialsDTO> dtoList = getStockMaterialById(null, materialsId);
        if (!dtoList.isEmpty()) {
            throw new ServiceException("物料已入库，无法删除");
        }
        List<OemContractItem> ociList = ociDao.selectByItemId(materialsId);
        if (ociList.isEmpty()) {
            List<SupplierContractItem> sciList = sciDao.selectByItemId(materialsId);
            if (!sciList.isEmpty()) {
                throw new ServiceException("物料已被合同使用，无法删除");
            }
        } else {
            throw new ServiceException("物料已被合同使用，无法删除");
        }
        
        BaseResponse<Void> response = materialConsumer.delMaterials(materialsId);
        if (response.success()) {
        } else {
            logger.error("delMaterials调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }
    
    public List<MaterialsDTO> qryMaterialsCount(String materialsName, String materialsCode, String regularModel, Integer type) {
        BaseResponse<List<MaterialsDTO>> response =
            materialConsumer.qryMaterialsCount(appkey, materialsName, materialsCode, regularModel, type);
        if (response.success()) {
            return response.getData();
        } else {
            logger.error("qryMaterialsCount调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }
    
    /**
     * 根据id删除仓库
     *
     * @param id
     */
    public void delWarehouseById(String id) {
        List<MaterialsDTO> materialList = getStockMaterialByWId(id);
        if (!materialList.isEmpty()) {
            BigDecimal count = materialList.stream().filter(m -> m.getRemainingNumber() != null).map(MaterialsDTO::getRemainingNumber)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
            if (count.compareTo(BigDecimal.ZERO) > 0)
                throw new ServiceException("仓库下已有库存，无法删除");
        }
        BaseResponse<Void> response = warehouseConsumer.delWarehouseById(appkey, id);
        if (response.success()) {
        } else {
            logger.error("delWarehouseById调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }
    
    /**
     * 根据id删除仓库等级
     *
     * @param id
     */
    public void delWarehouseLevelById(Integer id) {
        BaseResponse<Void> response = warehouseConsumer.delWarehouseLevenById(appkey, id);
        if (response.success()) {
        } else {
            logger.error("delWarehouseLevenById调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
    }
    
    /**
     * 组装查询数据
     *
     * @param id
     * @return
     */
    public List<WarehouseSelectorData> combineSelectorData(Integer id, Integer type) {
        SessionInfo user = tokenService.getUser();
        String installSupplierId = null;
        if (user != null) {
            installSupplierId = user.getAttribute().get("installSupplierId");
        }
        BaseResponse<List<WarehouseLevenDTO>> levelResponse = null;
        if (id == null) {
            levelResponse = warehouseConsumer.qryWarehouseLeven(0, appkey, null, null, null);
        } else {
            levelResponse = warehouseConsumer.qryWarehouseLeven(null, appkey, null, id, null);
        }
        WarehouseQuery warehouseQuery = new WarehouseQuery();
        warehouseQuery.setWarehouseLevenId(id + "");
        warehouseQuery.setAppkey(appkey);
        warehouseQuery.setIsEnable(1);
        warehouseQuery.setPage(1);
        warehouseQuery.setPagecount(1000);
        if (!StringUtils.isEmpty(installSupplierId))
            warehouseQuery.setServiceProvider(installSupplierId);
        BaseResponse<List<WarehouseDTO>> warehouseResponse = warehouseConsumer.qryWarehouseList(warehouseQuery);
        List<WarehouseSelectorData> returnList = new ArrayList<>();
        
        levelResponse.getData().stream().forEach(item -> {
            WarehouseSelectorData data = new WarehouseSelectorData();
            data.setName(item.getName());
            data.setId(item.getId());
            data.setLevelOr(true);
            if (item.getId().equals("0") || type == null)
                returnList.add(data);
        });
        
        warehouseResponse.getData().stream().forEach(item -> {
            WarehouseSelectorData data = new WarehouseSelectorData();
            data.setName(item.getName());
            data.setLevelOr(false);
            data.setId(item.getId());
            returnList.add(data);
        });
        return returnList;
    }
    
    /**
     * 启用/禁用
     *
     * @param id
     * @param enable
     */
    public void updateEnable(String id, Integer enable) {
        if (enable != null && enable.intValue() == 0) {
            List<MaterialsDTO> materialList = getStockMaterialByWId(id);
            if (!materialList.isEmpty()) {
                BigDecimal count = materialList.stream().filter(m -> m.getRemainingNumber() != null).map(MaterialsDTO::getRemainingNumber)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
                if (count.compareTo(BigDecimal.ZERO) > 0)
                    throw new ServiceException("仓库下已有库存，无法禁用");
            }
        }
        warehouseConsumer.updateWarehouseisEnabled(appkey, id, enable);
    }
    
    /**
     * 根据物料id查询详情
     *
     * @param id
     * @return
     */
    public MaterialsDTO getMaterialById(String id) {
        MaterialsQuery materialsQuery = new MaterialsQuery();
        materialsQuery.setId(id);
        materialsQuery.setPage(1);
        materialsQuery.setPagecount(1);
        List<MaterialsDTO> list = qryMaterials(materialsQuery);
        return list.isEmpty() ? null : list.get(0);
    }
    
    /**
     * 根据物料id及仓库id查询库存详情
     *
     * @param id
     * @return
     */
    public List<MaterialsDTO> getStockMaterialById(String warehouseId, String id) {
        MaterialsQuery materialsQuery = new MaterialsQuery();
        materialsQuery.setId(id);
        if (!StringUtils.isEmpty(warehouseId))
            materialsQuery.setWarehouseId(warehouseId);
        materialsQuery.setPage(1);
        materialsQuery.setPagecount(1000);
        materialsQuery.setIsBindWareHouse(1);
        List<MaterialsDTO> list = qryStockMaterials(materialsQuery);
        return list;
    }
    
    /**
     * 根据仓库id查询物料库存
     *
     * @param warehouseId
     * @return
     */
    public List<MaterialsDTO> getStockMaterialByWId(String warehouseId) {
        MaterialsQuery materialsQuery = new MaterialsQuery();
        materialsQuery.setWarehouseId(warehouseId);
        materialsQuery.setPage(1);
        materialsQuery.setPagecount(1000);
        materialsQuery.setIsBindWareHouse(1);
        List<MaterialsDTO> list = qryStockMaterials(materialsQuery);
        return list;
    }
    
    /**
     * 添加出库单
     *
     */
    public void addOutSingle(String orderId, String itemId, String warehouseId, int quantity) {
        SingleSaveQuery query = new SingleSaveQuery();
        query.setAppkey(appkey);
        if (tokenService.getUser() != null) {
            query.setAccountId(tokenService.getUser().getUserName());
        } else {
            query.setAccountId("未知");
        }
        query.setTransferIn(orderId);
        query.setTransferOut(warehouseId);
        query.setType(3);
        
        List<WarehouseMaterialsQuery> warehouseMaterialsQueryList = new ArrayList<>();
        WarehouseMaterialsQuery wmQuery = new WarehouseMaterialsQuery();
        wmQuery.setMaterialsId(itemId);
        wmQuery.setNumber(new BigDecimal(quantity));
        warehouseMaterialsQueryList.add(wmQuery);
        
        query.setWarehouseMaterialsQueryList(warehouseMaterialsQueryList);
        logger.info("addOutSingle查询参数:{}", JSONUtilsEx.serialize(query));
        BaseResponse<Void> response = singleConsumer.saveSingle(query);
        logger.info("addOutSingle返回数据:{}", JSONUtilsEx.serialize(response));
        if (!response.success()) {
            throw new ServiceException(response.getText());
        }
    }
    
    /**
     * 查询库存物料
     *
     * @param materialsQuery
     * @return
     */
    public List<MaterialsDTO> qryStockMaterials(MaterialsQuery materialsQuery) {
        SessionInfo user = tokenService.getUser();
        if (user != null && StringUtils.isEmpty(materialsQuery.getWarehouseId())) {
            String supplierId = user.getAttribute().get("installSupplierId");
            if (!StringUtils.isEmpty(supplierId)) {
                WarehouseQuery warehouseQuery = new WarehouseQuery();
                warehouseQuery.setPage(1);
                warehouseQuery.setPagecount(1000);
                warehouseQuery.setServiceProvider(supplierId);
                warehouseQuery.setIsEnable(1);
                List<WarehouseDTO> warehouseList = qryWarehouseList(warehouseQuery);
                List<String> warehouseIds = warehouseList.stream().map(WarehouseDTO::getId).collect(Collectors.toList());
                if (!warehouseIds.isEmpty())
                    materialsQuery.setWarehouseId(StringUtils.collectionToCommaDelimitedString(warehouseIds));
                else
                    return new ArrayList<>();
            }
        }
        materialsQuery.setAppkey(appkey);
        logger.info("qryStockMaterials请求数据:{}", JSONUtilsEx.serialize(materialsQuery));
        BaseResponse<List<MaterialsDTO>> response = materialConsumer.qryMaterialsStockList(materialsQuery);
        if (response.success()) {
            PageLimit pageLimit = response.getPageLimit();
            if (pageLimit != null)
                PageLimitHolderFilter.setContext(pageLimit.getCurrentPageNo(), pageLimit.getPageLength(), pageLimit.getTotalCount());
            PageLimit pageLimit1 = PageLimitHolderFilter.getContext();
            if (pageLimit1 != null)
                pageLimit1.setLimited(true);
        } else {
            logger.error("qryStockMaterials调用失败text={}", response.getText());
            throw new ServiceException(response.getText());
        }
        return response.getData();
    }
    
    /**
     * 更新剩余可用数量(生成安装订单时锁定充电桩，订单关闭时解锁充电桩)
     *
     * @param materialsId
     * @param number
     * @param type
     * @param warehouseId
     */
    public void materailNumberUpdate(String materialsId, int number, int type, String warehouseId) {
        logger.info("[materailNumberUpdate]materailNumberUpdate:{},number:{},type:{},warehouseId:{}", materialsId, number, type,
            warehouseId);
        BaseResponse<Void> response = materialConsumer.updateMaterialsRemainingNumber(appkey, warehouseId, materialsId, number, type);
        logger.info("[materailNumberUpdate]response:{}", JSONUtilsEx.serialize(response));
    }
}
