package com.starcharge.wios.task;

import com.ihidea.component.cache.redis.RedisClient;
import com.starcharge.wios.service.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;


/**
 * 定时任务
 * 
 * @author xgl
 * @version [版本号, 2020年4月8日]
 */
@Component
public class ScheduleTask {
    
    @Autowired
    private SettlementService settlementService;
    
    @Autowired
    private OrderService orderService;

    @Autowired
    private ScoreSupplierService scoreSupplierService;
    
    @Autowired
    private OemContractService oemContractService;
    
    @Autowired
    private SupplierContractService supplierContractService;
    
    @Autowired
    private SecretPhoneService spService;

    @Autowired
    private OrderInvoiceService orderInvoiceService;

    private Logger logger = LoggerFactory.getLogger(ScheduleTask.class);
    
    /**
     * 每天1:00:00执行
     *
     */
    @Scheduled(cron = "0 0 1 * * ?")
    public void generateSettlementDetail() {
        if (RedisClient.getLock("task:generateSettlementDetail3", 60)) {
            logger.info("=================开始生成主机厂结算单详情===============");
            Calendar cal = Calendar.getInstance();
            cal.add(Calendar.DATE, -1);
            Date date = cal.getTime();
            settlementService.generateOemDetail(date, date);
        }
    }
    
    /**
     * 每月第一天00:10:00执行（注释掉，改成手动生成）
     *
     */
    /*@Scheduled(cron = "0 10 0 1 * ?")
    public void generateSettlement() {
        logger.info("=================开始生成主机厂结算单===============");
        if (RedisClient.getLock("task:generateSettlement", 60)) {
            Calendar cal = Calendar.getInstance();
            cal.add(Calendar.MONTH, -1);
            cal.set(Calendar.DAY_OF_MONTH, 1);
            Date startDate = cal.getTime();
            cal.setTime(new Date());
            cal.set(Calendar.DAY_OF_MONTH, 0);
            Date endDate = cal.getTime();
            settlementService.generateOemSettlement(startDate, endDate);
        }
    }*/
    
    /**
     * 每天1:30:00执行
     *
     */
    @Scheduled(cron = "0 30 1 * * ?")
    public void generateSupplierSettlementDetail() {
        if (RedisClient.getLock("task:generateSupplierSettlementDetail", 60)) {
            logger.info("=================开始生成服务商结算单详情===============");
            Calendar cal = Calendar.getInstance();
            cal.add(Calendar.DATE, -1);
            Date date = cal.getTime();
            settlementService.generateSupplierDetail(date, date);
        }
    }
    
    /**
     * 每月第一天00:30:00执行
     *
     */
    @Scheduled(cron = "0 30 0 1 * ?")
    public void generateSupplierSettlement() {
        if (RedisClient.getLock("task:generateSupplierSettlement", 60)) {
            logger.info("=================开始生成服务商结算单===============");
            Calendar cal = Calendar.getInstance();
            cal.add(Calendar.MONTH, -1);
            cal.set(Calendar.DAY_OF_MONTH, 1);
            Date startDate = cal.getTime();
            cal.setTime(new Date());
            cal.set(Calendar.DAY_OF_MONTH, 0);
            Date endDate = cal.getTime();
            settlementService.generateSupplierSettlement(startDate, endDate);
        }
    }
    
    /**
     * 订单时效，每分钟执行
     *
     */
    @Scheduled(cron = "0 0/1 * * * ?")
    public void timingExecute() {
        if (RedisClient.getLock("task:timingExecute", 50)) {
            logger.info("=================开始执行时效配置任务====================");
            orderService.orderTiming(null);
        }
    }

    /**
     * 每季度21号1:00:00进行统计
     *
     */
    @Scheduled(cron = "0 0 1 21 1,4,7,10 ? ")
    public void generateBeginScoreSupplier() {
        if (RedisClient.getLock("task:generateBeginScoreSupplier", 60)) {
            logger.info("=================开始生成服务商季度评分表===============");
            scoreSupplierService.beginScoreSupplier();
        }
    }
    /**
     * 每季度26号1:00:00执行
     *
     */
    @Scheduled(cron = "0 0 1 26 1,4,7,10 ? ")
    public void generateEndScoreSupplier() {
        if (RedisClient.getLock("task:generateEndScoreSupplier", 60)) {
            logger.info("=================开始生成服务商季度评分结果===============");
            scoreSupplierService.endScoreSupplier();
        }
    }
    
    /**
     * 每天00:10:00执行
     * 
     * @throws ParseException
     *
     */
    @Scheduled(cron = "0 10 0 * * ?")
    public void contractTiming() throws ParseException {
        if (RedisClient.getLock("task:contractTiming", 60)) {
            logger.info("=================开始执行合同状态更新===============");
            oemContractService.contractTiming();
            supplierContractService.contractTiming();
        }
    }

    /**
     * 释放隐私号码，每个整点执行一次
     *
     */
    @Scheduled(cron = "0 0 0/1 * * ?")
    public void releaseSecretPhoneNo() throws Exception {
        if (RedisClient.getLock("task:releaseSecretPhoneNo", 60)) {
            logger.info("=================开始执行释放隐私号码===============");
            spService.releaseExpired();
        }
    }

    /**
     * （预约勘测，预约安装）预约失败后，如果超过跟进时间则第二天9点进行超时提醒
     *
     */
    @Scheduled(cron = "0 00 09 * * ?")
    public void reserveOverFollowTimeNotice() throws Exception {
        if (RedisClient.getLock("task:reserveOverFollowTimeNotice", 60)) {
            logger.info("=================开始执行9点跟进时间超时提醒===============");
            orderService.reserveOverFollowTimeNotice();
        }
    }

    /**
     * 发票状态更新，每分钟执行
     *
     */
    @Scheduled(cron = "0 0/1 * * * ?")
    public void invoiceUpdater() {
        if (RedisClient.getLock("task:invoiceUpdater", 50)) {
            logger.info("=================开始执行发票状态更新任务====================");
            orderInvoiceService.invoiceUpdater();
        }
    }
}
