package com.starcharge.wios.utils;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import com.aliyun.oss.OSSClient;
import com.aliyun.oss.model.OSSObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.component.datastore.DataStoreService;
import com.starcharge.component.datastore.dao.model.CptDataStore;
import com.starcharge.wios.bo.DataStorePath;

/**
 * 阿里云文件上传下载相关类
 * 
 * @author xgl
 * @version [版本号, 2019年2月20日]
 */
@Component
public class AliUtil {
    
    @Autowired
    DataStoreService dsService;
    
    private static final String DS = "ds_upload";
    
    private Logger log = LoggerFactory.getLogger(getClass());
    
    private CptDataStore settleStore;
    
    private CptDataStore dsStore;
    
    private CptDataStore phoenixStore;
    
    private Map<String, CptDataStore> storesMap = new HashMap<>();
    
    @PostConstruct
    private void construct() {
        // settleStore = dsService.getInfoByName(SETTLE);
        dsStore = dsService.getInfoByName(DS);
        // phoenixStore = dsService.getInfoByName(PHOENIX);
        // storesMap.put(SETTLE, settleStore);
        storesMap.put(DS, dsStore);
        // storesMap.put(PHOENIX, phoenixStore);
    }
    
    public CptDataStore getSettleStore() {
        return settleStore;
    }
    
    public DataStorePath getSettleStorePath() {
        return getPath(getSettleStore().getPath());
    }
    
    public CptDataStore getDsStore() {
        return dsStore;
    }
    
    public DataStorePath getDsStorePath() {
        return getPath(getDsStore().getPath());
    }
    
    public CptDataStore getPhoenixStore() {
        return phoenixStore;
    }
    
    public DataStorePath getPhoenixStorePath() {
        return getPath(getPhoenixStore().getPath());
    }
    
    private DataStorePath getPath(String path) {
        if (StringUtils.isEmpty(path)) {
            return null;
        }
        
        ObjectMapper mapper = new ObjectMapper();
        try {
            return mapper.readValue(path, DataStorePath.class);
        } catch (IOException e) {
            log.error("[AliUtil getPath]path={},ex:", path, e);
            throw new ServiceException(e.getMessage());
        }
    }
    
    /**
     * 下载oss文件
     *
     * @param response
     * @param storeName
     * @param fileName
     * @param filePath
     * @throws Exception
     */
    public void download(HttpServletResponse response, String storeName, String fileName, String filePath, String ossFileName,HttpServletRequest request)
        throws Exception {
        response.reset();// 清空输出流
        response.addHeader("Content-disposition", "attachment; filename=" + new String(fileName.getBytes(), "ISO-8859-1"));
        response.setContentType("application/vnd.ms-excel;charset=utf-8");
        response.addHeader("Access-Control-Allow-Origin", "*");
        // response.addHeader("Cache-Control", "no-cache");
        // response.addHeader("Access-Control-Allow-Headers", "X-Requested-With");
        // response.addHeader("Access-Control-Allow-Credentials", "true");
        // response.addHeader("Access-Control-Allow-Methods", "POST, GET, OPTIONS");
        String objectName = filePath + "/" + ossFileName;
        
        CptDataStore store = storesMap.get(storeName);
        if (store == null) {
            throw new ServiceException("没有找到" + storeName + "对应的store");
        }
        DataStorePath storePath = getPath(store.getPath());
        if (storePath == null) {
            throw new ServiceException("没有找到" + store.getPath() + "对应的storePath");
        }
        OSSClient ossClient = new OSSClient(storePath.getEndpoint(), storePath.getAccessKeyId(), storePath.getAccessKeySecret());
        OSSObject ossObject = ossClient.getObject(storePath.getBucketName(), objectName);
        
        OutputStream outputStream = response.getOutputStream();
        ByteArrayOutputStream swapStream = new ByteArrayOutputStream();
        byte[] buff = new byte[100]; // buff用于存放循环读取的临时数据
        int rc = 0;
        while ((rc = ossObject.getObjectContent().read(buff, 0, 100)) > 0) {
            swapStream.write(buff, 0, rc);
        }
        byte[] in_b = swapStream.toByteArray(); // in_b为转换之后的结果
        swapStream.close();
        
        outputStream.write(in_b);
        outputStream.flush();
        outputStream.close();
        // 关闭OSSClient。
        ossClient.shutdown();
    }
    
    /**
     * 上传文件至oss
     *
     * @param file
     * @param storeName
     * @param fileName
     * @param filePath
     * @throws Exception
     */
    public void upload(MultipartFile file, String storeName, String fileName, String filePath) throws Exception {
        String objectName = filePath + "/" + fileName;
        CptDataStore store = storesMap.get(storeName);
        if (store == null) {
            throw new ServiceException("没有找到" + storeName + "对应的store");
        }
        DataStorePath storePath = getPath(store.getPath());
        if (storePath == null) {
            throw new ServiceException("没有找到" + store.getPath() + "对应的storePath");
        }
        OSSClient ossClient = new OSSClient(storePath.getEndpoint(), storePath.getAccessKeyId(), storePath.getAccessKeySecret());
        // 上传文件流。
        ossClient.putObject(storePath.getBucketName(), objectName, file.getInputStream());
        // 关闭OSSClient。
        ossClient.shutdown();
    }
    
    /**
     * 读取上传到oss的文件内容
     *
     * @param store
     * @param filePath
     * @return
     */
    public String getContentFromOss(String storeName, String filePath) {
        CptDataStore store = storesMap.get(storeName);
        if (store == null) {
            throw new ServiceException("没有找到" + storeName + "对应的store");
        }
        DataStorePath storePath = getPath(store.getPath());
        if (storePath == null) {
            throw new ServiceException("没有找到" + store.getPath() + "对应的storePath");
        }
        OSSClient ossClient = new OSSClient(storePath.getEndpoint(), storePath.getAccessKeyId(), storePath.getAccessKeySecret());
        OSSObject ossObject = ossClient.getObject(storePath.getBucketName(), filePath);
        InputStream is = ossObject.getObjectContent();
        try {
            BufferedReader buff = new BufferedReader(new InputStreamReader(is, "UTF-8"));
            String line = "";
            StringBuffer content = new StringBuffer();
            while ((line = buff.readLine()) != null) {
                content.append(line);
            }
            return content.toString();
        } catch (IOException e) {
            log.error("[getContentFromOss]name={},ex", store.getName(), e);
            throw new ServiceException(e.getMessage());
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            // 关闭OSSClient。
            ossClient.shutdown();
        }
        
    }
}
