package com.starcharge.wios.validation;

import com.alibaba.fastjson.JSON;
import com.ihidea.core.support.exception.ServiceException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.validation.BindingResult;

import java.lang.reflect.Method;
import java.lang.reflect.Parameter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 类说明
 * 
 * @author yincheng lu
 * @time 2019年1月4日上午11:11:39
 * @Description
 */
@Aspect
@Component
@Order(2)
public class ParamsValidation {
	private static final Log logger = LogFactory.getLog(ParamsValidation.class);
	public static final String FILE_SUFFIX = "_file";


	@Pointcut("@annotation(com.starcharge.wios.validation.ParamsValidate)") // ||(execution(*
	public static void paramsValidate() {
	}

	@Autowired
	MessageSource messageSource;

	@Before("paramsValidate()")
	public void doBeforeInServiceLayer(JoinPoint joinPoint) {

	}

	@After("paramsValidate()")
	public void doAfterInServiceLayer(JoinPoint joinPoint) {

	}
	public static final String BaseEntityPackage="com.starcharge.wios";

	@Around("paramsValidate()")
	public Object doAround(ProceedingJoinPoint pjp) throws Throwable {
		logger.debug("start to validation params");
		Map<String, String> resultMap = new HashMap<>();
		Object[] arguments = pjp.getArgs();//实际参数
		Method realMethod = getProceedJoinPointMethod(pjp);
		Parameter[] parameters = realMethod.getParameters();//方法参数
		List<BindingResult> brs = getClassFromArray(arguments, BindingResult.class);
		if (brs != null && brs.size() > 0) {
			brs.forEach(br -> {
				if (br.hasErrors()) {
					br.getFieldErrors().forEach(e -> {

						resultMap.put(e.getField() + "", messageSource.getMessage(e, LocaleContextHolder.getLocale()));
					});
					br.getGlobalErrors().forEach(e -> {

						resultMap.put(e.getObjectName() + "",
								messageSource.getMessage(e, LocaleContextHolder.getLocale()));
					});
				}
			});
		}
		if (resultMap.size() > 0) {
			throw  new ServiceException("400", "参数错误:"+ JSON.toJSONString(resultMap));
		} else {
			Object result = pjp.proceed();// result的值就是被拦截方法的返回值
			return result;
		}

	}
	public Method getProceedJoinPointMethod(ProceedingJoinPoint pjp)
			throws NoSuchMethodException, SecurityException {
		Method method = ((MethodSignature) pjp.getSignature()).getMethod();
		Method realMethod = pjp.getTarget().getClass().getDeclaredMethod(method.getName(), method.getParameterTypes());
		return realMethod;
	}
	public static <T> List<T> getClassFromArray(Object[] data,Class<T> object) {
		List<T> result=new ArrayList<>();
		for(int i=0;i<data.length;i++) {
			if(object.isInstance(data[i])) {
				result.add( (T) data[i]);
			}
		}
		return result;
	}
}
