package com.starcharge.wios.vo;

import com.starcharge.wios.validation.CreateEntityGroup;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.Range;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;

/**
 * @projectName:xr-wios
 * @packageName:com.starcharge.wios.vo
 * @Description:TODO
 * @Author:luyincheng
 * @CreateDate: 2023/10/818:13
 */
@Data
public class DeliveryAddVo {

    /**
     * 发货单号
     *
     * @mbg.generated
     */
    @ApiModelProperty(value = "发货单号")
    private String deliveryOrder;

    /**
     * 安装单号
     *
     * @mbg.generated
     */
    @ApiModelProperty(value = "安装单号")
    @NotNull(groups = CreateEntityGroup.class,message = "缺少安装单号")
    @NotEmpty(groups = CreateEntityGroup.class,message = "缺少安装单号")
    private String installOrder;

    /**
     * 产品批次
     *
     * @mbg.generated
     */
    @ApiModelProperty(value = "产品批次",required = true)
    @NotNull(groups = CreateEntityGroup.class,message = "缺少产品批次")
    @NotEmpty(groups = CreateEntityGroup.class,message = "缺少产品批次")
    private String productBatch;

    /**
     * 发货数量
     *
     * @mbg.generated
     */
    @ApiModelProperty(value = "发货数量")
    @Min(value = 1,groups = CreateEntityGroup.class,message = "发货数量必须大于0")
    private Integer quantity;

    /**
     * 物料类别
     *
     * @mbg.generated
     */
    @ApiModelProperty(value = "物料类别",required = true)
    @NotNull(groups = CreateEntityGroup.class,message = "缺少物料类别")
    @NotEmpty(groups = CreateEntityGroup.class,message = "缺少物料类别")
    private String materialCategory;

    /**
     * 物料编码
     *
     * @mbg.generated
     */
    @ApiModelProperty(value = "物料编码",required = true)
    @NotNull(groups = CreateEntityGroup.class,message = "缺少物料编码")
    @NotEmpty(groups = CreateEntityGroup.class,message = "缺少物料编码")
    private String materialCode;

    /**
     * 物料名称
     *
     * @mbg.generated
     */
    @ApiModelProperty(value = "物料名称",required = true)
    @NotNull(groups = CreateEntityGroup.class,message = "缺少物料名称")
    @NotEmpty(groups = CreateEntityGroup.class,message = "缺少物料名称")
    private String materialName;

    /**
     * 物料厂商
     *
     * @mbg.generated
     */
    @ApiModelProperty(value = "物料厂商",required = true)
    @NotNull(groups = CreateEntityGroup.class,message = "缺少物料厂商")
    @NotEmpty(groups = CreateEntityGroup.class,message = "缺少物料厂商")
    private String materialCompany;

    /**
     * 物流单号
     *
     * @mbg.generated
     */
    @ApiModelProperty(value = "物流单号",required = true)
//    @NotNull(groups = CreateEntityGroup.class)
//    @NotEmpty(groups = CreateEntityGroup.class)
    private String logisticOrder;

    /**
     * 物流服务商
     *
     * @mbg.generated
     */
    @ApiModelProperty(value = "物流服务商",required = true)
//    @NotNull(groups = CreateEntityGroup.class)
//    @NotEmpty(groups = CreateEntityGroup.class)
    private String logisticCompany;

    /**
     * 物流状态  未发货  已发货
     *
     * @mbg.generated
     */
    @ApiModelProperty(value = "物流状态  未发货  已发货")
    private String logisticStatus;
    /**
     * 申请团队
     *
     * @mbg.generated
     */
    @ApiModelProperty(value = "申请团队")
    @NotNull(groups = CreateEntityGroup.class,message = "缺少申请团队")
    @NotEmpty(groups = CreateEntityGroup.class,message = "缺少申请团队")
    private String team;

    /**
     * 是否产品或辅料
     *
     * @mbg.generated
     */
    @ApiModelProperty(value = "是否产品或辅料 1 是 0 否")
    @NotNull(groups = CreateEntityGroup.class)
    @Range(min=0,max = 1,message = "是否产品或辅料只可填0或1",groups =CreateEntityGroup.class )
    private Integer isproduct;
}
