package com.starcharge.wios.wx.call;

import com.ihidea.core.util.HttpClientUtils;
import com.ihidea.core.util.JSONUtilsEx;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Value;

import java.text.MessageFormat;

/**
 * 微信公众号帮助类
 * @author Kevin
 *
 */
public class WeixinHelper{

	private static Log logger = LogFactory.getLog(WeixinHelper.class);

	/**
	 * 	通过code换取网页授权access_token
	 */
	private static final String SNS_API_URL_GET_ACCESS_TOKEN = "https://api.weixin.qq.com/sns/oauth2/access_token?appid={0}&secret={1}&code={2}&grant_type=authorization_code";

	private static String weixinAppId;

	@Value("${weixin.appId}")
	public void setWeixinAppId(String appId){
		WeixinHelper.weixinAppId = appId;
	}

	private static String weixinAppSecret;

	@Value("${weixin.appSecret}")
	public void setWeixinAppSecret(String appSecret){
		WeixinHelper.weixinAppSecret = appSecret;
	}

	/**
	 * 通过code换取网页授权access_token
	 * 
	 * @Title: getAccessTokenByCode
	 * @Description: 通过code换取网页授权access_token,此access_token与基础支持的access_token不同
	 * @param appid
	 *            公众号的唯一标识
	 * @param secret
	 *            公众号的appsecret
	 * @param code
	 *            获取的code参数
	 * @return AccessToken 网页授权接口调用凭证,注意：此access_token与基础支持的access_token不同
	 * @see AccessToken
	 */
	public static AccessToken getAccessTokenByCode(String appid, String secret, String code){
		String url = MessageFormat.format(SNS_API_URL_GET_ACCESS_TOKEN, appid, secret, code);
		AccessToken token = JSONUtilsEx.deserialize(HttpClientUtils.get(url, null, "UTF-8", "UTF-8"), AccessToken.class);
		return token;
	}

	public static String getOpenIdByCode(String code){
		AccessToken accessToken = getAccessTokenByCode(weixinAppId, weixinAppSecret, code);
		return accessToken!=null ? accessToken.getOpenid() : null;
	}

	
	public static void main(String[] args) {

	}
}
