package com.starcharge.wios.wx.permisson;

import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.utils.SpringContextUtil;
import org.apache.commons.lang.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.Ordered;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;

/**
 * @author kevin
 * @create 2020/10/30 18:12
 */
@Component
@Aspect
public class MinaPermissionAspect implements Ordered {

    @Autowired
    private TokenService tokenService;

    @Before(value = "@within(com.starcharge.wios.wx.permisson.MinaPermission) || @annotation(com.starcharge.wios.wx.permisson.MinaPermission)")
    public void beforeMethod(JoinPoint joinPoint){
        MethodSignature signature = (MethodSignature)joinPoint.getSignature();
        Method method = signature.getMethod();
        MinaPermission minaPermission = method.getAnnotation(MinaPermission.class);
        if(minaPermission == null){
            minaPermission = method.getDeclaringClass().getAnnotation(MinaPermission.class);
        }
        if(minaPermission != null && minaPermission.require()){
            String token = SpringContextUtil.getToken();
            if(StringUtils.isBlank(token) || !tokenService.verifyMinaToken(token)){
                throw new ServiceException("401", "登录失效，请重新登录");
            }
        }
    }

    @Override
    public int getOrder() {
        return Integer.MAX_VALUE;
    }
}
