package com.starcharge.wios.service;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.starcharge.wios.dao.OemContractDao;
import com.starcharge.wios.dao.OemContractItemDao;
import com.starcharge.wios.dao.OemContractPriceDao;
import com.starcharge.wios.dao.OrderDao;
import com.starcharge.wios.dao.OrderInstallDao;
import com.starcharge.wios.dao.RegionDao;
import com.starcharge.wios.dao.entity.OemContract;
import com.starcharge.wios.dao.entity.OemContractPrice;
import com.starcharge.wios.dao.entity.Order;
import com.starcharge.wios.dao.entity.Region;
import com.starcharge.wios.dao.entity.SettlementDetail;
import com.starcharge.wios.vo.OemContractItemVo;
import com.starcharge.wios.vo.OrderInstallVo;

@RunWith(SpringJUnit4ClassRunner.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class SettlementServiceTest {
    
    @Autowired
    SettlementService settlementService;
    
    private Logger log = LoggerFactory.getLogger(getClass());
    
    @Autowired
    OrderDao orderDao;
    
    @Autowired
    OemContractDao oemContractDao;
    
    @Autowired
    OemContractPriceDao oemContractPriceDao;
    
    @Autowired
    OrderInstallDao orderInstallDao;
    
    @Autowired
    OemContractItemDao oemContractItemDao;
    
    @Autowired
    RegionDao regionDao;
    
    @Test
    public void test() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, -10);
        Date date = cal.getTime();
        settlementService.generateOemDetail(date, new Date());
    }
    
    @Test
    public void test2() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, -10);
        Date date = cal.getTime();
        settlementService.generateOemSettlement(date, new Date());
    }
    
    @Test
    public void test3() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, -10);
        Date date = cal.getTime();
        settlementService.generateSupplierDetail(date, new Date());
    }
    
    @Test
    public void test4() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, -10);
        Date date = cal.getTime();
        settlementService.generateSupplierSettlement(date, new Date());
    }
    
    @Test
    public void test5() {
        Order order = orderDao.selectById("CA20201120141911");
        if (order.getContractId() == null)
            return;
        OemContract contract = oemContractDao.selectById(order.getContractId());
        List<OemContractPrice> prices = oemContractPriceDao.selectPureByContractId(contract.getId());// 查询合同关联的区域安装价格
        
        List<OrderInstallVo> orderInstallList = orderInstallDao.selectByOrderIds(order.getId());// 查询安装订单
        if (orderInstallList.isEmpty()) {
            log.error("[generateOemDetail]未找到安装订单,orderId={}", order.getId());
            return;
        }
        OrderInstallVo orderInstall = orderInstallList.get(0);
        SettlementDetail detail = new SettlementDetail();
        String wallboxItemId = orderInstall.getWallboxItemId();
        List<OemContractItemVo> contractItemList = oemContractItemDao.selectByContractIdAndItemId(contract.getId(), wallboxItemId);// 查询设备价格
        if (!contractItemList.isEmpty()) {
            OemContractItemVo contractItem = contractItemList.get(0);
            BigDecimal price = contractItem.getPrice();// 单价（不含税）
            BigDecimal ratePrice = contractItem.getRatePrice();// 单价（含税）
            detail.setOrderId(orderInstall.getOrderId());
            detail.setUserName(orderInstall.getUserName());
            detail.setUserPhone(orderInstall.getUserPhone());
            detail.setDeviceFee(price);
            detail.setDeviceFeeWithTax(ratePrice);
            detail.setInstallFinishTime(orderInstall.getInstallFinishTime());
            
            // 查询区域价格 没有的话 继续向上一级查找
            {
                Integer regionId = orderInstall.getRegionId();
                Region region = regionDao.selectById(regionId);
                String idTree = region.getIdTree();
                String[] idTreeArr = idTree.split("_");
                OemContractPrice contractPrice = null;
                for (int i = idTreeArr.length - 1; i >= 0; i--) {
                    final int iFinal = i;
                    List<OemContractPrice> filterResult = prices.stream()
                        .filter(item -> item.getRegionId().toString().equals(idTreeArr[iFinal])).collect(Collectors.toList());
                    if (!filterResult.isEmpty()) {
                        contractPrice = filterResult.get(0);
                        break;
                    }
                }
                if (contractPrice != null) {
                    detail.setInstallServiceFee(contractPrice.getInstallationPrice().add(contractPrice.getSurveyPrice()));
                    detail.setInstallServiceFeeWithTax(contractPrice.getInstallationRatePrice().add(contractPrice.getSurveyRatePrice()));
                } else {
                    log.error("[generateOemDetail]未找到contractPrice,orderId={},regionId={}，idTree={}", order.getId(), regionId, idTree);
                    // continue;
                }
            }
            detail.setTotalFee(
                detail.getDeviceFee().add(detail.getInstallServiceFee() == null ? BigDecimal.ZERO : detail.getInstallServiceFee()));
            detail.setTotalFeeWithTax(detail.getDeviceFeeWithTax()
                .add(detail.getInstallServiceFeeWithTax() == null ? BigDecimal.ZERO : detail.getInstallServiceFeeWithTax()));
            // settlementDetailDao.insert(detail);
        } else {
            log.error("[generateOemDetail]未找到设备价格,orderId={}，wallboxNo={}", order.getId(), wallboxItemId);
        }
    }
}
